@@ -40,7 +40,7 @@ struct _selector {
     GtkWidget *extra2;
     int code;
     int error;
-    unsigned long opts;
+    gretlopt opts;
     char *cmdlist;
     gpointer data;
 };
@@ -444,8 +444,10 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 
     if (MODEL_CODE(sr->code)) {
 	if (rows > 0) { 
-	    xlist = realloc(xlist, (rows + 1) * sizeof(int));
-	    if (xlist != NULL) xlist[0] = rows;
+	    xlist = realloc(xlist, (rows + 1) * sizeof *xlist);
+	    if (xlist != NULL) {
+		xlist[0] = rows;
+	    }
 	}
     }
     for (i=0; i<rows; i++) {
@@ -454,15 +456,18 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 	gtk_clist_get_text(GTK_CLIST(sr->rightvars), i, 0, &rvar);
 	add_to_cmdlist(sr, " ");
 	add_to_cmdlist(sr, rvar);
-	if (MODEL_CODE(sr->code) && xlist != NULL) 
+	if (MODEL_CODE(sr->code) && xlist != NULL) { 
 	    xlist[i+1] = atoi(rvar);
+	}
     }
 
     if (sr->code == TSLS || sr->code == VAR) {
 	rows = GTK_CLIST(sr->auxvars)->rows;
 	if (rows > 0) {
-	    auxlist = realloc(auxlist, (rows + 1) * sizeof(int));
-	    if (auxlist != NULL) auxlist[0] = rows;
+	    auxlist = realloc(auxlist, (rows + 1) * sizeof *auxlist);
+	    if (auxlist != NULL) {
+		auxlist[0] = rows;
+	    }
 	    add_to_cmdlist(sr, " ;");
 	    for (i=0; i<rows; i++) {
 		gchar *inst;
@@ -470,7 +475,9 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 		gtk_clist_get_text(GTK_CLIST(sr->auxvars), i, 0, &inst);
 		add_to_cmdlist(sr, " ");
 		add_to_cmdlist(sr, inst);
-		if (auxlist != NULL) auxlist[i+1] = atoi(inst);
+		if (auxlist != NULL) {
+		    auxlist[i+1] = atoi(inst);
+		}
 	    }
 	} else if (sr->code == TSLS) {
 	    errbox(_("You must specify a set of instrumental variables"));
@@ -497,7 +504,7 @@ static void destroy_selector (GtkWidget *w, selector *sr)
 {
     free(sr->cmdlist);
     free(sr);
-    open_dialog = NULL;
+    set_open_dialog(NULL);
 }
 
 static char *est_str (int cmdnum)
@@ -908,7 +915,7 @@ static void selector_init (selector *sr, guint code, const char *title)
     sr->opts = 0L;
 
     sr->dlg = gtk_dialog_new();
-    open_dialog = sr->dlg;
+    set_open_dialog(sr->dlg);
 
     gtk_window_set_title(GTK_WINDOW(sr->dlg), title);
 
@@ -1075,6 +1082,7 @@ void delete_selection_dialog (selector *sr)
 
 void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode) 
 {
+    GtkWidget *open_dialog;
     GtkWidget *right_vbox, *tmp;
     GtkWidget *big_hbox, *indepvar_hbox;
     GtkWidget *button_vbox, *scroller;
@@ -1082,6 +1090,7 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
     char topstr[48];
     int i;
 
+    open_dialog = get_open_dialog();
     if (open_dialog != NULL) {
 	gdk_window_raise(open_dialog->window);
 	return;
@@ -1359,11 +1368,13 @@ static GtkWidget *selection_top_label (int code)
 void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
 		       gpointer p) 
 {
+    GtkWidget *open_dialog;
     GtkWidget *left_vbox, *mid_vbox, *right_vbox, *tmp;
     GtkWidget *top_hbox, *big_hbox, *scroller;
     selector *sr;
     int i;
 
+    open_dialog = get_open_dialog();
     if (open_dialog != NULL) {
 	gdk_window_raise(open_dialog->window);
 	return;
@@ -1633,7 +1644,7 @@ gpointer selector_get_data (const selector *sr)
     return sr->data;
 }
 
-unsigned long selector_get_opts (const selector *sr)
+gretlopt selector_get_opts (const selector *sr)
 {
     return sr->opts;
 }
@@ -1642,3 +1653,19 @@ int selector_error (const selector *sr)
 {
     return sr->error;
 }
+
+void maybe_clear_selector (const int *dlist)
+{
+    int i, j;
+
+    if (xlist != NULL) {
+	for (i=1; i<=xlist[0]; i++) {
+	    for (j=1; j<=dlist[0]; j++) {
+		if (xlist[i] >= dlist[j]) {
+		    clear_selector();
+		    return;
+		}
+	    }
+	}
+    }
+}
