@@ -72,12 +72,6 @@ static double scalepos = 75.0; /* was 60 */
 static char boxfont[64] = "Helvetica";
 static int boxfontsize = 12;
 
-#ifndef OLD_GTK
-/* from gtkextra's gtkplotps.c (modified) */
-extern void psleave (GtkPlotPC *pc);
-extern gboolean psinit (GtkPlotPC *pc);
-#endif
-
 static int five_numbers (gpointer data);
 static int dump_boxplot (PLOTGROUP *grp);
 
@@ -199,7 +193,7 @@ static gint box_popup_activated (GtkWidget *w, gpointer data)
     }
 #endif
     else if (!strcmp(item, _("Help"))) {
-	context_help (NULL, GINT_TO_POINTER(BXPLOT));
+	context_help(NULL, GINT_TO_POINTER(BXPLOT));
     }
     else if (!strcmp(item, _("Close"))) { 
 	gtk_widget_destroy(grp->popup);
@@ -243,17 +237,13 @@ static GtkWidget *build_menu (PLOTGROUP *grp)
 	    continue;
 	}
 	item = gtk_menu_item_new_with_label(_(items[i]));
-#ifndef OLD_GTK	
         g_signal_connect(G_OBJECT(item), "activate",
 			 G_CALLBACK(box_popup_activated),
 			 _(items[i]));
 	g_object_set_data(G_OBJECT(item), "group", grp);
+#ifndef OLD_GTK	
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 #else
-        gtk_signal_connect(GTK_OBJECT(item), "activate",
-                           (GtkSignalFunc) box_popup_activated,
-                           _(items[i]));
-	gtk_object_set_data(GTK_OBJECT(item), "group", grp);
         gtk_menu_append(GTK_MENU(menu), item);
 #endif
 	gtk_widget_show(item);
@@ -715,13 +705,13 @@ make_area (PLOTGROUP *grp)
     
     gtk_widget_set_sensitive(grp->area, TRUE);
 
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(grp->area), "configure_event",
 		     G_CALLBACK(configure_event), grp);
 
     g_signal_connect(G_OBJECT(grp->area), "expose_event",
 		     G_CALLBACK(expose_event), grp);
 
+#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(grp->area), "button_press_event", 
 		     G_CALLBACK(box_popup), grp);
 
@@ -734,12 +724,6 @@ make_area (PLOTGROUP *grp)
     gtk_widget_set_size_request (GTK_WIDGET(grp->area),
 				 grp->width, grp->height); 
 #else
-    gtk_signal_connect(GTK_OBJECT(grp->area), "configure_event",
-		       GTK_SIGNAL_FUNC(configure_event), grp);
-
-    gtk_signal_connect(GTK_OBJECT(grp->area), "expose_event",
-		       GTK_SIGNAL_FUNC(expose_event), grp);
-
     gtk_signal_connect(GTK_OBJECT(grp->window), "button_press_event", 
 		       GTK_SIGNAL_FUNC(box_popup), grp);
 
@@ -1061,7 +1045,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
     plotgrp->saved = 0;
 
     plotgrp->nplots = list[0];
-    plotgrp->plots = mymalloc (plotgrp->nplots * sizeof(BOXPLOT));
+    plotgrp->plots = mymalloc(plotgrp->nplots * sizeof *plotgrp->plots);
     if (plotgrp->plots == NULL) {
 	free(plotgrp);
 	free(x);
@@ -1069,7 +1053,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
     }
 
     for (i=0, j=0; i<plotgrp->nplots; i++, j++) {
-	n = ztox(list[i+1], x, *pZ, pdinfo);
+	n = ztox(list[i+1], x, (const double **) *pZ, pdinfo);
 	if (n < 2) {
 	    sprintf(errtext, _("Dropping %s: insufficient observations"),
 		    pdinfo->varname[list[i+1]]);
@@ -1120,7 +1104,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 
     if (plotgrp->show_outliers) {
 	for (i=0; i<plotgrp->nplots; i++) {
-	    n = ztox(list[i+1], x, *pZ, pdinfo);
+	    n = ztox(list[i+1], x, (const double **) *pZ, pdinfo);
 	    qsort(x, n, sizeof *x, compare_doubles);
 	    add_outliers(x, n, &plotgrp->plots[i]);
 	}
@@ -1351,12 +1335,9 @@ static void read_boxrc (PLOTGROUP *grp)
 
     fp = fopen(".boxplotrc", "r");
     if (fp == NULL) {
-	char *homedir, boxrc[MAXLEN];
+	char boxrc[MAXLEN];
 
-	homedir = getenv("HOME");
-	if (homedir == NULL) homedir = paths.userdir;
-	sprintf(boxrc, "%s%s.boxplotrc", homedir,
-		(homedir[strlen(homedir)-1] != SLASH)? SLASHSTR : "");
+	sprintf(boxrc, "%s.boxplotrc", paths.userdir);
 	fp = fopen(boxrc, "r");
     }
 
@@ -1623,34 +1604,57 @@ static int special_varcount (const char *s)
 
 /* remove extra spaces around operators in boxplots line */
 
-static char *delete_spaces_in_parens (const char *line)
+static char *boxplots_fix_parentheses (const char *line)
 {
     char *s, *p;
     int inparen = 0;
+    int i, flen, len = strlen(line);
 
-    s = malloc(strlen(line) + 1);
+    /* make room to insert space before parens, if needed */
+    flen = len;
+    for (i=0; i<len; i++) {
+	if (i > 0 && line[i] == '(' && line[i-1] != ' ') {
+	    flen++;
+	}
+    }
+
+    s = malloc(flen + 1);
     if (s == NULL) return NULL;
 
     p = s;
-
-    while (*line) {
-	if (*line == '(') inparen = 1;
-	if (*line == ')') {
-	    if (inparen == 1) inparen = 0;
-	    else {
+    for (i=0; i<len; i++) {
+	if (line[i] == '(') {
+	    if (i > 0 && line[i-1] != ' ') {
+		*p++ = ' ';
+	    }
+	    inparen = 1;
+	}
+	if (line[i] == ')') {
+	    if (inparen == 1) {
+		inparen = 0;
+	    } else {
 		free(s);
 		return NULL;
 	    }
 	}
-	if (inparen && *line == ' ') ;
-	else *p++ = *line;
-	line++;
+	if (inparen && line[i] == ' ') ;
+	else *p++ = line[i];
     }
 
     *p = 0;
+
     return s;
 }
 
+#define BPSTRLEN 128
+
+static char boxplots_string[BPSTRLEN];
+
+const char *get_boxplots_string (void)
+{
+    return boxplots_string;
+}
+
 int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo, 
 		      int notches)
 {
@@ -1662,7 +1666,7 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
     if (!strncmp(str, "boxplots ", 9)) str += 9;
     else if (!strncmp(str, "boxplot ", 8)) str += 8;
 
-    s = delete_spaces_in_parens(str);
+    s = boxplots_fix_parentheses(str);
     if (s == NULL) return 1;
 
     nvars = special_varcount(s);
@@ -1678,12 +1682,18 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
-    for (i=0; i<nvars; i++) bools[i] = NULL;
+    for (i=0; i<nvars; i++) {
+	bools[i] = NULL;
+    }
 
     list[0] = nvars;
     i = 0;
     nbool = 0;
 
+    /* record the command string */
+    *boxplots_string = '\0';
+    strncat(boxplots_string, s, BPSTRLEN - 1);
+
     while (!err && (tok = strtok((i)? NULL : s, " "))) {
 	if (tok[0] == '(') {
 	    if (i) {
@@ -1718,8 +1728,6 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    free(s);
-
     /* now we add nbool new variables, with ID numbers origv,
        origv + 1, and so on.  These are the original variables
        that have boolean conditions attached, masked by those
@@ -1733,7 +1741,7 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 	    char formula[80];
 	    
 	    sprintf(formula, "bool_%d = %s", i-1, bools[i-1]);
-	    err = generate(pZ, pdinfo, formula, 0, NULL, 0);
+	    err = generate(pZ, pdinfo, formula, NULL);
 	    if (err) {
 		char errtxt[128];
 
@@ -1756,14 +1764,17 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!err) 
+    if (!err) {
 	err = boxplots(list, bools, pZ, pdinfo, notches);
+    } 
     
     free(list);
     free(bools); /* the bool[i]s are now attached to the plots */
+    free(s);
 
-    if (nbool) 
+    if (nbool) {
 	dataset_drop_vars(nbool, pZ, pdinfo);
+    }
     
     return err;
 }
