@@ -26,6 +26,8 @@
 #include "datafiles.h"
 #include "ssheet.h"
 #include "textbuf.h"
+#include "textutil.h"
+#include "boxplots.h"
 
 #ifdef OLD_GTK
 # include <gtkextra/gtkiconfilesel.h>
@@ -33,9 +35,6 @@
 # include "treeutils.h"
 #endif
 
-extern void do_samplebool (GtkWidget *widget, dialog_t *ddata);
-extern double **fullZ;
-
 /* these live in dialogs.c */
 extern GtkWidget *active_edit_id;
 extern GtkWidget *active_edit_name;
@@ -152,43 +151,37 @@ gboolean listbox_drag (GtkWidget *listbox, GdkEventMotion *event,
     return FALSE;
 }
 
-#else /* now an old gtk function */
+#else /* now comes an old gtk function */
 
 void selectrow (GtkCList *clist, gint row, gint column, 
 	        GdkEventButton *event, gpointer data) 
 {
-    gchar *numstr, *edttext, addvar[VNAMELEN];
+    gchar *numstr, *edttext;
     windata_t *win = (windata_t *) data;
 
     if (win == mdata) { /* main window */
-	gtk_clist_get_text(GTK_CLIST(clist), row, 0, &numstr);
+	gtk_clist_get_text(clist, row, 0, &numstr);
 	win->active_var = atoi(numstr);
     } else {
 	win->active_var = row;
     }
 
     if (active_edit_id != NULL) {
-	edttext = gtk_entry_get_text (GTK_ENTRY (active_edit_id));
+	gchar addvar[VNAMELEN];
+
+	edttext = gtk_entry_get_text(GTK_ENTRY(active_edit_id));
 	if (*edttext != '\0') {
 	    sprintf(addvar, " %d", win->active_var);
 	} else {
 	    sprintf(addvar, "%d", win->active_var);
 	}
-	gtk_entry_append_text(GTK_ENTRY (active_edit_id), addvar);
-    }
-    else if (active_edit_name != NULL) {
-	edttext = gtk_entry_get_text (GTK_ENTRY (active_edit_name));
-	gtk_entry_append_text(GTK_ENTRY (active_edit_name), 
+	gtk_entry_append_text(GTK_ENTRY(active_edit_id), addvar);
+    } else if (active_edit_name != NULL) {
+	edttext = gtk_entry_get_text (GTK_ENTRY(active_edit_name));
+	gtk_entry_append_text(GTK_ENTRY(active_edit_name), 
 			      datainfo->varname[win->active_var]);
-	gtk_entry_append_text(GTK_ENTRY (active_edit_name), " ");
+	gtk_entry_append_text(GTK_ENTRY(active_edit_name), " ");
     }
-# if 0
-    else if (active_edit_text != NULL) {
-	gtk_text_insert (GTK_TEXT (active_edit_text), fixed_font, 
-			 NULL, NULL, datainfo->varname[win->active_var],
-			 strlen(datainfo->varname[win->active_var]));
-    }
-# endif
 
     /* response to double-click */
     if (event != NULL && event->type == GDK_2BUTTON_PRESS 
@@ -197,6 +190,23 @@ void selectrow (GtkCList *clist, gint row, gint column,
     }
 }
 
+void unselectrow (GtkCList *clist, gint row, gint column, 
+		  GdkEventButton *event, gpointer data) 
+{
+    windata_t *win = (windata_t *) data;
+
+    if (win != mdata) { /* main window */
+	return;
+    } else {
+	gchar *numstr;
+
+	gtk_clist_get_text(clist, row, 0, &numstr);
+	if (win->active_var == atoi(numstr)) {
+	    win->active_var = clist->focus_row;
+	}
+    }
+}
+
 #endif /* old versus new gtk */
 
 /* ........................................................... */
@@ -274,6 +284,7 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
     case EXPORT_R:
     case EXPORT_R_ALT:
     case EXPORT_OCTAVE:
+    case EXPORT_DAT:
 	data_save_selection_wrapper(file_code);
 	break;
     case SAVE_TEX_TAB:
@@ -472,19 +483,19 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
 
     if (action == ADD || action == OMIT) {
 	strcpy(title, _("gretl: model tests"));
-	simple_selection (title, do_add_omit, action, vwin);
+	simple_selection(title, do_add_omit, action, vwin);
 	return;
     }
 
     if (action == COEFFSUM) {
 	strcpy(title, _("gretl: model tests"));
-	simple_selection (title, do_coeff_sum, action, vwin);
+	simple_selection(title, do_coeff_sum, action, vwin);
 	return;
     }
 
     if (action == GR_PLOT) {
 	strcpy(title, _("gretl: model tests"));
-	simple_selection (title, do_graph_from_selector, action, vwin);
+	simple_selection(title, do_graph_from_selector, action, vwin);
 	return;
     }
 
@@ -496,7 +507,6 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
 void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 {
     char title[64], query[MAXLABEL], defstr[MAXLEN];
-    char startdate[OBSLEN], enddate[OBSLEN];
     void (*okfunc)() = NULL;
     guint varclick = VARCLICK_NONE;
     windata_t *mydata = (windata_t *) data;
@@ -522,35 +532,12 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	       "pseudo-random number generator:"));
 	okfunc = do_seed;
 	break; 
-    case SIM:
-	if (mdata->active_var < 0) return;
-	if (mdata->active_var == 0) {
-	    errbox(_("You can't simulate with the constant"));
-	    return;
-	}
-	ntodate(startdate, datainfo->t1 + 1, datainfo);
-	ntodate(enddate, datainfo->n - 1, datainfo);
-	strcpy(title, _("gretl: simulate variable"));
-	strcpy(query, _("Enter spec. for simulation:\n"
-	       "(start end parameters)\n"
-	       "You will probably want to consult the "
-	       "help on sim first"));
-	sprintf(defstr, "%s %s %s", startdate, enddate, 
-		datainfo->varname[mdata->active_var]);
-	okfunc = do_sim;
-	break; 
     case NULLDATA:
 	strcpy(title, _("gretl: simulation data"));
 	strcpy(query, _("Series length for simulation data set:"));
 	strcpy(defstr, "100");
 	okfunc = do_simdata;
 	break;         
-    case ADF:
-	strcpy(title, _("gretl: ADF test"));
-	strcpy(query, _("Lag order for ADF test:"));
-	strcpy(defstr, "1");
-	okfunc = do_dialog_cmd;
-	break;
     case SPEARMAN:
 	strcpy(title, _("gretl: rank correlation"));
 	strcpy(query, _("Enter two variables by name or number:"));
@@ -604,6 +591,7 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	strcpy(query, _("Specify variables to plot:"));
 	okfunc = do_box_graph;
 	varclick = VARCLICK_INSERT_NAME;
+	strcpy(defstr, get_boxplots_string());
 	break;	
     case NLS:
 	strcpy(title, _("gretl: nonlinear least squares"));
@@ -611,6 +599,11 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	okfunc = do_nls_model;
 	varclick = VARCLICK_INSERT_TEXT;
 	break;	
+    case RESTRICT:
+	strcpy(title, _("gretl: linear restrictions"));
+	strcpy(query, _("Specify restrictions:"));
+	okfunc = do_restrict;
+	break;	
     default:
 	errbox("Bug: unrecognized action code in gretl_callback");
 	return;
@@ -622,43 +615,6 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 
 /* ........................................................... */
 
-void delete_var_by_id (int id)
-{
-    int list[2];
-    int renumber, resp;
-    gchar *msg;
-
-    if (fullZ != NULL) {
-	errbox(_("Can't delete a variable when in sub-sample"
-		 " mode\n"));
-	return;
-    }    
-
-    msg = g_strdup_printf(_("Really delete %s?"), datainfo->varname[id]);
-    resp = yes_no_dialog(_("gretl: delete"), msg, 0);
-    g_free(msg);
-
-    if (resp != GRETL_YES) return;
-
-    sprintf(line, "delete %d", id);
-    if (verify_and_record_command(line)) return;
-
-    list[0] = 1;
-    list[1] = id;
-
-    if (dataset_drop_listed_vars(list, &Z, datainfo, & renumber)) {
-	errbox(_("Out of memory reorganizing data set"));
-    } else {
-	refresh_data();
-	if (renumber) {
-	    infobox(_("Take note: variables have been renumbered"));
-	}
-	mark_dataset_as_modified();
-    }
-}
-
-/* ........................................................... */
-
 void text_copy_callback (GtkWidget *w, gpointer data)
 {
     text_copy(data, COPY_SELECTION, w);
@@ -706,16 +662,9 @@ void file_save_callback (GtkWidget *w, gpointer data)
     guint u = 0;
     windata_t *vwin = (windata_t *) data;
 
-#ifndef OLD_GTK
     if (g_object_get_data(G_OBJECT(vwin->dialog), "text_out")) {
 	u = SAVE_OUTPUT;
-    } 
-#else
-    if (gtk_object_get_data(GTK_OBJECT(vwin->dialog), "text_out")) {
-	u = SAVE_OUTPUT;
-    } 
-#endif
-    else {
+    } else {
 	switch (vwin->role) {
 	case EDIT_SCRIPT:
 	case VIEW_SCRIPT:
@@ -777,20 +726,17 @@ static void fix_obsstr (char *str)
     }
 }
 
-static void prep_spreadsheet (GtkWidget *widget, dialog_t *data)
+static void prep_spreadsheet (GtkWidget *widget, dialog_t *ddata)
 {
-#ifndef OLD_GTK
-    const gchar *edttext;
-#else
-    gchar *edttext;
-#endif
+    const gchar *buf;
     char dataspec[32];
     char *test, stobs[OBSLEN], endobs[OBSLEN], firstvar[VNAMELEN];
     double sd0, ed0;
 
-    edttext = gtk_entry_get_text(GTK_ENTRY(data->edit));
-    strncpy(dataspec, edttext, 31);
-    if (dataspec[0] == '\0') return;
+    buf = dialog_data_get_text(ddata);
+    if (buf == NULL) return;
+
+    strncpy(dataspec, buf, 31);
 
     /* check validity of dataspec */
     if (sscanf(dataspec, "%8s %8s %8s", stobs, endobs, firstvar) != 3) {
@@ -903,7 +849,7 @@ static void prep_spreadsheet (GtkWidget *widget, dialog_t *data)
 	}	    
     }
 
-    gtk_widget_destroy(data->dialog); 
+    close_dialog(ddata);
 
     strcpy(datainfo->stobs, stobs);
     strcpy(datainfo->endobs, endobs);
