@@ -21,12 +21,7 @@
 
 #include "gretl.h"
 #include "console.h"
-
-/* from library.c */
-extern int gui_exec_line (char *line, 
-			  LOOPSET *plp, int *plstack, int *plrun, 
-			  PRN *prn, int exec_code, 
-			  const char *myname);
+#include "menustate.h"
 
 static GtkWidget *console_view;
 static PRN *console_prn;
@@ -37,6 +32,10 @@ static gchar *cbuf;
 static char **cmd_history;
 static int hl, hlmax, hlines;
 
+static LOOPSET *loop;
+static int loopstack, looprun;
+static int echo_off;
+
 static int gretl_console_init (void)
 {
     int i;
@@ -61,6 +60,10 @@ static int gretl_console_init (void)
 
     cbuf = NULL;
 
+    loop = NULL;
+    loopstack = looprun = 0;
+    echo_off = 0;
+
     return 0;
 }
 
@@ -183,10 +186,51 @@ static void console_scroll_to_end (GtkTextBuffer *buf,
 
 #endif
 
+enum {
+    SAMPLE_RECORD,
+    SAMPLE_CHECK
+};
+
+static int console_sample (const DATAINFO *pdinfo, int code)
+{
+    static int pd, t1, t2, ts;
+    static double sd0;
+
+    int ret = 0;
+
+    if (code == SAMPLE_RECORD) {
+	pd = pdinfo->pd;
+	t1 = pdinfo->t1;
+	t2 = pdinfo->t2;
+	ts = pdinfo->time_series;
+	sd0 = pdinfo->sd0;
+    }
+    else if (code == SAMPLE_CHECK) {
+	if (pdinfo->pd != pd ||
+	    pdinfo->t1 != t1 ||
+	    pdinfo->t2 != t2 ||
+	    pdinfo->time_series != ts ||
+	    pdinfo->sd0 != sd0) {
+	    ret = 1;
+	}
+    }
+
+    return ret;
+}
+
+void console_record_sample (const DATAINFO *pdinfo)
+{
+    console_sample(pdinfo, SAMPLE_RECORD);
+}
+
+static int sample_changed (const DATAINFO *pdinfo)
+{
+    return console_sample(pdinfo, SAMPLE_CHECK);
+}
+
 static void console_exec (void)
 {
     static int redirected;
-    int loopstack = 0, looprun = 0;
     int oldv = datainfo->v;
     char execline[MAXLEN];
 #ifndef OLD_GTK
@@ -208,7 +252,10 @@ static void console_exec (void)
 
     top_n_tail(cbuf);
 
-    if (strcmp(cbuf, "quit") == 0 || strcmp(cbuf, "q") == 0) {
+    if (strcmp(cbuf, "quit") == 0 || 
+	strcmp(cbuf, "q") == 0 ||
+	strcmp(cbuf, "exit") == 0 ||
+	strcmp(cbuf, "x") == 0) {
 	gtk_widget_destroy(console_view->parent->parent->parent);
 	g_free(cbuf);
 	cbuf = NULL;
@@ -226,10 +273,24 @@ static void console_exec (void)
     g_free(cbuf);
     cbuf = NULL;
 
-    push_history_line(execline);    
-    gui_exec_line(execline, NULL, &loopstack, &looprun, console_prn, 
+    console_record_sample(datainfo);
+
+    push_history_line(execline); 
+
+    gui_exec_line(execline, &loop, &loopstack, &looprun, console_prn, 
 		  CONSOLE_EXEC, NULL);
 
+    if (looprun) { 
+	if (loop_exec(loop, execline, 
+		      &Z, &datainfo, models,
+		      &echo_off, console_prn)) {
+	    return;
+	}
+	gretl_loop_destroy(loop);
+	loop = NULL;
+	looprun = 0;
+    }    
+
     if (console_prn->fp == NULL) redirected = 0;
 
 #ifndef OLD_GTK
@@ -269,11 +330,11 @@ static void console_exec (void)
 
 #ifndef OLD_GTK
     gtk_text_buffer_insert_with_tags_by_name(buf, &start, 
-					     "\n? ", 3,
+					     (loopstack)? "\n> " : "\n? ", 3,
 					     "redtext", NULL);
 #else
     gtk_text_insert(GTK_TEXT(console_view), fixed_font,
-		    &red, NULL, "\n? ", 3);
+		    &red, NULL, (loopstack)? "\n> " : "\n? ", 3);
     gtk_text_thaw(GTK_TEXT(console_view));
 #endif
 
@@ -284,10 +345,15 @@ static void console_exec (void)
     console_scroll_to_end();
 #endif
 
-    /* update variable listing in main window */
+    /* update variable listing in main window if needed */
     if (datainfo->v != oldv || !strncmp(execline, "rename", 6)) {
 	populate_varlist();
     }
+
+    /* update sample info and options if needed */
+    if (sample_changed(datainfo)) {
+	set_sample_label(datainfo);
+    }
 }
 
 void show_gretl_console (void)
@@ -504,7 +570,7 @@ gint console_key_handler (GtkWidget *w, GdkEventKey *key, gpointer d)
     return FALSE;
 }
 
-#else /* gtk versions */
+#else /* alternate gtk versions */
 
 static void replace_command_line (int cw, const char *repl)
 {
