@@ -24,6 +24,7 @@
 #include "database.h"
 #include "datafiles.h"
 #include "webget.h"
+#include "menustate.h"
 
 #if !GLIB_CHECK_VERSION(2,0,0)
 # define OLD_GTK
@@ -53,8 +54,6 @@ static int populate_remote_series_list (windata_t *dbwin, char *buf);
 static int rats_populate_series_list (windata_t *dbwin);
 static SERIESINFO *get_series_info (windata_t *ddata, int action);
 static void update_statusline (windata_t *windat, char *str);
-static void data_compact_dialog (GtkWidget *w, int spd, int *target_pd, 
-				 gint *compact_method);
 
 enum db_data_actions {
     DB_DISPLAY,
@@ -176,7 +175,7 @@ static void display_dbdata (double ***dbZ, DATAINFO *dbdinfo)
 
     if (bufopen(&prn)) return;
 
-    printdata(NULL, dbZ, dbdinfo, 0, OPT_O, prn);
+    printdata(NULL, (const double **) *dbZ, dbdinfo, OPT_O, prn);
 
     view_buffer(prn, 36, 350, _("gretl: display database series"), PRINT,
 		NULL); 
@@ -206,7 +205,7 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbdinfo)
     lines[0] = 1;
     list[0] = 2; list[1] = 1; list[2] = 2;
     err = gnuplot(list, lines, NULL, dbZ, dbdinfo,
-		  &paths, &plot_count, GP_GUI);
+		  &plot_count, GP_GUI);
 
     if (err < 0) {
 	errbox(_("gnuplot command failed"));
@@ -222,10 +221,25 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbdinfo)
 
 /* ........................................................... */
 
+static void 
+init_datainfo_from_sinfo (DATAINFO *pdinfo, SERIESINFO *sinfo)
+{
+    pdinfo->pd = sinfo->pd;
+    strcpy(pdinfo->stobs, sinfo->stobs);
+    strcpy(pdinfo->endobs, sinfo->endobs);
+    colonize_obs(pdinfo->stobs);
+    colonize_obs(pdinfo->endobs);
+    pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
+    pdinfo->n = sinfo->nobs;
+    pdinfo->v = 2;
+}
+
+/* ........................................................... */
+
 static void add_dbdata (windata_t *dbwin, double **dbZ, SERIESINFO *sinfo)
 {
     gint err = 0;
-    double *xvec;
+    double *xvec = NULL;
     int n, t, start, stop, pad1 = 0, pad2 = 0;
     int compact_method = COMPACT_AVG;
     int overwrite = 0;
@@ -272,31 +286,33 @@ static void add_dbdata (windata_t *dbwin, double **dbZ, SERIESINFO *sinfo)
 		return;
 	    }
 
-	    data_compact_dialog(dbwin->w, sinfo->pd, &datainfo->pd, &compact_method);
+	    data_compact_dialog(dbwin->w, sinfo->pd, &datainfo->pd, NULL, 
+				&compact_method);
 
 	    if (compact_method == COMPACT_NONE) {
-		if (!overwrite) dataset_drop_vars(1, &Z, datainfo);
+		if (!overwrite) {
+		    dataset_drop_vars(1, &Z, datainfo);
+		}
 		return;
 	    }
-	    if (sinfo->pd == 12 && datainfo->pd == 4) {
-		mon_to_quart(&xvec, dbZ[1], sinfo, compact_method);
-	    }
-	    else if (datainfo->pd == 1) {
-		to_annual(&xvec, dbZ[1], sinfo, compact_method);
-	    }
+	    xvec = compact_db_series(dbZ[1], sinfo, datainfo->pd, 
+				     compact_method);
 	} else {  
 	    /* series does not need compacting */
 	    xvec = mymalloc(sinfo->nobs * sizeof *xvec);
-	    if (xvec == NULL) {
-		errbox(_("Out of memory attempting to add variable"));
-		if (!overwrite) dataset_drop_vars(1, &Z, datainfo);
-		return;
-	    }
-	    for (t=0; t<sinfo->nobs; t++) {
-		xvec[t] = dbZ[1][t];
+	    if (xvec != NULL) {
+		for (t=0; t<sinfo->nobs; t++) {
+		    xvec[t] = dbZ[1][t];
+		}
 	    }
 	}
 
+	if (xvec == NULL) {
+	    errbox(_("Out of memory attempting to add variable"));
+	    if (!overwrite) dataset_drop_vars(1, &Z, datainfo);
+	    return;
+	}
+
 	/* common stuff for adding a var */
 	strcpy(datainfo->varname[dbv], sinfo->varname);
 	strcpy(VARLABEL(datainfo, dbv), sinfo->descrip);
@@ -326,14 +342,7 @@ static void add_dbdata (windata_t *dbwin, double **dbZ, SERIESINFO *sinfo)
 	free(xvec);
     } else {  
 	/* no data open: start new data set with this db series */
-	datainfo->pd = sinfo->pd;
-	strcpy(datainfo->stobs, sinfo->stobs);
-	strcpy(datainfo->endobs, sinfo->endobs);
-	colonize_obs(datainfo->stobs);
-	colonize_obs(datainfo->endobs);
-	datainfo->sd0 = get_date_x(datainfo->pd, datainfo->stobs);
-	datainfo->n = sinfo->nobs;
-	datainfo->v = 2;
+	init_datainfo_from_sinfo(datainfo, sinfo);
 
 	/* time series data? */
 	set_time_series(datainfo);
@@ -464,7 +473,7 @@ static void db_view_codebook (GtkWidget *w, windata_t *dbwin)
 }
 
 static void 
-codebook_callback_wrapper (gpointer p, guint v, GtkWidget *w)
+book_callback_wrapper (gpointer p, guint u, GtkWidget *w)
 {
     db_view_codebook(w, p);
 }
@@ -527,19 +536,6 @@ static void build_db_popup (windata_t *win, int cb)
 
 /* ........................................................... */
 
-static void add_codebook_to_menu (windata_t *vwin)
-{
-    GtkItemFactoryEntry item;
-    const gchar *mpath = N_("/_Codebook");
-
-    item.callback = codebook_callback_wrapper;
-    item.callback_action = 0;
-    item.item_type = NULL;
-    item.path = g_strdup(_(mpath));
-    gtk_item_factory_create_item(vwin->ifac, &item, vwin, 1);
-    g_free(item.path);
-}
-
 static void set_up_db_menu (windata_t *win, int cb)
 {
 #ifndef OLD_GTK
@@ -547,7 +543,11 @@ static void set_up_db_menu (windata_t *win, int cb)
 	{ N_("/_Series/_Display"), NULL, gui_get_series, DB_DISPLAY, NULL, GNULL },
 	{ N_("/_Series/_Graph"), NULL, gui_get_series, DB_GRAPH, NULL, GNULL },
 	{ N_("/_Series/_Import"), NULL, gui_get_series, DB_IMPORT, NULL, GNULL },
-	{ N_("/_Find"), NULL, menu_find, 1, NULL, GNULL },
+	{ N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
+	{ N_("/Find/_Find in window"), NULL, menu_find, 1, "<StockItem>", GTK_STOCK_FIND },
+	{ N_("/_Codebook"), NULL, NULL, 0, "<Branch>", GNULL },    
+	{ N_("/Codebook/_Open"), NULL, book_callback_wrapper, 0, "<StockItem>", 
+	  GTK_STOCK_OPEN }
     };
 #else
     GtkItemFactoryEntry db_items[] = {
@@ -555,20 +555,25 @@ static void set_up_db_menu (windata_t *win, int cb)
 	{ N_("/_Series/_Graph"), NULL, gui_get_series, DB_GRAPH, NULL },
 	{ N_("/_Series/_Import"), NULL, gui_get_series, DB_IMPORT, NULL },
 	{ N_("/_Find"), NULL, menu_find, 1, NULL },
+	{ N_("/_Codebook"), NULL, book_callback_wrapper, 0, NULL }
     };
 #endif
     gint n_items = sizeof db_items / sizeof db_items[0];
 
+    if (!cb) {
+#ifndef OLD_GTK
+	n_items -= 2;
+#else
+	n_items--;
+#endif
+    }
+
     win->ifac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", NULL);
 #ifdef ENABLE_NLS
     gtk_item_factory_set_translate_func(win->ifac, menu_translate, NULL, NULL);
 #endif
     gtk_item_factory_create_items(win->ifac, n_items, db_items, win);
     win->mbar = gtk_item_factory_get_widget(win->ifac, "<main>");
-
-    if (cb) {
-	add_codebook_to_menu(win);
-    }
 }
 
 /* ........................................................... */
@@ -589,18 +594,20 @@ static void destroy_db_win (GtkWidget *w, gpointer data)
 
 /* ........................................................... */
 
-static int db_has_codebook (const char *fname)
+static void test_db_book (const char *fname, int *cb)
 {
-    char cbname[MAXLEN];
+    char testname[MAXLEN];
     FILE *fp;
 
-    strcpy(cbname, fname);
-    strcat(cbname, ".cb");
-    
-    fp = fopen(cbname, "r");
-    if (fp == NULL) return 0;
-    fclose(fp);
-    return 1;
+    strcpy(testname, fname);
+    strcat(testname, ".cb");
+    fp = fopen(testname, "r");
+    if (fp == NULL) {
+	*cb = 0;
+    } else {
+	*cb = 1;
+	fclose(fp);
+    }
 }
 
 /* ........................................................... */
@@ -633,7 +640,8 @@ static int display_db_series_list (int action, char *fname, char *buf)
     char *titlestr;
     windata_t *dbwin;
     int db_width = 700, db_height = 420;
-    int cb, err = 0;
+    int cb = 0;
+    int err = 0;
 
     dbwin = mymalloc(sizeof *dbwin);
     if (dbwin == NULL) return 1;
@@ -672,7 +680,7 @@ static int display_db_series_list (int action, char *fname, char *buf)
     gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 10);
     gtk_container_add (GTK_CONTAINER (dbwin->w), main_vbox);
 
-    cb = db_has_codebook(fname);
+    test_db_book(fname, &cb);
     set_up_db_menu(dbwin, cb);
     build_db_popup(dbwin, cb);
 
@@ -707,11 +715,9 @@ static int display_db_series_list (int action, char *fname, char *buf)
 
     if (action == NATIVE_SERIES) { 
 	err = populate_series_list(dbwin);
-    } 
-    else if (action == REMOTE_SERIES) { 
+    } else if (action == REMOTE_SERIES) { 
 	err = populate_remote_series_list(dbwin, buf);
-    }
-    else {
+    } else {
 	err = rats_populate_series_list(dbwin);
     } 
 
@@ -749,16 +755,18 @@ static int check_serinfo (char *str, char *sername)
 
 /* ........................................................... */
 
-static void end_trim (char *line)
+static char *end_trim (char *s)
 {
-    size_t i, n = strlen(line);
+    size_t i, n = strlen(s);
 
     for (i=n-1; i>0; i--) {
-	if (line[i] == ' ' || line[i] == '\n' || line[i] == '\r')
-	    line[i] = 0;
-	else
+	if (s[i] == ' ' || s[i] == '\n' || s[i] == '\r') {
+	    s[i] = '\0';
+	} else {
 	    break;
+	}
     }
+    return s;
 }
 
 /* ........................................................... */
@@ -777,15 +785,18 @@ static char *start_trim (char *s)
 static int my_utf_validate (char *s)
 {
     if (!g_utf8_validate(s, -1, NULL)) {
-	gsize wrote;
 	gchar *new;
 
 # if 0
 	fprintf(stderr, "database: string '%s' does not utf-8 validate\n", s);
 # endif
-	new = g_locale_to_utf8(s, -1, NULL, &wrote, NULL);
-	strcpy(s, new);
-	g_free(new);
+	new = my_locale_to_utf8(s);
+	if (new != NULL) {
+	    strcpy(s, new);
+	    g_free(new);
+	} else {
+	    *s = '\0';
+	}
 	return 1;
     }
     return 0;
@@ -917,8 +928,6 @@ static int populate_remote_series_list (windata_t *dbwin, char *buf)
     char sername[9], line1[150], line2[150];
     int n, err = 0;
 
-    getbufline(NULL, NULL, 1);
-
 #ifndef OLD_GTK
     store = GTK_LIST_STORE(gtk_tree_view_get_model 
 			   (GTK_TREE_VIEW(dbwin->listbox)));
@@ -928,11 +937,11 @@ static int populate_remote_series_list (windata_t *dbwin, char *buf)
     i = 0;
 #endif
 
-    while (1) {
-	if (getbufline(buf, line1, 0) == 0) break;
+    bufgets(NULL, 0, buf);
+
+    while (bufgets(line1, sizeof line1, buf)) {
 	if (line1[0] == '#') continue;
 
-	line1[149] = 0;
 	end_trim(line1);
 	charsub(line1, '\t', ' ');
 
@@ -942,14 +951,15 @@ static int populate_remote_series_list (windata_t *dbwin, char *buf)
 
 	if (sscanf(line1, "%8s", sername) != 1) break;
 
-	sername[8] = 0;
 	n = strlen(sername);
 	row[0] = sername;
 	row[1] = start_trim(line1 + n + 1);
 
-	getbufline(buf, line2, 0);
+	bufgets(line2, sizeof line2, buf);
 	row[2] = line2;
-	if (!err) err = check_serinfo(line2, sername);
+	if (!err) {
+	    err = check_serinfo(line2, sername);
+	}
 
 #ifndef OLD_GTK
 	gtk_list_store_append(store, &iter);
@@ -983,7 +993,6 @@ static void insert_and_free_db_table (db_table *tbl, GtkCList *clist)
     gchar *comment;
     GtkTreeIter iter;
     GtkListStore *store;
-    gsize bytes;
 #else
     gchar *row[3];
 #endif
@@ -995,7 +1004,7 @@ static void insert_and_free_db_table (db_table *tbl, GtkCList *clist)
 
     for (i=0; i<tbl->nrows; i++) {    
 #ifndef OLD_GTK
-	comment = g_locale_to_utf8(tbl->rows[i].comment, -1, NULL, &bytes, NULL);
+	comment = my_locale_to_utf8(tbl->rows[i].comment);
 
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter, 
@@ -1003,7 +1012,9 @@ static void insert_and_free_db_table (db_table *tbl, GtkCList *clist)
 			   1, comment,
 			   2, tbl->rows[i].obsinfo, 
 			   -1);
-	g_free(comment);
+	if (comment != NULL) {
+	    g_free(comment);
+	}
 #else
 	row[0] = tbl->rows[i].varname;
 	row[1] = tbl->rows[i].comment;
@@ -1230,6 +1241,7 @@ static SERIESINFO *get_series_info (windata_t *win, int action)
     if (pdc == 'M') sinfo->pd = 12;
     else if (pdc == 'Q') sinfo->pd = 4;
     else if (pdc == 'B') sinfo->pd = 5;
+    else if (pdc == 'S') sinfo->pd = 6;
     else if (pdc == 'D') sinfo->pd = 7;
     else if (pdc == 'U') sinfo->undated = 1;
 
@@ -1455,16 +1467,16 @@ static int parse_db_header (const char *buf, size_t *idxlen,
 
 static int ggz_extract (char *errbuf, char *ggzname)
 {
-    FILE *fidx, *fbin, *fcb;
+    gzFile fgz = NULL;
+    FILE *fidx = NULL, *fbin = NULL, *fcbk = NULL;
     size_t idxlen, datalen, cblen, bytesleft, bgot;
     char idxname[MAXLEN], binname[MAXLEN], cbname[MAXLEN];
     char gzbuf[GRETL_BUFSIZE];
-    gzFile fgz;
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-    size_t offset;
     netfloat nf;
     float val;
 #endif
+    int err = 0;
 
     switch_ext(idxname, ggzname, "idx");
     switch_ext(binname, ggzname, "bin");
@@ -1478,26 +1490,23 @@ static int ggz_extract (char *errbuf, char *ggzname)
 
     fidx = fopen(idxname, "wb");
     if (fidx == NULL) {
-        gzclose(fgz);
         sprintf(errbuf, _("Couldn't open %s for writing\n"), idxname);
-        return 1;
+	err = 1;
+	goto bailout;
     }
 
     fbin = fopen(binname, "wb");
     if (fbin == NULL) {
-        gzclose(fgz);
-        fclose(fidx);
         sprintf(errbuf, _("Couldn't open %s for writing\n"), binname);
-        return 1;
+	err = 1;
+	goto bailout;
     }
 
-    fcb = fopen(cbname, "wb");
-    if (fcb == NULL) {
-	gzclose(fgz);
-	fclose(fidx);
-	fclose(fbin);
+    fcbk = fopen(cbname, "wb");
+    if (fcbk == NULL) {
 	sprintf(errbuf, _("Couldn't open %s for writing\n"), cbname);
-	return 1;
+	err = 1;
+	goto bailout;
     } 
 
     memset(gzbuf, GRETL_BUFSIZE, 0);
@@ -1506,15 +1515,11 @@ static int ggz_extract (char *errbuf, char *ggzname)
     if (parse_db_header(gzbuf, &idxlen, &datalen, &cblen)) {
 	fputs("Error reading info buffer: failed to get byte counts\n",
 	      stderr);
-	gzclose(fgz);
-	fclose(fidx);
-	fclose(fbin);
-	fclose(fcb);
-	return 1;
+	err = 1;
+	goto bailout;
     }
 
     bytesleft = idxlen;
-
     while (bytesleft > 0) {
 	memset(gzbuf, 0, GRETL_BUFSIZE);
 	bgot = gzread(fgz, gzbuf, (bytesleft > GRETL_BUFSIZE)? 
@@ -1524,17 +1529,19 @@ static int ggz_extract (char *errbuf, char *ggzname)
 	fwrite(gzbuf, 1, bgot, fidx);
     }
 
-    fclose(fidx);
+    if (bytesleft > 0) {
+	fputs("Error reading database info buffer\n", stderr);
+	err = 1;
+	goto bailout;
+    }
 
     bytesleft = datalen;
-
     while (bytesleft > 0) {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
         if ((bgot = gzread(fgz, gzbuf, sizeof(long) + sizeof(short))) > 0) {
 	    /* read "netfloats" and write floats */
 	    memcpy(&(nf.frac), gzbuf, sizeof(long));
-	    offset = sizeof(long);
-	    memcpy(&(nf.exp), gzbuf + offset, sizeof(short));
+	    memcpy(&(nf.exp), gzbuf + sizeof(long), sizeof(short));
 	    val = retrieve_float(nf);
 	    fwrite(&val, sizeof(float), 1, fbin);
 	    bytesleft -= sizeof(dbnumber);
@@ -1549,6 +1556,12 @@ static int ggz_extract (char *errbuf, char *ggzname)
 #endif
     }
 
+    if (bytesleft > 0) {
+	fputs("Error reading database data\n", stderr);
+	err = 1;
+	goto bailout;
+    }
+
     bytesleft = cblen;
     while (bytesleft > 0) {
 	memset(gzbuf, 0, GRETL_BUFSIZE);
@@ -1556,12 +1569,20 @@ static int ggz_extract (char *errbuf, char *ggzname)
 		      GRETL_BUFSIZE : bytesleft);
 	if (bgot <= 0) break;
 	bytesleft -= bgot;
-	fwrite(gzbuf, 1, bgot, fcb);
+	fwrite(gzbuf, 1, bgot, fcbk);
     }
 
-    gzclose(fgz);
-    fclose(fbin);
-    fclose(fcb);
+    if (bytesleft > 0) {
+	fputs("Error reading database codebook\n", stderr);
+	err = 1;
+    }    
+
+ bailout:
+
+    if (fgz != NULL) gzclose(fgz);
+    if (fidx != NULL) fclose(fidx);
+    if (fbin != NULL) fclose(fbin);
+    if (fcbk != NULL) fclose(fcbk);
 
     if (cblen == 0) {
 	remove(cbname);
@@ -1569,7 +1590,7 @@ static int ggz_extract (char *errbuf, char *ggzname)
 
     remove(ggzname);
 
-    return 0;
+    return err;
 }
 
 /* ........................................................... */
@@ -1597,6 +1618,7 @@ void grab_remote_db (GtkWidget *w, gpointer data)
 
     build_path(paths.binbase, dbname, ggzname, ".ggz");
 
+    /* test write to gzipped file */
     errno = 0;
     fp = fopen(ggzname, "w");
     if (fp == NULL) {
@@ -1680,12 +1702,14 @@ static gchar *get_descrip (char *fname, const char *dbdir)
 
     fgets(tmp, 63, fp);
     fclose(fp);
+
     if (tmp[0] == '#') {
 	strcpy(line, tmp + 2);
 	/* the following line was ifdef'd with G_OS_WIN32 */
 	line[strlen(line) - 1] = 0;
 	return line;
     }
+
     return NULL;
 }
 
@@ -1782,7 +1806,8 @@ gint populate_dbfilelist (windata_t *win)
 #ifndef G_OS_WIN32
     /* pick up any databases in the user's personal dir */
     dbdir = paths.userdir;
-    if ((dir = opendir(dbdir)) != NULL) {
+    dir = opendir(dbdir);
+    if (dir != NULL) {
 	while ((dirent = readdir(dir)) != NULL) {
 	    fname = dirent->d_name;
 	    n = strlen(fname);
@@ -1834,523 +1859,48 @@ gint populate_dbfilelist (windata_t *win)
     return 0;
 }
 
-/* .................................................................. */
-
-static void set_compact_type (GtkWidget *w, gpointer data)
+static void set_compact_info_from_default (int method)
 {
-    gint *method = (gint *) data;
+    int i;
 
-    if (GTK_TOGGLE_BUTTON (w)->active) {
-#ifndef OLD_GTK
-        *method = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
-#else
-        *method = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "action"));
-#endif
-    }
-}
-
-/* .................................................................. */
-
-static void abort_compact (GtkWidget *w, gpointer data)
-{
-    gint *method = (gint *) data;
-
-    *method = COMPACT_NONE;
-}
-
-/* .................................................................. */
-
-static void set_target_pd (GtkWidget *w, gpointer data)
-{
-    gint *pd = (gint *) data;
-
-    if (GTK_TOGGLE_BUTTON (w)->active) {
-#ifndef OLD_GTK
-        *pd = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
-#else
-        *pd = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "action"));
-#endif
-    }
-}
-
-static void pd_buttons (dialog_t *d, int *target_pd)
-{    
-    GtkWidget *button, *hs;
-    GSList *group;
-    gint quart = 4, ann = 1;
-
-    button = gtk_radio_button_new_with_label(NULL, _("Quarterly"));
-    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d->dialog)->vbox), 
-			button, TRUE, TRUE, FALSE);
-
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-
-#ifndef OLD_GTK
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_target_pd), target_pd);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(quart));
-#else
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		       GTK_SIGNAL_FUNC(set_target_pd), target_pd);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(quart));
-#endif
-    gtk_widget_show (button);
-
-    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON (button));
-    button = gtk_radio_button_new_with_label (group, _("Annual"));
-    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d->dialog)->vbox), 
-			button, TRUE, TRUE, FALSE);
-
-#ifndef OLD_GTK
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_target_pd), target_pd);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(ann));
-#else
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		       GTK_SIGNAL_FUNC(set_target_pd), target_pd);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(ann));
-#endif
-    gtk_widget_show (button);
-
-    hs = gtk_hseparator_new();
-    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d->dialog)->vbox), 
-			hs, TRUE, TRUE, FALSE);
-    gtk_widget_show (hs);
-}
-
-/* .................................................................. */
-
-#ifndef OLD_GTK
-static gint dialog_unblock (GtkWidget *w, gpointer p)
-{
-    gtk_main_quit();
-    return FALSE;
-}
-#endif
-
-/* .................................................................. */
-
-static void data_compact_dialog (GtkWidget *w, int spd, int *target_pd, 
-				 gint *compact_method)
-{
-    dialog_t *d;
-    GtkWidget *button;
-    GtkWidget *tempwid;
-    GSList *group;
-    int show_pd_buttons = 0;
-    int all_vars = 0;
-    gchar *labelstr;
-
-    d = malloc(sizeof *d);
-    if (d == NULL) return;
-    
-    d->data = NULL;
-
-    d->dialog = gtk_dialog_new();
-
-    if (*target_pd != 0) {
-	/* importing series from database */
-	labelstr = g_strdup_printf(_("You are adding a %s series to %s dataset"),
-				   (spd == 4)? _("quarterly") : _("monthly"),
-				   (*target_pd == 4)? _("a quarterly"): _("an annual"));
-    } else {
-	/* compacting whole data set */
-	all_vars = 1;
-	if (spd == 4) {
-	    *target_pd = 1;
-	    labelstr = g_strdup(_("Compact quarterly data to annual"));
-	} else {
-	    /* source data are monthly */
-	    labelstr = g_strdup(_("Compact monthly data to:"));
-	    show_pd_buttons = 1;
-	    *target_pd = 4;
+    for (i=1; i<datainfo->v; i++) {
+	if (COMPACT_METHOD(datainfo, i) == COMPACT_NONE) {
+	    COMPACT_METHOD(datainfo, i) = method;
 	}
     }
-
-    gtk_window_set_title (GTK_WINDOW (d->dialog), _("gretl: compact data"));
-
-#ifndef OLD_GTK
-    gtk_window_set_resizable (GTK_WINDOW (d->dialog), FALSE);
-    gtk_container_set_border_width (GTK_CONTAINER 
-				    (GTK_DIALOG (d->dialog)->vbox), 10);
-    gtk_container_set_border_width (GTK_CONTAINER 
-				    (GTK_DIALOG (d->dialog)->action_area), 5);
-#else
-    gtk_window_set_policy (GTK_WINDOW (d->dialog), FALSE, FALSE, FALSE);
-    gtk_container_border_width (GTK_CONTAINER 
-				(GTK_DIALOG (d->dialog)->vbox), 10);
-    gtk_container_border_width (GTK_CONTAINER 
-				(GTK_DIALOG (d->dialog)->action_area), 5);
-#endif
-
-    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 5);
-    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 15);
-    gtk_box_set_homogeneous (GTK_BOX 
-			     (GTK_DIALOG (d->dialog)->action_area), TRUE);
-    gtk_window_set_position (GTK_WINDOW (d->dialog), GTK_WIN_POS_MOUSE);
-
-#ifndef OLD_GTK
-    g_signal_connect (G_OBJECT (d->dialog), "destroy", 
-		      G_CALLBACK (destroy_dialog_data), 
-		      d);
-    g_signal_connect (G_OBJECT (d->dialog), "destroy", 
-		      G_CALLBACK (dialog_unblock), NULL);
-#else
-    gtk_signal_connect (GTK_OBJECT (d->dialog), "destroy", 
-			GTK_SIGNAL_FUNC (destroy_dialog_data), 
-			d);
-#endif
-
-    tempwid = gtk_label_new(labelstr);
-    g_free(labelstr);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			tempwid, TRUE, TRUE, FALSE);
-    gtk_widget_show (tempwid);
-
-    if (show_pd_buttons) pd_buttons(d, target_pd);
-
-    button = gtk_radio_button_new_with_label (NULL, _("Compact by averaging"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, FALSE);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-#ifndef OLD_GTK
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_compact_type), compact_method);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(COMPACT_AVG));
-#else
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(set_compact_type), compact_method);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(COMPACT_AVG));
-#endif
-    gtk_widget_show (button);
-
-    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON (button));
-    button = gtk_radio_button_new_with_label (group, _("Compact by summing"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_compact_type), compact_method);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(COMPACT_SUM));
-#else
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		       GTK_SIGNAL_FUNC(set_compact_type), compact_method);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(COMPACT_SUM));
-#endif
-    gtk_widget_show (button);
-
-    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-    button = gtk_radio_button_new_with_label(group, _("Use end-of-period values"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_compact_type), compact_method);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(COMPACT_EOP));
-#else
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(set_compact_type), compact_method);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(COMPACT_EOP));
-#endif
-    gtk_widget_show (button);
-
-    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-    button = gtk_radio_button_new_with_label(group, _("Use start-of-period values"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_compact_type), compact_method);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(COMPACT_SOP));
-#else
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(set_compact_type), compact_method);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(COMPACT_SOP));
-#endif
-    gtk_widget_show (button);
-
-    /* Create the "OK" button */
-    tempwid = standard_button(GTK_STOCK_OK);
-    GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
-    g_signal_connect (G_OBJECT (tempwid), "clicked", 
-		      G_CALLBACK (delete_widget), 
-		      G_OBJECT (d->dialog));
-#else
-    gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked", 
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-			       GTK_OBJECT (d->dialog));
-#endif
-    gtk_widget_grab_default (tempwid);
-    gtk_widget_show (tempwid);
-
-    /* Create the "Cancel" button */
-    tempwid = standard_button(GTK_STOCK_CANCEL);
-    GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
-    g_signal_connect (G_OBJECT (tempwid), "clicked", 
-		      G_CALLBACK (abort_compact), compact_method);
-    g_signal_connect (G_OBJECT (tempwid), "clicked", 
-		      G_CALLBACK (delete_widget), 
-		      G_OBJECT (d->dialog));
-#else
-    gtk_signal_connect (GTK_OBJECT (tempwid), "clicked", 
-			GTK_SIGNAL_FUNC (abort_compact), compact_method);
-    gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked", 
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-			       GTK_OBJECT (d->dialog));
-#endif
-    gtk_widget_show (tempwid);
-
-    /* Create a "Help" button */
-    tempwid = standard_button(GTK_STOCK_HELP);
-    GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
-    g_signal_connect (G_OBJECT (tempwid), "clicked", 
-		      G_CALLBACK (context_help), 
-		      GINT_TO_POINTER (COMPACT));
-#else
-    gtk_signal_connect (GTK_OBJECT (tempwid), "clicked", 
-			GTK_SIGNAL_FUNC (context_help), 
-			GINT_TO_POINTER (COMPACT));
-#endif
-    gtk_widget_show (tempwid);
-
-    gtk_widget_show (d->dialog);
-    gtk_window_set_transient_for(GTK_WINDOW(d->dialog), GTK_WINDOW(w));
-    gtk_main();
 }
 
-static int compact_series (double **Z, int i, int n, int oldn,
-			   int startskip, int min_startskip, int cfac,
-			   int method)
+void do_compact_data_set (void)
 {
-    int t, j;
-    int idx = startskip;
-    int lead = startskip - min_startskip;
-    double *x;
-
-    x = malloc(n * sizeof *x);
-    if (x == NULL) return 1;
-
-    if (i == 0) {
-	for (t=0; t<n; t++) x[t] = 1.0;
-    } else {
-	for (t=0; t<n; t++) x[t] = NADBL;
-	for (t=lead; t<n; t++) {
-	    if (idx > oldn - 1) break;
-	    if (method == COMPACT_SOP || method == COMPACT_EOP) {
-		x[t] = Z[i][idx];
-	    }
-	    else {
-		if (idx + cfac - 1 > oldn - 1) break;
-		x[t] = 0.0;
-		for (j=0; j<cfac; j++) {
-		    x[t] += Z[i][idx + j];
-		}
-		if (method == COMPACT_AVG) {
-		    x[t] /= (double) cfac;	
-		}
-	    }
-	    idx += cfac;
-	}
-    }
-
-    free(Z[i]);
-    Z[i] = x;
-
-    return 0;
-}
-
-static void 
-get_startskip_etc (int cfac, int startper, int endper, 
-		   int oldn, int method, 
-		   int *startskip, int *newn) 
-{
-    int ss = cfac - (startper % cfac) + 1;
-    int es, n;
-
-    ss = ss % cfac;
-
-    if (method == COMPACT_EOP) {
-	if (ss > 0) {
-	    ss--;
-	} else {
-	    /* move to end of initial period */
-	    ss = cfac - 1;
-	}
-    }
-
-    es = endper % cfac;
-    if (method == COMPACT_SOP && es > 1) es--;
-
-    n = (oldn - ss - es) / cfac;
-    if (ss && method == COMPACT_EOP) n++;
-    if (es && method == COMPACT_SOP) n++;
-
-    *startskip = ss;
-    *newn = n;
-}
-
-static void 
-get_global_compact_params (int cfac, int startper, int endper,
-			   int oldn, int default_method, 
-			   int *min_startskip, int *max_n,
-			   int *any_eop, int *all_same)
-{
-    int i, startskip, n, cm;
-    int n_not_eop = 0;
-
-    for (i=0; i<datainfo->v; i++) {
-	if (i == 0) {
-	    get_startskip_etc(cfac, startper, endper, oldn, 
-			      default_method, &startskip, &n);
-	    if (default_method == COMPACT_EOP) *any_eop = 1;
-	} else {
-	    cm = COMPACT_METHOD(datainfo, i);
-	    if (cm != default_method && cm != COMPACT_NONE) {
-		get_startskip_etc(cfac, startper, endper, oldn, 
-				  cm, &startskip, &n);
-		*all_same = 0;
-		if (cm == COMPACT_EOP) *any_eop = 1;
-		else n_not_eop++;
-	    }
-	}
-	if (startskip < *min_startskip) *min_startskip = startskip;
-	if (n > *max_n) *max_n = n;
-    }
-
-    if (n_not_eop == datainfo->v - 1) *any_eop = 0;
-
-}
-
-void compact_data_set (void)
-{
-    int newpd, oldpd = datainfo->pd;
-    int newn, oldn = datainfo->n;
-    int cfac;
-    int startper, endper;
-    int startyr;
-    int any_eop, all_same;
-    int min_startskip = 0;
-    int i, err = 0;
     int default_method = COMPACT_AVG;
-    char stobs[OBSLEN], *p;
+    int err, newpd = 0, monstart = 1;
+    int *pmonstart = NULL;
 
     if (maybe_restore_full_data(COMPACT)) return;
 
-    newpd = 0;
-    data_compact_dialog(mdata->w, oldpd, &newpd, &default_method);
-    if (default_method == COMPACT_NONE) return;
+    if (dated_seven_day_data(datainfo)) {
+	pmonstart = &monstart;
+    }
 
-    cfac = oldpd / newpd;
-
-    /* figure starting year and sub-period */
-    startyr = atoi(datainfo->stobs);
-    p = strchr(datainfo->stobs, ':');
-    if (p == NULL) p = strchr(datainfo->stobs, '.');
-    if (p == NULL) return;
-    p++;
-    if (*p == '0') p++;
-    startper = atoi(p);
-
-    /* figure ending sub-period */
-    p = strchr(datainfo->endobs, ':');
-    if (p == NULL) p = strchr(datainfo->endobs, '.');
-    if (p == NULL) return;
-    p++;
-    if (*p == '0') p++;
-    endper = atoi(p);   
-
-    min_startskip = oldpd;
-    newn = 0;
-    any_eop = 0;
-    all_same = 1;
-    get_global_compact_params(cfac, startper, endper, oldn, default_method,
-			      &min_startskip, &newn, &any_eop, &all_same);
-    if (newn == 0) {
-	errbox(_("Compacted dataset would be empty"));
+    data_compact_dialog(mdata->w, datainfo->pd, &newpd, pmonstart, &default_method);
+    if (default_method == COMPACT_NONE) {
+	/* the user cancelled */
 	return;
-    }    
-    
-    if (newpd == 1) {
-	if (min_startskip > 0 && !any_eop) 
-	    startyr++;
-	sprintf(stobs, "%d", startyr);
-    } else if (newpd == 4) {
-	int mo = startper + min_startskip;
-	int qtr = mo / 3 + (mo % 3 > 0);
-
-	if (qtr > 4) {
-	    startyr++;
-	    qtr -= 4;
-	}
-	sprintf(stobs, "%d:%d", startyr, qtr);
     }
 
-    /* revise datainfo members */
-    strcpy(datainfo->stobs, stobs);
-    datainfo->pd = newpd;
-    datainfo->n = newn;
-    datainfo->sd0 = get_date_x(datainfo->pd, datainfo->stobs);
-    datainfo->t1 = 0;
-    datainfo->t2 = datainfo->n - 1;
-    ntodate(datainfo->endobs, datainfo->t2, datainfo);
+    err = compact_data_set(&Z, datainfo, newpd, default_method, monstart);
 
-    /* compact the individual data series */
-    for (i=0; i<datainfo->v && err == 0; i++) {
-	if (datainfo->vector[i]) {
-	    int this_method = default_method;
-	    int startskip = min_startskip;
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	data_status |= MODIFIED_DATA;
+	set_sample_label(datainfo);
 
-	    if (!all_same) {
-		if (COMPACT_METHOD(datainfo, i) != COMPACT_NONE) {
-		    this_method = COMPACT_METHOD(datainfo, i);
-		}
-
-		startskip = cfac - (startper % cfac) + 1;
-		startskip = startskip % cfac;
-		if (this_method == COMPACT_EOP) {
-		    if (startskip > 0) {
-			startskip--;
-		    } else {
-			startskip = cfac - 1;
-		    }
-		}
-	    }
-
-	    if (compact_series(Z, i, datainfo->n, oldn, startskip, min_startskip,
-			       cfac, this_method)) {
-		errbox(_("Out of memory!"));
-		err = 1;
-	    }
+	if (datainfo->pd == 1 || datainfo->pd == 52) {
+	    flip(mdata->ifac, "/Sample/Compact data...", FALSE);
 	}
-    }
 
-    data_status |= MODIFIED_DATA;
-    set_sample_label(datainfo);
-
-    if (datainfo->pd == 1) {
-	flip(mdata->ifac, "/Sample/Compact data...", FALSE);
+	set_compact_info_from_default(default_method);
     }
 }
 
-
