@@ -19,12 +19,14 @@
 
 /* datafiles.c : for gretl */
 
-/* #define COLL_DEBUG */
+#undef COLL_DEBUG
 
 #include "gretl.h"
 #include "datafiles.h"
 #include "database.h"
+#include "filelists.h"
 #include "webget.h"
+#include "menustate.h"
 
 #if !GLIB_CHECK_VERSION(2,0,0)
 # define OLD_GTK
@@ -161,6 +163,8 @@ static int get_title_from_descfile (file_collection *coll)
 	fclose(fp);
 	return err;
     }
+
+    chopstr(line);
     
     if (sscanf(line, "# %23[^:]", title) == 1) {
 	coll->title = malloc(strlen(title) + 1);
@@ -699,7 +703,7 @@ static void set_browser_status (windata_t *fdata, int status)
 
 /* ........................................................... */
 
-static void browser_ok (GtkWidget *w, gpointer data)
+static void free_browser (GtkWidget *w, gpointer data)
 {
     windata_t *vwin = (windata_t *) data;
 
@@ -797,24 +801,24 @@ static gint populate_remote_db_list (windata_t *win)
     GtkTreeIter iter;  
 #endif  
     int err;
-    char *getbuf;
+    char *buf;
     char fname[16], line[80], errbuf[80], status[20];
     gchar *row[3];
     gint i;
     time_t remtime;
 
-    getbuf = mymalloc(GRETL_BUFSIZE);
-    if (getbuf == NULL) return 1;
+    buf = mymalloc(GRETL_BUFSIZE);
+    if (buf == NULL) return 1;
 
-    memset(getbuf, 0, GRETL_BUFSIZE);
+    memset(buf, 0, GRETL_BUFSIZE);
 
     *errbuf = '\0';
 
-    err = list_remote_dbs(&getbuf, errbuf);
+    err = list_remote_dbs(&buf, errbuf);
 
     if (err) {
 	display_db_error(NULL, errbuf);
-	free(getbuf);
+	free(buf);
 	return err;
     }
 
@@ -828,16 +832,22 @@ static gint populate_remote_db_list (windata_t *win)
     gtk_clist_freeze(GTK_CLIST(win->listbox));
 #endif
 
+    bufgets(NULL, 0, buf);
     i = 0;
-    getbufline(NULL, NULL, 1);
-    while (getbufline(getbuf, line, 0)) {
+
+    while (bufgets(line, sizeof line, buf)) {
 	if (strstr(line, "idx")) continue;
 	if (parse_db_list_line(line, fname, &remtime))
 	    continue;
 	get_local_status(fname, status, remtime);
 	row[0] = strip_extension(fname);
-	if (!getbufline(getbuf, line, 0)) row[1] = NULL;
-	else row[1] = line + 2;
+
+	if (bufgets(line, sizeof line, buf)) {
+	    row[1] = line + 2;
+	} else {
+	    row[1] = NULL;
+	}
+
 	row[2] = status;
 #ifndef OLD_GTK
 	gtk_list_store_append(store, &iter);
@@ -856,7 +866,7 @@ static gint populate_remote_db_list (windata_t *win)
     gtk_clist_thaw(GTK_CLIST(win->listbox));
 #endif
 
-    free(getbuf);
+    free(buf);
 
     if (i == 0) errbox(_("No database files found"));
 #ifdef OLD_GTK
@@ -930,11 +940,11 @@ void display_files (gpointer data, guint code, GtkWidget *widget)
     fdata->w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 #ifndef OLD_GTK
     g_signal_connect (G_OBJECT (fdata->w), "destroy",
-		      G_CALLBACK (browser_ok),
+		      G_CALLBACK (free_browser),
 		      fdata);
 #else
     gtk_signal_connect (GTK_OBJECT (fdata->w), "destroy",
-			GTK_SIGNAL_FUNC (browser_ok),
+			GTK_SIGNAL_FUNC (free_browser),
 			fdata);
 #endif
     set_browser_status(fdata, BROWSER_BUSY);
@@ -1271,231 +1281,6 @@ static GtkWidget *files_window (windata_t *fdata)
 
 /* .................................................................. */
 
-static void really_set_panel_code (GtkWidget *w, dialog_t *d)
-{
-    DATAINFO *pdinfo = (DATAINFO *) d->data;
-
-    pdinfo->time_series = d->code;
-    set_sample_label(pdinfo);
-    d->data = NULL;
-}
-
-/* .................................................................. */
-
-static void set_panel_code (GtkWidget *w, dialog_t *d)
-{
-    gint i;
-
-    if (GTK_TOGGLE_BUTTON(w)->active) {
-#ifndef OLD_GTK
-	i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
-#else
-	i = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "action"));
-#endif
-	d->code = i;
-    }
-}
-
-static gint dialog_unblock (GtkWidget *w, gpointer p)
-{
-    gtk_main_quit();
-    return FALSE;
-}
-
-#ifndef OLD_GTK
-
-void panel_structure_dialog (DATAINFO *pdinfo, GtkWidget *w)
-{
-    dialog_t *d;
-    GtkWidget *button;
-    GtkWidget *tempwid;
-    GSList *group;
-
-    d = malloc(sizeof *d);
-    if (d == NULL) return;
-    
-    d->data = pdinfo;
-
-    d->dialog = gtk_dialog_new();
-    w = d->dialog;
-
-    d->code = (dataset_is_panel(pdinfo))? 
-	pdinfo->time_series : STACKED_TIME_SERIES;
-
-    gtk_window_set_title (GTK_WINDOW (d->dialog), _("gretl: panel structure"));
-    gtk_window_set_resizable (GTK_WINDOW (d->dialog), FALSE);
-
-    gtk_box_set_homogeneous (GTK_BOX 
-			     (GTK_DIALOG (d->dialog)->action_area), TRUE);
-    gtk_window_set_position (GTK_WINDOW (d->dialog), GTK_WIN_POS_MOUSE);
-
-    g_signal_connect (G_OBJECT (d->dialog), "destroy", 
-		      G_CALLBACK (destroy_dialog_data), 
-		      d);
-    g_signal_connect (G_OBJECT (d->dialog), "destroy", 
-		      G_CALLBACK (dialog_unblock), NULL);
-
-    button = gtk_radio_button_new_with_label (NULL, _("Stacked time series"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, 0);
-    if (d->code == STACKED_TIME_SERIES)
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_panel_code), d);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(STACKED_TIME_SERIES));
-    gtk_widget_show (button);
-
-    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-    button = gtk_radio_button_new_with_label(group, _("Stacked cross sections"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, 0);
-    if (d->code == STACKED_CROSS_SECTION)
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(set_panel_code), d);
-    g_object_set_data(G_OBJECT(button), "action", 
-		      GINT_TO_POINTER(STACKED_CROSS_SECTION));
-    gtk_widget_show (button);
-
-    /* Create the "OK" button */
-    tempwid = gtk_button_new_from_stock(GTK_STOCK_OK);
-    GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-    g_signal_connect(G_OBJECT(tempwid), "clicked",
-		     G_CALLBACK(really_set_panel_code), d);
-    g_signal_connect(G_OBJECT (tempwid), "clicked", 
-		     G_CALLBACK (delete_widget), 
-		     d->dialog);
-    gtk_widget_grab_default (tempwid);
-    gtk_widget_show (tempwid);
-
-    /* Create the "Cancel" button */
-    tempwid = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-    GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-    g_signal_connect (G_OBJECT (tempwid), "clicked", 
-		      G_CALLBACK (delete_widget), 
-		      d->dialog);
-    gtk_widget_show (tempwid);
-
-    /* Create a "Help" button */
-    tempwid = gtk_button_new_from_stock(GTK_STOCK_HELP);
-    GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-    g_signal_connect (G_OBJECT (tempwid), "clicked", 
-		      G_CALLBACK (context_help), 
-		      GINT_TO_POINTER (PANEL));
-    gtk_widget_show (tempwid);
-
-    gtk_widget_show (d->dialog);
-    gtk_window_set_transient_for(GTK_WINDOW(d->dialog), GTK_WINDOW(mdata->w));
-    gtk_main();
-}
-
-#else /* now the old gtk version */
-
-void panel_structure_dialog (DATAINFO *pdinfo, GtkWidget *w)
-{
-    dialog_t *d;
-    GtkWidget *button;
-    GtkWidget *tempwid;
-    GSList *group;
-
-    d = malloc(sizeof *d);
-    if (d == NULL) return;
-    
-    d->data = pdinfo;
-
-    d->dialog = gtk_dialog_new();
-    w = d->dialog;
-
-    d->code = (dataset_is_panel(pdinfo))? 
-	pdinfo->time_series : STACKED_TIME_SERIES;
-
-    gtk_window_set_title (GTK_WINDOW (d->dialog), _("gretl: panel structure"));
-    gtk_window_set_policy (GTK_WINDOW (d->dialog), FALSE, FALSE, FALSE);
-    gtk_container_border_width (GTK_CONTAINER 
-				(GTK_DIALOG (d->dialog)->vbox), 10);
-    gtk_container_border_width (GTK_CONTAINER 
-				(GTK_DIALOG (d->dialog)->action_area), 5);
-    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 5);
-    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 15);
-    gtk_box_set_homogeneous (GTK_BOX 
-			     (GTK_DIALOG (d->dialog)->action_area), TRUE);
-    gtk_window_set_position (GTK_WINDOW (d->dialog), GTK_WIN_POS_MOUSE);
-
-    gtk_signal_connect (GTK_OBJECT (d->dialog), "destroy", 
-			GTK_SIGNAL_FUNC (destroy_dialog_data), 
-			d);
-    gtk_signal_connect (GTK_OBJECT (d->dialog), "destroy", 
-			GTK_SIGNAL_FUNC (dialog_unblock), NULL);
-
-    button = gtk_radio_button_new_with_label (NULL, _("Stacked time series"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, 0);
-    if (d->code == STACKED_TIME_SERIES)
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(set_panel_code), d);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(STACKED_TIME_SERIES));
-    gtk_widget_show (button);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
-    button = gtk_radio_button_new_with_label(group, _("Stacked cross sections"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 
-			button, TRUE, TRUE, 0);
-    if (d->code == STACKED_CROSS_SECTION)
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(set_panel_code), d);
-    gtk_object_set_data(GTK_OBJECT(button), "action", 
-			GINT_TO_POINTER(STACKED_CROSS_SECTION));
-    gtk_widget_show(button);
-
-    /* Create the "OK" button */
-    tempwid = gtk_button_new_with_label ("OK");
-    GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-    gtk_signal_connect(GTK_OBJECT(tempwid), "clicked",
-                       GTK_SIGNAL_FUNC(really_set_panel_code), d);
-    gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked", 
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-			       GTK_OBJECT (d->dialog));
-    gtk_widget_grab_default (tempwid);
-    gtk_widget_show(tempwid);
-
-    /* Create the "Cancel" button */
-    tempwid = gtk_button_new_with_label (_("Cancel"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-    gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked", 
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-			       GTK_OBJECT (d->dialog));
-    gtk_widget_show(tempwid);
-
-    /* Create a "Help" button */
-    tempwid = gtk_button_new_with_label (_("Help"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 
-			tempwid, TRUE, TRUE, FALSE);
-    gtk_signal_connect (GTK_OBJECT (tempwid), "clicked", 
-			GTK_SIGNAL_FUNC (context_help), 
-			GINT_TO_POINTER (PANEL));
-    gtk_widget_show(tempwid);
-
-    gtk_widget_show(d->dialog);
-    gtk_main();
-}
-
-#endif /* old versus new gtk */
-
-/* .................................................................. */
-
 void panel_restructure_dialog (gpointer data, guint u, GtkWidget *w)
 {
     int resp;
