@@ -27,6 +27,8 @@ void browser_open_ps (GtkWidget *w, gpointer data);
 void destroy_file_collections (void);
 gint populate_filelist (windata_t *fdata, gpointer p);
 char *strip_extension (char *s);
-void panel_structure_dialog (DATAINFO *pdinfo, GtkWidget *w);
+
+void panel_restructure_dialog (gpointer data, guint u, GtkWidget *w);
+void time_series_dialog (gpointer data, guint u, GtkWidget *w);
 
 #endif /* DATAFILES_H */
