@@ -25,6 +25,8 @@
 #include "gpt_control.h"
 #include "session.h"
 #include "textbuf.h"
+#include "textutil.h"
+#include "filelists.h"
 
 #if (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 4)
 # ifndef G_OS_WIN32
@@ -67,7 +69,8 @@
 #define EXPORT_ACTION(i) (i == EXPORT_OCTAVE || \
                           i == EXPORT_R || \
                           i == EXPORT_R_ALT || \
-                          i == EXPORT_CSV)
+                          i == EXPORT_CSV || \
+                          i == EXPORT_DAT)
 
 extern int olddat; /* settings.c */
 
@@ -97,6 +100,7 @@ static struct extmap action_map[] = {
     { EXPORT_R,          ".R" },
     { EXPORT_R_ALT,      ".R" },
     { EXPORT_OCTAVE,     ".m" },
+    { EXPORT_DAT,        ".dat" },
     { SAVE_OUTPUT,       ".txt" },
     { SAVE_TEX_TAB,      ".tex" },
     { SAVE_TEX_EQ,       ".tex" },
@@ -120,7 +124,7 @@ static struct extmap action_map[] = {
 
 /* ........................................................... */
 
-static unsigned long action_to_opt (const int action)
+static gretlopt action_to_opt (const int action)
 {
     switch (action) {
     case SAVE_GZDATA:   return OPT_Z;
@@ -130,6 +134,7 @@ static unsigned long action_to_opt (const int action)
     case EXPORT_R:      return OPT_R;
     case EXPORT_R_ALT:  return OPT_A;
     case EXPORT_CSV:    return OPT_C;
+    case EXPORT_DAT:    return OPT_D;
     default: return 0L;
     }
 }
@@ -141,7 +146,8 @@ static const char *get_gp_ext (const char *termtype)
     if (!strncmp(termtype, "postscript", 10))    return ".eps";
     else if (!strcmp(termtype, "fig"))           return ".fig";
     else if (!strcmp(termtype, "latex"))         return ".tex";
-    else if (!strcmp(termtype, "png"))           return ".png";
+    else if (!strncmp(termtype, "png", 3))       return ".png";
+    else if (!strncmp(termtype, "emf", 3))       return ".emf";
     else if (!strcmp(termtype, "plot commands")) return ".plt";
     else return "*";
 }
@@ -257,7 +263,6 @@ static void save_editable_content (int action, const char *fname,
     FILE *fp;
     gchar *buf;
 #if defined(ENABLE_NLS) && !defined(OLD_GTK)
-    gsize bytes;
     gchar *trbuf;
 #endif
 
@@ -279,9 +284,11 @@ static void save_editable_content (int action, const char *fname,
     }
 
 #if defined(ENABLE_NLS) && !defined(OLD_GTK)
-    trbuf = g_locale_from_utf8(buf, -1, NULL, &bytes, NULL);
-    system_print_buf(trbuf, fp);
-    g_free(trbuf);
+    trbuf = my_locale_from_utf8(buf);
+    if (trbuf != NULL) {
+	system_print_buf(trbuf, fp);
+	g_free(trbuf);
+    }
 #else
     system_print_buf(buf, fp);
 #endif
@@ -402,6 +409,9 @@ static char *suggested_exportname (const char *fname, int action)
 	case EXPORT_CSV:
 	    test = ".csv";
 	    break;
+	case EXPORT_DAT:
+	    test = ".dat";
+	    break;
 	default:
 	    test = NULL;
 	    break;
@@ -439,15 +449,12 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
     if (OPEN_DATA_ACTION(action)) {
 	strcpy(trydatfile, fname);
 	verify_open_data(NULL, action);
-    }
-    else if (APPEND_DATA_ACTION(action)) {
+    } else if (APPEND_DATA_ACTION(action)) {
 	strcpy(trydatfile, fname);
 	do_open_data(NULL, NULL, action);
-    }
-    else if (action == OPEN_SCRIPT) {
+    } else if (action == OPEN_SCRIPT) {
 	filesel_open_script(fname);
-    }
-    else if (action == OPEN_SESSION) {
+    } else if (action == OPEN_SESSION) {
 	filesel_open_session(fname);
     }
 
@@ -455,51 +462,54 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
 
     /* now for the save options */
 
-    if (action > SAVE_BIN2 && dat_ext(fname, 1)) return;
+    if (action > SAVE_BIN2 && action != EXPORT_DAT && dat_ext(fname, 1)) { 
+	return;
+    }
 
-    if (check_maybe_add_ext(fname, action, data)) return;
+    if (check_maybe_add_ext(fname, action, data)) {
+	return;
+    }
 
     if (SAVE_DATA_ACTION(action)) {
 	int overwrite = 0;
 
-	if (!strcmp(fname, paths.datfile)) overwrite = 1;
+	if (!strcmp(fname, paths.datfile)) {
+	    overwrite = 1;
+	}
 	do_store(fname, action_to_opt(action), overwrite);
-    }
-    else if (action == SAVE_GNUPLOT) {
+    } else if (action == SAVE_GNUPLOT) {
 	int err = 0;
 	GPT_SPEC *plot = (GPT_SPEC *) data;
 
-	err = go_gnuplot(plot, fname, &paths);
-	if (err == 0) infobox(_("graph saved"));
-	else if (err == 1) errbox(_("gnuplot command failed"));
-	else if (err == 2) infobox(_("There were missing observations"));
-    }
-    else if (action == SAVE_THIS_GRAPH) {
+	err = go_gnuplot(plot, fname);
+	if (err == 0) {
+	    infobox(_("graph saved"));
+	} else if (err == 1) {
+	    errbox(_("gnuplot command failed"));
+	} else if (err == 2) {
+	    infobox(_("There were missing observations"));
+	}
+    } else if (action == SAVE_THIS_GRAPH) {
 	GPT_SPEC *plot = (GPT_SPEC *) data;
 
 	save_this_graph(plot, fname);
-    }
-    else if (action == SAVE_BOXPLOT_EPS || action == SAVE_BOXPLOT_PS) {
+    } else if (action == SAVE_BOXPLOT_EPS || action == SAVE_BOXPLOT_PS) {
 	int err;
 
 	err = ps_print_plots(fname, action, data);
 	if (!err) infobox(_("boxplots saved"));
 	else errbox(_("boxplot save failed"));
-    }
-    else if (action == SAVE_SESSION) {
+    } else if (action == SAVE_SESSION) {
 	save_session(fname);
-    }
-    else if (SAVE_TEX_ACTION(action)) {
+    } else if (SAVE_TEX_ACTION(action)) {
 	MODEL *pmod = (MODEL *) data;
 
 	do_save_tex(fname, action, pmod); 
-    }
-    else if (action == SET_PATH) {
+    } else if (action == SET_PATH) {
 	char *strvar = (char *) data;
 
 	filesel_set_path_callback(fname, strvar);
-    }
-    else {
+    } else {
 	windata_t *vwin = (windata_t *) data;
 
 	save_editable_content(action, fname, vwin);
@@ -533,6 +543,7 @@ static struct winfilter get_gp_filter (const char *termtype)
 	{ N_("xfig files (*.fig)"), "*.fig" },
 	{ N_("LaTeX files (*.tex)"), "*.tex" },
 	{ N_("PNG files (*.png)"), "*.png" },
+	{ N_("Windows metafiles (*.emf)"), "*.emf" },
 	{ N_("gnuplot files (*.plt)"), "*.plt" },
 	{ N_("all files (*.*)"), "*.*" }
     };
@@ -543,12 +554,14 @@ static struct winfilter get_gp_filter (const char *termtype)
 	return gpfilters[1];
     else if (!strcmp(termtype, "latex")) 
 	return gpfilters[2];
-    else if (!strcmp(termtype, "png")) 
+    else if (!strncmp(termtype, "png", 3)) 
 	return gpfilters[3];
-    else if (!strcmp(termtype, "plot commands")) 
+    else if (!strncmp(termtype, "emf", 3)) 
 	return gpfilters[4];
-    else 
+    else if (!strcmp(termtype, "plot commands")) 
 	return gpfilters[5];
+    else 
+	return gpfilters[6];
 }
 
 static struct winfilter get_filter (int action, gpointer data)
@@ -573,6 +586,7 @@ static struct winfilter get_filter (int action, gpointer data)
 	{EXPORT_R,     { N_("GNU R files (*.R)"), "*.R" }},
 	{EXPORT_R_ALT, { N_("GNU R files (*.R)"), "*.R" }},
 	{EXPORT_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
+	{EXPORT_DAT  , { N_("PcGive files (*.dat)"), "*.dat" }},
 	{SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
 	{SAVE_TEX_TAB, { N_("TeX files (*.tex)"), "*.tex" }},
 	{SAVE_TEX_EQ,  { N_("TeX files (*.tex)"), "*.tex" }},
@@ -615,6 +629,7 @@ static struct winfilter get_filter (int action, gpointer data)
 	    break;
 	}
     }
+
     return filter;
 }
 
@@ -687,9 +702,7 @@ void file_selector (const char *msg, int action, gpointer data)
     }	
 
     if (doing_nls()) {
-	gint wrote;
-
-	trmsg = g_locale_from_utf8 (msg, -1, NULL, &wrote, NULL);
+	trmsg = my_locale_from_utf8(msg);
     } else {
 	trmsg = g_strdup(msg);
     }
@@ -732,6 +745,10 @@ void file_selector (const char *msg, int action, gpointer data)
 	return;
     }
 
+#ifdef ENABLE_NLS
+    my_filename_to_utf8(fname);
+#endif
+
     file_selector_process_result(fname, action, data);
 }
 
