@@ -7,6 +7,7 @@ enum extra_cmds {
     GSETMISS,
     SMPLDUM,
     SMPLBOOL,
+    SMPLRAND,
     SETSEED,
     MARKERS,
     STORE_MODEL,
@@ -33,7 +34,6 @@ enum extra_cmds {
     H_TEST,
     TRAMO,
     X12A,
-    RANGE_MEAN,
     VIEW_SERIES,
     VIEW_SCALAR,
     VIEW_MODEL,
@@ -56,6 +56,10 @@ enum extra_cmds {
     MODELTABLE,
     GRAPHPAGE,
     CREATE_USERDIR,
+    LMTEST_LOGS,
+    LMTEST_SQUARES,
+    LMTEST_WHITE,
+    LMTEST_BG,
     GUI_CMD_MAX
 };
 
@@ -85,6 +89,7 @@ enum file_ops {
     EXPORT_R,
     EXPORT_R_ALT,
     EXPORT_CSV,
+    EXPORT_DAT,
     COPY_CSV,
     END_SAVE_DATA,  /* marker for end of data-saving section */
     SAVE_CMDS,
@@ -172,7 +177,7 @@ enum drag_types {
 };
 
 enum file_lists {
-    FILE_LIST_DATA = 1,
+    FILE_LIST_DATA,
     FILE_LIST_SESSION,
     FILE_LIST_SCRIPT,
 };
