@@ -97,8 +97,7 @@ void rtfprint_summary (GRETLSUMMARY *summ,
 		       PRN *prn)
 {
     char date1[OBSLEN], date2[OBSLEN], tmp[128];
-    double xbar, std, xcv;
-    int lo = summ->list[0], v, lv;
+    int i, vi;
 
     ntodate(date1, pdinfo->t1, pdinfo);
     ntodate(date2, pdinfo->t2, pdinfo);
@@ -108,7 +107,7 @@ void rtfprint_summary (GRETLSUMMARY *summ,
 
     pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n", tmp);
     
-    if (lo == 1) {
+    if (summ->list[0] == 1) {
 	sprintf(tmp, I_("for the variable %s (%d valid observations)"), 
 		pdinfo->varname[summ->list[1]], summ->n);
 	pprintf(prn, "%s\\par\n\n", tmp);
@@ -128,20 +127,20 @@ void rtfprint_summary (GRETLSUMMARY *summ,
 	    " \\intbl \\row\n",
 	    I_("Mean"), I_("Median"), I_("Minimum"), I_("Maximum"));
 
-    for (v=0; v<lo; v++) {
-	lv = summ->list[v+1];
-	xbar = summ->coeff[v];
-	if (lo > 1) {
-	    pprintf(prn, "\\intbl \\qc %s\\cell ", pdinfo->varname[lv]);
+    for (i=0; i<summ->list[0]; i++) {
+	vi = summ->list[i + 1];
+	if (summ->list[0] > 1) {
+	    pprintf(prn, "\\intbl \\qc %s\\cell ", pdinfo->varname[vi]);
 	}
-	printfrtf(xbar, prn, 0);
-	printfrtf(summ->xmedian[v], prn, 0);
-	printfrtf(summ->xpx[v], prn, 0);
-	printfrtf(summ->xpy[v], prn, 1);
+	printfrtf(summ->mean[i], prn, 0);
+	printfrtf(summ->median[i], prn, 0);
+	printfrtf(summ->low[i], prn, 0);
+	printfrtf(summ->high[i], prn, 1);
     }
 
-    if (lo > 1) pprintf(prn, "\\intbl \\qc %s\\cell",
-			I_("Variable"));
+    if (summ->list[0] > 1) pprintf(prn, "\\intbl \\qc %s\\cell",
+				   I_("Variable"));
+
     pprintf(prn, 
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
@@ -150,22 +149,15 @@ void rtfprint_summary (GRETLSUMMARY *summ,
 	    " \\intbl \\row\n",
 	    I_("Std. Dev."), I_("C.V."), I_("Skewness"), I_("Ex. kurtosis"));
 
-    for (v=0; v<lo; v++) {
-	lv = summ->list[v+1];
-	if (lo > 1) {
-	    pprintf(prn, "\\intbl \\qc %s\\cell ", pdinfo->varname[lv]);
+    for (i=0; i<summ->list[0]; i++) {
+	vi = summ->list[i + 1];
+	if (summ->list[0] > 1) {
+	    pprintf(prn, "\\intbl \\qc %s\\cell ", pdinfo->varname[vi]);
 	}
-	xbar = summ->coeff[v];
-	std = summ->sderr[v];
-	if (xbar != 0.0) {
-	    xcv = (xbar > 0)? std/xbar: -std/xbar;
-	} else {
-	    xcv = NADBL;
-	}
-	printfrtf(std, prn, 0);
-	printfrtf(xcv, prn, 0);
-	printfrtf(summ->xskew[v], prn, 0);
-	printfrtf(summ->xkurt[v], prn, 1);
+	printfrtf(summ->sd[i], prn, 0);
+	printfrtf(summ->cv[i], prn, 0);
+	printfrtf(summ->skew[i], prn, 0);
+	printfrtf(summ->xkurt[i], prn, 1);
     }
 
     pputs(prn, "}}\n");
@@ -201,8 +193,7 @@ void texprint_summary (GRETLSUMMARY *summ,
 		       PRN *prn)
 {
     char date1[OBSLEN], date2[OBSLEN], vname[16], tmp[128];
-    double xbar, std, xcv;
-    int lo = summ->list[0], v, lv;
+    int i, vi;
     char pt = get_local_decpoint();
 
     ntodate(date1, pdinfo->t1, pdinfo);
@@ -213,7 +204,7 @@ void texprint_summary (GRETLSUMMARY *summ,
 
     pprintf(prn, "\\begin{center}\n%s\\\\\n", tmp);
     
-    if (lo == 1) {
+    if (summ->list[0] == 1) {
 	tex_escape(vname, pdinfo->varname[summ->list[1]]);
 	sprintf(tmp, I_("for the variable %s (%d valid observations)"), 
 		vname, summ->n);
@@ -235,22 +226,26 @@ void texprint_summary (GRETLSUMMARY *summ,
 	    "   & \\multicolumn{1}{c}{%s} \\\\[1ex]\n",
 	    I_("Mean"), I_("Median"), I_("Minimum"), I_("Maximum"));
 
-    for (v=0; v<lo; v++) {
-	lv = summ->list[v+1];
-	xbar = summ->coeff[v];
-	if (lo > 1) {
-	    tex_escape(vname, pdinfo->varname[lv]);
+    for (i=0; i<summ->list[0]; i++) {
+	vi = summ->list[i + 1];
+	if (summ->list[0] > 1) {
+	    tex_escape(vname, pdinfo->varname[vi]);
 	    pprintf(prn, "%s & ", vname);
 	}
-	printftex(xbar, prn, 0);
-	printftex(summ->xmedian[v], prn, 0);
-	printftex(summ->xpx[v], prn, 0);
-	printftex(summ->xpy[v], prn, 1);
-	if (v == lo - 1) pputs(prn, "[10pt]\n\n");
-	else pputs(prn, "\n");
+	printftex(summ->mean[i], prn, 0);
+	printftex(summ->median[i], prn, 0);
+	printftex(summ->low[i], prn, 0);
+	printftex(summ->high[i], prn, 1);
+	if (i == summ->list[0] - 1) {
+	    pputs(prn, "[10pt]\n\n");
+	} else {
+	    pputc(prn, '\n');
+	}
     }
 
-    if (lo > 1) pprintf(prn, "%s & ", I_("Variable"));
+    if (summ->list[0] > 1) {
+	pprintf(prn, "%s & ", I_("Variable"));
+    }
 
     pprintf(prn, " \\multicolumn{1}{c}{%s}%%\n"
 	    " & \\multicolumn{1}{c}{%s}%%\n"
@@ -258,21 +253,17 @@ void texprint_summary (GRETLSUMMARY *summ,
 	    "   & \\multicolumn{1}{c}{%s} \\\\[1ex]\n",
 	    I_("Std.\\ Dev."), I_("C.V."), I_("Skewness"), I_("Ex.\\ kurtosis"));
 
-    for (v=0; v<lo; v++) {
-	lv = summ->list[v+1];
-	if (lo > 1) {
-	    tex_escape(vname, pdinfo->varname[lv]);
+    for (i=0; i<summ->list[0]; i++) {
+	vi = summ->list[i + 1];
+	if (summ->list[0] > 1) {
+	    tex_escape(vname, pdinfo->varname[vi]);
 	    pprintf(prn, "%s & ", vname);
 	}
-	xbar = summ->coeff[v];
-	std = summ->sderr[v];
-	if (xbar != 0.0) xcv = (xbar > 0)? std/xbar: (-1) * std/xbar;
-	else xcv = NADBL;
-	printftex(std, prn, 0);
-	printftex(xcv, prn, 0);
-	printftex(summ->xskew[v], prn, 0);
-	printftex(summ->xkurt[v], prn, 1);
-	pputs(prn, "\n");
+	printftex(summ->sd[i], prn, 0);
+	printftex(summ->cv[i], prn, 0);
+	printftex(summ->skew[i], prn, 0);
+	printftex(summ->xkurt[i], prn, 1);
+	pputc(prn, '\n');
     }
 
     pputs(prn, "\\end{tabular}\n\\end{center}\n");
@@ -388,10 +379,10 @@ rtfprint_matrix (const double *vec, const int *list,
 	}
 
 	/* print rectangular part, if any, of matrix */
-	for (j=1; j<=nf; j++) {
+	for (j=0; j<nf; j++) {
 	    pputs(prn, "\\intbl "); 
 	    if (pad) rtf_table_pad(pad, prn);
-	    for (k=1; k<=p; k++) {
+	    for (k=0; k<p; k++) {
 		index = ijton(j, nf+k, lo);
 		if (ci == CORR) {
 		    rtf_outxx(vec[index], prn);
@@ -399,15 +390,15 @@ rtfprint_matrix (const double *vec, const int *list,
 		    printfrtf(vec[index], prn, 0);
 		}
 	    }
-	    pprintf(prn, "\\ql (%d\\cell \\intbl \\row\n", list[j]);
+	    pprintf(prn, "\\ql (%d\\cell \\intbl \\row\n", list[j+1]);
 	}
 
 	/* print upper triangular part of matrix */
-	for (j=1; j<=p; ++j) {
+	for (j=0; j<p; ++j) {
 	    pputs(prn, "\\intbl "); 
-	    rtf_table_pad(pad + j - 1, prn);
+	    rtf_table_pad(pad + j, prn);
 	    ij2 = nf + j;
-	    for (k=j; k<=p; k++) {
+	    for (k=j; k<p; k++) {
 		index = ijton(ij2, nf+k, lo);
 		if (ci == CORR) {
 		    rtf_outxx(vec[index], prn);
@@ -415,7 +406,7 @@ rtfprint_matrix (const double *vec, const int *list,
 		    printfrtf(vec[index], prn, 0);
 		}
 	    }
-	    pprintf(prn, "\\ql (%d\\cell \\intbl \\row\n", list[ij2]);
+	    pprintf(prn, "\\ql (%d\\cell \\intbl \\row\n", list[ij2+1]);
 	}
     }
     pputs(prn, "}}\n");
@@ -511,8 +502,8 @@ texprint_matrix (const double *vec, const int *list,
 	}   
 
 	/* print rectangular part, if any, of matrix */
-	for (j=1; j<=nf; j++) {
-	    for (k=1; k<=p; k++) {
+	for (j=0; j<nf; j++) {
+	    for (k=0; k<p; k++) {
 		index = ijton(j, nf+k, lo);
 		if (ci == CORR) {
 		    tex_outxx(vec[index], prn);
@@ -520,14 +511,14 @@ texprint_matrix (const double *vec, const int *list,
 		    printftex(vec[index], prn, 0);
 		}
 	    }
-	    pprintf(prn, "(%d\\\\\n", list[j]);
+	    pprintf(prn, "(%d\\\\\n", list[j+1]);
 	}
 
 	/* print upper triangular part of matrix */
-	for (j=1; j<=p; ++j) {
+	for (j=0; j<p; ++j) {
 	    ij2 = nf + j;
-	    for (k=0; k<j-1; k++) pputs(prn, " & ");
-	    for (k=j; k<=p; k++) {
+	    for (k=0; k<j; k++) pputs(prn, " & ");
+	    for (k=j; k<p; k++) {
 		index = ijton(ij2, nf+k, lo);
 		if (ci == CORR) {
 		    tex_outxx(vec[index], prn);
@@ -535,7 +526,7 @@ texprint_matrix (const double *vec, const int *list,
 		    printftex(vec[index], prn, 0);
 		}
 	    }
-	    pprintf(prn, "(%d\\\\\n", list[ij2]);
+	    pprintf(prn, "(%d\\\\\n", list[ij2+1]);
 	}
 	pputs(prn, "\\\\\n");
     }
@@ -564,14 +555,10 @@ void tex_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
     ntodate(date2, fr->t2, pdinfo);
 
     pputs(prn, "\\begin{raggedright}\n");
-    pputs(prn, I_("Full data range:"));
-    pprintf(prn, " %s--%s ($n$ = %d)\\\\\n", 
-	    pdinfo->stobs, pdinfo->endobs, pdinfo->n);
     pputs(prn, I_("Model estimation range:"));
     pprintf(prn, " %s--%s", date1, date2);
 
-    if (fr->nobs == pdinfo->n) pputs(prn, "\\\\\n");
-    else pprintf(prn, " ($n$ = %d)\\\\\n", fr->nobs); 
+    pprintf(prn, " ($n$ = %d)\\\\\n", fr->nobs); 
 
     pprintf(prn, I_("Standard error of residuals = %g"), fr->sigma);
     pputs(prn, "\n\\end{raggedright}\n");
@@ -590,10 +577,6 @@ void rtf_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
     ntodate(date2, fr->t2, pdinfo);
 
     pputs(prn, "{\\rtf1\\par\n\\qc ");
-    pputs(prn, I_("Full data range:"));
-    pprintf(prn, " %s - %s\\par\n", pdinfo->stobs, pdinfo->endobs);
-
-    pputs(prn, "\\qc ");
     pputs(prn, I_("Model estimation range:")); 
     pprintf(prn, " %s - %s (n = %d)\\par\n", date1, date2, fr->nobs);
 
@@ -608,7 +591,6 @@ void
 texprint_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 {
     int t, anyast = 0;
-    int n = pdinfo->n;
     double xx;
     char vname[16];
 
@@ -624,15 +606,14 @@ texprint_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 	    "   \\multicolumn{1}{c}{%s}\\\\\n",
 	    vname, I_("fitted"), I_("residuals"));
 
-    for (t=0; t<n; t++) {
-	if (t == fr->t1 && t) pputs(prn, "\\\\\n");
-	if (t == fr->t2 + 1) pputs(prn, "\\\\\n");
-
-	print_obs_marker(t, pdinfo, prn);
+    for (t=0; t<fr->nobs; t++) {
+	print_obs_marker(t + fr->t1, pdinfo, prn);
 	pputs(prn, " & ");
-	
-	if (na(fr->actual[t]) || na(fr->fitted[t])) { 
+
+	if (na(fr->actual[t])) {
 	    pputs(prn, "\\\\\n");
+	} else if (na(fr->fitted[t])) {
+	    pprintf(prn, "%13.*f \\\\\n", fr->pmax, fr->actual[t]);
 	} else {
 	    int ast;
 
@@ -672,7 +653,7 @@ void rtfprint_fit_resid (const FITRESID *fr,
 {
     double xx;
     int anyast = 0;
-    int t, n = pdinfo->n;
+    int t;
 
     rtf_fit_resid_head(fr, pdinfo, prn);
 
@@ -686,20 +667,26 @@ void rtfprint_fit_resid (const FITRESID *fr,
 	    " \\intbl \\row\n",
 	    fr->depvar, I_("fitted"), I_("residual"));
 
-    for (t=0; t<n; t++) {
+    for (t=0; t<fr->nobs; t++) {
 	pputs(prn, "\\qr ");
-	print_obs_marker(t, pdinfo, prn);
+	print_obs_marker(t + fr->t1, pdinfo, prn);
 	pputs(prn, "\\cell"); 
-	
-	if (na(fr->actual[t]) || na(fr->fitted[t])) { 
+
+	if (na(fr->actual[t])) {
 	    pputs(prn, "\\qc \\cell \\qc \\cell \\qc \\cell \\ql \\cell"
 		  " \\intbl \\row\n"); 
+	} else if (na(fr->fitted[t])) {	 
+	    printfrtf(fr->actual[t], prn, 0);
+	    pputs(prn, "\\qc \\cell \\qc \\cell \\ql \\cell"
+		  " \\intbl \\row\n"); 
 	} else {
 	    int ast;
 
 	    xx = fr->actual[t] - fr->fitted[t];
 	    ast = (fabs(xx) > 2.5 * fr->sigma);
-	    if (ast) anyast = 1;
+	    if (ast) {
+		anyast = 1;
+	    }
 	    printfrtf(fr->actual[t], prn, 0);
 	    printfrtf(fr->fitted[t], prn, 0);
 	    printfrtf(xx, prn, 0);
