@@ -67,7 +67,8 @@ static char *needle;
 #ifndef OLD_GTK
 GtkItemFactoryEntry help_items[] = {
     { N_("/_Topics"), NULL, NULL, 0, "<Branch>", GNULL },    
-    { N_("/_Find"), NULL, menu_find, 0, NULL, GNULL },
+    { N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
+    { N_("/Find/_Find in window"), NULL, menu_find, 0, "<StockItem>", GTK_STOCK_FIND },
     { NULL, NULL, NULL, 0, NULL, GNULL }
 };
 #else
@@ -81,7 +82,8 @@ GtkItemFactoryEntry help_items[] = {
 #ifdef ENABLE_NLS
 # ifndef OLD_GTK
 GtkItemFactoryEntry english_help_items[] = {
-    { N_("/_Find"), NULL, menu_find, 0, NULL, GNULL },
+    { N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
+    { N_("/Find/_Find in window"), NULL, menu_find, 0, "<StockItem>", GTK_STOCK_FIND },
     { NULL, NULL, NULL, 0, NULL, GNULL }
 };
 # else
@@ -104,10 +106,10 @@ static struct gui_help_item gui_help_items[] = {
     { GR_PLOT,    "graphing" },
     { GR_XY,      "graphing" },
     { GR_DUMMY,   "factorized" },
+    { BXPLOT,     "boxplots" },
     { GR_BOX,     "boxplots" },
     { GR_NBOX,    "boxplots" },
     { GR_3D,      "3-D" },
-    { RANGE_MEAN, "range-mean" },
     { ONLINE,     "online" },
     { MARKERS,    "markers" },
     { EXPORT,     "export" },
@@ -144,9 +146,11 @@ static int extra_command_number (const char *s)
 {
     int i;
 
-    for (i=1; gui_help_items[i].code > 0; i++)
-	if (!strcmp(s, gui_help_items[i].string))
+    for (i=1; gui_help_items[i].code > 0; i++) {
+	if (!strcmp(s, gui_help_items[i].string)) {
 	    return gui_help_items[i].code;
+	}
+    }
 
     return -1;
 }
@@ -157,9 +161,11 @@ static char *help_string_from_cmd (int cmd)
 {
     int i;
 
-    for (i=1; gui_help_items[i].code > 0; i++)
-	if (cmd == gui_help_items[i].code)
+    for (i=1; gui_help_items[i].code > 0; i++) {
+	if (cmd == gui_help_items[i].code) {
 	    return gui_help_items[i].string;
+	}
+    }
 
     return NULL;    
 }
@@ -454,8 +460,7 @@ static int assemble_topic_list (struct help_head_t **heads, int nh,
 		pos++;
 	    }
 	}
-    }
-    else {
+    } else {
 	while (fgets(test, sizeof test, fp)) {
 	    if (topic == 1) {
 		sscanf(test, "%31s", word);
@@ -503,8 +508,7 @@ get_help_length (struct help_head_t ***pheads, int *pnh, int *length,
 		len++;
 	    }
 	}
-    }
-    else {
+    } else {
 	while (!err && fgets(test, sizeof test, fp)) {
 	    if (*test == '@') {
 		chopstr(test);
@@ -597,10 +601,13 @@ static char *get_gui_help_string (int pos)
 {
     int i, j;
 
-    for (i=0; gui_heads[i] != NULL; i++) 
-	for (j=0; j<(gui_heads[i])->ntopics; j++)
-	    if (pos == (gui_heads[i])->pos[j])
+    for (i=0; gui_heads[i] != NULL; i++) { 
+	for (j=0; j<(gui_heads[i])->ntopics; j++) {
+	    if (pos == (gui_heads[i])->pos[j]) {
 		return help_string_from_cmd((gui_heads[i])->topics[j]);
+	    }
+	}
+    }
 
     return NULL;
 }
@@ -720,7 +727,7 @@ static void add_help_topics (windata_t *hwin, int script)
 
 GtkItemFactoryEntry *get_help_menu_items (int code)
 {
-    if (code == CLI_HELP_ENGLISH || GUI_HELP_ENGLISH) {
+    if (code == CLI_HELP_ENGLISH || code == GUI_HELP_ENGLISH) {
 	return english_help_items;
     } else {
 	return help_items;
@@ -732,9 +739,18 @@ GtkItemFactoryEntry *get_help_menu_items (int code)
 static windata_t *helpwin (int script, int english) 
 {
     windata_t *vwin = NULL;
-    char *helpfile;
+    char *helpfile = NULL;
     int helpcode;
 
+#ifdef ENABLE_NLS
+    if (script) {
+	helpfile = (english)? english_script_helpfile : paths.cmd_helpfile;
+	helpcode = (english)? CLI_HELP_ENGLISH : CLI_HELP;
+    } else {
+	helpfile = (english)? english_gui_helpfile : paths.helpfile;
+	helpcode = (english)? GUI_HELP_ENGLISH : GUI_HELP;
+    }
+#else
     if (script) {
 	helpfile = paths.cmd_helpfile;
 	helpcode = CLI_HELP;
@@ -742,21 +758,10 @@ static windata_t *helpwin (int script, int english)
 	helpfile = paths.helpfile;
 	helpcode = GUI_HELP;
     }
+#endif
 
     if (helpfile == NULL) return NULL;
 
-#ifdef ENABLE_NLS
-    if (english) {
-	if (script) {
-	    helpcode = CLI_HELP_ENGLISH;
-	    helpfile = english_script_helpfile;
-	} else {
-	    helpcode = GUI_HELP_ENGLISH;
-	    helpfile = english_gui_helpfile;
-	}
-    }
-#endif
-
     vwin = view_file(helpfile, 0, 0, 80, 400, helpcode);
 
     if (!english) {
@@ -780,26 +785,35 @@ void context_help (GtkWidget *widget, gpointer data)
     int pos = 0;
 
     for (i=0; gui_heads[i] != NULL; i++) {
-	for (j=0; j<(gui_heads[i])->ntopics; j++)
-	    if (help_code == (gui_heads[i])->topics[j])
+	for (j=0; j<(gui_heads[i])->ntopics; j++) {
+	    if (help_code == (gui_heads[i])->topics[j]) {
 		pos = (gui_heads[i])->pos[j];
+		break;
+	    }
+	}
+	if (pos > 0) break;
     }
 
-    /* fallback */
-    if (!pos) {
+    /* special for gui-specific help items */
+    if (pos == 0) {
 	char *helpstr = help_string_from_cmd(help_code);
-	int altcode;
 
 	if (helpstr != NULL) {
-	    altcode = extra_command_number(helpstr);
-	    for (i=0; gui_heads[i] != NULL; i++)
-		for (j=0; j<(gui_heads[i])->ntopics; j++)
-		    if (altcode == (gui_heads[i])->topics[j])
+	    int altcode = extra_command_number(helpstr);
+
+	    for (i=0; gui_heads[i] != NULL; i++) {
+		for (j=0; j<(gui_heads[i])->ntopics; j++) {
+		    if (altcode == (gui_heads[i])->topics[j]) {
 			pos = (gui_heads[i])->pos[j];
+			break;
+		    }
+		}
+		if (pos > 0) break;
+	    }
 	}
     }
 
-    if (!pos) {
+    if (pos == 0) {
 	dummy_call();
     } else {
 	do_gui_help(NULL, pos, NULL);
@@ -832,15 +846,9 @@ static void real_do_help (guint pos, int cli)
 		w = gui_help_view = hwin->w;
 	    }
 	}
-#ifndef OLD_GTK
 	g_signal_connect(G_OBJECT(w), "destroy",
 			 G_CALLBACK(gtk_widget_destroyed),
 			 (cli)? &script_help_view : &gui_help_view);
-#else
-	gtk_signal_connect(GTK_OBJECT(w), "destroy",
-			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-			   (cli)? &script_help_view : &gui_help_view);	
-#endif	
     } else {
 	gdk_window_show(w->parent->window);
 	gdk_window_raise(w->parent->window);
@@ -879,10 +887,13 @@ static int pos_from_cmd (int cmd)
 {
     int i, j;
 
-    for (i=0; cli_heads[i] != NULL; i++)
-	for (j=0; j<(cli_heads[i])->ntopics; j++)
-	    if (cmd == (cli_heads[i])->topics[j])
+    for (i=0; cli_heads[i] != NULL; i++) {
+	for (j=0; j<(cli_heads[i])->ntopics; j++) {
+	    if (cmd == (cli_heads[i])->topics[j]) {
 		return (cli_heads[i])->pos[j];
+	    }
+	}
+    }
 
     return 0;
 }
@@ -1016,8 +1027,11 @@ static void find_in_help (GtkWidget *widget, gpointer data)
     haystack = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 0,
 	gtk_text_get_length(GTK_TEXT(vwin->w)));
 
-    if (vwin->role == CLI_HELP) help_length = script_help_length;
-    else if (vwin->role == GUI_HELP) help_length = gui_help_length;
+    if (vwin->role == CLI_HELP) {
+	help_length = script_help_length;
+    } else if (vwin->role == GUI_HELP) {
+	help_length = gui_help_length;
+    }
 
 # ifdef ENABLE_NLS
     if (vwin->role == CLI_HELP_ENGLISH) {
@@ -1364,6 +1378,7 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
 	gtk_object_set_data(GTK_OBJECT(find_window), "windat", mydat);
 #endif
 	parent_find(find_window, mydat);
+	gdk_window_raise(find_window->window);
 	return;
     }
 
