@@ -44,7 +44,7 @@ struct _selector {
     int code;
     int active_var;
     int error;
-    unsigned long opts;
+    gretlopt opts;
     char *cmdlist;
     gpointer data;
 };
@@ -653,7 +653,9 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
     if (MODEL_CODE(sr->code)) {
 	if (rows > 0) { 
 	    xlist = realloc(xlist, (rows + 1) * sizeof *xlist);
-	    if (xlist != NULL) xlist[0] = rows;
+	    if (xlist != NULL) {
+		xlist[0] = rows;
+	    }
 	}
     }
 
@@ -680,7 +682,9 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 	rows = varlist_row_count(sr->auxvars);
 	if (rows > 0) {
 	    auxlist = realloc(auxlist, (rows + 1) * sizeof *auxlist);
-	    if (auxlist != NULL) auxlist[0] = rows;
+	    if (auxlist != NULL) {
+		auxlist[0] = rows;
+	    }
 	    add_to_cmdlist(sr, " ;");
 	    for (i=0; i<rows; i++) {
 		gint inst;
@@ -722,6 +726,8 @@ void delete_selection_dialog (selector *sr)
 
 static void maybe_delete_dialog (GtkWidget *widget, selector *sr)
 {
+    GtkWidget *open_dialog = get_open_dialog();
+
     if (open_dialog != NULL && !sr->error) {
 	gtk_widget_destroy(sr->dlg);
     }
@@ -729,6 +735,8 @@ static void maybe_delete_dialog (GtkWidget *widget, selector *sr)
 
 static void cancel_selector (GtkWidget *widget, selector *sr)
 {
+    GtkWidget *open_dialog = get_open_dialog();
+
     if (open_dialog != NULL) {
 	gtk_widget_destroy(sr->dlg);
     }
@@ -739,9 +747,11 @@ static void destroy_selector (GtkWidget *w, selector *sr)
     if (SAVE_DATA_ACTION(sr->code)) {
 	gtk_main_quit();
     }
+
     free(sr->cmdlist);
     free(sr);
-    open_dialog = NULL;
+
+    set_open_dialog(NULL);
 }
 
 static char *est_str (int cmdnum)
@@ -1107,7 +1117,7 @@ static int screen_scalar (int i, int c)
 static void selector_init (selector *sr, guint code, const char *title)
 {
     GtkWidget *base, *hsep;
-    int dlgheight = 300;
+    int dlgheight = 320;
 
     if (MODEL_CODE(code) && datainfo->v > 10) dlgheight = 400;
     else if (code == WLS || code == AR) dlgheight = 350;
@@ -1132,7 +1142,7 @@ static void selector_init (selector *sr, guint code, const char *title)
 
     sr->code = code;
     sr->dlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    open_dialog = sr->dlg;
+    set_open_dialog(sr->dlg);
 
     gtk_window_set_title(GTK_WINDOW(sr->dlg), title);
 
@@ -1306,6 +1316,7 @@ build_selector_buttons (selector *sr, void (*okfunc)())
 
 void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode) 
 {
+    GtkWidget *open_dialog;
     GtkWidget *right_vbox, *tmp;
     GtkWidget *big_hbox;
     GtkWidget *button_vbox;
@@ -1315,6 +1326,7 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
     gchar *topstr;
     int i;
 
+    open_dialog = get_open_dialog();
     if (open_dialog != NULL) {
 	gdk_window_raise(open_dialog->window);
 	return;
@@ -1577,6 +1589,7 @@ static GtkWidget *selection_top_label (int code)
 void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
 		       gpointer p) 
 {
+    GtkWidget *open_dialog;
     GtkWidget *left_vbox, *mid_vbox, *right_vbox, *tmp;
     GtkWidget *top_hbox, *big_hbox, *remove_button;
     GtkListStore *store;
@@ -1584,6 +1597,7 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     selector *sr;
     int i;
 
+    open_dialog = get_open_dialog();
     if (open_dialog != NULL) {
 	gdk_window_raise(open_dialog->window);
 	return;
@@ -1731,7 +1745,10 @@ char *mdata_selection_to_string (int n_required)
     struct list_maker lmkr;
 
     lmkr.liststr = mymalloc(MAXLEN);
-    if (lmkr.liststr == NULL) return NULL;
+    if (lmkr.liststr == NULL) {
+	return NULL;
+    }
+
     lmkr.liststr[0] = 0;
     lmkr.n_items = lmkr.overflow = 0;
     lmkr.len = 0;
@@ -1829,7 +1846,7 @@ gpointer selector_get_data (const selector *sr)
     return sr->data;
 }
 
-unsigned long selector_get_opts (const selector *sr)
+gretlopt selector_get_opts (const selector *sr)
 {
     return sr->opts;
 }
@@ -1838,3 +1855,19 @@ int selector_error (const selector *sr)
 {
     return sr->error;
 }
+
+void maybe_clear_selector (const int *dlist)
+{
+    int i, j;
+
+    if (xlist != NULL) {
+	for (i=1; i<=xlist[0]; i++) {
+	    for (j=1; j<=dlist[0]; j++) {
+		if (xlist[i] >= dlist[j]) {
+		    clear_selector();
+		    return;
+		}
+	    }
+	}
+    }
+}
