@@ -20,6 +20,7 @@
 /* series_view.c for gretl */
 
 #include "gretl.h"
+#include "textutil.h"
 
 typedef struct {
     char label[OBSLEN];
@@ -142,9 +143,12 @@ void free_series_view (gpointer p)
 {
     series_view_t *sview = (series_view_t *) p;
 
-    if (sview == NULL || sview->points == NULL) return;
+    if (sview == NULL) return;
+
+    if (sview->points != NULL) {
+	free(sview->points);
+    }
 
-    free(sview->points);
     free(sview);
 }
 
