@@ -29,6 +29,8 @@
 #include "model_table.h"
 #include "graph_page.h"
 #include "textbuf.h"
+#include "cmdstack.h"
+#include "filelists.h"
 
 #include "var.h"
 
@@ -44,11 +46,6 @@
 # include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
 
-/* from gui_utils.c */
-extern void winstack_init (void);
-extern void winstack_destroy (void);
-extern int winstack_match_data (gpointer p);
-
 static void auto_save_gp (gpointer data, guint i, GtkWidget *w);
 
 #include "../pixmaps/model.xpm"
@@ -133,7 +130,14 @@ static char *model_items[] = {
     N_("Delete")
 };
 
-static char *table_items[] = {
+static char *model_table_items[] = {
+    N_("Display"),
+    N_("Clear"),
+    N_("Options"),
+    N_("Help")
+};
+
+static char *graph_page_items[] = {
     N_("Display"),
     N_("Clear"),
     N_("Help")
@@ -179,9 +183,10 @@ static GtkWidget *icon_table;
 static GtkWidget *global_popup;
 static GtkWidget *session_popup;
 static GtkWidget *model_popup;
-static GtkWidget *table_popup;
+static GtkWidget *model_table_popup;
 static GtkWidget *var_popup;
 static GtkWidget *graph_popup;
+static GtkWidget *graph_page_popup;
 static GtkWidget *boxplot_popup;
 static GtkWidget *data_popup;
 static GtkWidget *info_popup;
@@ -210,6 +215,20 @@ static void gretl_text_free (GRETL_TEXT *text);
 
 /* ........................................................... */
 
+static int session_saved;
+
+int session_is_saved (void)
+{
+    return session_saved;
+}
+
+void set_session_saved (int val)
+{
+    session_saved = val;
+}
+
+/* ........................................................... */
+
 static int rebuild_init (void)
 {
     rebuild.nmodels = 0;
@@ -247,8 +266,22 @@ static void free_rebuild (void)
 
 static void edit_session_notes (void)
 {
-    edit_buffer(&session.notes, 80, 400, _("gretl: session notes"),
-		EDIT_NOTES);
+    static GtkWidget *notes_window;
+
+    if (notes_window == NULL) {
+	windata_t *vwin;
+
+	vwin = edit_buffer(&session.notes, 80, 400, 
+			   _("gretl: session notes"),
+			   EDIT_NOTES);
+	notes_window = vwin->dialog;
+	g_signal_connect(G_OBJECT(vwin->dialog), "destroy",
+			 G_CALLBACK(gtk_widget_destroyed),
+			 &notes_window);
+    } else {
+	gdk_window_show(notes_window->window);
+	gdk_window_raise(notes_window->window);
+    }
 }
 
 /* .................................................................. */
@@ -302,10 +335,14 @@ int real_add_graph_to_session (const char *fname, const char *grname,
 	    session.graphs = mymalloc(sizeof *session.graphs);
 	}
 
-	if (session.graphs == NULL) return ADD_OBJECT_FAIL;
+	if (session.graphs == NULL) {
+	    return ADD_OBJECT_FAIL;
+	}
 
 	session.graphs[i] = mymalloc(sizeof **session.graphs);
-	if (session.graphs[i] == NULL) return ADD_OBJECT_FAIL;
+	if (session.graphs[i] == NULL) {
+	    return ADD_OBJECT_FAIL;
+	}
 
 	(session.graphs[i])->sort = code;
 
@@ -658,11 +695,12 @@ void do_open_session (GtkWidget *w, gpointer data)
     int status;
 
     if (data != NULL) {    
-	if (w == NULL) /* not coming from edit_dialog */
+	if (w == NULL) {
+	    /* not coming from edit_dialog */
 	    fwin = (windata_t *) data;
-	else {
+	} else {
 	    d = (dialog_t *) data;
-	    fwin = (windata_t *) d->data;
+	    fwin = (windata_t *) dialog_data_get_data(d);
 	}
     }
 
@@ -694,8 +732,9 @@ void do_open_session (GtkWidget *w, gpointer data)
 	return;
     }
 
-    if (recreate_session(scriptfile)) 
+    if (recreate_session(scriptfile)) { 
 	return;
+    }
 
     mkfilelist(FILE_LIST_SESSION, scriptfile);
 
@@ -795,6 +834,36 @@ void free_session (void)
 
     *session.name = '\0';
 }
+
+/* ........................................................... */
+
+int highest_numbered_variable_in_session (void)
+{
+    int i, mvm, vmax = 0;
+
+    if (session.models) {
+	for (i=0; i<session.nmodels; i++) {
+	    mvm = highest_numbered_var_in_model(session.models[i],
+						datainfo);
+	    if (mvm > vmax) {
+		vmax = mvm;
+	    }
+	}
+    }
+
+    if (session.vars) {
+	for (i=0; i<session.nvars; i++) {
+	    mvm = gretl_var_get_highest_variable(session.vars[i],
+						 datainfo);
+	    if (mvm > vmax) {
+		vmax = mvm;
+	    }
+	}
+    }	
+
+    return vmax;
+}
+
 /* ........................................................... */
 
 int session_file_is_open (void)
@@ -806,16 +875,21 @@ int session_file_is_open (void)
 
 void close_session (void)
 {
-    clear_data(); /* in gtk1 version: clear_data(1); */
+    clear_data(); 
     free_session();
     free_model_table_list(NULL);
     clear_graph_page();
 
     session_menu_state(FALSE);
     session_file_open = 0;
-    if (iconview != NULL) 
+
+    if (iconview != NULL) {
 	gtk_widget_destroy(iconview);
+    }
+
     session_changed(0);
+    set_session_saved(0);
+
     winstack_destroy();
     clear_selector();
 
@@ -1126,19 +1200,6 @@ void delete_session_callback (GtkWidget *w, guint i, gpointer data)
 
 /* ........................................................... */
 
-static void store_list (int *list, char *buf)
-{
-    int i;
-    char numstr[5];
-
-    for (i=1; i<=list[0]; i++) {
-        sprintf(numstr, "%d ", list[i]);
-        strcat(buf, numstr);
-    }
-}
-
-/* ........................................................... */
-
 static char *model_cmd_str (MODEL *pmod)
 {
     char *str;
@@ -1149,10 +1210,10 @@ static char *model_cmd_str (MODEL *pmod)
     sprintf(str, "%s ", gretl_command_word(pmod->ci));
 
     if (pmod->ci == AR) {
-        store_list(pmod->arinfo->arlist, str);
+        model_list_to_string(pmod->arinfo->arlist, str);
         strcat(str, "; ");
     }
-    store_list(pmod->list, str);    
+    model_list_to_string(pmod->list, str); 
 
     return str;
 }
@@ -1186,17 +1247,17 @@ static gchar *graph_str (GRAPHT *graph)
 	    buf = g_strdup_printf("%s %s %s", ylabel, _("versus"), xlabel);
 #else
 	    char *str = malloc(64);
-	    gsize bytes;
 
 	    if (str != NULL) {
 		sprintf(str, "%s %s %s", ylabel, _("versus"), xlabel);
-		buf = g_locale_to_utf8(str, -1, NULL, &bytes, NULL);
+		buf = my_locale_to_utf8(str);
 		free(str);
 	    }
 #endif /* OLD_GTK */
 	}
 	fclose(fp);
     }
+
     return buf;
 }
 
@@ -1236,8 +1297,7 @@ static void open_gui_model (gui_obj *gobj)
 
     if (bufopen(&prn)) return;
 
-    if (printmodel(pmod, datainfo, prn))
-	pmod->errcode = E_NAN;
+    printmodel(pmod, datainfo, OPT_NONE, prn);
     view_model(prn, pmod, 78, 400, gobj->name);
 }
 
@@ -1938,8 +1998,10 @@ static void object_popup_show (gui_obj *gobj, GdkEventButton *event)
 	w = model_popup; 
 	break;
     case 't': 
+	w = model_table_popup;
+	break;
     case 'q':
-	w = table_popup; 
+	w = graph_page_popup; 
 	break;
     case 'v': 
     case 'x':
@@ -1975,6 +2037,13 @@ static void display_model_table_wrapper (void)
     display_model_table(1);
 }
 
+static void view_script_default (void)
+{
+    if (dump_command_stack(cmdfile, 0)) return;
+
+    view_file(cmdfile, 0, 0, 78, 350, EDIT_SCRIPT);
+}
+
 static gboolean session_icon_click (GtkWidget *widget, 
 				    GdkEventButton *event,
 				    gpointer data)
@@ -2158,7 +2227,14 @@ static void object_popup_activated (GtkWidget *widget, gpointer data)
 	else if (obj->sort == 'q') {
 	    context_help(NULL, GINT_TO_POINTER(GRAPHPAGE));
 	}
-    }    
+    } 
+    else if (strcmp(item, _("Options")) == 0) {
+	if (obj->sort == 't') {
+	    model_table_dialog();
+	} else {
+	    dummy_call();
+	}
+    }     
 }
 
 /* ........................................................... */
@@ -2417,7 +2493,7 @@ static int silent_remember (MODEL **ppmod, DATAINFO *pdinfo)
     session.models[session.nmodels] = pmod;
     session.nmodels += 1;
 
-    tmp = gretl_model_new(pdinfo);
+    tmp = gretl_model_new();
     if (tmp == NULL) return 1;
 
     *ppmod = tmp; /* replaced */
@@ -2459,10 +2535,10 @@ int clear_or_save_model (MODEL **ppmod, DATAINFO *pdinfo,
 	    }
 	}
 	save = 1;
-	clear_model(*ppmod, pdinfo);
+	clear_model(*ppmod);
     } else {
 	/* no rebuild, no need to save */
-	clear_model(*ppmod, pdinfo);
+	clear_model(*ppmod);
     }
 
     return 0;
@@ -2595,6 +2671,15 @@ static void session_build_popups (void)
 	}
     }
 
+    if (model_table_popup == NULL) {
+	model_table_popup = gtk_menu_new();
+	for (i=0; i<sizeof model_table_items / sizeof model_table_items[0]; i++) {
+		create_popup_item(model_table_popup, 
+				  _(model_table_items[i]), 
+				  object_popup_activated);
+	}
+    }
+
     if (var_popup == NULL) {
 	var_popup = gtk_menu_new();
 	for (i=0; i<sizeof var_items / sizeof var_items[0]; i++) {
@@ -2604,15 +2689,6 @@ static void session_build_popups (void)
 	}
     }
 
-    if (table_popup == NULL) {
-	table_popup = gtk_menu_new();
-	for (i=0; i<sizeof table_items / sizeof table_items[0]; i++) {
-		create_popup_item(table_popup, 
-				  _(table_items[i]), 
-				  object_popup_activated);
-	}
-    }
-
     if (graph_popup == NULL) {
 	graph_popup = gtk_menu_new();
 	for (i=0; i<sizeof graph_items / sizeof graph_items[0]; i++) {
@@ -2622,6 +2698,15 @@ static void session_build_popups (void)
 	}
     }
 
+    if (graph_page_popup == NULL) {
+	graph_page_popup = gtk_menu_new();
+	for (i=0; i<sizeof graph_page_items / sizeof graph_page_items[0]; i++) {
+		create_popup_item(graph_page_popup, 
+				  _(graph_page_items[i]), 
+				  object_popup_activated);
+	}
+    }
+
     if (boxplot_popup == NULL) {
 	boxplot_popup = gtk_menu_new();
 	for (i=0; i<sizeof graph_items / sizeof graph_items[0]; i++) {
@@ -2762,11 +2847,27 @@ static void gtk_entry_set_has_frame (GtkEntry *entry, gboolean b)
 static void size_name_entry (GtkWidget *w, const char *name)
 {
     PangoLayout *layout;
-    PangoRectangle rect;
+    PangoRectangle logrect;
+    PangoFontDescription *pfd;
+#ifdef USE_GNOME
+    GtkSettings *settings;
+    gchar *fontname;
+
+    settings = gtk_settings_get_default();
+    g_object_get(G_OBJECT(settings), "gtk-font-name", &fontname, NULL);
+#endif    
 
     layout = gtk_entry_get_layout(GTK_ENTRY(w));
-    pango_layout_get_extents(layout, NULL, &rect);
-    gtk_widget_set_size_request(w, 2 + rect.width / PANGO_SCALE, -1); 
+#ifdef USE_GNOME
+    pfd = pango_font_description_from_string(fontname);
+    g_free(fontname);
+#else
+    pfd = pango_font_description_from_string(get_app_fontname());
+#endif
+    pango_layout_set_font_description(layout, pfd);
+
+    pango_layout_get_pixel_extents(layout, NULL, &logrect);
+    gtk_widget_set_size_request(w, 1.1 * logrect.width, -1); 
 }
 
 static gboolean object_name_return (GtkWidget *w,
@@ -2894,14 +2995,12 @@ static void create_gobj_icon (gui_obj *gobj, const char **xpm)
     if (gobj->sort == 'm' || gobj->sort == 'g' ||
 	gobj->sort == 'v' || gobj->sort == 'b') { 
 	gobj->label = gtk_entry_new();
-	gtk_widget_show(gobj->label);
 	/* on gtk 2.0.N, the text is/was not going into the selected font */
 	gtk_entry_set_text(GTK_ENTRY(gobj->label), gobj->name);
 	gtk_editable_set_editable(GTK_EDITABLE(gobj->label), FALSE);
 	gtk_entry_set_has_frame(GTK_ENTRY(gobj->label), FALSE);
 	gtk_entry_set_max_length(GTK_ENTRY(gobj->label), OBJNAMLEN);
 	size_name_entry(gobj->label, gobj->name);
-	gtk_widget_hide(gobj->label);
 	g_signal_connect(G_OBJECT(gobj->label), "button-press-event",
 			 G_CALLBACK(start_rename_object), gobj);
 	g_signal_connect(G_OBJECT(gobj->label), "key-press-event",
@@ -2992,7 +3091,6 @@ static void auto_save_gp (gpointer data, guint quiet, GtkWidget *w)
     gchar *msg, *savestuff;
     windata_t *mydata = (windata_t *) data;
 # ifdef ENABLE_NLS
-    gsize bytes;
     gchar *trbuf;
 # endif
 
@@ -3009,16 +3107,23 @@ static void auto_save_gp (gpointer data, guint quiet, GtkWidget *w)
     }
 
 # ifdef ENABLE_NLS
-    trbuf = g_locale_from_utf8(savestuff, -1, NULL, &bytes, NULL);
-    fprintf(fp, "%s", trbuf);
-    g_free(trbuf);
+    trbuf = force_locale_from_utf8(savestuff);
+    if (trbuf != NULL) {
+	fputs(trbuf, fp);
+	g_free(trbuf);
+    } else {
+	fputs(savestuff, fp);
+    }
 # else
-    fprintf(fp, "%s", savestuff);
+    fputs(savestuff, fp);
 # endif
 
     g_free(savestuff); 
     fclose(fp);
-    if (!quiet) infobox(_("plot commands saved"));
+
+    if (!quiet) {
+	infobox(_("plot commands saved"));
+    }
 }
 
 #endif /* gtk-2.0 branch */
