@@ -20,9 +20,13 @@
 /* settings.c for gretl */
 
 #include "gretl.h"
+#include "filelists.h"
 #include "webget.h"
+#include "toolbar.h"
 
 #include <unistd.h>
+#include <sys/types.h>
+#include <dirent.h>
 
 #if defined(USE_GNOME) && !defined(OLD_GTK)
 # define GNOME2
@@ -36,9 +40,7 @@
 # include <windows.h>
 # include "gretlwin32.h"
 #else
-# include <dirent.h>
 # include <sys/stat.h>
-# include <sys/types.h>
 # include <fcntl.h>
 # include <errno.h>
 # ifndef OLD_GTK
@@ -46,15 +48,13 @@
 # endif
 #endif
 
+extern const char *version_string;
+
 #if !defined(G_OS_WIN32) && !defined(USE_GNOME)
 char rcfile[MAXLEN];
 #endif
 
-extern GtkWidget *toolbar_box;
-
 extern int want_toolbar;
-extern char calculator[MAXSTR];
-extern char editor[MAXSTR];
 extern char Rcommand[MAXSTR];
 extern char dbproxy[21];
 
@@ -65,15 +65,6 @@ extern char tramodir[MAXSTR];
 
 int use_proxy;
 
-/* filelist stuff */
-#define MAXRECENT 4
-
-static void printfilelist (int filetype, FILE *fp);
-
-static char datalist[MAXRECENT][MAXSTR], *datap[MAXRECENT];
-static char sessionlist[MAXRECENT][MAXSTR], *sessionp[MAXRECENT];
-static char scriptlist[MAXRECENT][MAXSTR], *scriptp[MAXRECENT];
-
 static void make_prefs_tab (GtkWidget *notebook, int tab);
 static void apply_changes (GtkWidget *widget, gpointer data);
 #ifndef G_OS_WIN32
@@ -161,82 +152,88 @@ typedef struct {
    controls the sensitivity of the "dbproxy" entry widget. */
 
 RCVARS rc_vars[] = {
-    {"gretldir", N_("Main gretl directory"), NULL, paths.gretldir, 
-     ROOTSET, MAXLEN, 1, NULL},
-    {"userdir", N_("User's gretl directory"), NULL, paths.userdir, 
-     USERSET, MAXLEN, 1, NULL},
-    {"expert", N_("Expert mode (no warnings)"), NULL, &expert, 
-     BOOLSET, 0, 1, NULL},
-    {"updater", N_("Tell me about gretl updates"), NULL, &updater, 
-     BOOLSET, 0, 1, NULL},
-    {"toolbar", N_("Show gretl toolbar"), NULL, &want_toolbar, 
-     BOOLSET, 0, 1, NULL},
+    { "gretldir", N_("Main gretl directory"), NULL, paths.gretldir, 
+      ROOTSET, MAXLEN, 1, NULL },
+    { "userdir", N_("User's gretl directory"), NULL, paths.userdir, 
+      USERSET, MAXLEN, 1, NULL },
+    { "expert", N_("Expert mode (no warnings)"), NULL, &expert, 
+      BOOLSET, 0, 1, NULL },
+    { "updater", N_("Tell me about gretl updates"), NULL, &updater, 
+      BOOLSET, 0, 1, NULL },
+    { "toolbar", N_("Show gretl toolbar"), NULL, &want_toolbar, 
+      BOOLSET, 0, 1, NULL },
 #ifdef ENABLE_NLS
-    {"lcnumeric", N_("Use locale setting for decimal point"), NULL, &lcnumeric, 
-     BOOLSET, 0, 1, NULL},
+    { "lcnumeric", N_("Use locale setting for decimal point"), NULL, &lcnumeric, 
+      BOOLSET, 0, 1, NULL },
 #endif
 #ifdef G_OS_WIN32
-    {"wimp", N_("Emulate Windows look"), NULL, &wimp, BOOLSET, 0, 1, NULL},    
+    { "wimp", N_("Emulate Windows look"), NULL, &wimp, 
+      BOOLSET, 0, 1, NULL },    
 #endif
-    {"gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
-     ROOTSET, MAXLEN, 3, NULL},
-    {"Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
-     ROOTSET, MAXSTR, 3, NULL},
-    {"viewdvi", N_("Command to view DVI files"), NULL, viewdvi, 
-     ROOTSET, MAXSTR, 3, NULL},
+    { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
+      ROOTSET, MAXLEN, 3, NULL },
+    { "Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
+      ROOTSET, MAXSTR, 3, NULL },
+    { "viewdvi", N_("Command to view DVI files"), NULL, viewdvi, 
+      ROOTSET, MAXSTR, 3, NULL },
 #ifndef G_OS_WIN32
-    {"viewps", N_("Command to view postscript files"), NULL, viewps, 
-     ROOTSET, MAXSTR, 3, NULL},
+    { "viewps", N_("Command to view postscript files"), NULL, viewps, 
+      ROOTSET, MAXSTR, 3, NULL },
 #endif
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
-    {"midiplayer", N_("Program to play MIDI files"), NULL, midiplayer, 
-     USERSET, MAXSTR, 3, NULL},
-#endif
-    {"calculator", N_("Calculator"), NULL, calculator, USERSET, MAXSTR, 3, NULL},
-#ifdef SELECT_EDITOR
-    {"editor", N_("Editor"), NULL, editor, USERSET, MAXSTR, 3, NULL},
+    { "midiplayer", N_("Program to play MIDI files"), NULL, midiplayer, 
+      USERSET, MAXSTR, 3, NULL },
 #endif
+    { "calculator", N_("Calculator"), NULL, calculator, 
+      USERSET, MAXSTR, 3, NULL },
 #ifdef HAVE_X12A
-    {"x12a", N_("path to x12arima"), NULL, paths.x12a, ROOTSET, MAXSTR, 3, NULL},
+    { "x12a", N_("path to x12arima"), NULL, paths.x12a, 
+      ROOTSET, MAXSTR, 3, NULL },
 #endif
 #ifdef HAVE_TRAMO
-    {"tramo", N_("path to tramo"), NULL, tramo, ROOTSET, MAXSTR, 3, NULL},
+    { "tramo", N_("path to tramo"), NULL, tramo, ROOTSET, MAXSTR, 3, NULL},
 #endif
 #ifdef G_OS_WIN32
-    {"x12adir", N_("X-12-ARIMA working directory"), NULL, paths.x12adir, 
-     ROOTSET, MAXSTR, 3, NULL},
+    { "x12adir", N_("X-12-ARIMA working directory"), NULL, paths.x12adir, 
+      ROOTSET, MAXSTR, 3, NULL},
 #endif
 #ifdef G_OS_WIN32
-    {"tramodir", N_("TRAMO working directory"), NULL, tramodir, 
-     ROOTSET, MAXSTR, 3, NULL},
+    { "tramodir", N_("TRAMO working directory"), NULL, tramodir, 
+      ROOTSET, MAXSTR, 3, NULL},
 #endif
-    {"binbase", N_("gretl database directory"), NULL, paths.binbase, 
-     USERSET, MAXLEN, 2, NULL},
-    {"ratsbase", N_("RATS data directory"), NULL, paths.ratsbase, 
-     USERSET, MAXLEN, 2, NULL},
-    {"dbhost_ip", N_("Database server IP"), NULL, paths.dbhost_ip, 
-     USERSET, 16, 2, NULL},
-    {"dbproxy", N_("HTTP proxy (ipnumber:port)"), NULL, dbproxy, 
-     USERSET, 21, 2, NULL},
-    {"useproxy", N_("Use HTTP proxy"), NULL, &use_proxy, BOOLSET, 1, 2, NULL},
-    {"usecwd", N_("Use current working directory as default"), 
-     N_("Use gretl user directory as default"), &usecwd, BOOLSET, 0, 4, NULL},
-    {"olddat", N_("Use \".dat\" as default datafile suffix"), 
-     N_("Use \".gdt\" as default suffix"), &olddat, BOOLSET, 0, 5, NULL},
-    {"useqr", N_("Use QR decomposition"), 
-     N_("Use Cholesky decomposition"), &useqr, BOOLSET, 0, 1, NULL},
-    {"Fixed_font", N_("Fixed font"), NULL, fixedfontname, USERSET, MAXLEN, 0, NULL},
+    { "binbase", N_("gretl database directory"), NULL, paths.binbase, 
+      USERSET, MAXLEN, 2, NULL },
+    { "ratsbase", N_("RATS data directory"), NULL, paths.ratsbase, 
+      USERSET, MAXLEN, 2, NULL },
+    { "dbhost", N_("Database server name"), NULL, paths.dbhost, 
+      USERSET, 32, 2, NULL },
+    { "dbproxy", N_("HTTP proxy (ipnumber:port)"), NULL, dbproxy, 
+      USERSET, 21, 2, NULL },
+    { "useproxy", N_("Use HTTP proxy"), NULL, &use_proxy, 
+      BOOLSET, 1, 2, NULL },
+    { "usecwd", N_("Use current working directory as default"), 
+      N_("Use gretl user directory as default"), &usecwd, 
+      BOOLSET, 0, 4, NULL },
+    { "olddat", N_("Use \".dat\" as default datafile suffix"), 
+      N_("Use \".gdt\" as default suffix"), &olddat, 
+      BOOLSET, 0, 5, NULL },
+    { "useqr", N_("Use QR decomposition"), N_("Use Cholesky decomposition"), &useqr, 
+      BOOLSET, 0, 1, NULL },
+    { "Fixed_font", N_("Fixed font"), NULL, fixedfontname, 
+      USERSET, MAXLEN, 0, NULL },
 #if !defined(USE_GNOME) && !defined(OLD_GTK)
-    {"App_font", N_("Menu font"), NULL, appfontname, USERSET, MAXLEN, 0, NULL},
+    { "App_font", N_("Menu font"), NULL, appfontname, 
+      USERSET, MAXLEN, 0, NULL },
 #endif
-    {"DataPage", "Default data page", NULL, datapage, INVISET, 
-     sizeof datapage, 0, NULL},
-    {"ScriptPage", "Default script page", NULL, scriptpage, INVISET, 
-     sizeof scriptpage, 0, NULL},    
-    {"Png_font", N_("PNG graph font"), NULL, paths.pngfont, INVISET, 16, 0, NULL},
-    {"Gp_colors", N_("Gnuplot colors"), NULL, gpcolors, INVISET, 
-     sizeof gpcolors, 0, NULL},
-    {NULL, NULL, NULL, NULL, 0, 0, 0, NULL}
+    { "DataPage", "Default data page", NULL, datapage, 
+      INVISET, sizeof datapage, 0, NULL },
+    { "ScriptPage", "Default script page", NULL, scriptpage, 
+      INVISET, sizeof scriptpage, 0, NULL },    
+    { "Png_font", N_("PNG graph font"), NULL, paths.pngfont, 
+      INVISET, 32, 0, NULL },
+    { "Gp_colors", N_("Gnuplot colors"), NULL, gpcolors, 
+      INVISET, sizeof gpcolors, 0, NULL },
+    { NULL, NULL, NULL, NULL, 0, 0, 0, NULL }
 };
 
 /* ........................................................... */
@@ -263,6 +260,16 @@ const char *get_scriptpage (void)
 
 /* ........................................................... */
 
+static gretlopt set_paths_opt = OPT_X;
+
+void force_english_help (void)
+{
+    set_paths_opt |= OPT_N;
+    set_paths(&paths, set_paths_opt);
+}
+
+/* ........................................................... */
+
 void set_fixed_font (void)
 {
 #ifndef OLD_GTK
@@ -278,6 +285,12 @@ void set_fixed_font (void)
 /* ........................................................... */
 
 #if !defined(USE_GNOME) && !defined(OLD_GTK)
+
+const char *get_app_fontname (void)
+{
+    return appfontname;
+}
+
 void set_app_font (const char *fontname)
 {
     GtkSettings *settings;
@@ -308,6 +321,7 @@ void set_app_font (const char *fontname)
 	pango_font_description_free(pfd);
     }
 }
+
 #endif
 
 /* .................................................................. */
@@ -545,15 +559,9 @@ void options_dialog (gpointer data)
     gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(dialog)->action_area), TRUE);
     gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
 
-#ifndef OLD_GTK
-    g_signal_connect (G_OBJECT(dialog), "delete_event", 
-		      G_CALLBACK(delete_widget), 
-		      dialog);
-#else
-    gtk_signal_connect_object(GTK_OBJECT(dialog), "delete_event", 
-			      GTK_SIGNAL_FUNC(gtk_widget_destroy), 
-			      GTK_OBJECT(dialog));
-#endif
+    g_signal_connect(G_OBJECT(dialog), "delete_event", 
+		     G_CALLBACK(delete_widget), 
+		     dialog);
 
     notebook = gtk_notebook_new();
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), notebook, 
@@ -570,47 +578,34 @@ void options_dialog (gpointer data)
     GTK_WIDGET_SET_FLAGS(tempwid, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), 
 		       tempwid, TRUE, TRUE, 0);
-#ifndef OLD_GTK
+
     g_signal_connect(G_OBJECT(tempwid), "clicked", 
 		     G_CALLBACK(apply_changes), NULL);
     g_signal_connect(G_OBJECT(tempwid), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     dialog);
-#else
-    gtk_signal_connect(GTK_OBJECT(tempwid), "clicked", 
-		       GTK_SIGNAL_FUNC(apply_changes), NULL);
-    gtk_signal_connect_object(GTK_OBJECT(tempwid), "clicked", 
-			      GTK_SIGNAL_FUNC(gtk_widget_destroy), 
-			      GTK_OBJECT(dialog));
-#endif
+
     gtk_widget_show(tempwid);
 
     tempwid = standard_button(GTK_STOCK_CANCEL);
     GTK_WIDGET_SET_FLAGS(tempwid, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), 
 		       tempwid, TRUE, TRUE, 0);
-#ifndef OLD_GTK
+
     g_signal_connect(G_OBJECT(tempwid), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     dialog);
-#else
-    gtk_signal_connect_object(GTK_OBJECT(tempwid), "clicked", 
-			      GTK_SIGNAL_FUNC(gtk_widget_destroy), 
-			      GTK_OBJECT(dialog));
-#endif
+
     gtk_widget_show(tempwid);
 
     tempwid = standard_button(GTK_STOCK_APPLY);
     GTK_WIDGET_SET_FLAGS(tempwid, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), 
 		       tempwid, TRUE, TRUE, 0);
-#ifndef OLD_GTK
+
     g_signal_connect(G_OBJECT(tempwid), "clicked", 
 		     G_CALLBACK(apply_changes), NULL);
-#else
-    gtk_signal_connect(GTK_OBJECT(tempwid), "clicked", 
-		       GTK_SIGNAL_FUNC(apply_changes), NULL);
-#endif
+
     gtk_widget_grab_default(tempwid);
     gtk_widget_show(tempwid);
 
@@ -657,15 +652,9 @@ static GtkWidget *make_path_browse_button (RCVARS *rc)
     GtkWidget *b;
 
     b = gtk_button_new_with_label(_("Browse..."));
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(b), "clicked",
 		     G_CALLBACK(browse_button_callback), 
 		     rc);
-#else
-    gtk_signal_connect(GTK_OBJECT(b), "clicked",
-                       GTK_SIGNAL_FUNC(browse_button_callback), 
-		       rc);
-#endif
     return b;
 }
 
@@ -690,16 +679,17 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
     gtk_container_set_border_width(GTK_CONTAINER(box), 10);
     gtk_widget_show(box);
 
-    if (tab == 1)
+    if (tab == 1) {
 	tempwid = gtk_label_new(_("General"));
-    else if (tab == 2)
+    } else if (tab == 2) {
 	tempwid = gtk_label_new(_("Databases"));
-    else if (tab == 3)
+    } else if (tab == 3) {
 	tempwid = gtk_label_new(_("Programs"));
-    else if (tab == 4)
+    } else if (tab == 4) {
 	tempwid = gtk_label_new(_("Open/Save path"));
-    else if (tab == 5)
+    } else if (tab == 5) {
 	tempwid = gtk_label_new(_("Data files"));
+    }
     
     gtk_widget_show(tempwid);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, tempwid);   
@@ -737,30 +727,18 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 
 		/* special case: warning */
 		if (!strcmp(rc->key, "wimp") || !strcmp(rc->key, "lcnumeric")) {
-#ifndef OLD_GTK
 		    g_signal_connect(G_OBJECT(rc->widget), "toggled",
 				     G_CALLBACK(takes_effect_on_restart), 
 				     NULL);
-#else
-		    gtk_signal_connect(GTK_OBJECT(rc->widget), "toggled",
-				       GTK_SIGNAL_FUNC(takes_effect_on_restart), 
-				       NULL);
-#endif
 		}
 
 		/* special case: link between toggle and preceding entry */
 		if (rc->len && !(rc->type & FIXSET)) {
 		    gtk_widget_set_sensitive(rc_vars[i-1].widget,
 					     GTK_TOGGLE_BUTTON(rc->widget)->active);
-#ifndef OLD_GTK
 		    g_signal_connect(G_OBJECT(rc->widget), "clicked",
 				     G_CALLBACK(flip_sensitive),
 				     rc_vars[i-1].widget);
-#else
-		    gtk_signal_connect(GTK_OBJECT(rc->widget), "clicked",
-				       GTK_SIGNAL_FUNC(flip_sensitive),
-				       rc_vars[i-1].widget);
-#endif
 		} 
 
 		gtk_widget_show(rc->widget);
@@ -899,6 +877,12 @@ static void set_lcnumeric (void)
 		set = setlocale(LC_NUMERIC, "it");
 	    }
 	}
+	else if (lang != NULL && !strcmp(lang, "pl")) {
+	    set = setlocale(LC_NUMERIC, "Polish");
+	    if (set == NULL) {
+		set = setlocale(LC_NUMERIC, "pl");
+	    }
+	}
 
 	if (set == NULL) {
 	    setlocale(LC_NUMERIC, "");
@@ -937,7 +921,6 @@ static void set_gp_colors (void)
 static void apply_changes (GtkWidget *widget, gpointer data) 
 {
     const gchar *tempstr;
-    extern void show_toolbar (void);
     int i = 0;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
@@ -958,12 +941,7 @@ static void apply_changes (GtkWidget *widget, gpointer data)
     
     write_rc();
 
-    if (toolbar_box == NULL && want_toolbar)
-	show_toolbar();
-    else if (toolbar_box != NULL && !want_toolbar) {
-	gtk_widget_destroy(toolbar_box);
-	toolbar_box = NULL;
-    }
+    show_or_hide_toolbar(want_toolbar);
 
     set_use_qr(useqr);
 
@@ -987,31 +965,23 @@ static void str_to_boolvar (char *s, void *b)
     }	
 }
 
-static void boolvar_to_str (void *b, char *s)
-{
-    if (*(int *)b) strcpy(s, "true");
-    else strcpy(s, "false");
-}
-
 #endif
 
-/* .................................................................. */
-
-static void initialize_file_lists (void)
+static void boolvar_to_str (void *b, char *s)
 {
-    int i;
-
-    /* initialize lists of recently opened files */
-    for (i=0; i<MAXRECENT; i++) { 
-	datalist[i][0] = 0;
-	sessionlist[i][0] = 0;
-	scriptlist[i][0] = 0;
+    if (*(int *)b) {
+	strcpy(s, "true");
+    } else {
+	strcpy(s, "false");
     }
 }
 
-/* .................................................................. */
+/* next section: variant versions of write_rc and read_rc, depending
+   on both GTK version and platform
+*/
 
-#ifdef GNOME2
+/* first the gnome 2 versions */
+#ifdef GNOME2 
 
 void write_rc (void) 
 {
@@ -1026,19 +996,17 @@ void write_rc (void)
 	if (rc_vars[i].type == BOOLSET) {
 	    gboolean val = *(gboolean *) rc_vars[i].var;
 
-	    gconf_client_set_bool (client, key, val, NULL);
+	    gconf_client_set_bool(client, key, val, NULL);
 	} else {
-	    gconf_client_set_string (client, key, rc_vars[i].var, NULL);
+	    gconf_client_set_string(client, key, rc_vars[i].var, NULL);
 	}
     }
 
+    save_file_lists(client);
+
     g_object_unref(G_OBJECT(client));
 
-    printfilelist(FILE_LIST_DATA, NULL);
-    printfilelist(FILE_LIST_SESSION, NULL); 
-    printfilelist(FILE_LIST_SCRIPT, NULL);
-
-    set_paths(&paths, 0, 1);
+    set_paths(&paths, set_paths_opt);
 }
 
 static void read_rc (void) 
@@ -1048,12 +1016,12 @@ static void read_rc (void)
     GSList *flist = NULL;
     gchar *value;
     char key[MAXSTR];
-    int i;
-    static char *sections[] = {
+    const char *file_sections[] = {
 	"recent_data_files",
 	"recent_session_files",
 	"recent_script_files"
     };	
+    int i;
 
     client = gconf_client_get_default();
 
@@ -1062,7 +1030,7 @@ static void read_rc (void)
 	if (rc_vars[i].type == BOOLSET) {
 	    gboolean val;
 
-	    val = gconf_client_get_bool (client, key, &error);
+	    val = gconf_client_get_bool(client, key, &error);
 	    if (error) {
 		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
 		g_clear_error(&error);
@@ -1070,7 +1038,7 @@ static void read_rc (void)
 		*(int *) rc_vars[i].var = val;
 	    }
 	} else {
-	    value = gconf_client_get_string (client, key, &error);
+	    value = gconf_client_get_string(client, key, &error);
 
 	    if (error) {
 		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
@@ -1090,14 +1058,12 @@ static void read_rc (void)
     for (i=0; i<3; i++) {
 	int j;
 
-	sprintf(key, "/apps/gretl/%s", sections[i]);
-	flist = gconf_client_get_list (client, key,
-				       GCONF_VALUE_STRING, NULL);
+	sprintf(key, "/apps/gretl/%s", file_sections[i]);
+	flist = gconf_client_get_list(client, key,
+				      GCONF_VALUE_STRING, NULL);
 	if (flist != NULL) {
 	    for (j=0; j<MAXRECENT; j++) {
-		if (i == 0) strcpy(datalist[j], flist->data);
-		else if (i == 1) strcpy(sessionlist[j], flist->data);
-		else if (i == 2) strcpy(scriptlist[j], flist->data);
+		write_filename_to_list(i, j, flist->data);
 		flist = flist->next;
 	    }
 	    g_slist_free(flist);
@@ -1110,7 +1076,7 @@ static void read_rc (void)
     set_use_qr(useqr);
     set_gp_colors();
 
-    set_paths(&paths, 0, 1); /* 0 = not defaults, 1 = gui */
+    set_paths(&paths, set_paths_opt);
 
 # if defined(HAVE_TRAMO) || defined(HAVE_X12A)
     set_tramo_x12a_dirs();
@@ -1121,7 +1087,8 @@ static void read_rc (void)
 # endif
 }
 
-#elif defined(USE_GNOME)
+/* then the gnome 1 versions */
+#elif defined(USE_GNOME)  
 
 void write_rc (void) 
 {
@@ -1139,20 +1106,23 @@ void write_rc (void)
 	}
     }
 
-    printfilelist(FILE_LIST_DATA, NULL);
-    printfilelist(FILE_LIST_SESSION, NULL);
-    printfilelist(FILE_LIST_SCRIPT, NULL);
+    save_file_lists();
 
     gnome_config_sync();
 
-    set_paths(&paths, 0, 1);
+    set_paths(&paths, set_paths_opt);
 }
 
 static void read_rc (void) 
 {
     gchar *value = NULL;
     char gpath[MAXSTR];
-    int i;
+    const char *file_sections[] = {
+	"recent data files",
+	"recent session files",
+	"recent script files"
+    };	
+    int i, j;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 	sprintf(gpath, "/gretl/%s/%s", rc_vars[i].description, 
@@ -1171,35 +1141,22 @@ static void read_rc (void)
     initialize_file_lists();
 
     /* get recent file lists */
-    for (i=0; i<MAXRECENT; i++) {
-	sprintf(gpath, "/gretl/recent data files/%d", i);
-	if ((value = gnome_config_get_string(gpath)) != NULL) { 
-	    strcpy(datalist[i], value);
-	    g_free(value);
+    for (j=0; j<3; j++) {
+	for (i=0; i<MAXRECENT; i++) {
+	    sprintf(gpath, "/gretl/%s/%d", file_sections[j], i);
+	    if ((value = gnome_config_get_string(gpath)) != NULL) { 
+		write_filename_to_list(j, i, value);
+		g_free(value);
+	    } else {
+		break;
+	    }
 	}
-	else break;
     }    
-    for (i=0; i<MAXRECENT; i++) {
-	sprintf(gpath, "/gretl/recent session files/%d", i);
-	if ((value = gnome_config_get_string(gpath)) != NULL) { 
-	    strcpy(sessionlist[i], value);
-	    g_free(value);
-	}
-	else break;
-    } 
-    for (i=0; i<MAXRECENT; i++) {
-	sprintf(gpath, "/gretl/recent script files/%d", i);
-	if ((value = gnome_config_get_string(gpath)) != NULL) { 
-	    strcpy(scriptlist[i], value);
-	    g_free(value);
-	}
-	else break;
-    }
 
     set_use_qr(useqr);
     set_gp_colors();
 
-    set_paths(&paths, 0, 1); /* 0 = not defaults, 1 = gui */
+    set_paths(&paths, set_paths_opt);
 
 # if defined(HAVE_TRAMO) || defined(HAVE_X12A)
     set_tramo_x12a_dirs();
@@ -1243,11 +1200,9 @@ void write_rc (void)
 	}
     }
 
-    printfilelist(FILE_LIST_DATA, NULL); 
-    printfilelist(FILE_LIST_SESSION, NULL);
-    printfilelist(FILE_LIST_SCRIPT, NULL);
+    save_file_lists();
 
-    set_paths(&paths, 0, 1);
+    set_paths(&paths, set_paths_opt);
 }
 
 static int get_network_settings (void)
@@ -1305,8 +1260,13 @@ static int get_network_settings (void)
 
 void read_rc (void) 
 {
-    int i = 0;
     char rpath[MAXSTR], value[MAXSTR];
+    const char *file_sections[] = {
+	"recent data files",
+	"recent session files",
+	"recent script files"
+    };
+    int i, j;
 
     if (get_network_settings() && *paths.userdir != '\0') {
 	win32_make_user_dirs();
@@ -1350,29 +1310,21 @@ void read_rc (void)
     initialize_file_lists();
 
     /* get recent file lists */
-    for (i=0; i<MAXRECENT; i++) {
-	sprintf(rpath, "recent data files\\%d", i);
-	if (read_reg_val(HKEY_CURRENT_USER, "gretl", rpath, value) == 0) 
-	    strcpy(datalist[i], value);
-	else break;
+    for (j=0; j<3; j++) {
+	for (i=0; i<MAXRECENT; i++) {
+	    sprintf(rpath, "%s\\%d", file_sections[j], i);
+	    if (read_reg_val(HKEY_CURRENT_USER, "gretl", rpath, value) == 0) { 
+		write_filename_to_list(j, i, value);
+	    } else {
+		break;
+	    }
+	}
     }    
-    for (i=0; i<MAXRECENT; i++) {
-	sprintf(rpath, "recent session files\\%d", i);
-	if (read_reg_val(HKEY_CURRENT_USER, "gretl", rpath, value) == 0) 
-	    strcpy(sessionlist[i], value);
-	else break;
-    } 
-    for (i=0; i<MAXRECENT; i++) {
-	sprintf(rpath, "recent script files\\%d", i);
-	if (read_reg_val(HKEY_CURRENT_USER, "gretl", rpath, value) == 0) 
-	    strcpy(scriptlist[i], value);
-	else break;
-    }
 
     set_use_qr(useqr);
     set_gp_colors();
 
-    set_paths(&paths, 0, 1);
+    set_paths(&paths, set_paths_opt);
 
 # if defined(HAVE_TRAMO) || defined(HAVE_X12A)
     set_tramo_x12a_dirs();
@@ -1412,20 +1364,24 @@ void write_rc (void)
 	}
     }
 
-    printfilelist(FILE_LIST_DATA, rc);
-    printfilelist(FILE_LIST_SESSION, rc);
-    printfilelist(FILE_LIST_SCRIPT, rc);
+    save_file_lists(rc);
 
     fclose(rc);
-    set_paths(&paths, 0, 1);
+
+    set_paths(&paths, set_paths_opt);
 }
 
 static void read_rc (void) 
 {
     FILE *fp;
-    int i, j;
     char line[MAXLEN], key[32], linevar[MAXLEN];
+    const char *file_sections[] = {
+	"recent data files:",
+	"recent session files:",
+	"recent script files:"
+    };
     int gotrecent = 0;
+    int i, j;
 
     fp = fopen(rcfile, "r");
     if (fp == NULL) return;
@@ -1458,31 +1414,39 @@ static void read_rc (void)
     initialize_file_lists();
 
     if (gotrecent || (fgets(line, MAXLEN, fp) != NULL && 
-		      strncmp(line, "recent data files:", 18) == 0)) {
+		      strncmp(line, file_sections[0], 18) == 0)) {
 	i = 0;
 	while (fgets(line, MAXLEN, fp) && i<MAXRECENT) {
-	    if (strncmp(line, "recent session files:", 21) == 0)
+	    if (strncmp(line, file_sections[1], 21) == 0) {
 		break;
+	    }
 	    chopstr(line);
-	    if (*line) strcpy(datalist[i++], line);
+	    if (*line != '\0') {
+		write_filename_to_list(FILE_LIST_DATA, i++, line);
+	    }
 	}
     }
 
-    if (strncmp(line, "recent session files:", 21) == 0) {
+    if (strncmp(line, file_sections[1], 21) == 0) {
 	i = 0;
 	while (fgets(line, MAXLEN, fp) && i<MAXRECENT) {
-	    if (strncmp(line, "recent script files:", 20) == 0)
+	    if (strncmp(line, file_sections[2], 20) == 0) {
 		break;
+	    }
 	    chopstr(line);
-	    if (*line) strcpy(sessionlist[i++], line);
+	    if (*line != '\0') {
+		write_filename_to_list(FILE_LIST_SESSION, i++, line);
+	    }
 	}
     }
 
-    if (strncmp(line, "recent script files:", 20) == 0) {
+    if (strncmp(line, file_sections[2], 20) == 0) {
 	i = 0;
 	while (fgets(line, MAXLEN, fp) && i<MAXRECENT) {
 	    chopstr(line);
-	    if (*line) strcpy(scriptlist[i++], line);
+	    if (*line != '\0') {
+		write_filename_to_list(FILE_LIST_SCRIPT, i++, line);
+	    }
 	}
     }
 
@@ -1491,7 +1455,7 @@ static void read_rc (void)
     set_use_qr(useqr);
     set_gp_colors();
 
-    set_paths(&paths, 0, 1);
+    set_paths(&paths, set_paths_opt);
 
 # if defined(HAVE_TRAMO) || defined(HAVE_X12A)
     set_tramo_x12a_dirs();
@@ -1675,8 +1639,12 @@ void font_selector (gpointer data, guint u, GtkWidget *w)
 				   GTK_SIGNAL_FUNC(gtk_widget_destroy),
 				   GTK_OBJECT (fontsel));
     }
-    if (!GTK_WIDGET_VISIBLE (fontsel)) gtk_widget_show (fontsel);
-    else gtk_widget_destroy (fontsel);
+
+    if (!GTK_WIDGET_VISIBLE(fontsel)) {
+	gtk_widget_show(fontsel);
+    } else {
+	gtk_widget_destroy(fontsel);
+    }
 }
 
 # endif /* non-Windows, gtk version branches */
@@ -1769,426 +1737,6 @@ void font_selector (gpointer data, guint which, GtkWidget *widget)
 
 #endif /* end win32 font selection */
 
-/* .................................................................. */
-
-void init_fileptrs (void)
-{
-    int i;
-    
-    for (i=0; i<MAXRECENT; i++) {
-	datap[i] = datalist[i];
-	sessionp[i] = sessionlist[i];
-	scriptp[i] = scriptlist[i];
-    }
-}
-
-/* .................................................................. */
-
-static void clear_files_list (int filetype, char **filep)
-{
-    GtkWidget *w;
-    char tmpname[MAXSTR];
-    gchar itempath[80];
-    int i, pindex = -1;
-    const gchar *fpath[] = {
-	N_("/File/Open data"), 
-	N_("/Session"),
-	N_("/File/Open command file")
-    };
-
-    if (filetype == FILE_LIST_DATA) 
-	pindex = 0;
-    else if (filetype == FILE_LIST_SESSION)
-	pindex = 1;
-    else if (filetype == FILE_LIST_SCRIPT)
-	pindex = 2;
-
-    if (pindex == -1) return;
-
-    for (i=0; i<MAXRECENT; i++) {
-#ifndef OLD_GTK
-	sprintf(itempath, "%s/%d. %s", fpath[pindex],
-		i+1, endbit(tmpname, filep[i], 0)); 
-#else
-	sprintf(itempath, "%s/%d. %s", fpath[pindex],
-		i+1, endbit(tmpname, filep[i], -1)); 
-#endif
-	w = gtk_item_factory_get_widget(mdata->ifac, itempath);
-	if (w != NULL) {
-	    gtk_item_factory_delete_item(mdata->ifac, itempath);
-	}
-    }
-}
-
-/* .................................................................. */
-
-static char *cut_multiple_slashes (char *fname)
-{
-    int i, n = strlen(fname);
-#ifdef G_OS_WIN32
-    /* may be ok for a filename to start with a double backslash */
-    int start = 1;
-#else
-    int start = 0;
-#endif
-
-    for (i=start; i<n-1; i++) {
-	if (fname[i] == SLASH && fname[i+1] == SLASH) {
-	    memmove(fname + i, fname + i + 1, strlen(fname + i + 1) + 1);
-	    i--;
-	    n--;
-	}
-    }
-
-    return fname;
-}
-
-void mkfilelist (int filetype, char *fname)
-{
-    char *tmp[MAXRECENT-1];
-    char **filep;
-    int i, match = -1;
-
-    cut_multiple_slashes(fname);
-
-    if (filetype == FILE_LIST_DATA) filep = datap;
-    else if (filetype == FILE_LIST_SESSION) filep = sessionp;
-    else if (filetype == FILE_LIST_SCRIPT) filep = scriptp;
-    else return;
-
-    /* see if this file is already on the list */
-    for (i=0; i<MAXRECENT; i++) {
-        if (strcmp(filep[i], fname) == 0) {
-            match = i;
-            break;
-        }
-    }
-    if (match == 0) return; /* file is on top: no change in list */
-
-    /* clear menu files list before rebuilding */
-    clear_files_list(filetype, filep);
-    
-    /* save pointers to current order */
-    for (i=0; i<MAXRECENT-1; i++) tmp[i] = filep[i];
-
-    /* copy fname into array, if not already present */
-    if (match == -1) {
-        for (i=1; i<MAXRECENT; i++) {
-            if (filep[i][0] == '\0') {
-                strcpy(filep[i], fname);
-                match = i;
-                break;
-	    }
-	    if (match == -1) {
-		match = MAXRECENT - 1;
-		strcpy(filep[match], fname);
-	    }
-	}
-    } 
-
-    /* set first pointer to new file */
-    filep[0] = filep[match];
-
-    /* rearrange other pointers */
-    for (i=1; i<=match; i++) {
-	filep[i] = tmp[i-1];
-    }
-
-    add_files_to_menu(filetype);
-}
-
-/* .................................................................. */
-
-void delete_from_filelist (int filetype, const char *fname)
-{
-    char *tmp[MAXRECENT];
-    char **filep;
-    int i, match = -1;
-
-    if (filetype == FILE_LIST_DATA) filep = datap;
-    else if (filetype == FILE_LIST_SESSION) filep = sessionp;
-    else if (filetype == FILE_LIST_SCRIPT) filep = scriptp;
-    else return;
-
-    /* save pointers to current order */
-    for (i=0; i<MAXRECENT; i++) {
-	tmp[i] = filep[i];
-	if (!strcmp(filep[i], fname)) match = i;
-    }
-
-    if (match == -1) return;
-
-    /* clear menu files list before rebuilding */
-    clear_files_list(filetype, filep);
-
-    for (i=match; i<MAXRECENT-1; i++) {
-	filep[i] = tmp[i+1];
-    }
-
-    filep[MAXRECENT-1] = tmp[match];
-    filep[MAXRECENT-1][0] = '\0';
-
-    add_files_to_menu(filetype);
-    /* need to save to file at this point? */
-}
-
-/* .................................................................. */
-
-char *endbit (char *dest, char *src, int addscore)
-{
-    /* take last part of src filename */
-    if (strrchr(src, SLASH))
-	strcpy(dest, strrchr(src, SLASH) + 1);
-    else
-	strcpy(dest, src);
-
-    if (addscore != 0) {
-	/* then either double (1) or delete (-1) any underscores */
-	char mod[MAXSTR];
-	size_t i, j, n;
-
-	n = strlen(dest);
-	j = 0;
-	for (i=0; i<=n; i++) {
-	    if (dest[i] != '_')
-		mod[j++] = dest[i];
-	    else {
-		if (addscore == 1) {
-		    mod[j++] = '_';
-		    mod[j++] = dest[i];
-		} 
-	    }
-	}
-	strcpy(dest, mod);
-    }
-    return dest;
-}
-
-/* .................................................................. */
-
-#ifdef GNOME2
-
-static void printfilelist (int filetype, FILE *fp)
-     /* param fp is ignored */
-{
-    GConfClient *client;
-    GSList *flist = NULL;
-    gchar *key;
-    int i;
-    char **filep;
-    static char *sections[] = {
-	"recent_data_files",
-	"recent_session_files",
-	"recent_script_files"
-    };
-
-    switch (filetype) {
-    case FILE_LIST_DATA: filep = datap; break;
-    case FILE_LIST_SESSION: filep = sessionp; break;
-    case FILE_LIST_SCRIPT: filep = scriptp; break;
-    default: return;
-    }
-
-    client = gconf_client_get_default();
-
-    for (i=0; i<MAXRECENT; i++) {
-	flist = g_slist_append (flist, g_strdup(filep[i]));
-    }
-
-    key = g_strdup_printf("/apps/gretl/%s", sections[filetype - 1]);
-
-    gconf_client_set_list (client, key, GCONF_VALUE_STRING, 
-			   flist, NULL);
-
-    g_free(key);
-    g_slist_free(flist);
-    g_object_unref(G_OBJECT(client));
-}
-
-#elif defined(USE_GNOME)
-
-static void printfilelist (int filetype, FILE *fp)
-     /* fp is ignored */
-{
-    int i;
-    char **filep;
-    char gpath[MAXLEN];
-    static char *section[] = {
-	"recent data files",
-	"recent session files",
-	"recent script files"
-    };
-
-    switch (filetype) {
-    case FILE_LIST_DATA: filep = datap; break;
-    case FILE_LIST_SESSION: filep = sessionp; break;
-    case FILE_LIST_SCRIPT: filep = scriptp; break;
-    default: return;
-    }
-
-    for (i=0; i<MAXRECENT; i++) {
-	sprintf(gpath, "/gretl/%s/%d", section[filetype - 1], i);
-	gnome_config_set_string(gpath, filep[i]);
-    }
-}
-
-#elif defined(G_OS_WIN32)
-
-static void printfilelist (int filetype, FILE *fp)
-     /* param fp is ignored */
-{
-    int i;
-    char **filep;
-    char rpath[MAXLEN];
-    static char *sections[] = {
-	"recent data files",
-	"recent session files",
-	"recent script files"
-    };
-
-    switch (filetype) {
-    case FILE_LIST_DATA: filep = datap; break;
-    case FILE_LIST_SESSION: filep = sessionp; break;
-    case FILE_LIST_SCRIPT: filep = scriptp; break;
-    default: return;
-    }
-
-    for (i=0; i<MAXRECENT; i++) {
-	if (filep[i] == NULL) continue;
-	sprintf(rpath, "%s\\%d", sections[filetype - 1], i);
-	write_reg_val(HKEY_CURRENT_USER, "gretl", rpath, filep[i]);
-    }
-}
-
-#else /* "plain" version follows */
-
-static void printfilelist (int filetype, FILE *fp)
-{
-    int i;
-    char **filep;
-
-    if (filetype == FILE_LIST_DATA) {
-	fprintf(fp, "recent data files:\n");
-	filep = datap;
-    } else if (filetype == FILE_LIST_SESSION) {
-	fprintf(fp, "recent session files:\n");
-	filep = sessionp;
-    } else if (filetype == FILE_LIST_SCRIPT) {
-	fprintf(fp, "recent script files:\n");
-	filep = scriptp;
-    } else 
-	return;
-
-    for (i=0; i<MAXRECENT; i++) {
-	if (filep[i][0]) 
-	    fprintf(fp, "%s\n", filep[i]);
-	else break;
-    }
-}
-
-#endif 
-
-/* ........................................................... */
-
-static void set_data_from_filelist (gpointer data, guint i, 
-				    GtkWidget *widget)
-{
-    strcpy(trydatfile, datap[i]);
-    if (strstr(trydatfile, ".csv")) delimiter_dialog();
-    verify_open_data(NULL, 0);
-}
-
-/* ........................................................... */
-
-static void set_session_from_filelist (gpointer data, guint i, 
-				       GtkWidget *widget)
-{
-    strcpy(tryscript, sessionp[i]);
-    verify_open_session(NULL);
-}
-
-/* ........................................................... */
-
-static void set_script_from_filelist (gpointer data, guint i, 
-				      GtkWidget *widget)
-{
-    strcpy(tryscript, scriptp[i]);
-    do_open_script();
-}
-
-/* .................................................................. */
-
-void add_files_to_menu (int filetype)
-{
-    int i;
-    char **filep, tmp[MAXSTR];
-    void (*callfunc)();
-    GtkItemFactoryEntry fileitem;
-    GtkWidget *w;
-    const gchar *msep[] = {
-	N_("/File/Open data/sep"),
-	N_("/Session/sep"),
-	N_("/File/Open command file/sep")
-    };
-    const gchar *mpath[] = {
-	N_("/File/Open data"),
-	N_("/Session"),
-	N_("/File/Open command file")
-    };
-
-    fileitem.path = NULL;
-
-    if (filetype == FILE_LIST_DATA) {
-	callfunc = set_data_from_filelist;
-	filep = datap;
-    } else if (filetype == FILE_LIST_SESSION) {
-	callfunc = set_session_from_filelist;
-	filep = sessionp;
-    } else if (filetype == FILE_LIST_SCRIPT) {
-	callfunc = set_script_from_filelist;
-	filep = scriptp;
-    }
-    else return;
-
-    /* See if there are any files to add */
-    if (filep[0][0] == '\0') {
-	return;
-    } else {
-	gchar *itemtype = "<Separator>";
-	GtkWidget *w;
-
-	/* is a separator already in place? */
-	w = gtk_item_factory_get_widget(mdata->ifac, msep[filetype - 1]);
-	if (w == NULL) {
-	    fileitem.path = g_strdup(msep[filetype - 1]);
-	    fileitem.accelerator = NULL;
-	    fileitem.callback = NULL;
-	    fileitem.callback_action = 0;
-	    fileitem.item_type = itemtype;
-	    gtk_item_factory_create_item(mdata->ifac, &fileitem, NULL, 1);
-	    g_free(fileitem.path);
-	}
-    }
-
-    /* put the files under the menu separator */
-    for (i=0; i<MAXRECENT; i++) {
-	if (filep[i][0]) {
-	    fileitem.accelerator = NULL;
-	    fileitem.callback_action = i; 
-	    fileitem.item_type = NULL;
-	    fileitem.path = g_strdup_printf("%s/%d. %s", mpath[filetype - 1],
-					    i+1, endbit(tmp, filep[i], 1));
-	    fileitem.callback = callfunc; 
-	    gtk_item_factory_create_item(mdata->ifac, &fileitem, NULL, 1);
-	    g_free(fileitem.path);
-	    w = gtk_item_factory_get_widget_by_action(mdata->ifac, i);
-	    if (w != NULL) {
-		gretl_tooltips_add(w, filep[i]);
-	    } 
-	} else break;
-    }
-}
-
 /* graph color selection apparatus */
 
 static double scale_round (double val)
@@ -2346,11 +1894,7 @@ GtkWidget *color_patch_button (int colnum)
     } else {
 	button = gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(button), image);
-#ifndef OLD_GTK
 	g_object_set_data(G_OBJECT(button), "image", image);
-#else
-	gtk_object_set_data(GTK_OBJECT(button), "image", image);
-#endif
     }	
 
     return button;
@@ -2457,19 +2001,33 @@ void gnuplot_color_selector (GtkWidget *w, gpointer p)
 
 /* end graph color selection apparatus */
 
+static int dir_exists (const char *dname, FILE *fp)
+{
+    DIR *test;
+    int ok = 0;
+
+    test = opendir(dname);
+
+    if (test != NULL) {
+	ok = 1;
+	if (fp != NULL) {
+	    fprintf(fp, "Directory '%s' exists, OK\n", dname);
+	}
+	closedir(test);
+    } else if (fp != NULL) {
+	fprintf(fp, "Directory '%s' does not exist\n", dname);
+    }
+
+    return ok;
+}
+
 #ifndef G_OS_WIN32
 
 static int validate_dir (const char *dirname)
 {
-    DIR *test;
     int err = 0;
 
-    test = opendir(dirname);
-
-    if (test != NULL) {
-	fprintf(stderr, "Working dir already exists, OK\n");
-	closedir(test);
-    } else {
+    if (!dir_exists(dirname, NULL)) {
 	err = mkdir(dirname, 0755);
 	if (err) {
 	    sprintf(errtext, _("Couldn't create directory '%s'"), dirname);
@@ -2486,17 +2044,12 @@ static void real_set_userdir (GtkWidget *widget, dialog_t *ddata)
 {
     const gchar *dirname;
 
-    dirname = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
+    dirname = dialog_data_get_text(ddata);
 
     if (validate_dir(dirname)) {
 	return;
     } else {
-	size_t n = strlen(dirname);
-
-	strcpy(paths.userdir, dirname);
-	if (dirname[n - 1] != '/') {
-	    strcat(paths.userdir, "/");
-	}
+	set_gretl_user_dir(dirname, &paths);
 #if defined(HAVE_TRAMO) || defined(HAVE_X12A)
 	set_tramo_x12a_dirs();
 #endif
@@ -2527,3 +2080,58 @@ void first_time_set_user_dir (void)
 }
 
 #endif /* G_OS_WIN32 */
+
+void dump_rc (void) 
+{
+    FILE *fp;
+    char *tmp, *dumper = NULL;
+    char val[6];
+    int i;
+
+    if (dir_exists(paths.userdir, NULL)) {
+	dumper = g_strdup_printf("%sconfig-dump.txt", paths.userdir);
+    } else {
+	dumper = tempnam(NULL, "gretl");
+    }
+
+    if (dumper == NULL) {
+	errbox(_("Couldn't open config file for writing"));
+	return;
+    }
+
+    fp = fopen(dumper, "w");
+    if (fp == NULL) {
+	errbox(_("Couldn't open config file for writing"));
+	free(dumper);
+	return;
+    }
+
+    fprintf(fp, "gretl version %s\n", version_string);
+    fprintf(fp, "built with GTK %d.%d.%d\n", GTK_MAJOR_VERSION, GTK_MINOR_VERSION,
+	    GTK_MICRO_VERSION);
+
+    tmp = getenv("HOME");
+    if (tmp != NULL) {
+	fprintf(fp, "HOME='%s'\n", tmp);
+    } else {
+	fputs("HOME not set\n", fp);
+    }
+
+    for (i=0; rc_vars[i].var != NULL; i++) {
+	fprintf(fp, "# %s\n", rc_vars[i].description);
+	if (rc_vars[i].type == BOOLSET) {
+	    boolvar_to_str(rc_vars[i].var, val);
+	    fprintf(fp, "%s = %s\n", rc_vars[i].key, val);
+	} else {
+	    fprintf(fp, "%s = %s\n", rc_vars[i].key, (char *) rc_vars[i].var);
+	}
+    }
+
+    dir_exists(paths.userdir, fp);
+    dir_exists(paths.gretldir, fp);
+
+    printf("Config info written to %s\n", dumper);
+
+    fclose(fp);
+    free(dumper);
+}
