@@ -11,13 +11,9 @@ void set_rcfile (void);
 
 void write_rc (void);
 
-void mkfilelist (int filetype, char *newfile);
+void dump_rc (void);
 
-void init_fileptrs (void);
-
-void delete_from_filelist (int filetype, const char *fname);
-
-void add_files_to_menu (int filetype);
+void force_english_help (void);
 
 void options_dialog (gpointer data);
 
@@ -25,21 +21,28 @@ void font_selector (gpointer data, guint which, GtkWidget *widget);
 
 void set_fixed_font (void);
 
+#if !defined(OLD_GTK) && !defined(USE_GNOME)
+
 void set_app_font (const char *fontname);
 
+const char *get_app_fontname (void);
+
+#endif
+
 void gnuplot_color_selector (GtkWidget *w, gpointer p);
 
 GtkWidget *color_patch_button (int colnum);
 
-char *endbit (char *dest, char *src, int addscore);
-
 void get_default_dir (char *s);
 
 void filesel_set_path_callback (const char *setting, char *strvar);
 
 void set_datapage (const char *str);
+
 void set_scriptpage (const char *str);
+
 const char *get_datapage (void);
+
 const char *get_scriptpage (void);
 
 #ifdef HAVE_TRAMO
