@@ -381,14 +381,13 @@ static void real_add_new_obs (spreadsheet *sheet, const char *obsname)
 
 static void name_new_var (GtkWidget *widget, dialog_t *ddata) 
 {
-    spreadsheet *sheet = (spreadsheet *) ddata->data;
+    spreadsheet *sheet = (spreadsheet *) dialog_data_get_data(ddata);
     const gchar *buf;
     char varname[VNAMELEN];
 
-    buf = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
+    buf = dialog_data_get_text(ddata);
 
-    if (blank_entry(buf, ddata)) return;
-    if (validate_varname(buf)) return;
+    if (buf == NULL || validate_varname(buf)) return;
 
     *varname = 0;
     strncat(varname, buf, VNAMELEN - 1);
@@ -404,12 +403,12 @@ static void name_new_var (GtkWidget *widget, dialog_t *ddata)
 
 static void name_new_obs (GtkWidget *widget, dialog_t *ddata) 
 {
-    spreadsheet *sheet = (spreadsheet *) ddata->data;
+    spreadsheet *sheet = (spreadsheet *) dialog_data_get_data(ddata);
     const gchar *buf;
     char obsmarker[OBSLEN];
 
-    buf = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
-    if (blank_entry(buf, ddata)) return;
+    buf = dialog_data_get_text(ddata);
+    if (buf == NULL) return;
 
     *obsmarker = 0;
     strncat(obsmarker, buf, OBSLEN - 1);
@@ -902,6 +901,9 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
     if (mods & GDK_BUTTON3_MASK) {
 	GdkEventButton *bevent = (GdkEventButton *) event;
 
+	if (sheet->popup == NULL) 
+	    build_sheet_popup(sheet);
+
 	gtk_menu_popup (GTK_MENU(sheet->popup), NULL, NULL, NULL, NULL,
 			bevent->button, bevent->time);
 	return TRUE;
@@ -1003,6 +1005,9 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
     if (mods & GDK_BUTTON3_MASK) {
 	GdkEventButton *bevent = (GdkEventButton *) event;
 
+	if (sheet->popup == NULL)
+	    build_sheet_popup(sheet);
+
 	gtk_menu_popup (GTK_MENU(sheet->popup), NULL, NULL, NULL, NULL,
 			bevent->button, bevent->time);
 	return TRUE;
@@ -1174,7 +1179,10 @@ static void free_spreadsheet (GtkWidget *widget, spreadsheet **psheet)
 {
     spreadsheet *sheet = *psheet;
 
-    gtk_widget_destroy(sheet->popup);
+    if (sheet->popup != NULL) {
+	gtk_widget_destroy(sheet->popup);
+    }
+
     free(sheet);
     *psheet = NULL;
 }
@@ -1296,8 +1304,6 @@ void show_spreadsheet (DATAINFO *pdinfo)
     gtk_box_pack_start(GTK_BOX(main_vbox), mbar, FALSE, FALSE, 0);
     gtk_widget_show(mbar);
 
-    build_sheet_popup(sheet);
-
     status_box = gtk_hbox_new(FALSE, 1);
     gtk_container_set_border_width(GTK_CONTAINER(status_box), 0);
     gtk_box_pack_start(GTK_BOX(main_vbox), status_box, FALSE, FALSE, 0);
