@@ -24,7 +24,7 @@
    libgretl.
 */
 
-/* #define WDEBUG */
+#undef WDEBUG
 
 #ifdef UPDATER
 # include "version.h"
@@ -70,7 +70,7 @@ extern int h_errno;
 #ifndef UPDATER
 extern int use_proxy; /* gui_utils.c */
 #else
-const char *dbhost_ip = "152.17.150.2";
+const char *dbhost = "ricardo.ecn.wfu.edu";
 static char dbproxy[21];
 static int use_proxy;
 #endif /* UPDATER */
@@ -204,12 +204,12 @@ struct urlinfo
 {
     char *url;                    /* the URL */
     uerr_t proto;                 /* URL protocol */
-    char *host;                   /* hostname */
     unsigned short port;
     unsigned short saveopt;       /* save to buffer or file? */   
     char *path; 
     char *localfile;
     char **savebuf;
+    char host[32];
     char errbuf[80];
 };
 
@@ -355,6 +355,28 @@ int ws_startup (void)
 
 #endif /* WIN32 */
 
+static int get_db_host_ip (char *h_ip, const char *h_name)
+{
+    struct hostent *h_ent;
+
+    h_ent = gethostbyname(h_name);
+    if (h_ent == NULL) {
+	*h_ip = '\0';
+#ifndef WIN32
+	herror(NULL);
+#endif
+	return 1;
+    }
+
+    sprintf(h_ip, "%d.%d.%d.%d", 
+	   (unsigned char) h_ent->h_addr[0], 
+	   (unsigned char) h_ent->h_addr[1], 
+	   (unsigned char) h_ent->h_addr[2],
+	   (unsigned char) h_ent->h_addr[3]);
+
+    return 0;
+}
+
 /* ........................................................... */
 
 static char *time_str (time_t *tm)
@@ -802,13 +824,15 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 
     switch (err) {
     case HOSTERR:
-	sprintf(conn->errbuf, "%s: %s\n", conn->host, herrmsg(h_errno));
+	if (*conn->host != '\0') {
+	    sprintf(conn->errbuf, "%s: %s\n", conn->host, herrmsg(h_errno));
+	} else {
+	    sprintf(conn->errbuf, "%s\n", herrmsg(h_errno));
+	}
 	return HOSTERR;
-	break;
     case CONSOCKERR:
 	sprintf(conn->errbuf, "socket: %s\n", strerror(errno));
 	return CONSOCKERR;
-	break;
     case CONREFUSED:
 	sprintf(conn->errbuf, "Connection to %s:%hu refused\n", 
 		conn->host, conn->port);
@@ -818,7 +842,6 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	sprintf(conn->errbuf, "connect: %s\n", strerror(errno));
 	close(sock);
 	return CONERROR;
-	break;
     case NOCONERROR:
 	break;
     default:
@@ -842,17 +865,19 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
     } 
 
     if (proxy) {
-#ifdef UPDATER
-	path = mymalloc(strlen(dbhost_ip) + strlen(u->path) + 8);
-	sprintf(path, "http://%s%s", dbhost_ip, u->path);
-#else
-	path = mymalloc(strlen(paths.dbhost_ip) + strlen(u->path) + 8);
-	sprintf(path, "http://%s%s", paths.dbhost_ip, u->path);
-#endif
+	path = mymalloc(strlen(u->host) + strlen(u->path) + 8);
+	sprintf(path, "http://%s%s", u->host, u->path);
     } else {
 	path = u->path; 
     }
 
+    if (path == NULL) {
+	close(sock);
+	free(all_headers);
+	return NOTENOUGHMEM;
+    }
+	
+
     command = (*dt & HEAD_ONLY)? "HEAD" : "GET";
 
     if (*dt & SEND_NOCACHE) {
@@ -874,6 +899,13 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 		       + strlen(HTTP_ACCEPT)
 		       + strlen(pragma_h)
 		       + 64);
+
+    if (request == NULL) {
+	close(sock);
+	free(all_headers);
+	return NOTENOUGHMEM;
+    }
+
     sprintf(request, "%s %s HTTP/1.0\r\n"
 	    "User-Agent: %s\r\n"
 	    "Host: %s:%d\r\n"
@@ -1133,13 +1165,9 @@ static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy)
 	    continue;
 	    break;
 	case HOSTERR: case CONREFUSED: case PROXERR: case AUTHFAILED:
-	    FREEHSTAT(hstat);
-	    return err;
-	    break;
 	case FWRITEERR: case FOPENERR: case RETRCANCELED:
 	    FREEHSTAT(hstat);
 	    return err;
-	    break;
 	case RETRFINISHED:
 	    break;
 	default:
@@ -1199,6 +1227,21 @@ static size_t rbuf_flush (struct rbuf *rbuf, char *where, int maxsize)
 
 /* ........................................................... */
 
+static void url_init (struct urlinfo *u)
+{
+    u->url = NULL;
+    u->path = NULL;
+    u->localfile = NULL;
+    u->savebuf = NULL;
+
+    u->proto = URLHTTP;
+    u->port = DEFAULT_HTTP_PORT;
+    
+    u->host[0] = '\0';
+    u->errbuf[0] = '\0';
+    u->saveopt = 0;
+}
+
 static struct urlinfo *newurl (void)
 /* Allocate a new urlinfo structure, fill it with default values and
    return a pointer to it.  */
@@ -1206,7 +1249,9 @@ static struct urlinfo *newurl (void)
     struct urlinfo *u;
 
     u = mymalloc(sizeof *u);
-    memset(u, 0, sizeof *u);
+    if (u == NULL) return NULL;
+
+    url_init(u);
     
     return u;
 }
@@ -1221,11 +1266,11 @@ static void freeurl (struct urlinfo *u, int complete)
     if (u == NULL) return;
 
     free(u->url);
-    free(u->host);
     free(u->path);
     if (u->localfile) {
 	free(u->localfile);
     }
+
     if (complete) {
 	free(u);
     }
@@ -1456,21 +1501,33 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
     const char *cgi = "/gretl/cgi-bin/gretl_update.cgi";
     struct urlinfo *proxy = NULL; 
 
-    if (use_proxy && gretlproxy.host != NULL) {
+    if (use_proxy) {
 	proxy = &gretlproxy;
     }
 
     u = newurl();
-    u->proto = URLHTTP;
-    u->port = DEFAULT_HTTP_PORT;
-    u->host = mymalloc(16);
-#ifdef UPDATER
-    strcpy(u->host, dbhost_ip);
-#else
-    strcpy(u->host, paths.dbhost_ip);
-#endif
+    if (u == NULL) {
+	return 1;
+    }
+
     u->path = mymalloc(strlen(cgi) + 64);
 
+    if (u->path == NULL) {
+	freeurl(u, 1);
+	return 1;
+    }
+
+#ifdef UPDATER
+    err = get_db_host_ip(u->host, dbhost);
+#else
+    err = get_db_host_ip(u->host, paths.dbhost);
+#endif
+
+    if (err) {
+	freeurl(u, 1);
+	return err;
+    }
+
     if (queryopt == QUERY_VERBOSE) {
 	sprintf(u->path, "%s?opt=MANUAL_QUERY", cgi);
     } else {
@@ -1695,17 +1752,9 @@ int proxy_init (const char *dbproxy)
     char *p;
     size_t iplen;
 
-    gretlproxy.url = NULL;
-    gretlproxy.proto = URLHTTP;
-    gretlproxy.port = 0;
+    url_init(&gretlproxy);
+
     gretlproxy.saveopt = SAVE_TO_BUFFER;
-    gretlproxy.path = NULL; 
-    gretlproxy.localfile = NULL;
-    gretlproxy.savebuf = NULL;
-    if (gretlproxy.host) {
-	free(gretlproxy.host);
-    }
-    gretlproxy.host = NULL;
 
     if (!use_proxy || !strlen(dbproxy)) return 0;
 
@@ -1717,9 +1766,6 @@ int proxy_init (const char *dbproxy)
     }
 
     gretlproxy.port = atoi(p + 1);
-    gretlproxy.host = mymalloc(16);
-
-    if (gretlproxy.host == NULL) return 1;
 
     iplen = p - dbproxy;
     if (iplen > 15) {
@@ -1727,7 +1773,6 @@ int proxy_init (const char *dbproxy)
 	return 1;	
     }
 
-    gretlproxy.host[0] = '\0';
     strncat(gretlproxy.host, dbproxy, iplen);
 
 #ifdef UPDATER
@@ -1761,7 +1806,7 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
     int dt, err = 0;
     size_t fnlen = 0L;
 
-    if (use_proxy && gretlproxy.host != NULL) {
+    if (use_proxy) {
 	proxy = &gretlproxy;
     }
 
@@ -1773,15 +1818,27 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
     else cgi = datacgi;
 
     u = newurl();
-    u->proto = URLHTTP;
-    u->port = DEFAULT_HTTP_PORT;
-    u->host = mymalloc(16);
+    if (u == NULL) {
+	return 1;
+    }
+
     u->path = mymalloc(strlen(cgi) + fnlen + 64);
+    if (u->path == NULL) {
+	freeurl(u, 1);
+	return 1;
+    }
+
 #ifdef UPDATER
-    strcpy(u->host, dbhost_ip);
+    err = get_db_host_ip(u->host, dbhost);
 #else
-    strcpy(u->host, paths.dbhost_ip);
+    err = get_db_host_ip(u->host, paths.dbhost);
 #endif
+
+    if (err) {
+	freeurl(u, 1);
+	return err;
+    }
+
     sprintf(u->path, "%s?opt=%s", cgi, print_option(opt));
     u->saveopt = saveopt;
 
