@@ -41,37 +41,30 @@ enum aux_codes {
     AUX_WHITE, /* aux. regression for heteroskedasticity (White's test) */
     AUX_COINT, /* aux. regression for cointegreation test */
     AUX_ARCH,  /* aux. regression for ARCH test */
+    AUX_DF,    /* aux. regression for Dickey-Fuller test */
     AUX_ADF,   /* aux. regression for augmented Dickey-Fuller test */
+    AUX_KPSS,  /* aux. regression for KPSS test */
     AUX_OMIT,  /* aux. regression for omitting variables */
     AUX_RESET, /* aux. regression for Ramsey's RESET */
     AUX_SYS,   /* single equation from multivariate system */
     AUX_AUX    /* auxiliary regression not otherwise specified */
 };
 
-typedef struct {
-    int m1;        /* ID for first model */
-    int m2;        /* ID for second model */
-    int ci;        /* estimator code for the first */
-    int dfn;       /* numerator degrees of freedom */
-    int dfd;       /* denominator degrees of freedom */ 
-    double F;      /* F test statistic */
-    double chisq;  /* Chi-square test statistic */
-    double trsq;   /* T*R^2 test statistic */
-    int score;     /* "cases correct" for discrete models */
-} COMPARE;
-
 /* functions follow */
  
-int auxreg (LIST addvars, 
-	    MODEL *orig, MODEL *new, int *model_count, 
-	    double ***pZ, DATAINFO *pdinfo, 
-	    int aux_code, 
-	    PRN *prn, GRETLTEST *test, unsigned long opt);
+double robust_omit_F (const int *list, MODEL *pmod);
 
-int omit_test (LIST omitvars, MODEL *orig, MODEL *new, 
-	       int *model_count, 
+int add_test (int *addvars,  MODEL *orig, MODEL *new,
+	      double ***pZ, DATAINFO *pdinfo, 
+	      gretlopt opt, PRN *prn);
+
+int omit_test (int *omitvars, MODEL *orig, MODEL *new, 
 	       double ***pZ, DATAINFO *pdinfo, 
-	       PRN *prn, unsigned long opt);
+	       gretlopt opt, PRN *prn);
+
+int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+		       int aux_code, gretlopt opt, PRN *prn, 
+		       GRETLTEST *test);
 
 int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
 		PRN *prn, GRETLTEST *test);
@@ -86,27 +79,28 @@ int chow_test (const char *line, MODEL *pmod,
 
 int cusum_test (MODEL *pmod, 
 		double ***pZ, DATAINFO *pdinfo, 
-		PRN *prn, 
-		PATHS *ppaths, 
-		GRETLTEST *test);
+		PRN *prn, GRETLTEST *test);
 
 int hausman_test (MODEL *pmod, 
 		  double ***pZ, DATAINFO *pdinfo, 
 		  PRN *prn);
 
+int vif_test (MODEL *pmod, 
+	      double ***pZ, DATAINFO *pdinfo, 
+	      PRN *prn);
+
 int leverage_test (MODEL *pmod, 
 		   double ***pZ, DATAINFO *pdinfo, 
-		   PRN *prn, PATHS *ppaths, 
-		   unsigned long oflag);
+		   PRN *prn, gretlopt oflag);
 
 int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
 				    gretl_matrix *m, unsigned char opt);
 
-int mp_ols (const LIST list, const char *pos,
+int mp_ols (const int *list, const char *pos,
 	    double ***pZ, DATAINFO *pdinfo, 
 	    PRN *prn); 
 
-int sum_test (LIST sumvars, MODEL *pmod, 
+int sum_test (const int *sumvars, MODEL *pmod, 
 	      double ***pZ, DATAINFO *pdinfo, 
 	      PRN *prn);
 
