@@ -20,7 +20,7 @@
 #include <stdio.h>
 #include <string.h>
 
-enum data_options {
+typedef enum {
     GRETL_DATA_FLOAT = 1, /* single-precision binary data */
     GRETL_DATA_DOUBLE,    /* double-precision binary data */
     GRETL_DATA_OCTAVE,    /* data in Gnu Octave format */
@@ -28,10 +28,11 @@ enum data_options {
     GRETL_DATA_R,         /* data in Gnu R format */
     GRETL_DATA_R_ALT,     /* data in alternate Gnu R format */
     GRETL_DATA_GZIPPED,   /* gzipped data */
-    GRETL_DATA_TRAD       /* traditional (ESL-style) data */
-};
+    GRETL_DATA_TRAD,      /* traditional (ESL-style) data */
+    GRETL_DATA_DAT        /* data in PcGive format */
+} gretl_data_format;
 
-enum gretl_filetypes {
+typedef enum {
     GRETL_NATIVE_DATA,    /* gretl native format data file */
     GRETL_XML_DATA,       /* gretl xml format data file */
     GRETL_CSV_DATA,       /* comma-separated data file */
@@ -42,18 +43,24 @@ enum gretl_filetypes {
     GRETL_NATIVE_DB,      /* gretl database */
     GRETL_RATS_DB,        /* RATS 4.0 database */
     GRETL_UNRECOGNIZED    /* none of the above */
-};
+} gretl_filetype;
 
-enum clear_codes {
-    CLEAR_FULL,
-    CLEAR_SUBSAMPLE
-};
+typedef enum {
+    CLEAR_FULL,           /* fully clear the dataset */
+    CLEAR_SUBSAMPLE       /* dataset is sub-sampled: clear partially */
+} data_clear_code;
 
-enum data_open_codes {
-    DATA_NONE,
-    DATA_CLEAR,
-    DATA_APPEND
-};
+typedef enum {
+    DATA_NONE,    /* no dataset is currently open */
+    DATA_CLEAR,   /* dataset is open: dataset info should be cleared */
+    DATA_APPEND   /* dataset is open: attempt to append new data */
+} data_open_code;
+
+typedef enum {
+    VARNAME_RESERVED = 1, /* vername is a gretl reserved name */
+    VARNAME_FIRSTCHAR,    /* first character is not alphabetical */
+    VARNAME_BADCHAR       /* illegal character in second or subsequent place */
+} gretl_varname_errors;
 
 
 #define free_datainfo(p) do { if (p != NULL) { clear_datainfo(p, 0); free(p); } \
@@ -73,6 +80,10 @@ DATAINFO *create_new_dataset (double ***pZ, /* data matrix */
 
 void clear_datainfo (DATAINFO *pdinfo, int code);
 
+char **allocate_case_markers (int n);
+
+void destroy_dataset_markers (DATAINFO *pdinfo);
+
 int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample);
 
 int dateton (const char *date, const DATAINFO *pdinfo);
@@ -83,13 +94,13 @@ char *ntodate_full (char *datestr, int t, const DATAINFO *pdinfo);
 
 int get_info (const char *hdrfile, PRN *prn);
 
-int get_precision (double *x, int n, int placemax);
+int get_precision (const double *x, int n, int placemax);
 
 double get_date_x (int pd, const char *obs);
 
 int write_data (const char *fname, const int *list, 
-		double **Z, const DATAINFO *pdinfo, 
-	        int opt, PATHS *ppaths);
+		const double **Z, const DATAINFO *pdinfo, 
+	        gretlopt flag, PATHS *ppaths);
 
 int data_report (const DATAINFO *pdinfo, PATHS *ppaths, PRN *prn);
 
@@ -105,7 +116,7 @@ int merge_data (double ***pZ, DATAINFO *pdinfo,
 
 int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, 
 		    char *datfile, PATHS *ppaths, 
-		    int data_status, PRN *prn);
+		    data_open_code code, PRN *prn);
 
 int open_nulldata (double ***pZ, DATAINFO *pdinfo, 
 		   int data_status, int length,
