@@ -151,6 +151,7 @@ static int get_native_series_pd (SERIESINFO *sinfo, char pdc)
     if (pdc == 'M') sinfo->pd = 12;
     else if (pdc == 'Q') sinfo->pd = 4;
     else if (pdc == 'B') sinfo->pd = 5;
+    else if (pdc == 'S') sinfo->pd = 6;
     else if (pdc == 'D') sinfo->pd = 7;
     else if (pdc == 'U') sinfo->undated = 1;
     else return 1;
@@ -166,10 +167,14 @@ static int get_native_series_obs (SERIESINFO *sinfo,
 	const char *q = stobs;
 	const char *p = strchr(stobs, '/');
 
-	if (p - q == 4) strcpy(sinfo->stobs, q + 2);
+	if (p - q == 4) {
+	    strcpy(sinfo->stobs, q + 2);
+	}
 	q = endobs;
 	p = strchr(endobs, '/');
-	if (p && p - q == 4) strcpy(sinfo->endobs, q + 2);
+	if (p && p - q == 4) {
+	    strcpy(sinfo->endobs, q + 2);
+	}
     } else {
 	*sinfo->stobs = 0;
 	*sinfo->endobs = 0;
@@ -260,9 +265,10 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 
 /* ........................................................... */
 
+/* Figure the ending observation date of a series */
+
 static int get_endobs (char *datestr, int startyr, int startfrac, 
 		       int pd, int n)
-/* Figure the ending observation date of a series */
 {
     int endyr, endfrac;  
 
@@ -322,10 +328,15 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 
     if (dinfo->info == 4) {
 	sprintf(pdstr, ".%d", dinfo->month);
-	if (dinfo->month == 1) startfrac = 1;
-	else if (dinfo->month > 1 && dinfo->month <= 4) startfrac = 2;
-	else if (dinfo->month > 4 && dinfo->month <= 7) startfrac = 3;
-	else startfrac = 4;
+	if (dinfo->month == 1) {
+	    startfrac = 1;
+	} else if (dinfo->month > 1 && dinfo->month <= 4) {
+	    startfrac = 2;
+	} else if (dinfo->month > 4 && dinfo->month <= 7) {
+	    startfrac = 3;
+	} else {
+	    startfrac = 4;
+	}
     }
     else if (dinfo->info == 12) {
 	sprintf(pdstr, ".%02d", dinfo->month);
@@ -342,7 +353,9 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 	err = 1;
     }   
 
-    if (*pdstr) strcat(sinfo->stobs, pdstr);
+    if (*pdstr) {
+	strcat(sinfo->stobs, pdstr);
+    }
     get_endobs(sinfo->endobs, dinfo->year, startfrac, dinfo->info, n);
 
     sinfo->pd = dinfo->info;
@@ -665,8 +678,7 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
 		val = NADBL;
 		miss = 1;
 	    }
-	    Z[1][t] = val;
-	    t++;
+	    Z[1][t++] = val;
 	}
     }
 
@@ -678,7 +690,7 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
  * @fname: name of RATS 4.0 database to read from
  * @series_number: number of the series within the database
  * @sinfo: holds info on the given series (input)
- * @pZ: pointer to data matrix
+ * @Z: data matrix
  *
  * Read the actual data values for a series from a RATS database.
  * 
@@ -798,57 +810,93 @@ get_rats_data_by_offset (const char *fname,
     return ret;
 }
 
-/* ........................................................... */
+/* For importation of database series */
 
-#ifdef LIMIT_DIGITS
-static int get_places (double x)
+static double *get_compacted_xt (const double *src,
+				 int n, int method, int compfac,
+				 int skip)
 {
-    char numstr[16];
-    int i, n, p = 0;
+    int p, t;
+    double *x;
 
-    sprintf(numstr, "%.3f", x);
-    n = strlen(numstr);
-    for (i=n-3; i<n; i++) {
-	if (numstr[i] != '0') p++;
-	else break;
+    x = malloc(n * sizeof *x);
+    if (x == NULL) return NULL;
+
+    for (t=0; t<n; t++) {
+	p = (t + 1) * compfac;
+	x[t] = 0.0;
+	if (method == COMPACT_AVG || method == COMPACT_SUM) { 
+	    int i, st;
+
+	    for (i=1; i<=compfac; i++) {
+		st = p - i + skip;
+		if (na(src[st])) {
+		    x[t] = NADBL;
+		    break;
+		} else {
+		    x[t] += src[st];
+		}
+	    }
+	    if (method == COMPACT_AVG) {
+		x[t] /= (double) compfac;
+	    }
+	} else if (method == COMPACT_EOP) {
+	    x[t] = src[p - 1 + skip];
+	} else if (method == COMPACT_SOP) {
+	    x[t] = src[p - compfac + skip];
+	}
     }
 
-    return p;
+    return x;
 }
-#endif
 
-int mon_to_quart (double **pq, double *mvec, SERIESINFO *sinfo,
-		  int method)
+/* Compact a single series from a database, for importation 
+   into a working dataset of lower frequency.  At present
+   this is permitted only for the cases:
+
+     quarterly -> annual
+     monthly   -> quarterly
+     monthly   -> annual
+*/
+
+double *compact_db_series (const double *src, SERIESINFO *sinfo,
+			   int target_pd, int method)
 {
-    int t, p, m0, q0, y0, skip = 0, endskip, goodobs;
-    float q;
-    double val = 0.;
-#ifdef LIMIT_DIGITS
-    int pmax = 0;
-    char numstr[16];
+    int p0, y0, endskip, goodobs;
+    int skip = 0, compfac = sinfo->pd / target_pd;
+    double *x;
 
-    /* record the precision of the original data */
-    for (t=0; t<sinfo->nobs; t++) {
-	p = get_places(mvec[t]);
-	if (p > pmax) pmax = p;
-    }
-#endif
+    if (target_pd == 1) {
+	/* figure the annual dates */
+	y0 = atoi(sinfo->stobs);
+	p0 = atoi(sinfo->stobs + 5);
+	if (p0 != 1) {
+	    ++y0;
+	    skip = compfac - (p0 + 1);
+	}
+	sprintf(sinfo->stobs, "%d", y0);
+    } else if (target_pd == 4) {
+	/* figure the quarterly dates */
+	float q;
+	int q0;
 
-    /* figure the quarterly dates */
-    y0 = atoi(sinfo->stobs);
-    m0 = atoi(sinfo->stobs + 5);
-    q = 1.0 + m0/3.;
-    q0 = q + .5;
-    skip = ((q0 - 1) * 3) + 1 - m0;
-    if (q0 == 5) {
-	y0++;
-	q0 = 1;
+	y0 = atoi(sinfo->stobs);
+	p0 = atoi(sinfo->stobs + 5);
+	q = 1.0 + p0 / 3.;
+	q0 = q + .5;
+	skip = ((q0 - 1) * 3) + 1 - p0;
+	if (q0 == 5) {
+	    y0++;
+	    q0 = 1;
+	}
+	sprintf(sinfo->stobs, "%d.%d", y0, q0);
+    } else {
+	return NULL;
     }
 
-    endskip = (sinfo->nobs - skip) % 3;
-    goodobs = (sinfo->nobs - skip - endskip) / 3;
+    endskip = (sinfo->nobs - skip) % compfac;
+    goodobs = (sinfo->nobs - skip - endskip) / compfac;
     sinfo->nobs = goodobs;
-    sprintf(sinfo->stobs, "%d.%d", y0, q0);
 
 #ifdef DB_DEBUG
     fprintf(stderr, "startskip = %d\n", skip);
@@ -857,103 +905,15 @@ int mon_to_quart (double **pq, double *mvec, SERIESINFO *sinfo,
     fprintf(stderr, "starting date = %s\n", sinfo->stobs);
 #endif
 
-    *pq = malloc(goodobs * sizeof **pq);
-    if (*pq == NULL) return 1;
+    x = get_compacted_xt(src, goodobs, method, compfac, skip);
 
-    for (t=0; t<goodobs; t++) {
-	p = (t + 1) * 3;
-	if (method == COMPACT_AVG) 
-	    val = (mvec[p-3+skip] + mvec[p-2+skip] + mvec[p-1+skip]) / 3.0;
-	else if (method == COMPACT_SUM)
-	    val = mvec[p-3+skip] + mvec[p-2+skip] + mvec[p-1+skip];
-	else if (method == COMPACT_EOP)
-	    val = mvec[p-1+skip];
-	else if (method == COMPACT_SOP)
-	    val = mvec[p-3+skip];
-	/* do we want to limit the precision of the compacted
-	   data to that of the original data? */
-#ifdef LIMIT_DIGITS
-	sprintf(numstr, "%.*f", pmax, val);
-	(*pq)[t] = atof(numstr);
-#else
-	(*pq)[t] = val;
-#endif
-    }
+    sinfo->pd = target_pd;
 
-    sinfo->pd = 4;
-
-    return 0;
+    return x;
 }
 
 /* ........................................................... */
 
-int to_annual (double **pq, double *mvec, SERIESINFO *sinfo,
-	       int method)
-{
-    int i, t, p, p0, y0, skip = 0, endskip, goodobs;
-    int pd = sinfo->pd;
-    double val;
-#ifdef LIMIT_DIGITS
-    int pmax = 0;
-    char numstr[16];
-
-    /* record the precision of the original data */
-    for (t=0; t<sinfo->nobs; t++) {
-	p = get_places(mvec[t]);
-	if (p > pmax) pmax = p;
-    }
-#endif
-
-    /* figure the annual dates */
-    y0 = atoi(sinfo->stobs);
-    p0 = atoi(sinfo->stobs + 5);
-    if (p0 != 1) {
-	++y0;
-	skip = pd - (p0 + 1);
-    }
-
-    fprintf(stderr, "startskip = %d\n", skip);
-    endskip = (sinfo->nobs - skip) % pd;
-    fprintf(stderr, "endskip = %d\n", endskip);
-    goodobs = (sinfo->nobs - skip - endskip) / pd;
-    fprintf(stderr, "goodobs = %d\n", goodobs);
-    sinfo->nobs = goodobs;
-    sprintf(sinfo->stobs, "%d", y0);
-    fprintf(stderr, "starting date = %s\n", sinfo->stobs);
-
-    *pq = malloc(goodobs * sizeof **pq);
-    if (*pq == NULL) return 1;
-
-    for (t=0; t<goodobs; t++) {
-	p = (t + 1) * pd;
-	val = 0.;
-
-	if (method == COMPACT_AVG || method == COMPACT_SUM) { 
-	    for (i=1; i<=pd; i++) val += mvec[p-i+skip];
-	    if (method == COMPACT_AVG) {
-		val /= (double) pd;
-	    }
-	}
-	else if (method == COMPACT_EOP) {
-	    val = mvec[p-1+skip];
-	}
-	else if (method == COMPACT_SOP) {
-	    val = mvec[p-pd+skip];
-	}
-
-#ifdef LIMIT_DIGITS
-	sprintf(numstr, "%.*f", pmax, val);
-	(*pq)[t] = atof(numstr);
-#else
-	(*pq)[t] = val;
-#endif
-    }
-
-    sinfo->pd = 1;
-
-    return 0;
-}
-
 int set_db_name (const char *fname, int filetype, const PATHS *ppaths, 
 		 PRN *prn)
 {
@@ -1070,18 +1030,25 @@ get_compact_method_and_advance (const char *s, int *method)
 	    }
 	    p++;
 	}
-	comp[i] = 0;
+	comp[i] = '\0';
 
-	if (!strcmp(comp, "average")) *method = COMPACT_AVG;
-	else if (!strcmp(comp, "sum")) *method = COMPACT_SUM;
-	else if (!strcmp(comp, "first")) *method = COMPACT_SOP;
-	else if (!strcmp(comp, "last")) *method = COMPACT_EOP;
+	if (!strcmp(comp, "average")) {
+	    *method = COMPACT_AVG;
+	} else if (!strcmp(comp, "sum")) {
+	    *method = COMPACT_SUM;
+	} else if (!strcmp(comp, "first")) {
+	    *method = COMPACT_SOP;
+	} else if (!strcmp(comp, "last")) {
+	    *method = COMPACT_EOP;
+	}
 
 	p = strchr(p, ')');
 	if (p != NULL) p++;
     } else {
 	/* no compaction method given */
-	if ((p = strstr(s, "data "))) p += 5;
+	if ((p = strstr(s, "data "))) {
+	    p += 5;
+	}
     }
 
     return p;
@@ -1187,7 +1154,6 @@ void get_db_padding (SERIESINFO *sinfo, DATAINFO *pdinfo,
     *pad2 = pdinfo->n - sinfo->nobs - *pad1;
 } 
 
-
 int check_db_import (SERIESINFO *sinfo, DATAINFO *pdinfo)
 {
     double sd0, sdn_new, sdn_old;
@@ -1214,11 +1180,9 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 			    double ***pZ, DATAINFO *pdinfo,
 			    int compact_method, int dbv)
 {
-    int err = 0;
-    double *xvec;
+    double *xvec = NULL;
     int n, t, start, stop, pad1 = 0, pad2 = 0;
     int new = (dbv == pdinfo->v);
-    
 
     if (check_db_import(sinfo, pdinfo)) {
 	return 1;
@@ -1252,35 +1216,35 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	if (pdinfo->pd != 1 && pdinfo->pd != 4 &&
 	    sinfo->pd != 12) {
 	    strcpy(gretl_errmsg, _("Sorry, can't handle this conversion yet!"));
-	    if (new) dataset_drop_vars(1, pZ, pdinfo);
+	    if (new) {
+		dataset_drop_vars(1, pZ, pdinfo);
+	    }
 	    return 1;
 	}
 	if (compact_method == COMPACT_NONE) {
 	    sprintf(gretl_errmsg, _("%s: you must specify a compaction method"), 
 		    sinfo->varname);
-	    if (new) dataset_drop_vars(1, pZ, pdinfo);
+	    if (new) {
+		dataset_drop_vars(1, pZ, pdinfo);
+	    }
 	    return 1;
 	}
-	if (sinfo->pd == 12 && pdinfo->pd == 4) {
-	    mon_to_quart(&xvec, dbZ[1], sinfo, compact_method);
-	}
-	else if (pdinfo->pd == 1) {
-	    to_annual(&xvec, dbZ[1], sinfo, compact_method);
-	}
+	xvec = compact_db_series(dbZ[1], sinfo, pdinfo->pd, compact_method);
     } else {  
 	/* series does not need compacting */
 	xvec = malloc(sinfo->nobs * sizeof *xvec);
-	if (xvec == NULL) err = 1;
-	else {
+	if (xvec != NULL) {
 	    for (t=0; t<sinfo->nobs; t++) {
 		xvec[t] = dbZ[1][t];
 	    }
 	} 
     }
 
-    if (err) {
+    if (xvec == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory adding series"));
-	if (new) dataset_drop_vars(1, pZ, pdinfo);
+	if (new) {
+	    dataset_drop_vars(1, pZ, pdinfo);
+	}
 	return 1;
     }
 
@@ -1323,3 +1287,703 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
     return 0;
 }
 
+/* compact an individual series, in the context of converting an
+   entire working dataset to a lower frequency: used in all cases
+   except conversion from daily to monthly
+*/
+
+static double *compact_series (const double *src, int i, int n, int oldn,
+			       int startskip, int min_startskip, int compfac,
+			       int method)
+{
+    int t, idx;
+    int lead = startskip - min_startskip;
+    int to_weekly = (compfac >= 5 && compfac <= 7);
+    double *x;
+
+#ifdef DB_DEBUG
+    printf("compact_series: startskip=%d, min_startskip=%d, compfac=%d "
+	   "lead=%d\n", startskip, min_startskip, compfac, lead);
+#endif
+
+    x = malloc(n * sizeof *x);
+    if (x == NULL) return NULL;
+
+    for (t=0; t<n; t++) {
+	x[t] = (i == 0)? 1.0 : NADBL;
+    }
+
+    if (i == 0) {
+	return x;
+    }
+
+    idx = startskip;
+
+    for (t=lead; t<n && idx<oldn; t++) {
+	if (method == COMPACT_SOP) {
+	    if (to_weekly && na(src[idx]) && idx < oldn - 1) {
+		/* allow one day's slack */
+		x[t] = src[idx + 1];
+	    } else {
+		x[t] = src[idx];
+	    }
+	} else if (method == COMPACT_EOP) {
+	    if (to_weekly && na(src[idx]) && idx > startskip) {
+		/* one day's slack */
+		x[t] = src[idx - 1];
+	    } else {
+		x[t] = src[idx];
+	    }
+	} else if (method == COMPACT_SUM || method == COMPACT_AVG) {
+	    int j, st, den = compfac;
+
+	    if (idx + compfac - 1 > oldn - 1) break;
+
+	    x[t] = 0.0;
+	    for (j=0; j<compfac; j++) {
+		st = idx + j;
+		if (na(src[st])) {
+		    if (to_weekly) {
+			den--;
+		    } else {
+			x[t] = NADBL;
+			break;
+		    }
+		} else {
+		    x[t] += src[st];
+		}
+	    }
+	    if (method == COMPACT_AVG && !na(x[t])) {
+		if (den > 0) {
+		    x[t] /= den;
+		} else {
+		    x[t] = NADBL;
+		}
+	    }
+	}
+	idx += compfac;
+    }
+
+    return x;
+}
+
+/* specific apparatus for converting daily time series to monthly */
+
+static double *extend_series (const double *z, int n)
+{
+    double *x = malloc(n * sizeof *x);
+
+    if (x != NULL) {
+	int t;
+
+	x[0] = NADBL;
+	for (t=1; t<n; t++) {
+	    x[t] = z[t-1];
+	}
+    }
+
+    return x;
+}
+
+static double *
+daily_series_to_monthly (const double *src, DATAINFO *pdinfo, int i, 
+			 int nm, int yr, int mon, int offset, 
+			 int any_eop, int method)
+{
+    double *x;
+    const double *z;
+    double *tmp = NULL;
+    int t, mdbak;
+    int sopt, eopt;
+
+    x = malloc(nm * sizeof *x);
+    if (x == NULL) return NULL;
+
+    if (i == 0) {
+	for (t=0; t<nm; t++) {
+	    x[t] = 1.0;
+	}
+	return x;
+    }
+
+    if (offset < 0) {
+	tmp = extend_series(src, pdinfo->n + 1);
+	if (tmp == NULL) {
+	    free(x);
+	    return NULL;
+	}
+	/* this permits use of a negative offset */
+	z = tmp + 1;
+    } else {
+	z = src;
+    }
+
+    /* note: we can't necessarily assume that the first obs
+       is the first day of a month  
+    */
+
+    /* the "one day's slack" business with start-of-period
+       and end-of-period compaction is designed to allow for
+       the possibility that the first (or last) day of the
+       month may not have been a trading day
+    */
+
+    mdbak = 0;
+    sopt = offset;
+    eopt = offset - 1;
+
+#if 0
+    printf("offset=%d, sopt=%d\n", offset, sopt);
+#endif
+
+    for (t=0; t<nm; t++) {
+	int mdays = get_days_in_month(mon, yr, pdinfo->pd);
+
+	sopt += mdbak;
+	eopt += mdays;
+
+	if (t == 0 && offset > 0 && any_eop &&
+	    method != COMPACT_EOP) {
+	    x[t] = NADBL;
+	} else if (method == COMPACT_SOP) {
+	    /* allow one days's slack */
+	    if (na(z[sopt]) && sopt < pdinfo->n - 1) {
+		x[t] = z[sopt + 1];
+	    } else {
+		x[t] = z[sopt];
+	    }
+	} else if (method == COMPACT_EOP) {
+	    if (eopt >= pdinfo->n) {
+		x[t] = NADBL;
+	    } else {
+		/* allow one days's slack */
+		if (na(z[eopt]) && eopt > 0) {
+		    x[t] = z[eopt - 1];
+		} else {
+		    x[t] = z[eopt];
+		}
+	    }
+	} else if (method == COMPACT_SUM ||
+		   method == COMPACT_AVG) {
+	    int j, dayt, den = mdays;
+
+	    x[t] = 0.0;
+	    for (j=0; j<mdays; j++) {
+		dayt = sopt + j;
+		if (dayt >= pdinfo->n) {
+		    x[t] = NADBL;
+		    break;
+		} else if (na(z[dayt]) && method == COMPACT_AVG) {
+		    den--;
+		} else if (!na(z[dayt])) {
+		    x[t] += z[dayt];
+		}
+	    }
+	    if (method == COMPACT_AVG && !na(x[t])) {
+		if (den > 0) {
+		    x[t] /= (double) den;
+		} else {
+		    x[t] = NADBL;
+		}
+	    }
+	}
+
+	mdbak = mdays;
+
+	if (mon == 12) {
+	    mon = 1;
+	    yr++;
+	} else {
+	    mon++;
+	}
+    }
+
+    if (tmp != NULL) {
+	free(tmp);
+    }
+
+    return x;
+}
+
+static void 
+get_startskip_etc (int compfac, int startmin, int endmin, 
+		   int oldn, int method, 
+		   int *startskip, int *newn) 
+{
+    int ss = compfac - (startmin % compfac) + 1;
+    int es, n;
+
+    ss = ss % compfac;
+
+    if (method == COMPACT_EOP) {
+	if (ss > 0) {
+	    ss--;
+	} else {
+	    /* move to end of initial period */
+	    ss = compfac - 1;
+	}
+    }
+
+    es = endmin % compfac;
+    if (method == COMPACT_SOP && es > 1) {
+	es--;
+    }
+
+    n = (oldn - ss - es) / compfac;
+    if (ss && method == COMPACT_EOP) n++;
+    if (es && method == COMPACT_SOP) n++;
+
+    *startskip = ss;
+    *newn = n;
+}
+
+/* specific to compaction of daily time series */
+
+static void 
+get_daily_compact_params (int default_method, 
+			  int *any_eop, int *any_sop,
+			  int *all_same,
+			  const DATAINFO *pdinfo)
+{
+    int i, n_not_eop = 0, n_not_sop = 0;
+
+    *all_same = 1;
+    *any_eop = (default_method == COMPACT_EOP)? 1 : 0;
+    *any_sop = (default_method == COMPACT_SOP)? 1 : 0;
+
+    for (i=1; i<pdinfo->v; i++) {
+	int method = COMPACT_METHOD(pdinfo, i);
+
+	if (method != default_method && method != COMPACT_NONE) {
+	    *all_same = 0;
+	    if (method == COMPACT_EOP) {
+		*any_eop = 1;
+	    } else {
+		n_not_eop++;
+	    }
+	    if (method == COMPACT_SOP) {
+		*any_sop = 1;
+	    } else {
+		n_not_sop++;
+	    }
+	}
+    }
+
+    if (n_not_eop == pdinfo->v - 1) {
+	*any_eop = 0;
+    }
+
+    if (n_not_sop == pdinfo->v - 1) {
+	*any_sop = 0;
+    }
+}
+
+/* specific to non-daily time series (monthly or quarterly) */
+
+static void 
+get_global_compact_params (int compfac, int startmin, int endmin,
+			   int default_method, 
+			   int *min_startskip, int *max_n,
+			   int *any_eop, int *all_same,
+			   DATAINFO *pdinfo)
+{
+    int i, startskip, n, method;
+    int n_not_eop = 0;
+
+    for (i=0; i<pdinfo->v; i++) {
+	if (i == 0) {
+	    get_startskip_etc(compfac, startmin, endmin, pdinfo->n, 
+			      default_method, &startskip, &n);
+	    if (default_method == COMPACT_EOP) {
+		*any_eop = 1;
+	    }
+	} else {
+	    method = COMPACT_METHOD(pdinfo, i);
+	    if (method != default_method && method != COMPACT_NONE) {
+		get_startskip_etc(compfac, startmin, endmin, pdinfo->n, 
+				  method, &startskip, &n);
+		*all_same = 0;
+		if (method == COMPACT_EOP) {
+		    *any_eop = 1;
+		} else {
+		    n_not_eop++;
+		}
+	    }
+	}
+	if (startskip < *min_startskip) {
+	    *min_startskip = startskip;
+	}
+	if (n > *max_n) {
+	    *max_n = n;
+	}
+    }
+
+    if (n_not_eop == pdinfo->v - 1) {
+	*any_eop = 0;
+    }
+}
+
+static int get_obs_maj_min (const char *obs, int *maj, int *min)
+{
+    int np = sscanf(obs, "%d:%d", maj, min);
+
+    if (np < 2) {
+	np = sscanf(obs, "%d.%d", maj, min);
+    }
+
+    return (np == 2);
+}
+
+/* for daily data, figure the number of observations to
+   be skipped at the start of each series 
+*/
+
+static int get_daily_offset (const DATAINFO *pdinfo,
+			     int y, int m, int d, 
+			     int skip)
+{
+    int ret = 0;
+
+    if (skip) {
+	/* moving to start of next month: offset = no. of
+	   observations in the first month */
+	ret = days_in_month_after(y, m, d, pdinfo->pd) + 1;
+    } else {
+	/* offset = no. of obs missing at start of first month */
+	ret = days_in_month_before(y, m, d, pdinfo->pd);
+    }
+
+    return ret;
+}
+
+/* for daily data, figure the number of valid monthly 
+   observations that can be constructed by compaction
+*/
+
+static int get_n_ok_months (const DATAINFO *pdinfo, int default_method,
+			    int *startyr, int *startmon,
+			    int *endyr, int *endmon,
+			    int *offset, int *p_any_eop)
+{
+    int y1, m1, d1, y2, m2, d2;
+    int any_eop, any_sop, all_same;
+    int skip = 0, pad = 0, nm = -1;
+
+    if (sscanf(pdinfo->stobs, "%d/%d/%d", &y1, &m1, &d1) != 3) {
+	return -1;
+    }
+    if (sscanf(pdinfo->endobs, "%d/%d/%d", &y2, &m2, &d2) != 3) {
+	return -1;
+    }
+
+    if (y1 < 100) {
+	y1 = FOUR_DIGIT_YEAR(y1);
+    }
+    if (y2 < 100) {
+	y2 = FOUR_DIGIT_YEAR(y2);
+    }
+
+    nm = 12 * (y2 - y1) + m2 - m1 + 1;
+
+    get_daily_compact_params(default_method, &any_eop, &any_sop,
+			     &all_same, pdinfo);
+
+    *startyr = y1;
+    *startmon = m1;
+    *endyr = y2;
+    *endmon = m2;
+
+    if (!day_starts_month(d1, m1, y1, pdinfo->pd, &pad) && !any_eop) {
+	if (*startmon == 12) {
+	    *startmon = 1;
+	    *startyr += 1;
+	} else {
+	    *startmon += 1;
+	}
+	skip = 1;
+	nm--;
+    }
+
+    if (!day_ends_month(d2, m2, y2, pdinfo->pd) && !any_sop) {
+	if (*endmon == 1) {
+	    *endmon = 12;
+	    *endyr -= 1;
+	} else {
+	    *endmon -= 1;
+	}
+	nm--;
+    }
+
+    if (pad) {
+	*offset = -1;
+    } else {
+	*offset = get_daily_offset(pdinfo, y1, m1, d1, skip);
+    }
+
+    *p_any_eop = any_eop;
+
+    return nm;
+}
+
+/* driver for converting full dataset from daily to monthly */
+
+static int dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
+			       int default_method)
+{
+    int nm, startyr, startmon, endyr, endmon;
+    int offset, any_eop;
+    int i, err = 0;
+
+    nm = get_n_ok_months(pdinfo, default_method, &startyr, &startmon,
+			 &endyr, &endmon, &offset, &any_eop);
+
+    if (nm <= 0) {
+	gretl_errmsg_set(_("Compacted dataset would be empty"));
+	err = 1;
+    } else {
+	for (i=0; i<pdinfo->v && !err; i++) {
+	    int method;
+	    double *x;
+
+	    if (i > 0 && !pdinfo->vector[i]) {
+		continue;
+	    }
+
+	    method = COMPACT_METHOD(pdinfo, i);
+	    if (method == COMPACT_NONE) {
+		method = default_method;
+	    }
+
+	    x = daily_series_to_monthly((*pZ)[i], pdinfo, i, nm,
+					startyr, startmon, 
+					offset, any_eop, method);
+	    if (x == NULL) {
+		err = E_ALLOC;
+	    } else {
+		free((*pZ)[i]);
+		(*pZ)[i] = x;
+	    }
+	}
+    }
+
+    if (!err) {
+	pdinfo->n = nm;
+	pdinfo->pd = 12;
+	sprintf(pdinfo->stobs, "%04d:%02d", startyr, startmon);
+	sprintf(pdinfo->endobs, "%04d:%02d", endyr, endmon);
+	pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
+	pdinfo->t1 = 0;
+	pdinfo->t2 = pdinfo->n - 1;
+
+	destroy_dataset_markers(pdinfo);
+    }
+
+    return err;
+}
+
+static int get_daily_skip (const DATAINFO *pdinfo, int t)
+{
+    int dd = daily_obs_number(pdinfo->S[t], pdinfo) -
+	daily_obs_number(pdinfo->S[t-1], pdinfo);
+
+    return dd - 1;
+}
+
+static int 
+insert_missing_hidden_obs (double ***pZ, DATAINFO *pdinfo,
+			   int nmiss)
+{
+    int oldn = pdinfo->n;
+    double *tmp;
+    int i, s, t, skip;
+    int err = 0;
+
+    tmp = malloc(oldn * sizeof *tmp);
+    if (tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    err = grow_nobs(nmiss, pZ, pdinfo);
+    if (err) {
+	free(tmp);
+	return err;
+    }
+
+    for (i=1; i<pdinfo->v && !err; i++) {
+	for (s=0; s<oldn; s++) {
+	    tmp[s] = (*pZ)[i][s];
+	}
+
+	(*pZ)[i][0] = tmp[0];
+	t = 1;
+	for (s=1; s<oldn; s++) {
+	    skip = get_daily_skip(pdinfo, s);
+	    if (skip < 0) {
+		err = E_DATA;
+		break;
+	    }
+	    while (skip--) {
+		(*pZ)[i][t++] = NADBL;
+	    }
+	    (*pZ)[i][t++] = tmp[s];
+	}
+    }
+
+    free(tmp);
+
+    if (!err) {
+	destroy_dataset_markers(pdinfo);
+	pdinfo->t2 = pdinfo->n - 1;
+	ntodate_full(pdinfo->endobs, pdinfo->n - 1, pdinfo);
+    }
+
+    return err;
+}
+
+int maybe_expand_daily_data (double ***pZ, DATAINFO *pdinfo)
+{
+    int nmiss = n_hidden_missing_obs(pdinfo);
+    int err = 0;
+
+    if (nmiss < 0) {
+	err = 1;
+    } else if (nmiss > 0) {
+	err = insert_missing_hidden_obs(pZ, pdinfo, nmiss);
+    }
+
+    return err;
+}
+
+int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
+		      int default_method, int monstart)
+{
+    int newn, oldn = pdinfo->n, oldpd = pdinfo->pd;
+    int compfac;
+    int startmaj, startmin;
+    int endmaj, endmin;
+    int any_eop, all_same;
+    int min_startskip = 0;
+    char stobs[OBSLEN];
+    int i, err = 0;
+
+    *gretl_errmsg = '\0';
+
+    if (dated_daily_data(pdinfo)) {
+	/* allow for the possibility that the daily dataset
+	   contains "hidden" or suppressed missing observations
+	   (holidays are just skipped, not marked as NA)
+	*/
+	err = maybe_expand_daily_data(pZ, pdinfo);
+	if (err) return err;
+    }
+
+    if (newpd == 12 && oldpd >= 5 && oldpd <= 7) {
+	/* daily to monthly: special */
+	return dataset_to_monthly(pZ, pdinfo, default_method);
+    } else if (oldpd >= 5 && oldpd <= 7) {
+	/* daily to weekly */
+	compfac = oldpd;
+	if (dated_daily_data(pdinfo)) {
+	    startmin = get_day_of_week(pdinfo->stobs);
+	    if (oldpd == 7) {
+		if (monstart) {
+		    if (startmin == 0) startmin = 7;
+		} else {
+		    startmin++;
+		}
+	    }
+	} else {
+	    startmin = 1;
+	}
+    } else {
+	compfac = oldpd / newpd;
+	/* get starting obs major and minor components */
+	if (!get_obs_maj_min(pdinfo->stobs, &startmaj, &startmin)) {
+	    return 1;
+	}
+	/* get ending obs major and minor components */
+	if (!get_obs_maj_min(pdinfo->endobs, &endmaj, &endmin)) {
+	    return 1;
+	} 
+    }
+
+    min_startskip = oldpd;
+    newn = 0;
+    any_eop = 0;
+    all_same = 1;
+    get_global_compact_params(compfac, startmin, endmin, default_method,
+			      &min_startskip, &newn, &any_eop, &all_same, 
+			      pdinfo);
+    if (newn == 0) {
+	gretl_errmsg_set(_("Compacted dataset would be empty"));
+	return 1;
+    }    
+
+    if (newpd == 1) {
+	if (min_startskip > 0 && !any_eop) { 
+	    startmaj++;
+	}
+	sprintf(stobs, "%d", startmaj);
+    } else if (newpd == 52) {
+	strcpy(stobs, "1");
+    } else {
+	int m0 = startmin + min_startskip;
+	int minor = m0 / compfac + (m0 % compfac > 0);
+
+	if (minor > newpd) {
+	    startmaj++;
+	    minor -= newpd;
+	}
+	format_obs(stobs, startmaj, minor, newpd);
+    }
+
+    /* revise datainfo members */
+    strcpy(pdinfo->stobs, stobs);
+    pdinfo->pd = newpd;
+    pdinfo->n = newn;
+    pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
+    pdinfo->t1 = 0;
+    pdinfo->t2 = pdinfo->n - 1;
+    ntodate(pdinfo->endobs, pdinfo->t2, pdinfo);
+    
+    if (oldpd >= 5 && oldpd <= 7) {
+	/* remove any daily date strings */
+	destroy_dataset_markers(pdinfo);
+    }
+
+    /* compact the individual data series */
+    for (i=0; i<pdinfo->v && err == 0; i++) {
+	if (pdinfo->vector[i]) {
+	    int this_method = default_method;
+	    int startskip = min_startskip;
+	    double *x;
+
+	    if (!all_same) {
+		if (COMPACT_METHOD(pdinfo, i) != COMPACT_NONE) {
+		    this_method = COMPACT_METHOD(pdinfo, i);
+		}
+
+		startskip = compfac - (startmin % compfac) + 1;
+		startskip = startskip % compfac;
+		if (this_method == COMPACT_EOP) {
+		    if (startskip > 0) {
+			startskip--;
+		    } else {
+			startskip = compfac - 1;
+		    }
+		}
+	    }
+
+	    x = compact_series((*pZ)[i], i, pdinfo->n, oldn, startskip, 
+			       min_startskip, compfac, this_method);
+	    if (x == NULL) {
+		err = E_ALLOC;
+	    } else {
+		free((*pZ)[i]);
+		(*pZ)[i] = x;
+	    }
+	}
+    }
+
+    return err;
+}
