@@ -22,6 +22,8 @@
 #include "libgretl.h"
 #include "gretl_private.h"
 #include "gretl_matrix.h"
+#include "libset.h"
+#include "compat.h"
 
 #include <unistd.h>
 
@@ -33,18 +35,38 @@
 #include <glib.h>
 #endif
 
-/* ........................................................... */
-
 static int missvals (double *x, int n)
 {
     int t;
     
-    for (t=0; t<n; t++)
-	if (na(x[t])) return 1;
+    for (t=0; t<n; t++) {
+	if (na(x[t])) {
+	    return 1;
+	}
+    }
 	    
     return 0;
 }
 
+static int good_obs (const double *x, int n, double *x0)
+{
+    int t, good = n;
+
+    if (x0 != NULL) {
+	*x0 = NADBL;
+    }
+
+    for (t=0; t<n; t++) {
+	if (na(x[t])) {
+	    good--;
+	} else if (x0 != NULL && na(*x0)) {
+	    *x0 = x[t];
+	}
+    }
+
+    return good;
+}
+
 /**
  * gretl_median:
  * @x: data series.
@@ -60,15 +82,21 @@ double gretl_median (const double *x, int n)
     int t, n2, n2p;
 
     sx = malloc(n * sizeof *sx);
-    if (sx == NULL) return NADBL;
+    if (sx == NULL) {
+	return NADBL;
+    }
 
-    for (t=0; t<n; t++) {
-	sx[t] = x[t];
+    n2 = n;
+    n = 0;
+    for (t=0; t<n2; t++) {
+	if (!na(x[t])) {
+	    sx[n++] = x[t];
+	}
     }
 
     qsort(sx, n, sizeof *sx, gretl_compare_doubles); 
 
-    n2p = (n2 = n/2) + 1;
+    n2p = (n2 = n / 2) + 1;
     med = (n % 2)? sx[n2p - 1] : 0.5 * (sx[n2 - 1] + sx[n2p - 1]);
 
     free(sx);
@@ -76,23 +104,29 @@ double gretl_median (const double *x, int n)
     return med;
 }
 
-/* ........................................................... */
+/* k below is the "degrees of freedom loss": it will generally be one,
+   other than when dealing with a regression residual
+*/
 
-int moments (int t1, int t2, const double *zx, 
+int moments (int t1, int t2, const double *x, 
 	     double *xbar, double *std, 
 	     double *skew, double *kurt, int k)
-     /* k is the "degrees of freedom loss": it will generally be one,
-	other than when dealing with a regression residual */
 {
-    int t, n = t2 - t1 + 1;
+    int t, n;
     double dev, var;
     double s, s2, s3, s4;
     int allstats = 1;
 
-    if (skew == NULL && kurt == NULL) allstats = 0;
+    if (skew == NULL && kurt == NULL) {
+	allstats = 0;
+    }
 
-    if (gretl_isconst(t1, t2, zx)) {
-	*xbar = zx[t1];
+    while (na(x[t1]) && t1 <= t2) {
+	t1++;
+    }
+
+    if (gretl_isconst(t1, t2, x)) {
+	*xbar = x[t1];
 	*std = 0.0;
 	if (allstats) {
 	    *skew = *kurt = NADBL;
@@ -101,15 +135,34 @@ int moments (int t1, int t2, const double *zx,
     }
 
     s = 0.0;
-    for (t=t1; t<=t2; t++) s += zx[t];
+    n = 0;
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t])) {
+	    s += x[t];
+	    n++;
+	}
+    }
+
+    if (n == 0) {
+	*xbar = *std = NADBL;
+	if (allstats) {
+	    *skew = *kurt = 0.0;
+	}	
+	return 1;
+    }
 
     *xbar = s / n;
     var = 0.0;
-    if (allstats) *skew = *kurt = 0.0;
+    if (allstats) {
+	*skew = *kurt = 0.0;
+    }
 
     s2 = s3 = s4 = 0.0;
     for (t=t1; t<=t2; t++) {
-	dev = zx[t] - *xbar;
+	if (na(x[t])) {
+	    continue;
+	}
+	dev = x[t] - *xbar;
 	s2 += dev * dev;
 	if (allstats) {
 	    s3 += pow(dev, 3);
@@ -117,11 +170,13 @@ int moments (int t1, int t2, const double *zx,
 	}
     }
 
-    var = s2 / (n-k);
+    var = s2 / (n - k);
 
     if (var < 0.0) {
 	*std = NADBL;
-	if (allstats) *skew = *kurt = NADBL;
+	if (allstats) {
+	    *skew = *kurt = NADBL;
+	}
 	return 1;
     }
 
@@ -155,6 +210,23 @@ void free_freq (FREQDIST *freq)
     free(freq);
 }
 
+static FREQDIST *freq_new (void)
+{
+    FREQDIST *freq;
+
+    freq = malloc(sizeof *freq);
+    if (freq == NULL) return NULL;
+
+    freq->midpt = NULL;
+    freq->endpt = NULL;
+    freq->f = NULL;
+
+    freq->dist = 0;
+    freq->test = NADBL;
+
+    return freq;
+}
+
 static double rb1_to_z1 (double rb1, double n)
 {
     double b, w2, d, y, z1;
@@ -196,9 +268,9 @@ static double b2_to_z2 (double b1, double b2, double n)
     return z2;
 }
 
+/* Bowman-Shenton as modified by Doornik & Hansen */
+
 double doornik_chisq (double skew, double kurt, int n)
-     /* Bowman-Shenton as modified by Doornik & Hansen */
-	
 {
     double rb1, b1, b2, z1, z2;
 
@@ -213,12 +285,13 @@ double doornik_chisq (double skew, double kurt, int n)
 }
 
 /**
- * freqdist:
+ * get_freq:
+ * @varno: ID number of variable to process.
  * @pZ: pointer to data matrix
  * @pdinfo: information on the data set.
- * @varno: ID number of variable to process.
  * @params: degrees of freedom loss (generally = 1 unless we're dealing
  * with the residual from a regression)
+ * @opt: if & OPT_O, compare with gamma distribution, not normal
  *
  * Calculates the frequency distribution for the specified variable.
  *
@@ -226,40 +299,30 @@ double doornik_chisq (double skew, double kurt, int n)
  *
  */
 
-FREQDIST *freqdist (double ***pZ, const DATAINFO *pdinfo, 
-		    int varno, int params)
+FREQDIST *get_freq (int varno, const double **Z, const DATAINFO *pdinfo, 
+		    int params, gretlopt opt)
 {
     FREQDIST *freq;
-    double *x = NULL;
-    double xx, xmin, xmax, xbar, sdx;
+    const double *x = Z[varno];
+    double xx, xmin, xmax;
     double skew, kurt;
     double range, binwidth;
-    int i, k, n;
     int nbins;
+    int t, k, n;
 
-    freq = malloc(sizeof *freq);
+    freq = freq_new();
     if (freq == NULL) return NULL;
 
     gretl_errno = 0;
     gretl_errmsg[0] = '\0';
 
-    freq->midpt = NULL;
-    freq->endpt = NULL;
-    freq->f = NULL;
+    n = pdinfo->t2 - pdinfo->t1 + 1;
+    n = good_obs(x + pdinfo->t1, n, NULL);
 
-    x = malloc((pdinfo->t2 - pdinfo->t1 + 1) * sizeof *x);
-    if (x == NULL) {
-	sprintf(gretl_errmsg, _("Out of memory in frequency distribution"));
-	free(freq);
-	return NULL;
-    }
-
-    n = ztox(varno, x, *pZ, pdinfo);
     if (n < 8) {
 	gretl_errno = E_DATA;
 	sprintf(gretl_errmsg, _("Insufficient data to build frequency "
 		"distribution for variable %s"), pdinfo->varname[varno]);
-	free(x);
 	return freq;
     }
 
@@ -269,18 +332,38 @@ FREQDIST *freqdist (double ***pZ, const DATAINFO *pdinfo,
 
     strcpy(freq->varname, pdinfo->varname[varno]);
 
-    if (gretl_isconst(0, n-1, x)) {
+    if (gretl_isconst(pdinfo->t1, pdinfo->t2, x)) {
 	gretl_errno = 1;
 	sprintf(gretl_errmsg, _("%s is a constant"), freq->varname);
 	return freq;
     }    
     
-    moments(0, n-1, x, &freq->xbar, &freq->sdx, &skew, &kurt, params);
-    xbar = freq->xbar;
-    sdx = freq->sdx;
+    moments(pdinfo->t1, pdinfo->t2, x, 
+	    &freq->xbar, &freq->sdx, 
+	    &skew, &kurt, params);
 
-    gretl_minmax(0, n-1, x, &xmin, &xmax);
+    gretl_minmax(pdinfo->t1, pdinfo->t2, x, &xmin, &xmax);
     range = xmax - xmin;
+
+    /* calculate test stat for distribution */
+    if (freq->n > 7) {
+	if (opt & OPT_O) {
+	    freq->test = lockes_test(x, pdinfo->t1, pdinfo->t2);
+	    freq->dist = GAMMA;
+	} else {
+	    freq->test = doornik_chisq(skew, kurt, freq->n); 
+	    freq->dist = NORMAL;
+	}
+    } else {
+	freq->test = NADBL;
+	freq->dist = 0;
+    }
+
+    /* if the histogram is not wanted, we're done */
+    if (opt & OPT_Q) {
+	freq->numbins = 0;
+	return freq;
+    }
     
     if (n < 16) {
 	nbins = 5; 
@@ -290,7 +373,9 @@ FREQDIST *freqdist (double ***pZ, const DATAINFO *pdinfo,
 	nbins = 29;
     } else {
 	nbins = (int) sqrt((double) n);
-	if (nbins % 2 == 0) nbins++;
+	if (nbins % 2 == 0) {
+	    nbins++;
+	}
     }
 
     freq->numbins = nbins;
@@ -299,11 +384,9 @@ FREQDIST *freqdist (double ***pZ, const DATAINFO *pdinfo,
     freq->endpt = malloc((nbins + 1) * sizeof *freq->endpt);
     freq->midpt = malloc(nbins * sizeof *freq->midpt);
     freq->f = malloc(nbins * sizeof *freq->f);
-    if (freq->endpt == NULL || freq->midpt == NULL ||
-	freq->f == NULL) {
+    if (freq->endpt == NULL || freq->midpt == NULL || freq->f == NULL) {
 	gretl_errno = E_ALLOC;
 	strcpy(gretl_errmsg, _("Out of memory for frequency distribution"));
-	free(x);
 	return freq;
     }
     
@@ -327,8 +410,11 @@ FREQDIST *freqdist (double ***pZ, const DATAINFO *pdinfo,
 	freq->midpt[k] = freq->endpt[k] + .5 * binwidth;
     }
 
-    for (i=0; i<n; i++) {
-	xx = x[i];
+    for (t=pdinfo->t1; t<pdinfo->t2; t++) {
+	xx = x[t];
+	if (na(xx)) {
+	    continue;
+	}
 	if (xx < freq->endpt[1]) {
 	    freq->f[0] += 1;
 	    continue;
@@ -345,18 +431,62 @@ FREQDIST *freqdist (double ***pZ, const DATAINFO *pdinfo,
 	}
     }
 
-    if (freq->n > 7) {
-	freq->chisqu = doornik_chisq(skew, kurt, freq->n); 
-    } else {
-	freq->chisqu = NADBL;
-    }
-
-    free(x);
-
     return freq;
 }
 
-/* ...................................................... */
+int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
+	      int graph, PRN *prn, gretlopt opt)
+{
+    FREQDIST *freq;
+
+    freq = get_freq(varno, Z, pdinfo, 1, opt); 
+
+    if (freq == NULL) {
+	return E_ALLOC;
+    }
+
+    print_freq(freq, prn); 
+
+    if (graph && !(opt & OPT_Q)) {
+	if (plot_freq(freq, (opt)? GAMMA : NORMAL)) {
+	    pputs(prn, _("gnuplot command failed\n"));
+	}
+    }
+
+    free_freq(freq);
+
+    return 0;
+}
+
+int model_error_dist (const MODEL *pmod, double ***pZ,
+		      DATAINFO *pdinfo, PRN *prn)
+{
+    FREQDIST *freq = NULL;
+    int err = 0;
+
+    if (genr_fit_resid(pmod, pZ, pdinfo, GENR_RESID, 1)) {
+	pputs(prn, _("Out of memory attempting to add variable\n"));
+	err = E_ALLOC;
+    }
+
+    if (!err) {
+	freq = get_freq(pdinfo->v - 1, (const double **) *pZ, pdinfo, 
+			pmod->ncoeff, OPT_NONE);
+	if (freq == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (!err) {
+	print_freq(freq, prn); 
+	free_freq(freq);
+    }
+
+    dataset_drop_vars(1, pZ, pdinfo);
+
+    return err;
+
+}
 
 #ifdef PACF_BY_OLS
 
@@ -380,7 +510,7 @@ static int get_pacf (double *pacf, int m, int varnum,
 
     /* add appropriate number of lags to data set */
     for (i=1; i<=m; i++) {
-	int lnum = laggenr(varnum, i, 0, pZ, pdinfo);
+	int lnum = laggenr(varnum, i, pZ, pdinfo);
 
 	if (lnum < 0) {
 	    free(list);
@@ -488,8 +618,9 @@ static double gretl_acf (int n, int k, const double *y)
     int t;
     double z, ybar, num = 0.0, den = 0.0;
 
-    if (n == 0 || gretl_isconst(0, n-1, y)) 
+    if (n == 0 || gretl_isconst(0, n - 1, y)) { 
 	return NADBL;
+    }
 
     ybar = gretl_mean(0, n-1, y);
 
@@ -510,7 +641,6 @@ static double gretl_acf (int n, int k, const double *y)
  * @order: integer order for autocorrelation function.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @ppaths: struct containing path information.
  * @batch: if = 1, use ASCII graphic rather than gnuplot graph.
  * @prn: gretl printing struct.
  *
@@ -522,8 +652,7 @@ static double gretl_acf (int n, int k, const double *y)
  */
 
 int corrgram (int varno, int order, double ***pZ, 
-	      DATAINFO *pdinfo, PATHS *ppaths, 
-	      int batch, PRN *prn)
+	      DATAINFO *pdinfo, int batch, PRN *prn)
 {
     double *acf, box, pm;
     double *pacf = NULL;
@@ -613,8 +742,11 @@ int corrgram (int varno, int order, double ***pZ,
     } 
 
     /* determine lag order for pacf (may have to be shorter than acf_m) */
-    if (acf_m > nobs / 2 - 1) pacf_m = nobs / 2 - 1;
-    else pacf_m = acf_m;
+    if (acf_m > nobs / 2 - 1) {
+	pacf_m = nobs / 2 - 1;
+    } else {
+	pacf_m = acf_m;
+    }
 
     /* generate (and if not in batch mode) plot partial 
        autocorrelation function */
@@ -645,21 +777,19 @@ int corrgram (int varno, int order, double ***pZ,
 	}
     }
     pputc(prn, '\n');
-    if (pacf_m % 5 > 0) pputc(prn, '\n');
+    if (pacf_m % 5 > 0) {
+	pputc(prn, '\n');
+    }
 
     if (batch) {
 	goto acf_getout;
-    } else if (gnuplot_init(ppaths, PLOT_CORRELOGRAM, &fq)) {
+    } else if (gnuplot_init(PLOT_CORRELOGRAM, &fq)) {
 	err = E_FOPEN;
 	goto acf_getout;
     }
 
     /* for confidence bands */
-#if 0
-    pm = 1.0 / sqrt((double) nobs);
-#else
     pm = 1.96 / sqrt((double) nobs);
-#endif
 
     fprintf(fq, "# correlogram\n");
 
@@ -720,7 +850,7 @@ int corrgram (int varno, int order, double ***pZ,
 
     fclose(fq);
 
-    err = gnuplot_display(ppaths);
+    err = gnuplot_make_graph();
 
  acf_getout:
     free(acf);
@@ -741,16 +871,20 @@ static int roundup_mod (int i, double x)
 static int fract_int (int n, double *hhat, double *omega, PRN *prn)
 {
     double xx, tstat, **tmpZ = NULL;
-    DATAINFO tmpdinfo;
+    DATAINFO *tmpdinfo;
     MODEL tmp;
     int t, err = 0, list[4];
 
-    tmpdinfo.vector = NULL;
-    tmpdinfo.n = n;
-    tmpdinfo.v = 3;
-    tmpdinfo.pd = 1;
-    if (start_new_Z(&tmpZ, &tmpdinfo, 1))
+    tmpdinfo = datainfo_new();
+    if (tmpdinfo == NULL) {
 	return 1;
+    }
+
+    tmpdinfo->n = n;
+    tmpdinfo->v = 3;
+    if (start_new_Z(&tmpZ, tmpdinfo, 1)) {
+	return 1;
+    }
 
     /* Test from Geweke and Porter-Hudak, as set out in
        Greene, Econometric Analysis 4e, p. 787 */
@@ -767,8 +901,7 @@ static int fract_int (int n, double *hhat, double *omega, PRN *prn)
     list[2] = 0;
     list[3] = 2;
 
-    gretl_model_init(&tmp, &tmpdinfo);
-    tmp = lsq(list, &tmpZ, &tmpdinfo, OLS, OPT_A, 0);
+    tmp = lsq(list, &tmpZ, tmpdinfo, OLS, OPT_A, 0);
 
     if (!tmp.errcode) {
 	tstat = -tmp.coeff[1] / tmp.sderr[1];
@@ -783,9 +916,10 @@ static int fract_int (int n, double *hhat, double *omega, PRN *prn)
 	err = tmp.errcode;
     }
 
-    clear_model(&tmp, &tmpdinfo);
-    free_Z(tmpZ, &tmpdinfo);
-    clear_datainfo(&tmpdinfo, CLEAR_FULL);
+    clear_model(&tmp);
+    free_Z(tmpZ, tmpdinfo);
+    clear_datainfo(tmpdinfo, CLEAR_FULL);
+    free(tmpdinfo);
 
     return err;
 }
@@ -795,7 +929,6 @@ static int fract_int (int n, double *hhat, double *omega, PRN *prn)
  * @varno: ID number of variable to process.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @ppaths: struct containing path information.
  * @batch: if non-zero, don't show gnuplot graph.
  * @opt: if non-zero, use Bartlett lag window for periodogram.
  * @prn: gretl printing struct.
@@ -807,8 +940,7 @@ static int fract_int (int n, double *hhat, double *omega, PRN *prn)
  */
 
 int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo, 
-		 PATHS *ppaths, int batch, 
-		 int opt, PRN *prn)
+		 int batch, int opt, PRN *prn)
 {
     double *autocov, *omega, *hhat, *savexx = NULL;
     double xx, yy, varx, w;
@@ -872,7 +1004,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 
     xmax = roundup_mod(nobs, 2.0);
 
-    if (do_graph && gnuplot_init(ppaths, PLOT_PERIODOGRAM, &fq) == 0) {
+    if (do_graph && gnuplot_init(PLOT_PERIODOGRAM, &fq) == 0) {
 	char titlestr[80];
 
 	fprintf(fq, "# periodogram\n");
@@ -964,7 +1096,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 
 	fclose(fq);
 	free(savexx);
-	err = gnuplot_display(ppaths);
+	err = gnuplot_make_graph();
     }
 
     if (opt == 0 && fract_int(nT, hhat, omega, prn)) {
@@ -983,16 +1115,15 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 
 static void printf15 (double zz, PRN *prn)
 {
-    if (na(zz)) pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 15), 
-			_("undefined"));
-    else {
+    if (na(zz)) {
+	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 15), 
+		_("undefined"));
+    } else {
 	pputc(prn, ' ');
 	gretl_print_fullwidth_double(zz, 5, prn);	
     }
 }
 
-/* ............................................................. */
-
 #define LINEWID 78
 
 static void center_line (char *str, PRN *prn, int dblspc)
@@ -1005,10 +1136,14 @@ static void center_line (char *str, PRN *prn, int dblspc)
 
 	for (i=0; i<pad; i++) cstr[i] = ' ';
 	strcpy(cstr + i, str);
-	if (dblspc) strcat(cstr, "\n");
+	if (dblspc) {
+	    strcat(cstr, "\n");
+	}
 	pprintf(prn, "%s\n", cstr);
     } else {
-	if (dblspc) strcat(str, "\n");
+	if (dblspc) {
+	    strcat(str, "\n");
+	}
 	pprintf(prn, "%s\n", str);
     }
 }
@@ -1029,18 +1164,17 @@ static void prhdr (const char *str, const DATAINFO *pdinfo,
     center_line(tmp, prn, 0);
 
     if (ci == CORR) {
-	strcpy(tmp, _("(missing values denoted by -999 will be skipped)"));
+	strcpy(tmp, _("(missing values will be skipped)"));
 	center_line(tmp, prn, 1);
     }
 }
 
-static void print_summary_single (GRETLSUMMARY *summ,
+static void print_summary_single (const GRETLSUMMARY *summ,
 				  const DATAINFO *pdinfo,
 				  PRN *prn)
 {
     char obs1[OBSLEN], obs2[OBSLEN], tmp[128];
     double vals[8];
-    double xbar, sd, cv = NADBL;
     const char *labels[] = {
 	N_("Mean"),
 	N_("Median"),
@@ -1061,17 +1195,13 @@ static void print_summary_single (GRETLSUMMARY *summ,
 	    pdinfo->varname[summ->list[1]], summ->n);
     center_line(tmp, prn, 1);
 
-    xbar = summ->coeff[0];
-    sd = summ->sderr[0];
-    if (xbar != 0.0 && !na(sd)) cv = fabs(sd/xbar);
-
-    vals[0] = xbar;
-    vals[1] = summ->xmedian[0];
-    vals[2] = summ->xpx[0];
-    vals[3] = summ->xpy[0];
-    vals[4] = sd;
-    vals[5] = cv;
-    vals[6] = summ->xskew[0];
+    vals[0] = summ->mean[0];
+    vals[1] = summ->median[0];
+    vals[2] = summ->low[0];
+    vals[3] = summ->high[0];
+    vals[4] = summ->sd[0];
+    vals[5] = summ->cv[0];
+    vals[6] = summ->skew[0];
     vals[7] = summ->xkurt[0];
 
     for (i=0; i<8; i++) {
@@ -1098,71 +1228,83 @@ static void print_summary_single (GRETLSUMMARY *summ,
  * print_summary:
  * @summ: gretl summary statistics struct.
  * @pdinfo: information on the data set.
- * @pause: if non-zero, pause after showing each screen of info.
  * @prn: gretl printing struct.
  *
  * Print the summary statistics for a given variable.
  *
  */
 
-void print_summary (GRETLSUMMARY *summ,
+void print_summary (const GRETLSUMMARY *summ,
 		    const DATAINFO *pdinfo,
-		    int pause, PRN *prn)
+		    PRN *prn)
 {
-    double xbar, std, xcv;
-    int lo = summ->list[0], v, lv, lineno = 4;
+    int pause = gretl_get_text_pause();
+    int i, vi, lineno;
     char tmp[128];
 
-    if (lo == 1) {
+    if (summ->list[0] == 1) {
 	print_summary_single(summ, pdinfo, prn);
 	return;
     }
 
     prhdr(_("Summary Statistics"), pdinfo, SUMMARY, prn);
-    strcpy(tmp, _("(missing values denoted by -999 will be skipped)"));
+    strcpy(tmp, _("(missing values will be skipped)"));
     center_line(tmp, prn, 1);
     pprintf(prn, "\n%s  ", _("Variable"));
 
     pputs(prn, _("      MEAN           MEDIAN           MIN"
             "             MAX\n\n"));
 
-    for (v=0; v<lo; v++) {
-	if (pause) page_break(1, &lineno, 0);
-	lineno++;
-	lv = summ->list[v+1];
-	pprintf(prn, "%-10s", pdinfo->varname[lv]);
-	xbar = summ->coeff[v];
-	printf15(xbar, prn);
-	printf15(summ->xmedian[v], prn);
-	printf15(summ->xpx[v], prn);
-	printf15(summ->xpy[v], prn);
+    lineno = 1;
+    for (i=0; i<summ->list[0]; i++) {
+	vi = summ->list[i + 1];
+	if (pause && (lineno % PAGELINES == 0)) {
+	    takenotes(0);
+	    lineno = 1;
+	}
+	pprintf(prn, "%-10s", pdinfo->varname[vi]);
+	printf15(summ->mean[i], prn);
+	printf15(summ->median[i], prn);
+	printf15(summ->low[i], prn);
+	printf15(summ->high[i], prn);
 	pputc(prn, '\n');
+	lineno++;
     }
-
-    if (pause) page_break(lo + 2, &lineno, 0);
-    lineno += 2;
     pputc(prn, '\n');
 
+    if (pause) {
+	takenotes(0);
+    }
+
     pprintf(prn, "\n%s  ", _("Variable"));
     pputs(prn, _("      S.D.            C.V.           "
 	 " SKEW          EXCSKURT\n\n"));
 
-    for (v=0; v<lo; v++) {
-	if (pause) page_break(1, &lineno, 0);
-	lineno++;
-	lv = summ->list[v+1];
-	pprintf(prn, "%-10s", pdinfo->varname[lv]);
+    lineno = 1;
+    for (i=0; i<summ->list[0]; i++) {
+	double cv;
 
-	xbar = summ->coeff[v];
-	std = summ->sderr[v];
-	if (xbar != 0.0) xcv = (xbar > 0)? std/xbar: (-1) * std/xbar;
-	else xcv = NADBL;
+	vi = summ->list[i + 1];
 
-	printf15(std, prn);
-	printf15(xcv, prn);
-	printf15(summ->xskew[v], prn);
-	printf15(summ->xkurt[v], prn);
+	if (pause && (lineno % PAGELINES == 0)) {
+	    takenotes(0);
+	    lineno = 1;
+	}
+
+	pprintf(prn, "%-10s", pdinfo->varname[vi]);
+
+	if (summ->mean[i] != 0.0 && !na(summ->sd[i])) {
+	    cv = fabs(summ->sd[i] / summ->mean[i]);
+	} else {
+	    cv = NADBL;
+	}
+
+	printf15(summ->sd[i], prn);
+	printf15(cv, prn);
+	printf15(summ->skew[i], prn);
+	printf15(summ->xkurt[i], prn);
 	pputc(prn, '\n');
+	lineno++;
     }
     pputc(prn, '\n');
 }
@@ -1177,17 +1319,62 @@ void print_summary (GRETLSUMMARY *summ,
 
 void free_summary (GRETLSUMMARY *summ)
 {
-    free(summ->xskew);
-    free(summ->xkurt);
-    free(summ->xmedian);
-    free(summ->coeff);
-    free(summ->sderr);
-    free(summ->xpx);
-    free(summ->xpy); 
     free(summ->list);
+
+    free(summ->mean);
+    free(summ->median);
+    free(summ->sd);
+    free(summ->skew);
+    free(summ->xkurt);
+    free(summ->low);
+    free(summ->high); 
+    free(summ->cv);
+
     free(summ);
 }
 
+static GRETLSUMMARY *summary_new (const int *list)
+{
+    GRETLSUMMARY *summ;
+    int nv = list[0];
+
+    summ = malloc(sizeof *summ);
+    if (summ == NULL) {
+	return NULL;
+    }
+
+    summ->list = copylist(list);
+    if (summ->list == NULL) {
+	free(summ);
+	return NULL;
+    }
+
+    summ->n = 0;
+
+    summ->mean = summ->median = summ->sd = NULL;
+    summ->skew = summ->xkurt = NULL;
+    summ->low = summ->high = summ->cv = NULL;
+
+    summ->mean = malloc(nv * sizeof *summ->mean);
+    summ->median = malloc(nv * sizeof *summ->median);
+    summ->sd = malloc(nv * sizeof *summ->sd);
+    summ->skew = malloc(nv * sizeof *summ->skew);
+    summ->xkurt = malloc(nv * sizeof *summ->xkurt);
+    summ->low = malloc(nv * sizeof *summ->low);
+    summ->high = malloc(nv * sizeof *summ->high);
+    summ->cv = malloc(nv * sizeof *summ->cv);
+
+    if (summ->mean == NULL || summ->median == NULL ||
+	summ->sd == NULL || summ->skew == NULL ||
+	summ->xkurt == NULL || summ->low == NULL ||
+	summ->high == NULL || summ->cv == NULL) {
+	free_summary(summ);
+	return NULL;
+    }
+
+    return summ;
+}
+
 /**
  * summary:
  * @list: list of variables to process.
@@ -1201,70 +1388,68 @@ void free_summary (GRETLSUMMARY *summ)
  *
  */
 
-GRETLSUMMARY *summary (LIST list, 
-		       double ***pZ, const DATAINFO *pdinfo,
+GRETLSUMMARY *summary (const int *list, 
+		       const double **Z, const DATAINFO *pdinfo,
 		       PRN *prn) 
 {
-    int lo = list[0];
-    int v, *tmp = NULL;
     GRETLSUMMARY *summ;
-    double xbar, std, low, high, skew, kurt, *x = NULL;
+    int i, vi, sn;
 
-    summ = malloc(sizeof *summ);
-    if (summ == NULL) return NULL;
-    summ->list = NULL;
-
-    if ((summ->xskew = malloc(lo * sizeof(double))) == NULL) return NULL;
-    if ((summ->xkurt = malloc(lo * sizeof(double))) == NULL) return NULL;
-    if ((summ->xmedian = malloc(lo * sizeof(double))) == NULL) return NULL;
-    if ((summ->coeff = malloc(lo * sizeof(double))) == NULL) return NULL;
-    if ((summ->sderr = malloc(lo * sizeof(double))) == NULL) return NULL;
-    if ((summ->xpx = malloc(lo * sizeof(double))) == NULL) return NULL;
-    if ((summ->xpy = malloc(lo * sizeof(double))) == NULL) return NULL;
-    if ((x = malloc((pdinfo->t2 - pdinfo->t1 + 1) * sizeof *x)) == NULL) 
+    summ = summary_new(list);
+    if (summ == NULL) {
 	return NULL;
+    }
 
-    for (v=0; v<lo; v++)  {
-	summ->n = ztox(list[v+1], x, *pZ, pdinfo);
-	if (summ->n < 2) { /* zero or one observations */
-	    if (summ->n == 0)
+    for (i=0; i<summ->list[0]; i++)  {
+	double x0;
+
+	vi = summ->list[i + 1];
+
+	sn = pdinfo->t2 - pdinfo->t1 + 1;
+	sn = good_obs(Z[vi] + pdinfo->t1, sn, &x0);
+
+	if (sn > summ->n) {
+	    summ->n = sn;
+	}
+
+	if (sn < 2) { 
+	    /* zero or one observations */
+	    if (summ->n == 0) {
 		pprintf(prn, _("Dropping %s: sample range contains no valid "
-			"observations\n"), pdinfo->varname[list[v+1]]);
-	    else
+			"observations\n"), pdinfo->varname[vi]);
+	    } else {
 		pprintf(prn, _("Dropping %s: sample range has only one "
-			"obs, namely %g\n"), pdinfo->varname[list[v+1]], x[0]);
-	    list_exclude(v+1, list);
-	    if (list[0] == 0) {
+			"obs, namely %g\n"), pdinfo->varname[vi], x0);
+	    }
+	    list_exclude(i + 1, summ->list);
+	    if (summ->list[0] == 0) {
 		free_summary(summ);
-		free(x);
 		return NULL;
 	    } else {
-		lo--;
-		v--;
+		i--;
 		continue;
 	    }
 	}
 
-	gretl_minmax(0, summ->n-1, x, &low, &high);	
-	moments(0, summ->n-1, x, &xbar, &std, &skew, &kurt, 1);
+	gretl_minmax(pdinfo->t1, pdinfo->t2, Z[vi], 
+		     &summ->low[i], 
+		     &summ->high[i]);
+	
+	moments(pdinfo->t1, pdinfo->t2, Z[vi], 
+		&summ->mean[i], 
+		&summ->sd[i], 
+		&summ->skew[i], 
+		&summ->xkurt[i], 1);
 
-	summ->xpx[v] = low;
-	summ->xpy[v] = high;
-	summ->coeff[v] = xbar;
-	summ->sderr[v] = std;
-	summ->xskew[v] = skew;
-	summ->xkurt[v] = kurt;
-
-	if (summ->n > 1) {
-	    summ->xmedian[v] = gretl_median(x, summ->n);
+	if (!floateq(summ->mean[i], 0.0)) {
+	    summ->cv[i] = fabs(summ->sd[i] / summ->mean[i]);
 	} else {
-	    summ->xmedian[v] = x[1];
+	    summ->cv[i] = NADBL;
 	}
-    } /* end loop over variables in list */
 
-    copylist(&tmp, list);
-    summ->list = tmp;
-    free(x);
+	summ->median[i] = gretl_median(Z[vi] + pdinfo->t1, 
+				       pdinfo->t2 - pdinfo->t1 + 1);
+    } 
 
     return summ;
 }
@@ -1299,7 +1484,7 @@ void free_corrmat (CORRMAT *corrmat)
  * 
  */
 
-CORRMAT *corrlist (LIST list, double ***pZ, const DATAINFO *pdinfo)
+CORRMAT *corrlist (int *list, const double **Z, const DATAINFO *pdinfo)
 {
     CORRMAT *corrmat;
     int *p = NULL;
@@ -1307,9 +1492,11 @@ CORRMAT *corrlist (LIST list, double ***pZ, const DATAINFO *pdinfo)
     int t1 = pdinfo->t1, t2 = pdinfo->t2; 
 
     corrmat = malloc(sizeof *corrmat);
-    if (corrmat == NULL) return NULL;
+    if (corrmat == NULL) {
+	return NULL;
+    }
 
-    copylist(&p, list);
+    p = copylist(list);
     if (p == NULL) {
 	free(corrmat);
 	return NULL;
@@ -1317,7 +1504,7 @@ CORRMAT *corrlist (LIST list, double ***pZ, const DATAINFO *pdinfo)
 
     /* drop any constants from list */
     for (i=1; i<=p[0]; i++) {
-	if (gretl_isconst(t1, t2, (*pZ)[p[i]])) {
+	if (gretl_isconst(t1, t2, Z[p[i]])) {
 	    list_exclude(i, p);
 	    i--;
 	}
@@ -1327,22 +1514,27 @@ CORRMAT *corrlist (LIST list, double ***pZ, const DATAINFO *pdinfo)
     lo = corrmat->list[0];  
     corrmat->n = t2 - t1 + 1;
     mm = (lo * (lo + 1))/2;
+
     corrmat->xpx = malloc(mm * sizeof(double));
     if (corrmat->xpx == NULL) {
 	free_corrmat(corrmat);
 	return NULL;
     }
-    for (i=1; i<=lo; i++) {   
+
+    for (i=1; i<=lo; i++) {  
+	int vi = corrmat->list[i];
+
 	for (j=i; j<=lo; j++)  {
-	    nij = ijton(i, j, lo);
+	    int vj = corrmat->list[j];
+
+	    nij = ijton(i-1, j-1, lo);
 	    if (i == j) {
 		corrmat->xpx[nij] = 1.0;
 		continue;
 	    }
-	    corrmat->xpx[nij] = 
-		gretl_corr(corrmat->n, 
-			   &(*pZ)[corrmat->list[i]][t1],
-			   &(*pZ)[corrmat->list[j]][t1]);
+	    corrmat->xpx[nij] = gretl_corr(corrmat->n, 
+					   Z[vi] + t1,
+					   Z[vj] + t1);
 	}
     }
 
@@ -1356,7 +1548,6 @@ CORRMAT *corrlist (LIST list, double ***pZ, const DATAINFO *pdinfo)
  * matrix_print_corr:
  * @corr: gretl correlation matrix
  * @pdinfo: data information struct.
- * @pause: = 1 to pause after showing each screen of info.
  * @prn: gretl printing struct.
  *
  * Prints a gretl correlation matrix.
@@ -1364,7 +1555,7 @@ CORRMAT *corrlist (LIST list, double ***pZ, const DATAINFO *pdinfo)
  */
 
 void matrix_print_corr (CORRMAT *corr, const DATAINFO *pdinfo,
-			int pause, PRN *prn)
+			PRN *prn)
 {
     char tmp[96];
 
@@ -1372,15 +1563,14 @@ void matrix_print_corr (CORRMAT *corr, const DATAINFO *pdinfo,
     sprintf(tmp, _("5%% critical value (two-tailed) = "
 	    "%.4f for n = %d"), rhocrit95(corr->n), corr->n);
     center_line(tmp, prn, 1);
-    text_print_matrix(corr->xpx, corr->list, NULL, pdinfo, pause, prn);
+    text_print_matrix(corr->xpx, corr->list, NULL, pdinfo, prn);
 }
 
 /**
- * esl_corrmx:
+ * gretl_corrmx:
  * @list: gives the ID numbers of the variables to process.
  * @pZ: pointer to the data matrix.
  * @pdinfo: data information struct.
- * @pause: if non-zero, pause after showing each screen of info.
  * @prn: gretl printing struct.
  *
  * Computes and prints the correlation matrix for the specified list
@@ -1389,15 +1579,19 @@ void matrix_print_corr (CORRMAT *corr, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int esl_corrmx (LIST list, double ***pZ, const DATAINFO *pdinfo, 
-		int pause, PRN *prn)
+int gretl_corrmx (int *list, const double **Z, const DATAINFO *pdinfo, 
+		  PRN *prn)
 {
     CORRMAT *corr;
 
-    corr = corrlist(list, pZ, pdinfo);
-    if (corr == NULL) return 1;
-    matrix_print_corr(corr, pdinfo, pause, prn);
+    corr = corrlist(list, Z, pdinfo);
+    if (corr == NULL) {
+	return 1;
+    }
+
+    matrix_print_corr(corr, pdinfo, prn);
     free_corrmat(corr);
+
     return 0;
 }
 
@@ -1415,8 +1609,8 @@ int esl_corrmx (LIST list, double ***pZ, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion, error code on error.
  */
 
-int means_test (LIST list, double **Z, const DATAINFO *pdinfo, 
-		unsigned long vardiff, PRN *prn)
+int means_test (const int *list, const double **Z, const DATAINFO *pdinfo, 
+		gretlopt vardiff, PRN *prn)
 {
     double m1, m2, s1, s2, skew, kurt, se, mdiff, t, pval;
     double *x = NULL, *y = NULL;
@@ -1492,7 +1686,7 @@ int means_test (LIST list, double **Z, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion, error code on error.
  */
 
-int vars_test (LIST list, double **Z, const DATAINFO *pdinfo, 
+int vars_test (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	       PRN *prn)
 {
     double m, skew, kurt, s1, s2, var1, var2, F;
