@@ -23,13 +23,15 @@
 #define INDEXNUM 996
 
 int generate (double ***pZ, DATAINFO *pdinfo, 
-	      const char *line, int model_count, 
-	      MODEL *pmod, unsigned long oflag);
+	      const char *line, MODEL *pmod); 
+
+int genrtime (double ***pZ, DATAINFO *pdinfo, int tm);
 
 int dummy (double ***pZ, DATAINFO *pdinfo);
 
-int paneldum (double ***pZ, DATAINFO *pdinfo, 
-	      unsigned long opt);
+int paneldum (double ***pZ, DATAINFO *pdinfo); 
+
+int panel_unit_dummies (double ***pZ, DATAINFO *pdinfo);
 
 int panel_unit_first_obs (int t, const DATAINFO *pdinfo);
 
@@ -40,27 +42,13 @@ void varlist (const DATAINFO *pdinfo, PRN *prn);
 
 int varindex (const DATAINFO *pdinfo, const char *varname);
 
-int logs (const LIST list, 
-	  double ***pZ, DATAINFO *pdinfo);
-
-int lags (const LIST list, 
-	  double ***pZ, DATAINFO *pdinfo);
-
-int xpxgenr (const LIST list, 
-	     double ***pZ, DATAINFO *pdinfo, 
-	     const int opt, const int nodup);
-
-int rhodiff (char *param, const LIST list, 
-	     double ***pZ, DATAINFO *pdinfo);
-
-int simulate (char *cmd, 
-	      double ***pZ, DATAINFO *pdinfo); 
-
-int genr_fit_resid (MODEL *pmod, 
+int genr_fit_resid (const MODEL *pmod, 
 		    double ***pZ, DATAINFO *pdinfo,
 		    int code, int undo);
 
-int genr_scalar_index (int opt, int put);
+int get_generated_value (const char *argv, double *val,
+			 double ***pZ, DATAINFO *pdinfo,
+			 MODEL *pmod, int t);
 
 #endif /* GENERATE_H */
 
