@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
+ *  Copyright (c) by Allin Cottrell
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -31,9 +31,10 @@ void dprintf (const char *format, ...);
 
 #define ATOMLEN 32  /* length of auxiliary string in genr atom */
 
-typedef struct _genatom genatom;
+typedef struct genatom_ genatom;
+typedef struct atomset_ atomset;
 
-struct _genatom {
+struct genatom_ {
     char level;
     char scalar;
     int varnum;
@@ -45,6 +46,7 @@ struct _genatom {
     char popped;
     char str[ATOMLEN];
     genatom *parent;
+    atomset *aset;
 };
 
 /* below: matches funcs[] in generate.c */
@@ -75,6 +77,7 @@ enum transformations {
     T_STDERR,
     T_CUM, 
     T_MISSING,
+    T_OK,
     T_MISSZERO,
     T_CORR,
     T_VCV,
@@ -84,6 +87,7 @@ enum transformations {
     T_MEDIAN,
     T_ZEROMISS,
     T_PVALUE,
+    T_CRIT,
     T_OBSNUM,
     T_MPOW,
     T_DNORM,
@@ -96,20 +100,50 @@ enum transformations {
     T_IDENTITY
 };
 
+#define VALSTACK_SIZE 32
+
+enum genr_flags {
+    GENR_SAVE   = 1 << 0,
+    GENR_SCALAR = 1 << 1,
+    GENR_LOCAL  = 1 << 2,
+    GENR_WARN   = 1 << 3
+};
+
+typedef struct _GENERATE GENERATE;
+
+struct _GENERATE {
+    int err;
+    unsigned char flags;
+    double *xvec;
+    int varnum;
+    int obs;
+    char varname[VNAMELEN];
+    char label[MAXLABEL];
+    int tmpv;
+    double **tmpZ;
+    DATAINFO *pdinfo;
+    double ***pZ;
+    MODEL *pmod;
+    atomset *aset;
+    double valstack[VALSTACK_SIZE];
+    int nvals;
+};
+
+int attach_atomset (GENERATE *genr);
 int push_atom (genatom *atom);
-genatom *pop_atom (void);
+genatom *pop_atom (GENERATE *genr);
 genatom *pop_child_atom (genatom *atom);
 genatom *peek_child_atom (genatom *atom);
-void reset_atom_stack (void);
-void destroy_atom_stack (void);
-void atom_stack_set_parentage (void);
+void reset_atom_stack (GENERATE *genr);
+void destroy_atom_stack (GENERATE *genr);
+void atom_stack_set_parentage (GENERATE *genr);
 void atom_eat_children (genatom *atom);
-void atom_stack_bookmark (void);
-void atom_stack_resume (void);
-int atom_stack_check_for_scalar (void);
+void atom_stack_bookmark (GENERATE *genr);
+void atom_stack_resume (GENERATE *genr);
+int atom_stack_check_for_scalar (GENERATE *genr);
 
-int calc_push (double x);
-double calc_pop (void);
-void reset_calc_stack (void);
+int calc_push (double x, GENERATE *genr);
+double calc_pop (GENERATE *genr);
+void reset_calc_stack (GENERATE *genr);
 
 #endif /* GENSTACK_H */
