@@ -39,7 +39,8 @@ enum gptspec_flags {
     GPTSPEC_Y2AXIS        = 1 << 1,
     GPTSPEC_AUTO_OLS      = 1 << 2,
     GPTSPEC_OLS_HIDDEN    = 1 << 3,
-    GPTSPEC_BORDER_HIDDEN = 1 << 4
+    GPTSPEC_BORDER_HIDDEN = 1 << 4,
+    GPTSPEC_PNG_OUTPUT    = 1 << 5
 }; 
 
 #define MAXTITLE 128
@@ -67,7 +68,6 @@ typedef struct {
 typedef struct {
     FILE *fp;
     char fname[MAXLEN];        /* for gui purposes */
-    int edit;                  /* 1 for editing existing plot */
     int code;                  /* to deal with FREQ, FCASTERR... */
     unsigned char flags;       /* bitwise OR of options (gptspec_flags) */
     int t1, t2;                /* starting and ending obs */
@@ -109,65 +109,61 @@ enum plot_type_codes {
     PLOT_LEVERAGE,
     PLOT_TYPE_MAX
 };
+
+#define frequency_plot_code(c) (c == PLOT_FREQ_SIMPLE || \
+				c == PLOT_FREQ_NORMAL || \
+				c == PLOT_FREQ_GAMMA)
+
+#define set_png_output(p) (p->flags |= GPTSPEC_PNG_OUTPUT)
+#define get_png_output(p) (p->flags & GPTSPEC_PNG_OUTPUT) 
     
 /* functions follow */
  
-int plot (const LIST list, 
-	  double **Z, const DATAINFO *pdinfo, 
-	  unsigned long oflag, int pause, PRN *prn);
+const char *get_gretl_png_term_line (int plottype);
 
-int graph (const LIST list, 
-	   double **Z, const DATAINFO *pdinfo, 
-	   unsigned long oflag, PRN *prn);
-
-const char *get_gretl_png_term_line (const PATHS *ppaths, int plottype);
+const char *get_gretl_emf_term_line (int plottype, int color);
 
 const char *get_timevar_name (DATAINFO *pdinfo);
 
-int gnuplot_init (PATHS *ppaths, int plottype, FILE **fpp);
+int gnuplot_init (int plottype, FILE **fpp);
 
-int gnuplot_display (const PATHS *ppaths);
+int gnuplot_make_graph (void);
 
-int gnuplot (LIST list, const int *lines, const char *literal,
-	     double ***pZ, DATAINFO *pdinfo, PATHS *ppaths, 
+int gnuplot (int *list, const int *lines, const char *literal,
+	     double ***pZ, DATAINFO *pdinfo, 
 	     int *plot_count, unsigned char flags);
 
-int multi_scatters (const LIST list, int pos, 
+int multi_scatters (const int *list, int pos, 
 		    double ***pZ, const DATAINFO *pdinfo, 
-		    PATHS *ppaths, int *plot_count, 
-		    unsigned char flags);
+		    int *plot_count, unsigned char flags);
 
-int gnuplot_3d (LIST list, const char *literal,
-		double ***pZ, DATAINFO *pdinfo, PATHS *ppaths, 
+int gnuplot_3d (int *list, const char *literal,
+		double ***pZ, DATAINFO *pdinfo, 
 		int *plot_count, unsigned char flags);
 
-int plot_freq (FREQDIST *freq, PATHS *ppaths, int dist);
+int plot_freq (FREQDIST *freq, int dist);
 
-int garch_resid_plot (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
-		      PATHS *ppaths);
+int garch_resid_plot (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo); 
 
 int print_plotspec_details (const GPT_SPEC *spec, FILE *fp);
 
-int go_gnuplot (GPT_SPEC *spec, char *fname, PATHS *ppaths);
+int go_gnuplot (GPT_SPEC *spec, char *fname);
 
 void free_plotspec (GPT_SPEC *spec);
 
-int termtype_to_termstr (const char *termtype, char *termstr,
-			 const PATHS *ppaths);
+int get_termstr (const GPT_SPEC *spec, char *termstr);
 
-int rmplot (const LIST list, double **Z, DATAINFO *pdinfo, PRN *prn,
-	    PATHS *ppaths);
+int rmplot (const int *list, double **Z, DATAINFO *pdinfo, PRN *prn);
 
 int plot_fcast_errs (int n, const double *obs, 
 		     const double *depvar, const double *yhat, 
 		     const double *maxerr, const char *varname, 
-		     int time_series, PATHS *ppaths);
+		     int time_series);
 
 int 
 gretl_var_plot_impulse_response (GRETL_VAR *var,
 				 int targ, int shock, int periods,
-				 const DATAINFO *pdinfo,
-				 PATHS *ppaths);
+				 const DATAINFO *pdinfo);
 
 int is_auto_ols_string (const char *s);
 
@@ -181,3 +177,11 @@ const char *get_gnuplot_pallette (int i, int plottype);
 
 int gnuplot_test_command (const char *cmd);
 
+#ifdef ENABLE_NLS
+
+void pprint_gnuplot_encoding (const char *termstr, PRN *prn);
+
+void fprint_gnuplot_encoding (const char *termstr, FILE *fp);
+
+#endif
+
