@@ -25,6 +25,7 @@ enum gretl_cmd_codes {
     ADD,
     ADDTO,
     ADF,
+    APPEND,
     AR,      
     ARCH,
     ARMA,
@@ -52,6 +53,7 @@ enum gretl_cmd_codes {
     FCASTERR, 
     FIT,     
     FREQ, 
+    FUNC,
     GARCH,
     GENR,     
     GNUPLOT, 
@@ -64,6 +66,7 @@ enum gretl_cmd_codes {
     IF,
     IMPORT,
     INFO,
+    KPSS,
     LABEL,
     LABELS, 
     LAD,
@@ -99,6 +102,7 @@ enum gretl_cmd_codes {
     QUIT, 
     RENAME,
     RESET,
+    RESTRICT,
     RHODIFF, 
     RMPLOT,
     RUN,
@@ -122,6 +126,7 @@ enum gretl_cmd_codes {
     VAR,
     VARLIST,
     VARTEST,
+    VIF,
     WLS,
     NC
 };
@@ -151,6 +156,7 @@ enum gretl_cmd_codes {
                         c == FREQ || \
                         c == GRAPH || \
                         c == HAUSMAN || \
+                        c == KPSS || \
                         c == LEVERAGE || \
                         c == LMTEST || \
                         c == MEANTEST || \
@@ -168,6 +174,7 @@ enum gretl_cmd_codes {
                         c == RUNS || \
                         c == SPEARMAN || \
                         c == TESTUHAT || \
+                        c == VIF || \
                         c == VARTEST)
 
 #define LIMDEP(c) (c == LOGIT || c == PROBIT || c == TOBIT)
