@@ -79,8 +79,9 @@ enum gretl_error_codes {
     E_NOMERGE,     /* 52 */
     E_NOCONV,      /* 53 */
     E_CANCEL,      /* 54 */
-    E_NAN,         /* 55 */
-    E_MAX          /* 56 */
+    E_MISSDATA,    /* 55 */
+    E_NAN,         /* 56 */
+    E_MAX          /* 57 */
 }; 
 
 /* functions follow */
@@ -91,8 +92,12 @@ void errmsg (const int errcode, PRN *prn);
 
 int get_gretl_errno (void);
 
-char *get_gretl_errmsg (void);
+const char *get_gretl_errmsg (void);
 
-char *get_gretl_msg (void);
+const char *get_gretl_msg (void);
+
+int print_gretl_msg (PRN *prn);
+
+int print_gretl_errmsg (PRN *prn);
 
 void gretl_errmsg_set (const char *str);
