@@ -30,6 +30,7 @@ enum gretl_matrix_errors {
     GRETL_MATRIX_NOMEM,
     GRETL_MATRIX_NON_CONFORM,
     GRETL_MATRIX_RANGE,
+    GRETL_MATRIX_SINGULAR,
     GRETL_MATRIX_ERR 
 };
 
@@ -43,24 +44,27 @@ typedef struct _gretl_matrix gretl_matrix;
 typedef struct _gretl_matrix gretl_vector;
 
 #define gretl_vector_alloc(i) gretl_matrix_alloc(1,(i))
+#define gretl_column_vector_alloc(i) gretl_matrix_alloc((i),1)
 #define gretl_vector_free(v) gretl_matrix_free(v)
-#define gretl_vector_get(v,i) gretl_matrix_get((v),0,(i))
-#define gretl_vector_set(v,i,x) gretl_matrix_set((v),0,(i),(x))
-
 
 gretl_matrix *gretl_matrix_alloc (int rows, int cols);
 
 gretl_matrix *gretl_packed_matrix_alloc (int rows);
 
-gretl_matrix *gretl_matrix_copy (gretl_matrix *m);
+gretl_matrix *gretl_matrix_copy (const gretl_matrix *m);
 
-gretl_matrix *gretl_diagonal_matrix (const double *d, int n, int mod);
+gretl_vector *gretl_column_vector_from_array (const double *x, 
+					      int n, int mod);
 
 gretl_matrix *gretl_matrix_from_2d_array (const double **X, 
 					  int rows, int cols);
 
 void gretl_matrix_zero (gretl_matrix *m);
 
+int gretl_matrix_zero_upper (gretl_matrix *m);
+
+int gretl_matrix_zero_lower (gretl_matrix *m);
+
 void gretl_matrix_multiply_by_scalar (gretl_matrix *m, double x);
 
 void gretl_matrix_divide_by_scalar (gretl_matrix *m, double x);
@@ -74,10 +78,19 @@ int gretl_matrix_copy_values (gretl_matrix *targ,
 
 double gretl_matrix_get (const gretl_matrix *m, int i, int j);
 
+double gretl_vector_get (const gretl_vector *v, int i);
+
 int gretl_matrix_set (gretl_matrix *m, int i, int j, double x);
 
+int gretl_vector_set (gretl_vector *v, int i, double x);
+
 int gretl_matrix_add_to (gretl_matrix *targ, const gretl_matrix *src);
 
+int 
+gretl_matrix_subtract_from (gretl_matrix *targ, const gretl_matrix *src);
+
+int gretl_square_matrix_transpose (gretl_matrix *m);
+
 int gretl_matrix_add_self_transpose (gretl_matrix *m);
 
 int gretl_matrix_multiply_mod (const gretl_matrix *a, int aflag,
@@ -93,11 +106,13 @@ double gretl_matrix_dot_product (const gretl_matrix *a, int aflag,
 
 gretl_matrix *gretl_matrix_vcv (gretl_matrix *m);
 
+double gretl_LU_determinant (gretl_matrix *a);
+
 int gretl_LU_solve (gretl_matrix *a, gretl_vector *b);
 
-int gretl_invert_general_matrix (gretl_matrix *m);
+int gretl_invert_general_matrix (gretl_matrix *a);
 
-int gretl_invert_symmetric_matrix (gretl_matrix *m);
+int gretl_invert_symmetric_matrix (gretl_matrix *a);
 
 double *gretl_general_matrix_eigenvals (gretl_matrix *m, gretl_matrix *ev);
 
@@ -106,7 +121,24 @@ double *gretl_symmetric_matrix_eigenvals (gretl_matrix *m,
 
 int gretl_matrix_cholesky_decomp (gretl_matrix *a);
 
-void gretl_matrix_print (gretl_matrix *m, const char *msg, PRN *prn);
+int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
+		      gretl_vector *b, gretl_matrix *vcv,
+		      gretl_vector *uhat);
+
+double gretl_scalar_b_prime_X_b (const gretl_vector *b, const gretl_matrix *X,
+				 int *err);
+
+gretl_matrix *
+gretl_matrix_A_X_A_prime (const gretl_matrix *A, const gretl_matrix *X,
+			  int *err);
+
+gretl_matrix *
+gretl_vcv_matrix_from_model (MODEL *pmod, const char *select);
+
+gretl_vector *
+gretl_coeff_vector_from_model (const MODEL *pmod, const char *select);
+
+void gretl_matrix_print (const gretl_matrix *m, const char *msg, PRN *prn);
 
 void gretl_matrix_set_int (gretl_matrix *m, int t);
 
@@ -118,4 +150,8 @@ int gretl_matrix_cols (const gretl_matrix *m);
 
 int gretl_matrix_rows (const gretl_matrix *m);
 
+int gretl_is_identity_matrix (const gretl_matrix *m);
+
+int gretl_is_zero_vector (const gretl_vector *v);
+
 #endif /* GRETL_MATRIX_H */
