@@ -28,20 +28,24 @@
  */
 
 #define free_model(p) if (p != NULL) { \
-                             clear_model(p, NULL); \
+                             clear_model(p); \
                              free(p); \
                           }
 
+#define AR_MODEL(c) (c == AR || c == CORC || c == PWE || c == HILU)
 
-MODEL *gretl_model_new (const DATAINFO *pdinfo);
 
-void gretl_model_init (MODEL *pmod, const DATAINFO *pdinfo);
+MODEL *gretl_model_new (void);
+
+void gretl_model_init (MODEL *pmod);
+
+void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo);
 
 void gretl_model_set_auxiliary (MODEL *pmod, int aux);
 
 void exchange_smpl (MODEL *pmod, DATAINFO *pdinfo);
 
-void clear_model (MODEL *pmod, const DATAINFO *pdinfo);
+void clear_model (MODEL *pmod);
 
 int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size);
 
@@ -61,4 +65,21 @@ int copy_model (MODEL *targ, const MODEL *src, const DATAINFO *pdinfo);
 
 int swap_models (MODEL **targ, MODEL **src);
 
+int command_ok_for_model (int test_ci, int model_ci);
+
+int get_model_count (void);
+
+void reset_model_count (void);
+
+int model_count_plus (void);
+
+void model_count_minus (void);
+
+void set_model_id (MODEL *pmod);
+
+void model_list_to_string (int *list, char *buf);
+
+int highest_numbered_var_in_model (const MODEL *pmod, 
+				   const DATAINFO *pdinfo);
+
 #endif /* GRETL_MODEL_H */
