@@ -1,4 +1,4 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
+/*
  * Copyright (C) 1999-2004 Ramu Ramanathan and Allin Cottrell
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +20,7 @@
 #ifndef GRETL_UTILS_H
 #define GRETL_UTILS_H
 
-#include <stdio.h>
+#include "libgretl.h"
 
 enum model_selection_criteria {
     C_SGMASQ = 0,
@@ -41,7 +41,7 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains time series
  * data (1) or not (0).
  */
-#define dataset_is_time_series(p) (p->time_series == TIME_SERIES)
+#define dataset_is_time_series(p) ((p)->time_series == TIME_SERIES)
 
 /**
  * dataset_is_daily:
@@ -51,7 +51,27 @@ enum model_selection_criteria {
  * data (1) or not (0).
  */
 #define dataset_is_daily(p) (p->time_series == TIME_SERIES \
-                             && (p->pd == 5 || p->pd == 7))
+                             && (p->pd == 5 || p->pd == 6 || p->pd == 7))
+
+/**
+ * dataset_is_weekly:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set contains weekly time series
+ * data (1) or not (0).
+ */
+#define dataset_is_weekly(p) (p->time_series == TIME_SERIES \
+                              && p->pd == 52)
+
+/**
+ * dataset_is_hourly:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set contains hourly time series
+ * data (1) or not (0).
+ */
+#define dataset_is_hourly(p) (p->time_series == TIME_SERIES \
+                              && p->pd == 24)
 
 /**
  * dated_daily_data:
@@ -61,8 +81,19 @@ enum model_selection_criteria {
  * data (1) or not (0).
  */
 #define dated_daily_data(p) (p->time_series == TIME_SERIES \
-                             && (p->pd == 5 || p->pd == 7) && \
-                             p->sd0 > 10000.0)
+                             && (p->pd == 5 || p->pd == 6 || p->pd == 7) \
+                             && p->sd0 > 10000.0)
+
+/**
+ * dated_seven_day_data:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set contains dated daily 
+ * (seven-day) time series data (1) or not (0).
+ */
+#define dated_seven_day_data(p) (p->time_series == TIME_SERIES \
+                                 && p->pd == 7 && \
+                                 p->sd0 > 10000.0)
 
 /**
  * dataset_is_panel:
@@ -71,25 +102,25 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains panel
  * data (1) or not (0).
  */
-#define dataset_is_panel(p) (p->time_series == STACKED_TIME_SERIES \
-                            || p->time_series == STACKED_CROSS_SECTION)
-
+#define dataset_is_panel(p) ((p)->time_series == STACKED_TIME_SERIES \
+                            || (p)->time_series == STACKED_CROSS_SECTION)
 
 #include <float.h>
+
 #define floateq(x, y) (fabs((x) - (y)) < DBL_EPSILON)
 #define floatneq(x, y) (fabs((x) - (y)) > DBL_EPSILON)
 #define floatgt(x, y) ((x) - (y) > DBL_EPSILON)
 #define floatlt(x, y) ((y) - (x) > DBL_EPSILON)
-#define na(x) (fabs((x) + 999.0) < DBL_EPSILON)
-#define NADBL -999.0
-
-#define want_vcv(o) (o & OPT_O)
 
 /* functions follow */
+
+void libgretl_init (CMD *cmd);
+
+void libgretl_cleanup (CMD *cmd);
  
 double date (int nt, int pd, const double sd0);
 
-int isdummy (double *x, int t1, int t2);
+int isdummy (const double *x, int t1, int t2);
 
 void printlist (const int *list, const char *msg);
 
@@ -97,12 +128,11 @@ int print_list_to_buffer (const int *list, char *buf, size_t len);
 
 void list_exclude (int n, int *list);
 
-int set_obs (const char *line, DATAINFO *pdinfo, unsigned long opt);
+char *format_obs (char *obs, int maj, int min, int pd);
 
-void set_miss (LIST list, const char *param, double **Z,
-	       DATAINFO *pdinfo, PRN *prn);
+int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt);
 
-int copylist (int **target, const int *src);
+int *copylist (const int *src);
 
 int grow_nobs (int newobs, double ***pZ, DATAINFO *pdinfo);
 
@@ -132,13 +162,13 @@ FITRESID *get_fcast_with_errs (const char *str, const MODEL *pmod,
 
 int fcast_with_errs (const char *str, const MODEL *pmod, 
 		     double ***pZ, DATAINFO *pdinfo, PRN *prn,
-		     PATHS *ppaths, int plot);
+		     int plot);
 
 int is_model_cmd (const char *line);
 
 int is_model_ref_cmd (int ci);
 
-int save_model_spec (MODEL *pmod, MODELSPEC *spec, DATAINFO *fullinfo);
+int is_quiet_model_test (int ci, gretlopt opt);
 
 int re_estimate (char *model_spec, MODEL *tmpmod, 
 		 double ***pZ, DATAINFO *pdinfo);
@@ -147,16 +177,15 @@ double *copyvec (const double *src, int n);
 
 int ijton (int i, int j, int nrows);
 
-int ztox (int i, double *px, 
-	  double **Z, const DATAINFO *pdinfo);
+int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo);
 
 int get_panel_structure (DATAINFO *pdinfo, int *nunits, int *T);
 
-int set_panel_structure (unsigned char flag, DATAINFO *pdinfo, PRN *prn); 
+int set_panel_structure (gretlopt opt, DATAINFO *pdinfo, PRN *prn); 
 
 int balanced_panel (const DATAINFO *pdinfo);
 
-double get_xvalue (int i, double **Z, const DATAINFO *pdinfo);
+double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo);
 
 void free_gretl_mp_results (mp_results *mpvals);
 
@@ -172,6 +201,12 @@ CONFINT *get_model_confints (const MODEL *pmod);
 
 void free_confint (CONFINT *cf);
 
+double gretl_stddev (int t1, int t2, const double *x);
+
+double gretl_variance (int t1, int t2, const double *x);
+
+double gretl_sst (int t1, int t2, const double *x);
+
 #ifndef WIN32
 int gretl_spawn (const char *cmdline);
 int gretl_spawn_quiet (const char *cmdline);
