@@ -21,7 +21,9 @@
 
 #include "libgretl.h"
 #include "var.h"
+#include "gretl_func.h"
 #include "gretl_private.h"
+#include "compat.h"
 
 /* equipment for the "shell" command */
 #ifndef WIN32
@@ -38,7 +40,8 @@ static int make_full_list (const DATAINFO *pdinfo, CMD *command);
 static void get_optional_filename (const char *line, CMD *cmd);
 
 typedef struct {
-    int lag;
+    int firstlag;
+    int lastlag;
     int varnum;
     char varname[VNAMELEN];
 } LAGVAR;
@@ -51,20 +54,26 @@ static int trydatafile (char *line, int *ignore)
     char datfile[MAXLEN];
 
     *datfile = '\0';
+
     for (i=0; i<n; i++) {
 	if ((n - i) > 4 && strncmp(line+i, "DATA", 4) == 0) {
 	    sscanf(line + i, "%s", datfile);
 	    m = strlen(datfile);
-	    if (datfile[m-1] == ',') datfile[m-1] = '\0';
+	    if (datfile[m-1] == ',') {
+		datfile[m-1] = '\0';
+	    }
 	    lower(datfile);
 	    i += 4;
+	} else if (line[i] == '*' && line[i+1] == ')') {
+	    *ignore = 0;
 	}
-	else if (line[i] == '*' && line[i+1] == ')') *ignore = 0;
     }
+
     if (*datfile) {
 	sprintf(line, "open %s.gdt", datfile);
 	return 1;
     } 
+
     return 0;
 }
 
@@ -75,7 +84,9 @@ static int filter_comments (char *line, int *ignore)
     int i, j = 0, n = strlen(line);
     char tmpstr[MAXLEN], datfile[MAXLEN];
 
-    if (n >= MAXLEN) return 0;
+    if (n >= MAXLEN) {
+	return 0;
+    }
     
     for (i=0; i<n; i++) {
 	if (line[i] == '(' && line [i+1] == '*') {
@@ -96,6 +107,7 @@ static int filter_comments (char *line, int *ignore)
 	    j++;
 	}
     }
+
     tmpstr[j] = '\0';
     strcpy(line, tmpstr);
 
@@ -109,10 +121,14 @@ static int get_rhodiff_param (char *str, CMD *cmd)
 {
     int k;
 
-    if ((k = haschar(';', str)) < 0) return 1;
+    if ((k = haschar(';', str)) < 0) {
+	return 1;
+    }
 
     cmd->param = realloc(cmd->param, k+1);
-    if (cmd->param == NULL) return E_ALLOC;
+    if (cmd->param == NULL) {
+	return E_ALLOC;
+    }
 
     *cmd->param = 0;
     strncat(cmd->param, str, k);
@@ -124,9 +140,23 @@ static int get_rhodiff_param (char *str, CMD *cmd)
 
 /* ........................................................... */
 
-int subsetted_command (const char *cmd)
+static int restrict_bnum (const char *s)
+{
+    if (*s == 'b' && isdigit((unsigned char) *(s+1))) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
+static int subsetted_command (const char *cmd)
 {    
-    if (strcmp(cmd, "deriv") == 0) return NLS;
+    if (strcmp(cmd, "deriv") == 0)    return NLS;
+    if (strcmp(cmd, "identity") == 0) return SYSTEM;
+    if (strcmp(cmd, "endog") == 0)    return SYSTEM;
+    if (strcmp(cmd, "instr") == 0)    return SYSTEM;
+    if (restrict_bnum(cmd))           return RESTRICT;
+
     return 0;
 }
 
@@ -199,19 +229,23 @@ static int aliased (char *cmd)
 	               c == PANEL || \
 	               c == OPEN || \
 	               c == IMPORT || \
+                       c == APPEND || \
 	               c == ENDLOOP || \
 	               c == SIM || \
                        c == RENAME || \
 	               c == TESTUHAT || \
                        c == RESET || \
+                       c == RESTRICT || \
                        c == SYSTEM || \
                        c == LEVERAGE || \
+                       c == VIF || \
                        c == MODELTAB || \
                        c == NLS || \
                        c == DATA || \
 	               c == GENR || \
                        c == SET || \
                        c == PRINTF || \
+                       c == FUNC || \
                        c == OUTFILE)
 
 /* ........................................................... */
@@ -221,21 +255,25 @@ static int flow_control (const char *line, double ***pZ,
 {
     /* clear to proceed */
     if (!ifstate(IS_FALSE) && 
-	cmd->ci != IF && cmd->ci != ELSE && cmd->ci != ENDIF)
+	cmd->ci != IF && cmd->ci != ELSE && cmd->ci != ENDIF) {
 	return 0;
+    }
 
     if (cmd->ci == IF) {
 	int ret = if_eval(line, pZ, pdinfo);
 
-	if (ret == -1 || ifstate((ret)? SET_TRUE : SET_FALSE)) 
+	if (ret == -1 || ifstate((ret)? SET_TRUE : SET_FALSE)) { 
 	    cmd->errcode = E_SYNTAX;
+	}
     }
 
-    else if (cmd->ci == ELSE && ifstate(SET_ELSE)) 
+    else if (cmd->ci == ELSE && ifstate(SET_ELSE)) {
 	cmd->errcode = E_SYNTAX;
+    }
 
-    else if (cmd->ci == ENDIF && ifstate(SET_ENDIF)) 
+    else if (cmd->ci == ENDIF && ifstate(SET_ENDIF)) {
 	cmd->errcode = E_SYNTAX;
+    }
 
     return 1;
 }
@@ -249,11 +287,15 @@ static void get_savename (char *s, CMD *cmd)
 
 	quote = (*s == '"');
 	len = strcspn(s, "<");
-	if (len < 2) return;
+	if (len < 2) {
+	    return;
+	}
 	n = len - 1 - quote;
 	if (n > MAXSAVENAME - 1) n = MAXSAVENAME - 1;
 	strncat(cmd->savename, s + quote, n);
-	if (cmd->savename[n-1] == '"') cmd->savename[n-1] = 0;
+	if (cmd->savename[n-1] == '"') {
+	    cmd->savename[n-1] = 0;
+	}
 	shift_left(s, len + 3);
     }
 }
@@ -274,24 +316,159 @@ static void grab_gnuplot_literal_block (char *line, CMD *command)
     }
 }
 
-static int parse_lagvar (const char *s, LAGVAR *plagv, DATAINFO *pdinfo)
+static int parse_lagvar (const char *s, LAGVAR *plagv, 
+			 const DATAINFO *pdinfo)
 {
-    int ret = 0;
+    int ret = 1;
 
     *plagv->varname = 0;
-    plagv->lag = 0;
+    plagv->firstlag = 0;
+    plagv->lastlag = 0;
     plagv->varnum = 0;
 
-    if (sscanf(s, "%8[^(](-%d)", plagv->varname, &plagv->lag) == 2) {
+    if (sscanf(s, "%8[^(](-%d to -%d)", plagv->varname, 
+	       &plagv->firstlag, &plagv->lastlag) == 3) {
 	plagv->varnum = varindex(pdinfo, plagv->varname);
 	if (plagv->varnum < pdinfo->v) {
-	    ret = 1;
+	    ret = 0;
+	} 
+    } else if (sscanf(s, "%8[^(](-%d)", plagv->varname, 
+		      &plagv->firstlag) == 2) {
+	plagv->varnum = varindex(pdinfo, plagv->varname);
+	if (plagv->varnum < pdinfo->v) {
+	    plagv->lastlag = plagv->firstlag;
+	    ret = 0;
 	} 
     }
 
     return ret;
 }
 
+static int expand_command_list (CMD *cmd, int nadd)
+{
+    int oldn = cmd->list[0];
+    int *list;
+
+    list = realloc(cmd->list, (oldn + nadd) * sizeof *list);
+    if (list == NULL) {
+	cmd->errcode = E_ALLOC;
+	strcpy (gretl_errmsg, 
+		_("Memory allocation failed for command list"));
+	return 1;
+    }
+
+    /* one of the vars was "already assumed" */
+    list[0] += (nadd - 1);
+    
+    cmd->list = list;
+
+    return 0;
+}
+
+int auto_lag_ok (const char *s, int *lnum,
+		 double ***pZ, DATAINFO *pdinfo,
+		 CMD *cmd, PRN *prn)
+{
+    LAGVAR lagvar;
+    int nlags, i;
+    int llen = *lnum;
+    int ok = 1;
+	
+    if (parse_lagvar(s, &lagvar, pdinfo)) return 0;
+
+    nlags = lagvar.lastlag - lagvar.firstlag + 1;
+    if (nlags <= 0) {
+	cmd->errcode = E_PARSE;
+	return 0;
+    }
+
+    if (nlags > 1 && expand_command_list(cmd, nlags)) {
+	return 0;
+    }
+
+    for (i=0; i<nlags && ok; i++) {
+	int laglen = lagvar.firstlag + i;
+	int vnum;
+
+	vnum = laggenr(lagvar.varnum, laglen, pZ, pdinfo);
+
+	if (vnum < 0) {
+	    cmd->errcode = 1;
+	    sprintf(gretl_errmsg, _("generation of lag variable failed"));
+	    ok = 0;
+	} else {
+	    cmd->list[llen++] = vnum;
+	    if (newlag) {
+		/* newlag lives in transforms. c */
+		pprintf(prn, "genr %s\n", VARLABEL(pdinfo, vnum));
+	    }
+	}
+    }
+
+    if (ok) {
+	*lnum = llen;
+    }
+
+    return ok;
+} 
+
+static int plot_var_ok (const char *s, int *lnum,
+			double ***pZ, DATAINFO *pdinfo,
+			CMD *cmd)
+{
+    int pnum, ok = 1;
+    
+    if (strcmp(s, "qtrs") &&
+	strcmp(s, "months") &&
+	strcmp(s, "time")) 
+	return 0;
+
+    pnum = plotvar(pZ, pdinfo, s);
+
+    if (pnum < 0) {
+	cmd->errcode = 1;
+	ok = 0;
+	sprintf(gretl_errmsg, 
+		_("Failed to add plotting index variable"));
+    } else {
+	cmd->list[*lnum] = pnum;
+	*lnum += 1;
+    }
+
+    return ok;
+} 
+
+#if defined(USE_GTK2) || defined (HAVE_FNMATCH_H)
+
+static int wildcard_expand_ok (const char *s, int *lnum,
+			       const DATAINFO *pdinfo, CMD *cmd)
+{
+    int ok = 0;
+
+    if (strchr(s, '*') != NULL) {
+	int *wildlist = varname_match_list(pdinfo, s);
+
+	if (wildlist != NULL) {
+	    int k, nw = wildlist[0];
+	    int llen = *lnum;
+
+	    if (expand_command_list(cmd, nw)) {
+		return 0;
+	    }
+	    for (k=1; k<=nw; k++) {
+		cmd->list[llen++] = wildlist[k];
+	    }
+	    free(wildlist);
+	    *lnum = llen;
+	    ok = 1;
+	}
+    }
+
+    return ok;
+}
+
+#endif 
+
 static void parse_rename_cmd (const char *line, CMD *cmd, 
 			      const DATAINFO *pdinfo)
 {
@@ -387,6 +564,57 @@ static void parse_logistic_ymax (char *line, CMD *cmd)
     }
 }
 
+static int field_from_line (char *field, const char *s)
+{
+    const char *p;
+    int ret = 0;
+
+    sscanf(s, "%s", field);
+    
+    p = strchr(field, '(');
+    if (p == NULL) return 0; /* no parens? */
+
+    p = strchr(p + 1, ')');
+    if (p != NULL) return 0; /* balanced parens? */
+
+    /* fields that need to be glued together */
+    p = s + strlen(field);
+    if (!strncmp(p, " to ", 4)) {
+	char tmp[8];
+
+	if (sscanf(p, " to %7s", tmp)) {
+	    strcat(field, " to ");
+	    strcat(field, tmp);
+	    ret = 2;
+	}
+    }
+
+    return ret;
+}
+
+#define COMMAND_CAN_END(c) (c == SYSTEM || \
+                            c == NLS || \
+			    c == RESTRICT || \
+			    c == FUNC)
+
+static void check_end_command (CMD *cmd)
+{
+    if (*cmd->param) {
+	int cmdcode = gretl_command_number(cmd->param);
+
+	if (cmdcode == LOOP) {
+	    cmd->ci = ENDLOOP;
+	} else if (!COMMAND_CAN_END(cmdcode)) {
+	    cmd->errcode = 1;
+	    sprintf(gretl_errmsg, _("command 'end %s' not recognized"), 
+		    cmd->param);
+	}
+    } else {
+	cmd->errcode = 1;
+	strcpy(gretl_errmsg, _("end: nothing to end")); 
+    }
+}
+
 /**
  * getcmd:
  * @line: the command line (string).
@@ -395,20 +623,19 @@ static void parse_logistic_ymax (char *line, CMD *cmd)
  * @ignore: pointer to int indicating whether (1) or not (0) we're
  * in comment mode, and @line should not be parsed.
  * @pZ: pointer to data matrix.
- * @cmds: pointer to gretl printing struct.
+ * @cmdprn: pointer to gretl printing struct.
  *
  * Parses @line and fills out @command accordingly.  In case
  * of error, @command->errcode gets a non-zero value.
  */
 
 void getcmd (char *line, DATAINFO *pdinfo, CMD *command, 
-	     int *ignore, double ***pZ, PRN *cmds)
+	     int *ignore, double ***pZ, PRN *cmdprn)
 {
     int i, j, nf, linelen, n, v, lnum;
     int gotdata = 0, ar = 0, poly = 0;
     int spacename = 0;
     char field[10], *remainder;
-    LAGVAR lagvar;
 
     command->ci = 0;
     command->errcode = 0;
@@ -427,7 +654,7 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
     if (!gotdata) {
 	if (filter_comments(line, ignore)) {
 	    command->nolist = 1;
-	    command->ci = -2;
+	    command->ci = CMD_COMMENT;
 	    return;
 	}
     }
@@ -435,7 +662,7 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
     /* also new-style comments */
     if (*line == '#') {
 	command->nolist = 1;
-	command->ci = -1;
+	command->ci = CMD_COMMENT;
 	return;
     }    
 
@@ -445,7 +672,7 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
     /* no command here? */
     if (sscanf(line, "%s", command->cmd) != 1) {
 	command->nolist = 1;
-	command->ci = -1;
+	command->ci = CMD_NULL;
 	return;
     }
 
@@ -489,7 +716,7 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
     /* if, else, endif controls */
     if (flow_control(line, pZ, pdinfo, command)) {
 	command->nolist = 1;
-	command->ci = -1;
+	command->ci = CMD_NULL;
 	return;
     }
 
@@ -572,6 +799,9 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
 		sscanf(line + n + 1, "%s", command->param);
 	    }
 	}
+	if (command->ci == END) {
+	    check_end_command(command);
+	}
 	return;
     }
 
@@ -648,6 +878,7 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
     */
     if ((command->ci == STORE && !spacename) ||
 	command->ci == ADF ||
+	command->ci == KPSS ||
 	command->ci == ARCH ||
 	command->ci == COINT ||
 	command->ci == COINT2 ||
@@ -699,6 +930,7 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
 
     /* now assemble the command list */
     for (j=1,lnum=1; j<=nf; j++) {
+	int skip;
 
 	strcpy(remainder, line + n + 1);
 
@@ -709,81 +941,44 @@ void getcmd (char *line, DATAINFO *pdinfo, CMD *command,
 	    sscanf(remainder, "%s", command->param);
 	    break;
 	}
-	
-	sscanf(remainder, "%s", field);
-	/* fprintf(stderr, "remainder: %s\n", remainder); */
+
+	skip = field_from_line(field, remainder);
+	if (skip > 0) {
+	    nf -= skip;
+	    command->list[0] -= skip;
+	}
 
 	if (isalpha((unsigned char) *field)) {
 	    /* should be the name of a variable */
 	    if (field[strlen(field) - 1] == ';')
 		field[strlen(field) - 1] = '\0';
-	    if ((v = varindex(pdinfo, field)) <= pdinfo->v - 1) {
+	    if ((v = varindex(pdinfo, field)) < pdinfo->v) {
 		/* yes, it's an existing variable */
 		command->list[lnum++] = v;
-	    } else {
-		/* no: an auto-generated variable? */
+	    } else { /* possibly an auto-generated variable? */
+
 		/* Case 1: automated lags:  e.g. 'var(-1)' */
-		if (parse_lagvar(field, &lagvar, pdinfo)) {
-		    extern int newlag; /* generate.c */
-		    int lagnum;
+		if (auto_lag_ok(field, &lnum, pZ, pdinfo, command, cmdprn)) {
+		    /* handled, get on with it */
+		    n += strlen(field) + 1;
+		    continue; 
+		}
 
-		    lagnum = laggenr(lagvar.varnum, lagvar.lag, 1, pZ, pdinfo);
-		    if (lagnum < 0) {
-			command->errcode = 1;
-			sprintf(gretl_errmsg, 
-				_("generation of lag variable failed"));
-		    } else { 
-			command->list[lnum++] = lagnum;
-			if (newlag && cmds != NULL) {
-			    pprintf(cmds, "genr %s\n", VARLABEL(pdinfo, lagnum));
-			}
-			/* fully handled, get on with it */
-			n += strlen(field) + 1;
-			continue; 
-		    }
-		} 
 		/* Case 2: special plotting variable */
-		else if (!command->errcode && (!strcmp(field, "qtrs") || 
-		    !strcmp(field, "months") || !strcmp(field, "time"))) {
-		    int pnum = plotvar(pZ, pdinfo, field);
-
-		    if (pnum < 0) {
-			command->errcode = 1;
-			sprintf(gretl_errmsg, 
-				_("Failed to add plotting index variable"));
-		    } else {
-			command->list[lnum++] = pnum;
-			/* fully handled, get on with it */
-			n += strlen(field) + 1;
-			continue; 
-		    }
+		else if (!command->errcode && 
+			 plot_var_ok(field, &lnum, pZ, pdinfo, command)) {
+		    /* handled, get on with it */
+		    n += strlen(field) + 1;
+		    continue; 
 		} 
+
 #if defined(USE_GTK2) || defined (HAVE_FNMATCH_H)
 		/* wildcard expansion? */
-		else if (strchr(field, '*') != NULL) {
-		    int *wildlist = varname_match_list(pdinfo, field);
-
-		    if (wildlist != NULL) {
-			int k, nw = wildlist[0];
-
-			command->list = realloc(command->list, (command->list[0] + nw) *
-						sizeof *command->list);
-			if (command->list == NULL) {
-			    command->errcode = E_ALLOC;
-			    strcpy (gretl_errmsg, 
-				    _("Memory allocation failed for command list"));
-			    free(remainder);
-			    return;
-			}
-			command->list[0] += (nw - 1);
-			for (k=1; k<=nw; k++) {
-			    command->list[lnum++] = wildlist[k];
-			}
-			free(wildlist);
-			/* fully handled, get on with it */
-			n += strlen(field) + 1;
-			continue; 			
-		    }
+		else if (!command->errcode && 
+			 wildcard_expand_ok(field, &lnum, pdinfo, command)) {
+		    /* handled, get on with it */
+		    n += strlen(field) + 1;
+		    continue; 			
 		}
 #endif
 		/* last chance: try abbreviating the varname? */
@@ -940,15 +1135,20 @@ int help (const char *cmd, const char *helpfile, PRN *prn)
     char line[MAXLEN], cmdcopy[9];
     int i, ok;
 
-    if (cmd == NULL) {
+    if (cmd == NULL || *cmd == '\0') {
 	pputs(prn, _("\nValid gretl commands are:\n"));
 	for (i=1; i<NC; i++) {
 	    pprintf(prn, "%-9s", gretl_command_word(i));
-	    if (i%8 == 0) pputs(prn, "\n");
-	    else pputs(prn, " ");
+	    if (i % 8 == 0) {
+		pputc(prn, '\n');
+	    } else {
+		pputc(prn, ' ');
+	    }
 	}
+
 	pputs(prn, _("\n\nFor help on a specific command, type: help cmdname"));
 	pputs(prn, _(" (e.g. help smpl)\n"));
+
 	return 0;
     }
 
@@ -957,12 +1157,10 @@ int help (const char *cmd, const char *helpfile, PRN *prn)
 
     ok = 0;
 
-    if (gretl_command_number(cmd) > 0) ok = 1;
-
-    if (!ok && aliased(cmdcopy)) {
-	if (gretl_command_number(cmdcopy) > 0) {
-	    ok = 1;
-	}
+    if (gretl_command_number(cmd) > 0) {
+	ok = 1;
+    } else if (aliased(cmdcopy) && gretl_command_number(cmdcopy) > 0) {
+	ok = 1;
     }
 
     if (!ok) {
@@ -978,11 +1176,13 @@ int help (const char *cmd, const char *helpfile, PRN *prn)
     ok = 0;
     while (fgets(line, MAXLEN, fq) != NULL) {
 	nl_strip(line);
-	if (strcmp(cmdcopy, line) == 0) {
+	if (!strcmp(cmdcopy, line)) {
 	    ok = 1;
 	    pputs(prn, "\n");
 	    while (fgets(line, MAXLEN, fq)) {
-		if (*line == '#') break;
+		if (*line == '#') {
+		    break;
+		}
 		nl_strip(line);
 		if (*line != '@') {
 		    pprintf(prn, "%s\n", line);
@@ -997,13 +1197,14 @@ int help (const char *cmd, const char *helpfile, PRN *prn)
     }
 
     fclose(fq);
+
     return 0;
 }
 
 /* ........................................................... */
 
-static int parse_criteria (const char *line, const DATAINFO *pdinfo, 
-			   double ***pZ, PRN *prn)
+static int parse_criteria (const char *line, const double **Z,
+			   const DATAINFO *pdinfo, PRN *prn)
 {
     double ess;
     int i, T, k;
@@ -1014,30 +1215,42 @@ static int parse_criteria (const char *line, const DATAINFO *pdinfo,
     }
 
     if (isalpha((unsigned char) *essstr) && 
-	(i = varindex(pdinfo, essstr)) < pdinfo->v) 
-	    ess = get_xvalue(i, *pZ, pdinfo);
-    else if (isdigit(*essstr)) ess = atof(essstr);
-    else return 1;
+	(i = varindex(pdinfo, essstr)) < pdinfo->v) {
+	ess = get_xvalue(i, Z, pdinfo);
+    } else if (isdigit(*essstr)) {
+	ess = atof(essstr);
+    } else {
+	return 1;
+    }
+
     if (ess < 0) {
 	pputs(prn, _("ess: negative value is out of bounds.\n"));
 	return 1;
     }
 
     if (isalpha((unsigned char) *Tstr) &&
-	(i = varindex(pdinfo, Tstr)) < pdinfo->v) 
-	    T = (int) get_xvalue(i, *pZ, pdinfo);
-    else if (isdigit(*Tstr)) T = atoi(Tstr);
-    else return 1;
+	(i = varindex(pdinfo, Tstr)) < pdinfo->v) { 
+	T = (int) get_xvalue(i, Z, pdinfo);
+    } else if (isdigit(*Tstr)) {
+	T = atoi(Tstr);
+    } else {
+	return 1;
+    }
+
     if (T < 0) {
 	pputs(prn, _("T: negative value is out of bounds.\n"));
 	return 1;
     }
 
     if (isalpha((unsigned char) *kstr) &&
-	(i = varindex(pdinfo, kstr)) < pdinfo->v) 
-	    k = (int) get_xvalue(i, *pZ, pdinfo);
-    else if (isdigit(*kstr)) k = atoi(kstr);
-    else return 1;
+	(i = varindex(pdinfo, kstr)) < pdinfo->v) {
+	k = (int) get_xvalue(i, Z, pdinfo);
+    } else if (isdigit(*kstr)) {
+	k = atoi(kstr);
+    } else {
+	return 1;
+    }
+
     if (k < 0) {
 	pputs(prn, _("k: negative value is out of bounds.\n"));
 	return 1;
@@ -1086,12 +1299,15 @@ int fcast (const char *line, const MODEL *pmod, DATAINFO *pdinfo,
 	t2 = pdinfo->t2;
     }
 
-    if (check_varname(varname)) return -1;
+    if (check_varname(varname)) {
+	return -1;
+    }
 
     vi = varindex(pdinfo, varname);
 
-    if (vi >= pdinfo->v && dataset_add_vars(1, pZ, pdinfo)) 
+    if (vi >= pdinfo->v && dataset_add_vars(1, pZ, pdinfo)) { 
 	return -1 * E_ALLOC;
+    }
 
     strcpy(pdinfo->varname[vi], varname);
     strcpy(VARLABEL(pdinfo, vi), _("predicted values"));
@@ -1112,16 +1328,27 @@ static int make_full_list (const DATAINFO *pdinfo, CMD *command)
    except for the constant and any "hidden" variables or scalars */
 {
     int i, n = 1;
+    int *list;
 
-    command->list = realloc(command->list, pdinfo->v * sizeof(int));
-    if (command->list == NULL) return E_ALLOC;
+    list = realloc(command->list, pdinfo->v * sizeof *list);
+    if (list == NULL) {
+	return E_ALLOC;
+    }
+
+    command->list = list;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (hidden_var(i, pdinfo)) continue;
-	if (pdinfo->vector[i] == 0) continue;
+	if (hidden_var(i, pdinfo)) {
+	    continue;
+	}
+	if (pdinfo->vector[i] == 0) {
+	    continue;
+	}
 	command->list[n++] = i;
     }
+
     command->list[0] = n - 1;
+
     return 0;
 }
 
@@ -1169,6 +1396,8 @@ int parseopt (const char **argv, int argc, char *fname,
 	opt = OPT_DBOPEN;
     else if (strcmp(s, "-w") == 0 || strncmp(s, "--webdb", 7) == 0) 
 	opt = OPT_WEBDB;
+    else if (strcmp(s, "-c") == 0 || strncmp(s, "--dump", 6) == 0) 
+	opt = OPT_DUMP;
 
     if (opt != 0) {
 	argv++;
@@ -1192,46 +1421,54 @@ int shell (const char *arg)
     char shellnam[40];
     const char *theshell, *namep; 
 
-    old1 = signal (SIGINT, SIG_IGN);
-    old2 = signal (SIGQUIT, SIG_IGN);
+    old1 = signal(SIGINT, SIG_IGN);
+    old2 = signal(SIGQUIT, SIG_IGN);
 
     if ((pid = fork()) == 0) {
-	for (pid = 3; pid < 20; pid++)
-	    (void) close(pid);
-	(void) signal(SIGINT, SIG_DFL);
-	(void) signal(SIGQUIT, SIG_DFL);
+	for (pid = 3; pid < 20; pid++) {
+	    close(pid);
+	}
+
+	signal(SIGINT, SIG_DFL);
+	signal(SIGQUIT, SIG_DFL);
+
 	theshell = getenv("SHELL");
-	if (theshell == NULL)
+	if (theshell == NULL) {
 #ifdef HAVE_PATHS_H
 	    theshell =_PATH_BSHELL;
 #else
 	    theshell = "/bin/sh"; 
 #endif
+	}
 	namep = strrchr(theshell, '/');
-	if (namep == NULL)
+	if (namep == NULL) {
 	    namep = theshell;
-	(void) strcpy(shellnam,"-");
-	(void) strcat(shellnam, ++namep);
-	if (strcmp(namep, "sh") != 0)
+	}
+	strcpy(shellnam, "-");
+	strcat(shellnam, ++namep);
+	if (strcmp(namep, "sh") != 0) {
 	    shellnam[0] = '+';
+	}
 	if (arg) {
 	    execl(theshell, shellnam, "-c", arg, NULL);
-	}
-	else {
+	} else {
 	    execl(theshell, shellnam, NULL);
 	}
 	perror(theshell);
 	return 1;
     }
 
-    if (pid > 0) while (wait(NULL) != pid);
+    if (pid > 0) {
+	while (wait(NULL) != pid);
+    }
 
-    (void) signal(SIGINT, old1);
-    (void) signal(SIGQUIT, old2);
+    signal(SIGINT, old1);
+    signal(SIGQUIT, old2);
 
     if (pid == -1) {
 	perror(_("Try again later"));
     }
+
     return 0;
 }
 
@@ -1255,19 +1492,27 @@ static void trim_to_length (char *s, int oklen)
 
 static void 
 real_safe_print_line (const char *line, int cli, int batch, 
-		      int script, PRN *prn)
+		      int script, int loopstack, PRN *prn)
 {
     char tmp[SAFELEN];
     const char *split = " \\";
+    const char *leader;
+    const char *leaders[] = { "? ", "> " };
     const char *p, *q;
     int n, out, rem;
 
     if (!cli && batch) return;
 
+    if (loopstack) {
+	leader = leaders[1];
+    } else {
+	leader = leaders[0];
+    }
+
     if (cli) {
-	printf("%s", (batch)? "? " : " ");
+	printf("%s", (batch)? leader : " ");
     } else if (script) {
-	pprintf(prn, "\n? ");
+	pputs(prn, leader); 
     }	
 
     rem = n = strlen(line);
@@ -1291,9 +1536,9 @@ real_safe_print_line (const char *line, int cli, int batch,
     }
 }
 
-void safe_print_line (const char *line, PRN *prn)
+void safe_print_line (const char *line, int loopstack, PRN *prn)
 {
-    real_safe_print_line(line, 0, 0, 1, prn);
+    real_safe_print_line(line, 0, 0, 1, loopstack, prn);
 }
 
 /**
@@ -1303,6 +1548,7 @@ void safe_print_line (const char *line, PRN *prn)
  * @line: "raw" command line to be echoed.
  * @batch: set to 1 for batch mode, 0 for interactive.
  * @gui: 1 for the gretl GUI, 0 for command-line program.
+ * @loopstack: 1 if stacking commands for a loop, else 0.
  * @prn: pointer to gretl printing struct.
  *
  * Echoes the user command represented by @pcmd and @line.
@@ -1313,17 +1559,31 @@ void safe_print_line (const char *line, PRN *prn)
                        c == MPOLS || c == SCATTERS || c == GNUPLOT || \
                        c == LOGISTIC || c == GARCH || c == EQUATION)
 
-void echo_cmd (CMD *cmd, const DATAINFO *pdinfo, const char *line, 
-	       int batch, int gui, PRN *prn)
-     /* echo a given command: depending on the circumstances, either
-	to stdout or to a buffer, or both */
+/* The following may appear to be absurdly complicated.  Nonetheless,
+   I _think_ it is only as complex as it has to be, given the several 
+   dimensions in play:
 
+   * batch mode vs interactive mode ("batch" param)
+   * command client program vs gui program ("gui" param)
+   * stacking loop commands or not ("loopstack" param)
+   
+   If you are tempted to optimize one or other aspect, please make
+   sure you have not broken one of the others in the process.
+*/
+
+void echo_cmd (CMD *cmd, const DATAINFO *pdinfo, const char *line, 
+	       int batch, int gui, int loopstack, PRN *prn)
 {
     int i, err, gotsep = 1;
+    char leadchar = '?';
     int cli = !gui;
 
     if (line == NULL) return;
 
+    if (loopstack) {
+	leadchar = '>';
+    }
+
 #if 0
     fprintf(stderr, "echo_cmd: line='%s', gui=%d, cmd->opt=%ld, batch=%d, "
 	    "param='%s', nolist=%d\n", line, gui, cmd->opt, batch, cmd->param,
@@ -1332,16 +1592,16 @@ void echo_cmd (CMD *cmd, const DATAINFO *pdinfo, const char *line,
 #endif
 
     /* special case: gui "store" command, which could overflow the
-       "line" length; also I'm not sure whether we should record
-       gui "store" in the command script; we'll record it, but
-       commented out.
+       line length; also I'm not sure whether we should record gui
+       "store" in the command script.  As a compromise we'll record it,
+       but commented out. (FIXME: loop context?)
     */
-    if (gui && !batch && cmd->ci == STORE) {  /* FIXME monte carlo loop? */
+    if (gui && !batch && cmd->ci == STORE) {  
 	pprintf(prn, "# store '%s'", cmd->param);
 	if (cmd->opt) { 
 	    const char *flagstr = print_flags(cmd->opt, cmd->ci);
 
-	    pprintf(prn, "%s", flagstr);
+	    pputs(prn, flagstr);
 	}
 	pputc(prn, '\n');
 	return;
@@ -1354,7 +1614,7 @@ void echo_cmd (CMD *cmd, const DATAINFO *pdinfo, const char *line,
 	gotsep = 0;
     }
 
-    /* command is preceded by a "savename" to which a object will
+    /* command is preceded by a "savename" to which an object will
        be assigned */
     if (*cmd->savename && gui && !batch) {
 	pprintf(prn, "%s <- ", cmd->savename);
@@ -1364,32 +1624,42 @@ void echo_cmd (CMD *cmd, const DATAINFO *pdinfo, const char *line,
 	/* command has a list of args to be printed */
 	if (cli) {
 	    if (batch) {
-		printf("\n? %s", cmd->cmd);
+		printf("\n%c %s", leadchar, cmd->cmd);
 	    } else {
 		printf(" %s", cmd->cmd);
 	    }
-	    if (cmd->ci == RHODIFF) printf(" %s;", cmd->param);
-	    else if (*cmd->param && !hold_param(cmd->ci)) {
+	    if (cmd->ci == RHODIFF) {
+		printf(" %s;", cmd->param);
+	    } else if (*cmd->param && !hold_param(cmd->ci)) {
 		printf(" %s", cmd->param);
 	    }
 	}
 	if (!batch) {
 	    pprintf(prn, "%s", cmd->cmd);
-	    if (cmd->ci == RHODIFF) pprintf(prn, " %s;", cmd->param);
-	    else if (*cmd->param && !hold_param(cmd->ci)) {
+	    if (cmd->ci == RHODIFF) {
+		pprintf(prn, " %s;", cmd->param);
+	    } else if (*cmd->param && !hold_param(cmd->ci)) {
 		pprintf(prn, " %s", cmd->param);
 	    }
 	}
 
 	/* if list is very long, break it up over lines */
 	if (cmd->ci == STORE) {
-	    if (cli) printf(" \\\n");
-	    if (!batch) pputs(prn, " \\\n");
+	    if (cli) {
+		printf(" \\\n");
+	    }
+	    if (!batch) {
+		pputs(prn, " \\\n");
+	    }
 	}
 	for (i=1; i<=cmd->list[0]; i++) {
 	    if (cmd->list[i] == LISTSEP) {
-		if (cli) printf(" ;");
-		if (!batch) pputs(prn, " ;");
+		if (cli) {
+		    printf(" ;");
+		}
+		if (!batch) {
+		    pputs(prn, " ;");
+		}
 		gotsep = (cmd->ci != MPOLS)? 1 : 0;
 		continue;
 	    }
@@ -1418,8 +1688,12 @@ void echo_cmd (CMD *cmd, const DATAINFO *pdinfo, const char *line,
 	/* corrgm and gnuplot: param comes last */
 	if ((cmd->ci == CORRGM || cmd->ci == GNUPLOT || cmd->ci == LOGISTIC)
 	    && *cmd->param) { 
-	    if (cli) printf(" %s", cmd->param);
-	    if (!batch) pprintf(prn, " %s", cmd->param);
+	    if (cli) {
+		printf(" %s", cmd->param);
+	    }
+	    if (!batch) {
+		pprintf(prn, " %s", cmd->param);
+	    }
 	}
 
 	/* check for duplicated vars */
@@ -1429,38 +1703,50 @@ void echo_cmd (CMD *cmd, const DATAINFO *pdinfo, const char *line,
 		   err);
 	    cmd->ci = VARDUP;
 	}
-    } /* end if !cmd->nolist */
+    } /* end if command has list of variables */
 
     else if ((cmd->ci == GENR || cmd->ci == SMPL) && 
 	     strlen(line) > SAFELEN - 2) {
-	real_safe_print_line(line, cli, batch, 0, prn);
+	real_safe_print_line(line, cli, batch, 0, loopstack, prn);
     }
 
     else if (strcmp(cmd->cmd, "quit")) {
 	if (cli) {
-	    if (batch) printf("? %s", line);
-	    else printf(" %s", line);
+	    if (batch) {
+		printf("%c %s", leadchar, line);
+	    } else {
+		printf(" %s", line);
+	    }
+	}
+	if (!batch) {
+	    pputs(prn, line);
 	}
-	if (!batch) pputs(prn, line);
     }
 
+    /* add printout of any options to the command */
     if (cmd->opt) {
 	const char *flagstr;
 	int ci = cmd->ci;
 
-	if (ci == END && !strcmp(cmd->param, "nls")) {
+	if (cmd->ci == END && !strcmp(cmd->param, "nls")) {
 	    ci = NLS;
 	}
 	flagstr = print_flags(cmd->opt, ci);
-	if (cli) fputs(flagstr, stdout);
-	if (!batch) pputs(prn, flagstr);
+	if (cli) {
+	    fputs(flagstr, stdout);
+	}
+	if (!batch) {
+	    pputs(prn, flagstr);
+	}
     }
 
-    if (cli) putchar('\n');
+    if (cli) {
+	putchar('\n');
+    }
 
     if (!batch) {
 	pputc(prn, '\n');
-	if (prn != NULL && prn->fp) {
+	if (prn != NULL && prn->fp != NULL) {
 	    fflush(prn->fp);
 	}
     }
@@ -1479,13 +1765,9 @@ static const char *flag_present (const char *s, char f, int *quoted)
     while (*s) {
 	if (*s == '"') inquote = !inquote;
 	if (!inquote) {
-	    if (*s == '-') gotdash = 1;
-	    else if (gotdash && *s == f && *(s+1)) {
-#if 0
-		/* blank out the flag and following in the
-		   original string? */
-		*s = 0;
-#endif
+	    if (*s == '-') {
+		gotdash = 1;
+	    } else if (gotdash && *s == f && *(s+1)) {
 		s++;
 		while (*s) {
 		    if (isspace(*s)) s++;
@@ -1499,8 +1781,9 @@ static const char *flag_present (const char *s, char f, int *quoted)
 		    *quoted = 0;
 		    return s;
 		}
+	    } else {
+		gotdash = 0;
 	    }
-	    else gotdash = 0;
 	}
 	s++;
     }
@@ -1584,8 +1867,6 @@ static void get_optional_filename (const char *line, CMD *cmd)
     }    
 }
 
-/* .......................................................... */
-
 static void showlabels (const DATAINFO *pdinfo, PRN *prn)
 {
     int i;
@@ -1605,12 +1886,14 @@ static void do_print_string (char *str, PRN *prn)
 
     if (*str == '"') str++;
     len = strlen(str);
-    if (str[len-1] == '"') str[len-1] = 0;
+    if (str[len-1] == '"') {
+	str[len-1] = 0;
+    }
 
     pprintf(prn, "%s\n", str);
 }
 
-static int do_outfile_command (unsigned long flag, char *fname,
+static int do_outfile_command (gretlopt flag, char *fname,
 			       PRN *prn)
 {
     static char outname[MAXLEN];
@@ -1682,12 +1965,12 @@ static int do_outfile_command (unsigned long flag, char *fname,
 /* ........................................................ */
 
 int call_pca_plugin (CORRMAT *corrmat, double ***pZ,
-		     DATAINFO *pdinfo, unsigned long *pflag,
+		     DATAINFO *pdinfo, gretlopt *pflag,
 		     PRN *prn)
 {
     void *handle = NULL;
     int (*pca_from_corrmat) (CORRMAT *, double ***, DATAINFO *,
-			     unsigned long *, PRN *);
+			     gretlopt *, PRN *);
     int err = 0;
 
     *gretl_errmsg = 0;
@@ -1703,14 +1986,13 @@ int call_pca_plugin (CORRMAT *corrmat, double ***pZ,
     return err;
 }
 
-/* ........................................................ */
+/* common code for command-line and GUI client programs, where the
+   command doesn't require special handling on the client side 
+*/
 
 int simple_commands (CMD *cmd, const char *line, 
-		     double ***pZ, DATAINFO *datainfo, PATHS *paths,
-		     int pause, PRN *prn)
-     /* common code for command-line and GUI client programs, where
-	the command doesn't require special handling on the client
-	side */
+		     double ***pZ, DATAINFO *datainfo,
+		     PRN *prn)
 {
     int err = 0, order = 0;
     CORRMAT *corrmat;
@@ -1725,7 +2007,7 @@ int simple_commands (CMD *cmd, const char *line,
 	}
 	/* flag an error if cmd->list[0] > 1? */
 	order = atoi(cmd->param);
-	err = adf_test(order, cmd->list[1], pZ, datainfo, prn);
+	err = adf_test(order, cmd->list[1], pZ, datainfo, cmd->opt, prn);
 	break;
 
     case COINT:
@@ -1740,12 +2022,13 @@ int simple_commands (CMD *cmd, const char *line,
 
     case CORR:
 	if (cmd->list[0] > 3) {
-	    err = esl_corrmx(cmd->list, pZ, datainfo, pause, prn);
+	    err = gretl_corrmx(cmd->list, (const double **) *pZ, datainfo, 
+			       prn);
 	    if (err) 
 		pputs(prn, _("Error in generating correlation matrix\n"));
 	    break;
 	}
-	corrmat = corrlist(cmd->list, pZ, datainfo);
+	corrmat = corrlist(cmd->list, (const double **) *pZ, datainfo);
 	if (corrmat == NULL) 
 	    pputs(prn, _("Couldn't allocate memory for correlation matrix.\n"));
 	else printcorr(corrmat, datainfo, prn);
@@ -1753,7 +2036,7 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case PCA:
-	corrmat = corrlist(cmd->list, pZ, datainfo);
+	corrmat = corrlist(cmd->list, (const double **) *pZ, datainfo);
 	if (corrmat == NULL) {
 	    pputs(prn, _("Couldn't allocate memory for correlation matrix.\n"));
 	} else {
@@ -1766,7 +2049,7 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case CRITERIA:
-	err = parse_criteria(line, datainfo, pZ, prn);
+	err = parse_criteria(line, (const double **) *pZ, datainfo, prn);
 	if (err) 
 	    pputs(prn, _("Error in computing model selection criteria.\n"));
 	break;
@@ -1786,6 +2069,15 @@ int simple_commands (CMD *cmd, const char *line,
 	else varlist(datainfo, prn);
 	break;
 
+    case KPSS:
+	if (!isdigit((unsigned char) *cmd->param)) {
+	    pputs(prn, _("kpss: lag order must be given first\n"));
+	    break;
+	}
+	order = atoi(cmd->param);
+	err = kpss_test(order, cmd->list[1], pZ, datainfo, cmd->opt, prn);
+	break;
+
     case LDIFF:
 	err = list_ldiffgenr(cmd->list, pZ, datainfo);
 	if (err) 
@@ -1794,21 +2086,24 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case LAGS:
-	err = lags(cmd->list, pZ, datainfo); 
+	err = list_laggenr(cmd->list, pZ, datainfo); 
 	if (err) 
 	    pputs(prn, _("Error adding lags of variables.\n"));
 	else varlist(datainfo, prn);
 	break;
 
     case LOGS:
-	err = logs(cmd->list, pZ, datainfo);
-	if (err < cmd->list[0]) 
+	err = list_loggenr(cmd->list, pZ, datainfo);
+	if (err) 
 	    pputs(prn, _("Error adding logs of variables.\n"));
-	if (err > 0) { 
-	    varlist(datainfo, prn);
-	    err = 0;
-	} else
-	    err = 1;
+	else varlist(datainfo, prn);
+	break;
+
+    case SQUARE:
+	err = list_xpxgenr(cmd->list, pZ, datainfo, cmd->opt);
+	if (err) 
+	    pputs(prn, _("Failed to generate squares\n"));
+	else varlist(datainfo, prn);
 	break;
 
     case MULTIPLY:
@@ -1817,11 +2112,13 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case GRAPH:
-	graph(cmd->list, *pZ, datainfo, cmd->opt, prn);
+	ascii_graph(cmd->list, (const double **) *pZ, datainfo, 
+		    cmd->opt, prn);
 	break;
 
     case PLOT:
-	plot(cmd->list, *pZ, datainfo, cmd->opt, pause, prn);
+	ascii_plot(cmd->list, (const double **) *pZ, datainfo, 
+		   cmd->opt, prn);
 	break;
 
     case RMPLOT:
@@ -1829,7 +2126,7 @@ int simple_commands (CMD *cmd, const char *line,
 	    pputs(prn, _("This command requires one variable.\n"));
 	    err = 1;
 	} else {
-	    err = rmplot(cmd->list, *pZ, datainfo, prn, paths);
+	    err = rmplot(cmd->list, *pZ, datainfo, prn);
 	}
 	break;
 
@@ -1856,40 +2153,80 @@ int simple_commands (CMD *cmd, const char *line,
 	if (strlen(cmd->param)) {
 	    do_print_string(cmd->param, prn);
 	} else {
-	    printdata(cmd->list, pZ, datainfo, pause, cmd->opt, prn);
+	    printdata(cmd->list, (const double **) *pZ, datainfo, 
+		      cmd->opt, prn);
 	}
 	break;
 
+    case RHODIFF:
+	err = rhodiff(cmd->param, cmd->list, pZ, datainfo);
+	if (err) errmsg(err, prn);
+	else varlist(datainfo, prn);
+	break;
+
+    case SIM:
+	err = simulate(line, *pZ, datainfo);
+	if (err) errmsg(err, prn);
+	else print_gretl_msg(prn);
+	break;
+
     case SUMMARY:
-	summ = summary(cmd->list, pZ, datainfo, prn);
+	summ = summary(cmd->list, (const double **) *pZ, datainfo, prn);
 	if (summ == NULL) 
 	    pputs(prn, _("generation of summary stats failed\n"));
 	else {
-	    print_summary(summ, datainfo, pause, prn);
+	    print_summary(summ, datainfo, prn);
 	    free_summary(summ);
 	}
 	break; 
 
     case MEANTEST:
-	err = means_test(cmd->list, *pZ, datainfo, cmd->opt, prn);
+	err = means_test(cmd->list, (const double **) *pZ, datainfo, 
+			 cmd->opt, prn);
 	break;	
 
     case VARTEST:
-	err = vars_test(cmd->list, *pZ, datainfo, prn);
+	err = vars_test(cmd->list, (const double **) *pZ, datainfo, 
+			prn);
 	break;
 
     case RUNS:
-	err = runs_test(cmd->list[1], *pZ, datainfo, prn);
+	err = runs_test(cmd->list[1], (const double **) *pZ, datainfo, 
+			prn);
 	break;
 
     case SPEARMAN:
-	err = spearman(cmd->list, *pZ, datainfo, cmd->opt, prn);
+	err = spearman(cmd->list, (const double **) *pZ, datainfo, 
+		       cmd->opt, prn);
 	break;
 
     case OUTFILE:
 	err = do_outfile_command(cmd->opt, cmd->param, prn);
 	break;
 
+    case STORE:
+	if (*cmd->param != '\0') {
+	    if ((cmd->opt & OPT_Z) && !has_gz_suffix(cmd->param)) {
+		pprintf(prn, _("store: using filename %s.gz\n"), cmd->param);
+	    } else {
+		pprintf(prn, _("store: using filename %s\n"), cmd->param);
+	    }
+	} else {
+	    pprintf(prn, _("store: no filename given\n"));
+	    break;
+	}
+	if (write_data(cmd->param, cmd->list, (const double **) *pZ,
+		       datainfo, cmd->opt, NULL)) {
+	    pprintf(prn, _("write of data file failed\n"));
+	    err = 1;
+	    break;
+	}
+	pprintf(prn, _("Data written OK.\n"));
+	if (((cmd->opt & OPT_O) || (cmd->opt & OPT_S)) && datainfo->markers) 
+	    pprintf(prn, _("Warning: case markers not saved in "
+			   "binary datafile\n"));
+	break;
+
     default:
 	break;
     }
@@ -1905,6 +2242,8 @@ int ready_for_command (const char *line)
 	"nulldata", 
 	"import", 
 	"pvalue",
+	"print",
+	"printf",
 	"eval",
 	"!",
 	"(*", 
@@ -1913,25 +2252,52 @@ int ready_for_command (const char *line)
 	"set", 
 	"critical", 
 	"seed", 
-	"genr",
+	"function",
+	"noecho",
 	NULL 
     };
     const char **p = ok_cmds;
 
-    if (string_is_blank(line)) return 1;
+    if (string_is_blank(line) || gretl_compiling_function()) {
+	return 1;
+    }
 
-    if (*line == 'q' || *line == 'x' || 
-	*line == '\0' || *line == '#') return 1;
+    if (*line == 'q' || *line == 'x' || *line == '#') {
+	return 1;
+    }
 
     while (*p) {
-	if (strncmp(line, *p, strlen(*p)) == 0)
+	if (strncmp(line, *p, strlen(*p)) == 0) {
 	    return 1;
+	}
 	p++;
     }
 
     return 0;
 }
 
+int gretl_cmd_init (CMD *cmd)
+{
+    cmd->list = malloc(sizeof *cmd->list);
+    if (cmd->list == NULL) {
+	return 1;
+    }
+
+    cmd->param = malloc(1);
+    if (cmd->param == NULL) {
+	free(cmd->list);
+	return 1;
+    }
+
+    return 0;
+}
+
+void gretl_cmd_free (CMD *cmd)
+{
+    free(cmd->list);
+    free(cmd->param);
+}
+
 
 
 
