@@ -24,13 +24,15 @@
 
 #define MAXSAVENAME 32
 #define VARDUP     999
+#define CMD_NULL    -1
+#define CMD_COMMENT -2
 
 typedef struct _CMD CMD;
 
 struct _CMD {
     char cmd[9];                /* command word */
     int ci;                     /* command index number */
-    unsigned long opt;          /* option flags */
+    gretlopt opt;               /* option flags */
     char savename[MAXSAVENAME]; /* name used to save an object from the command */
     char str[4];                /* used, e.g., in "multiply" command */
     int nolist;                 /* = 1 if the command does not take a list */
@@ -46,13 +48,14 @@ enum option_codes {
     OPT_VERSION,
     OPT_RUNIT,
     OPT_DBOPEN,
-    OPT_WEBDB
+    OPT_WEBDB,
+    OPT_DUMP
 };
     
 /* functions follow */
  
 void getcmd (char *line, DATAINFO *pdinfo, CMD *command, 
-	     int *ignore, double ***pZ, PRN *cmds);
+	     int *ignore, double ***pZ, PRN *cmdprn);
 
 int command_number (const char *cmd);
 
@@ -66,19 +69,19 @@ int parseopt (const char **argv, int argc, char *fname, int *english);
 int shell (const char *arg);
 
 void echo_cmd (CMD *pcmd, const DATAINFO *pdinfo, const char *line, 
-	       int batch, int gui, PRN *prn);
+	       int batch, int gui, int loopstack, PRN *prn);
 
 int simple_commands (CMD *cmd, const char *line, 
-		     double ***pZ, DATAINFO *datainfo, PATHS *paths,
-		     int batch, PRN *prn);
+		     double ***pZ, DATAINFO *datainfo,
+		     PRN *prn);
 
 int call_pca_plugin (CORRMAT *corrmat, double ***pZ,
-		     DATAINFO *pdinfo, unsigned long *pflag,
+		     DATAINFO *pdinfo, gretlopt *pflag,
 		     PRN *prn);
 
 int ready_for_command (const char *line);
 
-void safe_print_line (const char *line, PRN *prn);
+void safe_print_line (const char *line, int loopstack, PRN *prn);
 
 #endif /* INTERACT_H */
 
