@@ -65,11 +65,13 @@
 # endif /* ENABLE_NLS */
 #endif /* __GNOME_I18N_H__ */
 
+#define MAXLINE 1024  /* maximum length of command line */
 #define MAXLABEL 128  /* maximum length of descriptive labels for variables */
-#define MAXLEN   512  /* max length of "long" strings */
+#define MAXLEN   512  /* max length of regular "long" strings */
 #define ERRLEN   256  /* max length of libgretl error messages */
 #define MAXDISP   20  /* max length of "display names" for variables */
-#define VNAMELEN   9  /* space allocated for var names (including termination) */
+#define VNAMELEN  14  /* space allocated for var names (including termination) */
+#define USER_VLEN  9  /* max allocation for varname constructed by user */
 #define OBSLEN    11  /* space allocated for obs strings (including termination) */
 
 #ifndef M_PI
@@ -78,6 +80,7 @@
 
 #define LISTSEP            999
 #define PMAX_NOT_AVAILABLE 666
+#define OPT_NONE           0L
 
 /* numbers smaller than the given limit will print as zero */
 #define screen_zero(x)  ((fabs(x) > 1.0e-13)? x : 0.0)
@@ -101,7 +104,8 @@ enum gretl_print_formats {
 enum ts_codes {
     TIME_SERIES = 1,
     STACKED_TIME_SERIES,
-    STACKED_CROSS_SECTION
+    STACKED_CROSS_SECTION,
+    STRUCTURE_UNKNOWN,
 };
 
 enum auto_genr {
@@ -157,39 +161,38 @@ enum gretl_opt_flags {
     OPT_Z = 1 << 18
 };
 
-typedef int *LIST; 
+typedef unsigned long gretlopt;
 
-typedef struct _VARINFO VARINFO;
-typedef struct _DATAINFO DATAINFO;
-typedef struct _PATHS PATHS;
-typedef struct _GRETLTEST GRETLTEST;
-typedef struct _GRETLSUMMARY GRETLSUMMARY;
-typedef struct _CORRMAT CORRMAT;
-typedef struct _SAMPLE SAMPLE;
-typedef struct _ARINFO ARINFO;
-typedef struct _MODEL MODEL;
-typedef struct _MODELSPEC MODELSPEC;
-typedef struct _GRAPHT GRAPHT;
-typedef struct _PRN PRN;
-typedef struct _FITRESID FITRESID;
-typedef struct _CONFINT CONFINT;
-typedef struct _VCV VCV;
-typedef struct _DATASET DATASET;
-typedef struct _GRETL_VAR GRETL_VAR;
+typedef struct VARINFO_ VARINFO;
+typedef struct DATAINFO_ DATAINFO;
+typedef struct PATHS_ PATHS;
+typedef struct GRETLTEST_ GRETLTEST;
+typedef struct CORRMAT_ CORRMAT;
+typedef struct SAMPLE_ SAMPLE;
+typedef struct ARINFO_ ARINFO;
+typedef struct MODEL_ MODEL;
+typedef struct GRAPHT_ GRAPHT;
+typedef struct PRN_ PRN;
+typedef struct FITRESID_ FITRESID;
+typedef struct CONFINT_ CONFINT;
+typedef struct VCV_ VCV;
+typedef struct DATASET_ DATASET;
+typedef struct GRETL_VAR_ GRETL_VAR;
 
-typedef struct _mp_results mp_results;
-typedef struct _model_data_item model_data_item;
+typedef struct mp_results_ mp_results;
+typedef struct model_data_item_ model_data_item;
 
 
 /* information on individual variable */
-struct _VARINFO {
+struct VARINFO_ {
     char label[MAXLABEL];
     char display_name[MAXDISP];
-    int compact_method;
+    char compact_method;
+    char stack_level;
 };
 
 /* information on data set */
-struct _DATAINFO { 
+struct DATAINFO_ { 
     int v;              /* number of variables */
     int n;              /* number of observations */
     int pd;             /* periodicity or frequency of data */
@@ -213,12 +216,12 @@ struct _DATAINFO {
 };
 
 /* wrapper for the two main elements of a gretl data set */
-struct _DATASET {
+struct DATASET_ {
     DATAINFO *dinfo;
     double **Z;
 };
 
-struct _PATHS {
+struct PATHS_ {
     char currdir[MAXLEN];
     char userdir[MAXLEN];
     char gretldir[MAXLEN];
@@ -227,7 +230,6 @@ struct _PATHS {
     char helpfile[MAXLEN];
     char cmd_helpfile[MAXLEN];
     char datfile[MAXLEN];
-    char plotfile[MAXLEN];
     char binbase[MAXLEN];
     char ratsbase[MAXLEN];
     char gnuplot[MAXLEN];
@@ -235,12 +237,11 @@ struct _PATHS {
     char x12a[MAXLEN];
     char x12adir[MAXLEN];
 #endif
-    char dbhost_ip[16];
-    char pngfont[16];
-    unsigned char status;
+    char dbhost[32];
+    char pngfont[32];
 };
 
-struct _GRETLTEST {
+struct GRETLTEST_ {
     char type[72];
     char h_0[64];
     char param[VNAMELEN];
@@ -250,42 +251,31 @@ struct _GRETLTEST {
     double pvalue;
 };
 
-struct _GRETLSUMMARY {
-    int n;
-    int *list;
-    double *xskew; 
-    double *xkurt;
-    double *xmedian;
-    double *coeff;
-    double *sderr;
-    double *xpx;
-    double *xpy;
-};
-
-struct _CORRMAT {
+struct CORRMAT_ {
     int n, t1, t2;
     int *list;
     double *xpx;
 };
 
-struct _SAMPLE {
+struct SAMPLE_ {
     int t1;
     int t2;
 };
 
-struct _ARINFO {
+struct ARINFO_ {
     int *arlist;                /* list of autoreg lags */
     double *rho;                /* array of autoreg. coeffs. */
     double *sderr;              /* and their standard errors */
 };
 
 /* struct to hold model results */
-struct _MODEL {
+struct MODEL_ {
     int ID;                      /* ID number for model */
     int t1, t2, nobs;            /* starting observation, ending
                                     observation, and number of obs */
     char *subdum;                /* keep track of sub-sample in force
                                     when model was estimated */
+    char *missmask;              /* missing observations mask */
     SAMPLE smpl;                 /* numeric start and end of current sample
                                     when model was estimated */
     int ncoeff, dfn, dfd;        /* number of coefficents; degrees of
@@ -331,19 +321,14 @@ struct _MODEL {
     model_data_item **data_items; /* pointer to additional data */
 };
 
-struct _MODELSPEC {
-    char *cmd;
-    char *subdum;
-};
-
-struct _GRAPHT {
+struct GRAPHT_ {
     int ID;
     int sort;
     char name[32];
     char fname[MAXLEN];
 }; 
 
-struct _PRN {
+struct PRN_ {
     FILE *fp;
     FILE *fpaux;    
     char *buf;
@@ -351,7 +336,7 @@ struct _PRN {
     int format;
 };
 
-struct _mp_results {
+struct mp_results_ {
     int ncoeff;
     int t1, t2, ifc;
     int dfn, dfd;
@@ -365,7 +350,7 @@ struct _mp_results {
     double fstt;
 };
 
-struct _FITRESID {
+struct FITRESID_ {
     double *actual;
     double *fitted;
     double *sderr;
@@ -379,7 +364,7 @@ struct _FITRESID {
     char depvar[VNAMELEN];
 };
 
-struct _CONFINT {
+struct CONFINT_ {
     int *list;
     double *coeff;
     double *maxerr;
@@ -387,7 +372,7 @@ struct _CONFINT {
     int ifc;
 };
 
-struct _VCV {
+struct VCV_ {
     int ci;
     int *list;
     double *vec;
@@ -396,11 +381,14 @@ struct _VCV {
 #define VARLABEL(p,i)  ((p->varinfo[i])->label)
 #define DISPLAYNAME(p,i)  ((p->varinfo[i])->display_name)
 #define COMPACT_METHOD(p,i) ((p->varinfo[i])->compact_method)
+#define STACK_LEVEL(p,i) ((p->varinfo[i])->stack_level)
 
 #include "gretl_commands.h"
 #include "gretl_errors.h"
+#include "interact.h"
 #include "estimate.h"
 #include "generate.h"
+#include "gretl_list.h"
 #include "gretl_paths.h"
 #include "gretl_utils.h"
 #include "gretl_model.h"
@@ -413,7 +401,6 @@ struct _VCV {
 #include "printout.h"
 #include "modelprint.h"
 #include "texprint.h"
-#include "interact.h"
 #include "graphing.h"
 #include "random.h"
 #include "monte_carlo.h"
@@ -425,6 +412,8 @@ struct _VCV {
 #include "system.h"
 #include "plugins.h"
 #include "nls.h"
+#include "missing.h"
+#include "transforms.h"
 #ifdef WIN32
 # include "gretl_win32.h"
 #endif
