@@ -31,7 +31,15 @@ enum vcv_codes {
 
 int get_garch_vcv_version (void);
 
+int get_force_hc (void);
+
+int get_halt_on_error (void);
+
 double get_hp_lambda (void);
 
+void gretl_set_text_pause (int p);
+
+int gretl_get_text_pause (void);
+
 #endif /* LIBSET_H */
 
