@@ -39,7 +39,7 @@ struct _cmplx {
 #define NO_RBAR_SQ(a) (a == AUX_SQ || a == AUX_LOG || a == AUX_WHITE || a == AUX_AR)
 
 static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			int c, PRN *prn);
+			int c, int longnames, PRN *prn);
 static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
 			    int c, PRN *prn);
 static void depvarstats (const MODEL *pmod, PRN *prn);
@@ -418,9 +418,11 @@ static int least_signif_coeff (const MODEL *pmod)
 	    k = i;
 	}
     }
+
     if (tprob(tmin, pmod->dfd) > .10) {
 	return pmod->list[k+2];
     }
+
     return 0;
 }
 
@@ -429,25 +431,16 @@ static int least_signif_coeff (const MODEL *pmod)
 static void pval_max_line (const MODEL *pmod, const DATAINFO *pdinfo, 
 			   PRN *prn)
 {
-    if (!TEX_FORMAT(prn->format)) {
-	int k = pmod->ncoeff - pmod->ifc;
+    int k = pmod->ncoeff - pmod->ifc;
 
-	if (k < 2) return;
+    if (k < 2) return;
 
-	if ((k = least_signif_coeff(pmod))) {
-	    char tmp[128];
+    if ((k = least_signif_coeff(pmod))) {
+	char tmp[128];
 
-	    if (PLAIN_FORMAT(prn->format)) {
-		sprintf(tmp, _("Excluding the constant, p-value was highest "
-			"for variable %d (%s)"), k, pdinfo->varname[k]);
-		pprintf(prn, "%s\n\n", tmp);
-	    }
-	    else if (RTF_FORMAT(prn->format)) {
-		sprintf(tmp, I_("Excluding the constant, p-value was highest "
-			"for variable %d (%s)"), k, pdinfo->varname[k]);
-		pprintf(prn, "\\par %s\\par\n", tmp);
-	    }
-	}
+	sprintf(tmp, _("Excluding the constant, p-value was highest "
+		       "for variable %d (%s)"), k, pdinfo->varname[k]);
+	pprintf(prn, "%s\n\n", tmp);
     }
 }
 
@@ -475,7 +468,14 @@ static const char *aux_string (int aux, int format)
     }
     else if (aux == AUX_ADF) {
 	if (TEX_FORMAT(format)) return N_("Augmented Dickey--Fuller regression");
-	return N_("Augmented Dickey-Fuller regression");
+	else return N_("Augmented Dickey-Fuller regression");
+    }
+    else if (aux == AUX_DF) {
+	if (TEX_FORMAT(format)) return N_("Dickey--Fuller regression");
+	else return N_("Dickey-Fuller regression");
+    }
+    else if (aux == AUX_KPSS) {
+	return N_("KPSS regression");
     }
     else if (aux == AUX_RESET) {
 	return N_("Auxiliary regression for RESET specification test");
@@ -702,16 +702,8 @@ static void print_model_tests (const MODEL *pmod, PRN *prn)
 
 /* ......................................................... */
 
-static void modelprint_setup_obs (const MODEL *pmod, int *t2)
-{
-    if (pmod->data != NULL) {
-	*t2 += get_misscount(pmod);
-    }
-}
-
-/* ......................................................... */
-
-static int print_tsls_instruments (LIST list, const DATAINFO *pdinfo, PRN *prn)
+static int 
+print_tsls_instruments (const int *list, const DATAINFO *pdinfo, PRN *prn)
 {
     int i, j, pos = 0;
     int ccount = 0;
@@ -742,14 +734,21 @@ static int print_tsls_instruments (LIST list, const DATAINFO *pdinfo, PRN *prn)
 		}
 	    }
 	    if (!dup) {
-		if (tex) tex_escape(vname, pdinfo->varname[list[i]]);
-		else strcpy(vname, pdinfo->varname[list[i]]);
+		if (tex) {
+		    tex_escape(vname, pdinfo->varname[list[i]]);
+		} else {
+		    strcpy(vname, pdinfo->varname[list[i]]);
+		}
 		pprintf(prn, "%s ", vname);
 		ccount += strlen(vname) + 1;
 		if (ccount >= 76) {
-		    if (tex) pputs(prn, "\\\\\n");
-		    else if (RTF_FORMAT(prn->format)) pputs(prn, "\\par\n");
-		    else pputs(prn, "\n  "); 
+		    if (tex) {
+			pputs(prn, "\\\\\n");
+		    } else if (RTF_FORMAT(prn->format)) {
+			pputs(prn, "\\par\n");
+		    } else {
+			pputs(prn, "\n  "); 
+		    }
 		    ccount = 0;
 		}
 	    }
@@ -757,9 +756,13 @@ static int print_tsls_instruments (LIST list, const DATAINFO *pdinfo, PRN *prn)
     }
 
     if (ccount > 0) {
-	if (tex) pputs(prn, "\\\\\n");
-	else if (RTF_FORMAT(prn->format)) pputs(prn, "\\par\n");
-	else pputs(prn, "\n");
+	if (tex) {
+	    pputs(prn, "\\\\\n");
+	} else if (RTF_FORMAT(prn->format)) {
+	    pputs(prn, "\\par\n");
+	} else {
+	    pputs(prn, "\n");
+	}
     }
 
     return 0;
@@ -859,8 +862,6 @@ static void print_model_heading (const MODEL *pmod,
     int tex = TEX_FORMAT(prn->format);
     int utf = PLAIN_FORMAT(prn->format);
 
-    modelprint_setup_obs(pmod, &t2);
-
     ntodate(startdate, t1, pdinfo);
     ntodate(enddate, t2, pdinfo);
 
@@ -871,6 +872,8 @@ static void print_model_heading (const MODEL *pmod,
     case AUX_CHOW:
     case AUX_COINT:
     case AUX_ADF:
+    case AUX_DF:
+    case AUX_KPSS:
     case AUX_RESET:
 	if (utf) {
 	    pprintf(prn, "\n%s\n", _(aux_string(pmod->aux, prn->format)));
@@ -927,17 +930,38 @@ static void print_model_heading (const MODEL *pmod,
 		_(gretl_system_short_string(pmod)),
 		pmod->nobs, startdate, (tex)? "--" : "-", enddate);
     } else if (!dataset_is_panel(pdinfo)) {
-	pprintf(prn, (utf)?
-		_("%s estimates using the %d observations %s%s%s") :
-		I_("%s estimates using the %d observations %s%s%s"),
-		_(my_estimator_string(pmod, prn->format)), 
-		pmod->nobs, startdate, (tex)? "--" : "-", enddate);
+	if (pmod->missmask != NULL) {
+	    int mc = model_missval_count(pmod);
+
+	    pprintf(prn, (utf)?
+		    _("%s estimates using %d observations from %s%s%s") :
+		    I_("%s estimates using %d observations from %s%s%s"),
+		    _(my_estimator_string(pmod, prn->format)), 
+		    pmod->nobs, startdate, (tex)? "--" : "-", enddate);
+	    model_print_newline(prn);
+	    pprintf(prn, "%s: %d",
+		    (utf)? _("Missing or incomplete observations dropped") :
+		    I_("Missing or incomplete observations dropped"), mc);
+	} else {
+	    pprintf(prn, (utf)?
+		    _("%s estimates using the %d observations %s%s%s") :
+		    I_("%s estimates using the %d observations %s%s%s"),
+		    _(my_estimator_string(pmod, prn->format)), 
+		    pmod->nobs, startdate, (tex)? "--" : "-", enddate);
+	}
     } else {
+	int effn = gretl_model_get_int(pmod, "n_included_units");
+
 	pprintf(prn, (utf)?
 		_("%s estimates using %d observations") :
 		I_("%s estimates using %d observations"),
 		_(my_estimator_string(pmod, prn->format)), 
 		pmod->nobs);
+	if (effn > 0) {
+	    model_print_newline(prn);
+	    pprintf(prn, (utf)? _("Included %d cross-sectional units") :
+		    I_("Included %d cross-sectional units"), effn);
+	}
     }
 
     model_print_newline(prn);
@@ -995,6 +1019,16 @@ static void print_model_heading (const MODEL *pmod,
 	garch_vcv_line(pmod, prn);
     }
 
+    /* WLS on panel data */
+    else if (gretl_model_get_int(pmod, "unit_weights")) {
+	if (tex) {
+	    pputs(prn, "\\\\\n");
+	}
+	pprintf(prn, (utf)? _("Weights based on per-unit error variances") : 
+		I_("Weights based on per-unit error variances"));
+	pputc(prn, '\n');
+    }
+
     /* weight variable for WLS and ARCH */
     else if (pmod->ci == WLS || pmod->ci == ARCH) {
 	if (tex) {
@@ -1009,7 +1043,7 @@ static void print_model_heading (const MODEL *pmod,
     /* rhohat for CORC and HILU (TeX) */
     else if (pmod->ci == CORC || pmod->ci == HILU || pmod->ci == PWE) {
 	if (tex) {
-	    pprintf(prn, "\\\\ \n$\\hat{\\rho}$ = %g\n", 
+	    pprintf(prn, "$\\hat{\\rho}$ = %g\n", 
 		    gretl_model_get_double(pmod, "rho_in"));
 	}
     } 
@@ -1021,28 +1055,26 @@ static void print_model_heading (const MODEL *pmod,
 	pprintf(prn, "%s\n", gretl_msg);
     }
 
-    if (gretl_model_get_int(pmod, "wt_dummy")) { /* FIXME alt formats */
+    if (pmod->missmask == NULL && gretl_model_get_int(pmod, "wt_dummy")) { 
+	/* FIXME alt formats */
 	pprintf(prn, "%s %d\n", 
 		(utf)? _("Weight var is a dummy variable, effective obs =") :
 		I_("Weight var is a dummy variable, effective obs ="),
 		pmod->nobs);
-    }
+    } 
 
-    if (RTF_FORMAT(prn->format)) pputs(prn, "\\par\n");
-    else pputc(prn, '\n');
+    if (RTF_FORMAT(prn->format)) {
+	pputs(prn, "\\par\n");
+    } else {
+	pputc(prn, '\n');
+    }
 }
 
 static void model_format_start (PRN *prn)
 {
     if (TEX_FORMAT(prn->format)) {
 	if (STANDALONE(prn->format)) {
-	    pputs(prn, "\\documentclass{article}\n"
-		    "\\usepackage{dcolumn}\n");
-#ifdef ENABLE_NLS
-	    pputs(prn, "\\usepackage[latin1]{inputenc}\n");
-#endif
-	    pputs(prn, "\\begin{document}\n\n"
-		    "\\thispagestyle{empty}\n");
+	    gretl_tex_preamble(prn, 0);
 	}
 	pputs(prn, "\\begin{center}\n");
     }
@@ -1156,11 +1188,9 @@ static void print_coeff_table_end (PRN *prn)
 {
     if (PLAIN_FORMAT(prn->format)) {
 	pputs(prn, "\n");
-    }
-    else if (TEX_FORMAT(prn->format)) {
+    } else if (TEX_FORMAT(prn->format)) {
 	pputs(prn, "\\end{tabular*}\n\n");
-    }
-    else if (RTF_FORMAT(prn->format)) {
+    } else if (RTF_FORMAT(prn->format)) {
 	pputs(prn, "}\n\n");
     }
 }
@@ -1179,28 +1209,44 @@ static void model_format_end (PRN *prn)
     }
 } 
 
-static int print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
+static int 
+print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 {
     int i, err = 0, gotnan = 0;
     int n = pmod->ncoeff;
+    int longnames = 0;
     int gn = -1;
 
-    if (pmod->ci == GARCH) gn = pmod->list[0] - 4;
+    if (pmod->ci == GARCH) {
+	gn = pmod->list[0] - 4;
+    }
+
+    if (PLAIN_FORMAT(prn->format) && 
+	pmod->aux != AUX_ARCH &&
+	pmod->ci != NLS && 
+	pmod->ci != ARMA && 
+	pmod->ci != GARCH) {
+	for (i=0; i<n; i++) {
+	    int len = strlen(pdinfo->varname[pmod->list[i+2]]);
+
+	    if (len > 8) {
+		longnames = 1;
+	    }
+	}
+    }
 
     for (i=0; i<n; i++) {
 	if (PLAIN_FORMAT(prn->format)) {
 	    if (i == gn) {
 		pputc(prn, '\n');
 	    }
-	    err = print_coeff(pdinfo, pmod, i + 2, prn);
-	}
-	else if (TEX_FORMAT(prn->format)) {
+	    err = print_coeff(pdinfo, pmod, i + 2, longnames, prn);
+	} else if (TEX_FORMAT(prn->format)) {
 	    if (i == gn) {
 		pputs(prn, "\\\\ \n");
 	    }
 	    err = tex_print_coeff(pdinfo, pmod, i + 2, prn);
-	}
-	else if (RTF_FORMAT(prn->format)) {
+	} else if (RTF_FORMAT(prn->format)) {
 	    if (i == gn) {
 		pputc(prn, '\n');
 	    }
@@ -1323,19 +1369,23 @@ static void print_whites_results (const MODEL *pmod, PRN *prn)
  * printmodel:
  * @pmod: pointer to gretl model.
  * @pdinfo: data information struct.
+ * @opt: may contain OPT_O to print covariance matrix.
  * @prn: gretl printing struct.
  *
  * Print to @prn the estimates in @pmod plus associated statistics.
  * 
- * Returns: 0 on success, 1 if some of the values to print were NAN.
+ * Returns: 0 on success, 1 if some of the values to print were %NaN.
  */
 
-int printmodel (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
+int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt, 
+		PRN *prn)
 {
     int gotnan = 0;
     int is_discrete = (pmod->ci == PROBIT || pmod->ci == LOGIT);
 
-    if (prn == NULL) return 0;
+    if (prn == NULL || (opt & OPT_Q)) {
+	return 0;
+    }
 
     if (prn->format != GRETL_PRINT_FORMAT_PLAIN) {
 	model_format_start(prn);
@@ -1360,7 +1410,8 @@ int printmodel (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
     print_coeff_table_end (prn);
 
     if (pmod->aux == AUX_ARCH || pmod->aux == AUX_ADF || 
-	pmod->aux == AUX_RESET || pmod->aux == AUX_SCR) {
+	pmod->aux == AUX_RESET || pmod->aux == AUX_SCR ||
+	pmod->aux == AUX_DF || pmod->aux == AUX_KPSS) {
 	goto close_format;
     }
 
@@ -1374,7 +1425,6 @@ int printmodel (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 	depvarstats(pmod, prn);
 	print_tobit_stats(pmod, prn);
 	print_middle_table_end(prn);
-	print_model_tests(pmod, prn);
 	goto close_format;
     }	
 
@@ -1515,24 +1565,43 @@ int printmodel (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
     }
 
     if (!(pmod->ci == ARMA && !na(pmod->lnL))) {
-	if (PLAIN_FORMAT(prn->format)) print_aicetc(pmod, prn);
-	else if (TEX_FORMAT(prn->format)) tex_print_aicetc(pmod, prn);
-	else if (RTF_FORMAT(prn->format)) rtf_print_aicetc(pmod, prn);
+	if (PLAIN_FORMAT(prn->format)) {
+	    print_aicetc(pmod, prn);
+	} else if (TEX_FORMAT(prn->format)) {
+	    tex_print_aicetc(pmod, prn);
+	} else if (RTF_FORMAT(prn->format)) {
+	    rtf_print_aicetc(pmod, prn);
+	}
     }
 
-    if (pmod->ci != ARMA && pmod->ci != NLS) {
+    if (PLAIN_FORMAT(prn->format) && 
+	pmod->ci != ARMA && pmod->ci != NLS) {
 	pval_max_line(pmod, pdinfo, prn);
     }
-    
-    print_model_tests(pmod, prn);
 
  close_format:
+
+    if (pmod->ci != LAD && (opt & OPT_O)) {
+	outcovmx(pmod, pdinfo, prn);
+    }
+
+    if (pmod->ntests > 0) {
+	print_model_tests(pmod, prn);
+    }
+
+#if 0
     if (PLAIN_FORMAT(prn->format) && pmod->aux == AUX_ADF) {
 	print_aicetc(pmod, prn);
     }
+#endif
+
     if (!PLAIN_FORMAT(prn->format)) {
 	model_format_end(prn);
     }
+    
+    if (gotnan) {
+	pmod->errcode = E_NAN;
+    }
 
     return gotnan;
 }
@@ -1548,7 +1617,7 @@ static void print_aicetc_value (double x, PRN *prn)
 
     if ((p = strstr(dstr, "+00")) ||
 	(p = strstr(dstr, "-00"))) {
-	memmove(p+1, p+2, strlen(p+2));
+	memmove(p+1, p+2, strlen(p+1));
     }    
 
     len = strlen(dstr) - 1;
@@ -1565,32 +1634,43 @@ static void print_aicetc (const MODEL *pmod, PRN *prn)
 {
     if (pmod->aux == AUX_SQ || pmod->aux == AUX_LOG ||
 	pmod->aux == AUX_COINT || pmod->aux == AUX_WHITE ||
-	pmod->aux == AUX_AR) return;
+	pmod->aux == AUX_AR) {
+	return;
+    }
 
     if (pmod->dfd == 0) {
 	pputs(prn, "\n");
 	return;
     }
 
-    pprintf(prn, "  %s\n\n", _("MODEL SELECTION STATISTICS"));	
+    pprintf(prn, "  %s\n\n", _("MODEL SELECTION STATISTICS"));
+	
     pputs(prn, "  SGMASQ    ");
     print_aicetc_value(pmod->criterion[C_SGMASQ], prn);
+
     pputs(prn, "     AIC       ");
     print_aicetc_value(pmod->criterion[C_AIC], prn);
+
     pputs(prn, "     FPE       ");
     print_aicetc_value(pmod->criterion[C_FPE], prn);
+
     pputs(prn, "\n  HQ        ");
     print_aicetc_value(pmod->criterion[C_HQ], prn);
+
     pputs(prn, "     SCHWARZ   ");
     print_aicetc_value(pmod->criterion[C_BIC], prn);
+
     pputs(prn, "     SHIBATA   ");
     print_aicetc_value(pmod->criterion[C_SHIBATA], prn);
+
     pputs(prn, "\n  GCV       ");
     print_aicetc_value(pmod->criterion[C_GCV], prn);
+
     pputs(prn, "     RICE      ");
-    if (pmod->criterion[C_RICE] > 0.0) {
+    if (!na(pmod->criterion[C_RICE])) {
 	print_aicetc_value(pmod->criterion[C_RICE], prn);
     } else {
+	pputs(prn, "  ");
 	pputs(prn, _("undefined"));
     }
     pputs(prn, "\n\n");
@@ -1634,14 +1714,14 @@ static void print_pval_str (double pval, char *str)
 /* ......................................................... */ 
 
 static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			int c, PRN *prn)
+			int c, int longnames, PRN *prn)
 {
     double t, pvalue = 999.0;
     int gotnan = 0;
     int do_pval = (pmod->ci != LOGIT && pmod->ci != PROBIT);
-    char varname[12];
+    char varname[24];
 
-    /* special treatment for ARCH model coefficients, NLS, ARMA */
+    /* special treatment for ARCH model coefficients, NLS, ARMA, VAR */
     if (pmod->aux == AUX_ARCH) {
 	make_cname(pdinfo->varname[pmod->list[c]], varname);
     } else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
@@ -1650,7 +1730,9 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	strcpy(varname, pdinfo->varname[pmod->list[c]]);
     }
 
-    if (pmod->ci == GARCH) {
+    if (longnames) {
+	pprintf(prn, " %13s ", varname);
+    } else if (pmod->ci == GARCH) {
 	pprintf(prn, "      %8s ", varname);
     } else if (pmod->ci == ARMA) {
 	pprintf(prn, " %3d) %8s ", c - 1, varname);
@@ -1658,14 +1740,14 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	pprintf(prn, " %3d) %8s ", pmod->list[c], varname);
     }
 
-    bufspace(2, prn);
+    bufspace((strlen(varname) > 12)? 1 : 2, prn);
 
     /* print coeff value if well-defined */
     if (isnan(pmod->coeff[c-2]) || na(pmod->coeff[c-2])) {
 	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 17), _("undefined"));
 	gotnan = 1;
     } else {
-	gretl_print_value (pmod->coeff[c-2], prn);
+	gretl_print_value(pmod->coeff[c-2], prn);
     }
 
     bufspace(2, prn);
@@ -1689,9 +1771,15 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	} else {
 	    pprintf(prn, " %7.3f", t);
 	}
-	if (pmod->aux == AUX_ADF) {
+	if (pmod->aux == AUX_ADF || pmod->aux == AUX_DF) {
+	    if (c == gretl_model_get_int(pmod, "dfnum")) {
+		char pvalstr[16];
+
+		pvalue = gretl_model_get_double(pmod, "dfpval");
+		print_pval_str(pvalue, pvalstr);
+		pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 12), pvalstr);
+	    } 
 	    do_pval = 0;
-	    pprintf(prn, "%*s", UTF_WIDTH(_("unknown"), 12), _("unknown"));
 	}
 	if (do_pval) {
 	    char pvalstr[16];
@@ -1747,7 +1835,7 @@ static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     char varname[12];
 
 
-    /* special treatment for ARCH model coefficients, NLS, ARMA */
+    /* special treatment for ARCH model coefficients, NLS, ARMA, VAR */
     if (pmod->aux == AUX_ARCH) {
 	make_cname(pdinfo->varname[pmod->list[c]], varname);
     } else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
@@ -1781,7 +1869,7 @@ static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     if (pmod->sderr[c-2] > 0.) {
 	t = pmod->coeff[c-2] / pmod->sderr[c-2];
 	pprintf(prn, " \\qc %.4f\\cell", t);
-	if (pmod->aux == AUX_ADF) {
+	if (pmod->aux == AUX_ADF || pmod->aux == AUX_DF) {
 	    do_pval = 0;
 	    pprintf(prn, " \\qc %s\\cell", I_("unknown"));
 	}
@@ -2174,10 +2262,11 @@ static void tex_print_aicetc (const MODEL *pmod, PRN *prn)
 	    pmod->criterion[C_BIC], pmod->criterion[C_SHIBATA],
 	    pmod->criterion[C_GCV]);
 
-    if (pmod->criterion[C_RICE] > 0.0) 
+    if (!na(pmod->criterion[C_RICE])) {
 	pprintf(prn, "\\textsc{rice}    & %g\n", pmod->criterion[C_RICE]);
-    else
+    } else {
 	pprintf(prn, "\\textsc{rice}    & %s\n", I_("undefined"));
+    }
     
     pputs(prn, "\\end{tabular*}\n\n");
 }
@@ -2224,10 +2313,11 @@ static void rtf_print_aicetc (const MODEL *pmod, PRN *prn)
 	    crit_strs[0], crit_strs[1], 
 	    crit_strs[2], crit_strs[3], 
 	    crit_strs[4], crit_strs[5], crit_strs[6]);
-    if (pmod->criterion[7] > 0.0) 
+    if (!na(pmod->criterion[C_RICE])) {
 	pprintf(prn, " \\qc %s\\cell", crit_strs[7]);
-    else
+    } else {
 	pprintf(prn, " \\qc %s\\cell", I_("undefined"));
+    }
     pputs(prn, " \\qr \\cell \\qr \\cell");
 
     pputs(prn, " \\intbl \\row}\n\n");
@@ -2260,11 +2350,13 @@ static void mp_other_stats (const mp_results *mpvals, PRN *prn)
 
     sprintf(fstr, "F(%d, %d)", mpvals->dfn, mpvals->dfd);
     pprintf(prn, "%-*s", len, fstr);
+
     if (na(mpvals->fstt)) {
 	pprintf(prn, "            %s", _("undefined"));
     } else {
 	gretl_print_fullwidth_double(mpvals->fstt, GRETL_MP_DIGITS, prn);
     }
+
     pputs(prn, "\n\n");
 }
 
