@@ -76,12 +76,12 @@ static int nls_auto_gen (int i)
     int err;
 
     if (i == 0) {
-	sprintf(formula, "$nls_y = %s", nlspec.nlfunc);
+	sprintf(formula, "$nl_y = %s", nlspec.nlfunc);
     } else {
-	sprintf(formula, "$nls_x%d = %s", i, nlspec.terms[i-1].deriv);
+	sprintf(formula, "$nl_x%d = %s", i, nlspec.terms[i-1].deriv);
     }
 
-    err = generate(pZ, pdinfo, formula, 0, NULL, 0);
+    err = generate(pZ, pdinfo, formula, NULL);
 
     if (err) {
 	errmsg(err, prn);
@@ -97,7 +97,7 @@ static int add_term_from_nlfunc (const char *vname)
     int i, v, nt = nlspec.nparam + 1; 
 
     /* if term is math function or constant, skip */
-    if (get_function(vname) || !strcmp(vname, "pi")) return 0;
+    if (get_genr_function(vname) || !strcmp(vname, "pi")) return 0;
 
     v = varindex(pdinfo, vname);
     if (v >= pdinfo->v) {
@@ -186,7 +186,9 @@ static int check_for_missing_vals (void)
 	else break;
     }
 
-    if (t2 - t1 + 1 < nlspec.nparam) return E_DF;
+    if (t2 - t1 + 1 < nlspec.nparam) {
+	return E_DF;
+    }
 
     for (t=t1; t<=t2; t++) {
 	if (na((*pZ)[v][t])) {
@@ -222,7 +224,7 @@ static int get_resid (double *fvec)
 
     if (nls_auto_gen(0)) return 1;
 
-    v = varindex(pdinfo, "$nls_y");
+    v = varindex(pdinfo, "$nl_y");
     if (v == pdinfo->v) return 1;
 
     nlspec.ess = 0.0;
@@ -232,6 +234,7 @@ static int get_resid (double *fvec)
 	nlspec.ess += fvec[j] * fvec[j];
 	j++;
     }
+
     print_iter_ess();
 
     return 0;
@@ -244,7 +247,7 @@ static int get_deriv (int i, double *deriv)
 
     if (nls_auto_gen(i + 1)) return 1;
 
-    sprintf(varname, "$nls_x%d", i + 1);
+    sprintf(varname, "$nl_x%d", i + 1);
     v = varindex(pdinfo, varname);
     if (v == pdinfo->v) return 1;
 
@@ -252,8 +255,11 @@ static int get_deriv (int i, double *deriv)
 
     j = 0;
     for (t=nlspec.t1; t<=nlspec.t2; t++) {
-	if (vec) deriv[j] = - (*pZ)[v][t];
-	else deriv[j] = - (*pZ)[v][0];
+	if (vec) {
+	    deriv[j] = - (*pZ)[v][t];
+	} else {
+	    deriv[j] = - (*pZ)[v][0];
+	}
 	j++;
     }
 
@@ -310,6 +316,7 @@ static void add_stats_to_model (MODEL *pmod)
 	d = (*pZ)[nlspec.depvar][t] - pmod->ybar;
 	tss += d * d;
     }    
+
     pmod->rsq = 1.0 - nlspec.ess / tss;
     pmod->adjrsq = NADBL;
 }
@@ -321,10 +328,14 @@ static int add_std_errs_to_model (MODEL *pmod)
     if (pmod->vcv == NULL && makevcv(pmod)) return E_ALLOC;
 
     for (i=0; i<pmod->ncoeff; i++) {
-	k = ijton(i+1, i+1, pmod->ncoeff);
-	if (pmod->vcv[k] == 0.0) pmod->sderr[i] = 0.0;
-	else if (pmod->vcv[k] > 0.0) pmod->sderr[i] = sqrt(pmod->vcv[k]);
-	else pmod->sderr[i] = NADBL;
+	k = ijton(i, i, pmod->ncoeff);
+	if (pmod->vcv[k] == 0.0) {
+	    pmod->sderr[i] = 0.0;
+	} else if (pmod->vcv[k] > 0.0) {
+	    pmod->sderr[i] = sqrt(pmod->vcv[k]);
+	} else {
+	    pmod->sderr[i] = NADBL;
+	}
     }
 
     return 0;
@@ -400,9 +411,17 @@ static MODEL GNR (double *fvec, double *fjac)
     nlspec.t1 = 0;
     nlspec.t2 = pdinfo->n - 1;
 
+    if (gretl_iszero(0, T-1, fvec)) {
+	pputs(prn, _("Perfect fit achieved\n"));
+	for (t=0; t<T; t++) {
+	    fvec[t] = 1.0;
+	}
+	nlspec.ess = 0.0;
+    }
+
     ninfo = create_new_dataset(&nZ, nlspec.nparam + 1, pdinfo->n, 0);
     if (ninfo == NULL) {
-	gretl_model_init(&gnr, NULL);
+	gretl_model_init(&gnr);
 	gnr.errcode = E_ALLOC;
 	return gnr;
     }
@@ -411,7 +430,7 @@ static MODEL GNR (double *fvec, double *fjac)
     if (nlist == NULL) {
 	free_Z(nZ, ninfo);
 	free_datainfo(ninfo);
-	gretl_model_init(&gnr, NULL);
+	gretl_model_init(&gnr);
 	gnr.errcode = E_ALLOC;
 	return gnr;
     }
@@ -424,8 +443,11 @@ static MODEL GNR (double *fvec, double *fjac)
 	    /* dependent variable (NLS residual) */
 	    j = 0;
 	    for (t=0; t<pdinfo->n; t++) {
-		if (t < t1 || t > t2) nZ[i][t] = NADBL;
-		else nZ[i][t] = fvec[j++];
+		if (t < t1 || t > t2) {
+		    nZ[i][t] = NADBL;
+		} else {
+		    nZ[i][t] = fvec[j++];
+		}
 	    }
 	} else {
 	    if (nlspec.mode == ANALYTIC_DERIVS) {
@@ -433,8 +455,11 @@ static MODEL GNR (double *fvec, double *fjac)
 	    } else {
 		j = T * (i - 1);
 		for (t=0; t<pdinfo->n; t++) {
-		    if (t < t1 || t > t2) nZ[i][t] = NADBL;
-		    else nZ[i][t] = fjac[j++];
+		    if (t < t1 || t > t2) {
+			nZ[i][t] = NADBL;
+		    } else {
+			nZ[i][t] = fjac[j++];
+		    }
 		}
 	    }
 	}
@@ -517,12 +542,13 @@ static int nls_spec_start (const char *nlfunc, const DATAINFO *dinfo)
 	clear_nls_spec();
     }
 
-    if (strncmp(nlfunc, "nls ", 4) == 0) 
+    if (strncmp(nlfunc, "nls ", 4) == 0) { 
 	p = nlfunc + 4;
-    else 
+    } else {
 	p = nlfunc;
+    }
 
-    if (sscanf(p, "%8s = %*s", depvarname) != 1) {
+    if (strchr(p, '=') == NULL || sscanf(p, "%8s = %*s", depvarname) != 1) {
 	sprintf(gretl_errmsg, _("parse error in '%s'\n"), p);
 	return E_PARSE;
     }
@@ -538,6 +564,7 @@ static int nls_spec_start (const char *nlfunc, const DATAINFO *dinfo)
 
     p = strchr(p, '=') + 1;
     while (isspace(*p)) p++;
+
     sprintf(nlspec.nlfunc, "%s - (%s)", depvarname, p);
 
     nlspec.depvar = v;
@@ -554,12 +581,13 @@ static int nls_spec_start (const char *nlfunc, const DATAINFO *dinfo)
 static int parse_deriv_line (const char *line, int i, nls_term *term,
 			     const double **Z, const DATAINFO *dinfo)
 {
-    int v;
-    int err = 0;
     const char *p;
+    int v, err = 0;
 
     term->deriv = malloc(strlen(line) - 10);
-    if (term->deriv == NULL) return E_ALLOC;
+    if (term->deriv == NULL) {
+	return E_ALLOC;
+    }
 
     if (sscanf(line, "deriv %8s = %s", term->name, term->deriv) != 2) {
 	free(term->deriv);
@@ -625,8 +653,7 @@ int nls_parse_line (const char *line, const double **Z,
 {
     if (strncmp(line, "deriv", 5) == 0) {
 	return nls_spec_add_term(line, Z, dinfo);
-    }
-    else {
+    } else {
 	return nls_spec_start(line, dinfo);
     }
 }
@@ -637,15 +664,15 @@ static int check_derivs (integer m, integer n, double *x,
 {
     integer mode = 1;
     integer iflag;
-    doublereal *xp;
-    doublereal *err;
-    doublereal *fvecp;
-    int i;
-    int badcount = 0, zerocount = 0;
+    doublereal *xp = NULL;
+    doublereal *err = NULL;
+    doublereal *fvecp = NULL;
+    int i, badcount = 0, zerocount = 0;
 
     xp = malloc(m * sizeof *xp);
     err = malloc(m * sizeof *err);
     fvecp = malloc(m * sizeof *fvecp);
+
     if (xp == NULL || err == NULL || fvecp == NULL) {
 	free(err);
 	free(xp);
@@ -656,21 +683,27 @@ static int check_derivs (integer m, integer n, double *x,
     iflag = 1;
     nls_calc(&m, &n, x, fvec, fjac, &ldfjac, &iflag);
     if (iflag == -1) goto chkderiv_abort;
+
     chkder_(&m, &n, x, fvec, fjac, &ldfjac, xp, fvecp, &mode, err);
 
     iflag = 2;
     nls_calc(&m, &n, x, fvec, fjac, &ldfjac, &iflag);
     if (iflag == -1) goto chkderiv_abort;
+
     iflag = 1;
     nls_calc(&m, &n, xp, fvecp, fjac, &ldfjac, &iflag);
     if (iflag == -1) goto chkderiv_abort; 
+
     mode = 2;
     chkder_(&m, &n, x, fvec, fjac, &ldfjac, xp, fvecp, &mode, err);
 
     /* examine "err" vector */
     for (i=0; i<m; i++) {
-	if (err[i] == 0.0) zerocount++;
-	else if (err[i] < 0.35) badcount++;
+	if (err[i] == 0.0) {
+	    zerocount++;
+	} else if (err[i] < 0.35) {
+	    badcount++;
+	}
     }
 
     if (zerocount > 0) {
@@ -734,10 +767,10 @@ static int lm_calculate (double *fvec, double *fjac)
     case 1:
     case 2:
     case 3:
+    case 4: /* is this right? */
 	pprintf(prn, _("Convergence achieved after %d iterations\n"),
 		nlspec.iters);
 	break;
-    case 4:
     case 5:
     case 6:
     case 7:
@@ -857,7 +890,8 @@ MODEL nls (double ***mainZ, DATAINFO *maininfo, PRN *mainprn)
     int origv = maininfo->v;
     int err = 0;
 
-    gretl_model_init(&nlsmod, maininfo);
+    gretl_model_init(&nlsmod);
+    gretl_model_smpl_init(&nlsmod, maininfo);
 
     if (nlspec.nlfunc == NULL) {
 	strcpy(gretl_errmsg, _("No regression function has been specified"));
@@ -874,8 +908,11 @@ MODEL nls (double ***mainZ, DATAINFO *maininfo, PRN *mainprn)
 	err = get_params_from_nlfunc();
 	if (err) {
 	    clear_nls_spec();
-	    if (err == 1) nlsmod.errcode = E_PARSE;
-	    else nlsmod.errcode = err;
+	    if (err == 1) {
+		nlsmod.errcode = E_PARSE;
+	    } else {
+		nlsmod.errcode = err;
+	    }
 	    return nlsmod;
 	}
     }
@@ -931,7 +968,9 @@ MODEL nls (double ***mainZ, DATAINFO *maininfo, PRN *mainprn)
     dataset_drop_vars(pdinfo->v - origv, pZ, pdinfo);
 
     *mainZ = *pZ;
-  
+
+    set_model_id(&nlsmod);
+
     return nlsmod;
 }
 
