@@ -20,11 +20,12 @@
 #ifndef OPTIONS_H
 #define OPTIONS_H
 
-int catchflags (char *line, unsigned long *oflag);
+gretlopt get_gretl_options (char *line, int *err);
 
-const char *print_flags (unsigned long flags, int ci);
+const char *print_flags (gretlopt oflags, int ci);
 
 const char **get_opts_for_command (int ci);
 
+int check_for_loop_only_options (int ci, gretlopt opt, PRN *prn);
 
 #endif /* OPTIONS_H */
