@@ -22,6 +22,8 @@
 #define GRETL_DIGITS 6
 #define GRETL_MP_DIGITS 12
 
+#define PAGELINES 21
+
 /* functions follow */
  
 void session_time (FILE *fp);
@@ -30,14 +32,15 @@ void logo (void);
 
 void lib_logo (void);
 
+void gui_script_logo (PRN *prn);
+
 void gui_logo (FILE *fp);
 
 void text_print_model_confints (const CONFINT *cf,
 				const DATAINFO *pdinfo, 
 				PRN *prn);
 
-void printfreq (FREQDIST *freq, 
-		PRN *prn);
+void print_freq (const FREQDIST *freq, PRN *prn);
 
 void printcorr (const CORRMAT *corrmat, 
 		const DATAINFO *pdinfo, 
@@ -48,15 +51,14 @@ void print_smpl (const DATAINFO *pdinfo,
 
 int outcovmx (MODEL *pmod, 
 	      const DATAINFO *pdinfo, 
-	      int pause, PRN *prn);
+	      PRN *prn);
 
-int page_break (int n, int *lineno, int quit_option);
+int page_pause (void);
 
 void print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn);
 
-int printdata (LIST list, 
-	       double ***pZ, const DATAINFO *pdinfo, 
-	       int pause, unsigned long oflag, PRN *prn);
+int printdata (int *list, const double **Z, const DATAINFO *pdinfo, 
+	       gretlopt oflag, PRN *prn);
 
 int text_print_fit_resid (const FITRESID *fr, 
 			  const DATAINFO *pdinfo, 
@@ -68,11 +70,11 @@ int print_fit_resid (const MODEL *pmod,
 
 int text_print_fcast_with_errs (const FITRESID *fr, 
 				double ***pZ, DATAINFO *pdinfo, PRN *prn,
-				PATHS *ppaths, int plot);
+				int plot);
 
 void text_print_matrix (const double *rr, const int *list, 
 			MODEL *pmod, const DATAINFO *pdinfo, 
-			int pause, PRN *prn);
+			PRN *prn);
 
 void gretl_print_fullwidth_double (double x, int digits, PRN *prn);
 
@@ -98,6 +100,8 @@ int do_printf (const char *line, double ***pZ,
 	       DATAINFO *pdinfo, MODEL *pmod,
 	       PRN *prn);
 
+int generate_obs_markers (double ***pZ, DATAINFO *pdinfo, char *s);
+
 int in_usa (void);
 
 char *bufgets (char *s, size_t size, const char *buf);
