@@ -22,6 +22,7 @@
 */  
 
 #include "libgretl.h" 
+#include "gretl_private.h"
 #include "../../cephes/libprob.h"
  
 static void putxx (double xx);
@@ -97,8 +98,15 @@ double tprob (double x, int df)
 
 double fdist (double x, int dfn, int dfd)
 {
-    if (x < 0) return 1.0;
-    else return fdtrc(dfn, dfd, x);
+    double ret = 1.0;
+
+    if (dfn < 0 || dfd < 0) {
+	ret = -1.0;
+    } else if (x > 0.0) {
+	ret = fdtrc(dfn, dfd, x);
+    }
+
+    return ret;
 }
 
 /**
@@ -112,8 +120,15 @@ double fdist (double x, int dfn, int dfd)
 
 double chisq (double x, int df)
 {
-    if (x < 0) return 1.0;
-    else return 1.0 - chdtr(df, x);
+    double ret = 1.0;
+
+    if (df < 0) {
+	ret = -1.0;
+    } else if (x > 0.0) {
+	ret = 1.0 - chdtr(df, x);
+    }
+
+    return ret;
 }
 
 /**
@@ -140,6 +155,28 @@ double normal_pdf (double x)
     return (1.0 / sqrt(2.0 * M_PI)) * exp(-0.5 * x * x);
 }
 
+double log_normal_pdf (double x)
+{
+    return (x * x) / 2.0 - 0.91893853320467274178;
+}
+
+static double get_number_or_val (const char *s, 
+				 const double **Z,
+				 const DATAINFO *pdinfo)
+{
+    if (numeric_string(s)) {
+	return dot_atof(s);
+    } else {
+	int v = varindex(pdinfo, s);
+
+	if (v > 0 && v < pdinfo->v && !pdinfo->vector[v]) {
+	    return Z[v][0];
+	}
+    } 
+
+    return NADBL;
+}
+
 /**
  * batch_pvalue:
  * @str: the command line, which should be of one of the following forms:
@@ -159,7 +196,7 @@ double normal_pdf (double x)
  */
 
 double batch_pvalue (const char *str, 
-		     double **Z, const DATAINFO *pdinfo, 
+		     const double **Z, const DATAINFO *pdinfo, 
                      PRN *prn)
 {
     int i, df1 = 0, df2 = 0;
@@ -254,12 +291,15 @@ double batch_pvalue (const char *str,
 	    err = 1;
 	}
     } else {
-	if (*fstr && check_atof(fstr)) err = 1;
-	else xval = atof(fstr);
+	if (*fstr && check_atof(fstr)) {
+	    err = 1;
+	} else {
+	    xval = atof(fstr);
+	}
     }
 
     if (err) {
-	pprintf(prn, "%s\n", gretl_errmsg);
+	print_gretl_errmsg(prn);
 	return NADBL;
     }
 
@@ -288,16 +328,17 @@ double batch_pvalue (const char *str,
 	    pputs(prn, _("\npvalue for t: missing parameter\n"));
 	    return -1;
 	}
-	xx = tprob(xval, df1);
+	xx = tprob(xval, df1); /* this is two-tailed */
 	if (xx < 0) {
 	    pputs(prn, _("\np-value calculation failed\n"));
 	    return -1;
 	}
+	xx *= 0.5;
 	pprintf(prn, _("\nt(%d): area to the %s of %g = %g\n"), 
 		df1, (xval > 0)? _("right"): _("left"),
-		xval, 0.5 * xx);
+		xval, xx);
 	pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
-		xx, 1.0 - xx);
+		2.0 * xx, 1.0 - 2.0 * xx);
 	return xx;
 
     case '3':
@@ -547,13 +588,39 @@ static void getdf (const char *str)
 
 double f_crit_a (double a, int df1, int df2)
 {
-    double x = 1.0;
+    double add = 0.5, x = 1.0;
+
+    if (fdist(x, df1, df2) < a) return NADBL;
+
+    while (add > 1.0e-7) {
+	while (fdist(x, df1, df2) > a) x += add; 
+	if (x > add) x -= add;
+	if (add == 0.5) {
+	    add /= 5.0;
+	} else {
+	    add /= 10.0;
+	}
+    }
+
+    return x;
+}
+
+static double chi_crit_a (double a, int df)
+{
+    double add = 0.5, x = 1.0;
+
+    if (chisq(x, df) < a) return NADBL;
+
+    while (add > 1.0e-7) {
+	while (chisq(x, df) > a) x += add; 
+	if (x > add) x -= add;
+	if (add == 0.5) {
+	    add /= 5.0;
+	} else {
+	    add /= 10.0;
+	}
+    }
 
-    while (fdist(x, df1, df2) > a) x += .5;
-    if (x > .5) x -= .5;
-    while (fdist(x, df1, df2) > a) x += .1; 
-    if (x > .5) x -= .1;
-    while (fdist(x, df1, df2) > a) x += .01;
     return x;
 }
 
@@ -569,9 +636,8 @@ static int parse_critical_input (const char *str, int *i,
     else if (sscanf(str, "critical t %d", df) == 1) *i = 1;
     else if (sscanf(str, "critical d %d", n) == 1) *i = 4;
     else if (!strncmp(str, "critical N", 10)) *i = 0;
-    
-    if (*i >= 0) return 0;
-    else return 1;
+
+    return (*i == -1);
 }
 
 /**
@@ -670,6 +736,89 @@ int print_critical (const char *line, PRN *prn)
     return 0;
 }
 
+static int parse_genr_critical_input (const char *str, 
+				      const double **Z, 
+				      const DATAINFO *pdinfo,
+				      int *i, int *dfn, int *dfd, 
+				      double *a)
+{
+    char dfnstr[VNAMELEN], dfdstr[VNAMELEN];
+    char astr[VNAMELEN];
+    double val;
+    *i = -1;
+
+    dfnstr[0] = dfdstr[0] = astr[0] = '\0';
+
+    if (sscanf(str, "F,%8[^,],%8[^,],%8s", dfnstr, dfdstr, astr) == 3) {
+	*i = 3;
+    } else if (sscanf(str, "X,%8[^,],%8s", dfnstr, astr) == 2) {
+	*i = 2;
+    } else if (sscanf(str, "t,%8[^,],%8s", dfnstr, astr) == 2) {
+	*i = 1;
+    } else if (sscanf(str, "N,%8s", astr) == 1) {
+	*i = 0;
+	*dfn = 500;
+    }
+
+    if (*i == -1) return 1;
+
+    if (dfnstr[0] != '\0') {
+	val = get_number_or_val(dfnstr, Z, pdinfo);
+	if (na(val)) {
+	    *i = -1;
+	} else {
+	    *dfn = val;
+	}
+    }
+    if (dfdstr[0] != '\0') {
+	val = get_number_or_val(dfdstr, Z, pdinfo);
+	if (na(val)) {
+	    *i = -1;
+	} else {
+	    *dfd = val;
+	}
+    }
+    if (astr[0] != '\0') {
+	*a = get_number_or_val(astr, Z, pdinfo);
+	if (na(*a) || *a < 0.0) {
+	    *i = -1;
+	}
+    }
+
+    return (*i == -1);
+}
+
+double genr_get_critical (const char *line, const double **Z, 
+			  const DATAINFO *pdinfo)
+{
+    double alpha, ret = NADBL;
+    int st, dfn = -1, dfd = -1;
+
+    if (parse_genr_critical_input(line, Z, pdinfo,
+				  &st, &dfn, &dfd, &alpha)) {
+	return NADBL;
+    }
+
+    if ((0 < st && st < 4 && dfn <= 0) || (st == 3 && dfd <= 0)) {
+	strcpy(gretl_errmsg, _("Invalid degrees of freedom\n"));
+	return NADBL;
+    }
+
+    if (st == 3) {
+	ret = f_crit_a(alpha, dfn, dfd);
+    } else if (st == 2) {
+	ret = chi_crit_a(alpha, dfn);
+    } else {
+	ret = sqrt(f_crit_a(2.0 * alpha, 1, dfn));
+    } 
+
+    if (ret < 0) {
+	ret = NADBL;
+    }
+
+    return ret;
+}
+
 /* ........................................................ */
 
 /* Functions relating to the gamma distribution.
