@@ -31,6 +31,8 @@ double normal_cdf (double x);
 
 double normal_pdf (double x);
 
+double log_normal_pdf (double x);
+
 double gamma_dist (double s1, double s2, double x, int control);
 
 double tcrit95 (int df);
@@ -38,7 +40,7 @@ double tcrit95 (int df);
 double rhocrit95 (int n);
 
 double batch_pvalue (const char *str, 
-		     double **Z, const DATAINFO *pdinfo, 
+		     const double **Z, const DATAINFO *pdinfo, 
                      PRN *prn);
 
 void interact_pvalue (void);
@@ -47,4 +49,7 @@ double f_crit_a (double a, int df1, int df2);
 
 int print_critical (const char *line, PRN *prn);
 
+double genr_get_critical (const char *line, const double **Z, 
+			  const DATAINFO *pdinfo);
+
 
