@@ -22,6 +22,7 @@
 #include "gretl_matrix.h"
 #include "gretl_matrix_private.h"
 #include "gretl_private.h"
+#include "libset.h"
 
 #define QR_RCOND_MIN 1e-15 /* experiment with this? */
 #define ESSZERO      1e-22 /* SSR < this counts as zero */
@@ -38,7 +39,9 @@ static double get_tss (const double *y, int n, int ifc)
     double x, tss = 0.0;
     int i;
 
-    if (ifc) ymean = gretl_mean(0, n-1, y);
+    if (ifc) {
+	ymean = gretl_mean(0, n-1, y);
+    }
 
     for (i=0; i<n; i++) {
 	x = y[i] - ymean;
@@ -48,6 +51,20 @@ static double get_tss (const double *y, int n, int ifc)
     return tss;
 }
 
+static void qr_compute_f_stat (MODEL *pmod, gretlopt opts)
+{
+    if (pmod->dfd > 0 && pmod->dfn > 1) {
+	if (opts & OPT_R) {
+	    pmod->fstt = robust_omit_F(NULL, pmod);
+	} else {
+	    pmod->fstt = (pmod->tss - pmod->ess) * pmod->dfd / 
+		(pmod->ess * pmod->dfn);
+	}
+    } else {
+	pmod->fstt = NADBL;
+    }
+}
+
 static void qr_compute_r_squared (MODEL *pmod, const double *y, int n)
 {
     if (pmod->dfd > 0) {
@@ -66,11 +83,8 @@ static void qr_compute_r_squared (MODEL *pmod, const double *y, int n)
 		pmod->adjrsq = 1.0 - ((1 - alt) * (n - 1.0) / pmod->dfd);
 	    }
 	}
-	pmod->fstt = (pmod->tss - pmod->ess) * pmod->dfd / 
-	    (pmod->ess * pmod->dfn);
     } else {
 	pmod->rsq = 1.0;
-	pmod->fstt = NADBL;
     }
 }
 
@@ -79,11 +93,11 @@ int get_vcv_index (MODEL *pmod, int i, int j, int n)
     int k, vi, vj;
 
     if (pmod->ifc) {
-	vi = 1 + (i + 1) % n;
-	vj = 1 + (j + 1) % n;
+	vi = (i + 1) % n;
+	vj = (j + 1) % n;
     } else {
-	vi = i + 1;
-	vj = j + 1;
+	vi = i;
+	vj = j;
     }
 
     k = ijton(vi, vj, n);
@@ -125,10 +139,6 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
     int pwe = gretl_model_get_int(pmod, "pwe");
     int yvar = pmod->list[1];
     double u, y;
-#ifdef OUT_OF_SAMPLE_OK
-    int j;
-    double x;
-#endif
 
     if (dwt) dwt = pmod->nwt;
 
@@ -153,14 +163,16 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 	}
     } else if (pmod->nwt) {
 	for (t=0; t<fulln; t++) {
-	    if (t < pmod->t1 || t > pmod->t2) {
+	    if (t < pmod->t1 || t > pmod->t2 || model_missing(pmod, t)) {
 		pmod->yhat[t] = pmod->uhat[t] = NADBL;
 	    } else {
 		y = Z[yvar][t];
 		if (dwt && Z[dwt][t] == 0.0) {
 		    pmod->yhat[t] = NADBL;
 		} else {
-		    if (!dwt) y *= Z[pmod->nwt][t];
+		    if (!dwt) {
+			y *= Z[pmod->nwt][t];
+		    }
 		    pmod->yhat[t] = yhat->val[i];
 		    pmod->uhat[t] = y - yhat->val[i];
 		    pmod->ess += pmod->uhat[t] * pmod->uhat[t];
@@ -169,36 +181,8 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 	    }
 	}
     } else {
-#ifdef OUT_OF_SAMPLE_OK
 	for (t=0; t<fulln; t++) {
-	    if (t < pmod->t1 || t > pmod->t2) {
-		y = Z[yvar][t];
-		if (na(y)) {
-		    pmod->yhat[t] = pmod->uhat[t] = NADBL;
-		    continue;
-		}
-		pmod->yhat[t] = 0.0;
-		for (j=0; j<pmod->ncoeff; j++) {
-		    int xno = pmod->list[j+2];
-
-		    x = Z[xno][t];
-		    if (na(x)) {
-			pmod->yhat[t] = pmod->uhat[t] = NADBL;
-			continue;
-		    }
-		    pmod->yhat[t] += pmod->coeff[j] * x;
-		}
-		pmod->uhat[t] = y - pmod->yhat[t];  
-	    } else {
-		pmod->yhat[t] = yhat->val[i];
-		pmod->uhat[t] = Z[yvar][t] - pmod->yhat[t];
-		pmod->ess += pmod->uhat[t] * pmod->uhat[t];
-		i++;
-	    }
-	}
-#else
-	for (t=0; t<fulln; t++) {
-	    if (t < pmod->t1 || t > pmod->t2) {
+	    if (t < pmod->t1 || t > pmod->t2 || model_missing(pmod, t)) {
 		pmod->yhat[t] = pmod->uhat[t] = NADBL;
 	    } else {
 		pmod->yhat[t] = yhat->val[i];
@@ -207,7 +191,6 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 		i++;
 	    }
 	}
-#endif
     }
 
     /* if SSR is small enough, treat it as zero */
@@ -219,23 +202,28 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 static gretl_matrix *make_data_X (const MODEL *pmod, const double **Z)
 {
     gretl_matrix *X;
-    int i, t, j = 0;
+    int i, j, t;
     int start = (pmod->ifc)? 3 : 2;
 
     X = gretl_matrix_alloc(pmod->nobs, pmod->ncoeff);
     if (X == NULL) return NULL;
 
     /* copy independent vars into matrix X */
+    j = 0;
     for (i=start; i<=pmod->list[0]; i++) {
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    X->val[j++] = Z[pmod->list[i]][t];
+	    if (!model_missing(pmod, t)) {
+		X->val[j++] = Z[pmod->list[i]][t];
+	    }
 	}
     }
 
     /* insert constant */
     if (pmod->ifc) {
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    X->val[j++] = 1.0;
+	    if (!model_missing(pmod, t)) {
+		X->val[j++] = 1.0;
+	    }
 	}
     }
 
@@ -348,6 +336,28 @@ static int qr_make_hac (MODEL *pmod, const double **Z, gretl_matrix *xpxinv)
     return err;
 }
 
+/* Multiply X transpose into D, treating D as if it were
+   a diagonal matrix -- although in fact it is just a vector,
+   for economy of storage.  Result into R.
+*/
+
+static void do_X_prime_diag (const gretl_matrix *X,
+			     const gretl_vector *D,
+			     gretl_matrix *R)
+{
+    const double *d;
+    double x;
+    int i, j;
+
+    for (i=0; i<R->rows; i++) {
+	d = D->val;
+	for (j=0; j<R->cols; j++) {
+	    x = X->val[mdx(X, j, i)];
+	    R->val[mdx(R, i, j)] = x * (*d++);
+	}
+    }
+}
+
 /* Heteroskedasticity-Consistent Covariance Matrices: See Davidson
    and MacKinnon, Econometric Theory and Methods, chapter 5, esp.
    page 200.  Implements HC0, HC1, HC2 and HC3.
@@ -368,11 +378,12 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
     X = make_data_X(pmod, Z);
     if (X == NULL) return 1;
 
-    diag = gretl_diagonal_matrix(pmod->uhat, m, GRETL_MOD_SQUARE);
+    diag = gretl_column_vector_from_array(pmod->uhat + pmod->t1, m,
+					  GRETL_MOD_SQUARE);
     if (diag == NULL) {
 	err = 1;
 	goto bailout;
-    }   
+    }  
 
     tmp1 = gretl_matrix_alloc(n, m);
     tmp2 = gretl_matrix_alloc(n, n);
@@ -390,7 +401,7 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
 
     if (hc_version == 1) {
 	for (t=0; t<m; t++) {
-	    diag->val[mdx(diag, t, t)] *= (double) m / (m - n);
+	    diag->val[t] *= (double) m / (m - n);
 	}
     } else if (hc_version > 1) {
 	/* do the h_t calculations */
@@ -402,16 +413,15 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
 		ht += q * q;
 	    }
 	    if (hc_version == 2) {
-		diag->val[mdx(diag, t, t)] /= (1.0 - ht);
+		diag->val[t] /= (1.0 - ht);
 	    } else { /* HC3 */
-		diag->val[mdx(diag, t, t)] /= (1.0 - ht) * (1.0 - ht);
+		diag->val[t] /= (1.0 - ht) * (1.0 - ht);
 	    }
 	}
     }
 
-    gretl_matrix_multiply_mod(X, GRETL_MOD_TRANSPOSE,
-			      diag, GRETL_MOD_NONE,
-			      tmp1);
+    do_X_prime_diag(X, diag, tmp1);
+
     gretl_matrix_multiply(tmp1, X, tmp2);
     gretl_matrix_multiply(xpxinv, tmp2, tmp3); 
     gretl_matrix_multiply(tmp3, xpxinv, tmp2);
@@ -476,6 +486,9 @@ static double get_model_data (MODEL *pmod, const double **Z,
     j = 0;
     for (i=start; i<=pmod->list[0]; i++) {
 	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (model_missing(pmod, t)) {
+		continue;
+	    }
 	    x = Z[pmod->list[i]][t];
 	    if (dwt) {
 		if (Z[dwt][t] == 0.0) continue;
@@ -495,7 +508,10 @@ static double get_model_data (MODEL *pmod, const double **Z,
     /* insert constant last (numerical issues) */
     if (pmod->ifc) {
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    x = 1.0;
+	    if (model_missing(pmod, t)) {
+		continue;
+	    }	    
+	    x = Z[0][t]; /* in some special cases the constant is pre-transformed */
 	    if (dwt) {
 		if (Z[dwt][t] == 0.0) continue;
 	    } else if (pmod->nwt) {
@@ -514,6 +530,9 @@ static double get_model_data (MODEL *pmod, const double **Z,
     /* copy dependent variable into y vector */
     j = 0;
     for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (model_missing(pmod, t)) {
+	    continue;
+	}		
 	x = Z[pmod->list[1]][t];
 	if (dwt) {
 	    if (Z[dwt][t] == 0.0) continue;
@@ -551,7 +570,7 @@ static void save_coefficients (MODEL *pmod, gretl_matrix *b,
 }
 
 int gretl_qr_regress (MODEL *pmod, const double **Z, int fulln,
-		      unsigned long opts)
+		      gretlopt opts)
 {
     integer info, lwork;
     integer m, n, lda;
@@ -693,7 +712,11 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, int fulln,
 
     /* standard error of regression */
     if (m - n > 0) {
-	pmod->sigma = sqrt(pmod->ess / (m - n));
+	if (gretl_model_get_int(pmod, "no-df-corr")) {
+	    pmod->sigma = sqrt(pmod->ess / m);
+	} else {
+	    pmod->sigma = sqrt(pmod->ess / (m - n));
+	}
     } else {
 	pmod->sigma = 0.0;
     }
@@ -712,7 +735,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, int fulln,
     /* VCV and standard errors */
     if (opts & OPT_R) { 
 	gretl_model_set_int(pmod, "robust", 1);
-	if (opts & OPT_T) {
+	if ((opts & OPT_T) && !get_force_hc()) {
 	    qr_make_hac(pmod, Z, xpxinv);
 	} else {
 	    qr_make_hccme(pmod, Z, Q, xpxinv);
@@ -721,8 +744,12 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, int fulln,
 	qr_make_regular_vcv(pmod, xpxinv);
     }
 
-    /* get R^2 and F-stat */
+    gretl_matrix_free(Q);
+    Q = NULL;
+
+    /* get R^2 */
     qr_compute_r_squared(pmod, Z[pmod->list[1]], m);
+    qr_compute_f_stat(pmod, opts);
 
  qr_cleanup:
     gretl_matrix_free(Q);
@@ -741,7 +768,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, int fulln,
     return err;    
 }
 
-int qr_tsls_vcv (MODEL *pmod, const double **Z, unsigned long opts)
+int qr_tsls_vcv (MODEL *pmod, const double **Z, gretlopt opts)
 {
     integer info, lwork;
     integer m, n, lda;
