@@ -24,29 +24,23 @@
 
 /* functions follow */
 
-int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo, int n);
-
-int model_sample_issue (const MODEL *pmod, MODELSPEC *spec,
-			const DATAINFO *pdinfo);
+int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo);
 
 int restrict_sample (const char *line, 
-		     double ***oldZ, double ***newZ,
-		     DATAINFO *oldinfo, DATAINFO *newinfo,
-		     const int *list, unsigned long oflag);
+		     double ***pZ, DATAINFO **ppdinfo,
+		     const int *list, gretlopt oflag);
+
+int complex_subsampled (void);
+
+int get_full_length_n (void);
 
 int set_sample (const char *line, DATAINFO *pdinfo);
 
-int restore_full_sample (double ***subZ, double ***fullZ, double ***Z,
-			 DATAINFO **subinfo, DATAINFO **fullinfo,
-			 DATAINFO **datainfo); 
+int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt); 
 
 int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn);
 
-int allocate_case_markers (char ***S, int n);
-
-int add_subsampled_dataset_to_model (MODEL *pmod, 
-				     const double **fullZ, 
-				     const DATAINFO *fullinfo);
+int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo);
 
 void free_model_dataset (MODEL *pmod);
 
