@@ -168,14 +168,20 @@ static void tex_garch_coeff_name (char *targ, const char *src,
     int lag;
 
     if (sscanf(src, "%[^(](%d)", vname, &lag) == 2) {
+	/* e.g. "alpha(0)" */
 	if (!inmath) {
 	    sprintf(targ, "$\\%s_%d$", vname, lag);
 	} else {
 	    sprintf(targ, "\\%s_%d", vname, lag);
 	}
     } else {
+	/* regular variable name */
 	tex_escape(vnesc, src);
-	strcpy(targ, vnesc);
+	if (inmath) {
+	    sprintf(targ, "\\mbox{%s}", vnesc);
+	} else {
+	    strcpy(targ, vnesc);
+	}
     }
 }
 
@@ -213,6 +219,28 @@ static void tex_arma_coeff_name (char *targ, const char *src,
     }
 }
 
+static void tex_lagname (char *s, const DATAINFO *pdinfo, int v)
+{
+    const char *lbl = VARLABEL(pdinfo, v);
+    int gotit = 0;
+
+    if (strlen(lbl) > 2) {
+	char myvar[24], tmp[9];
+	int lag;
+
+	lbl += 2;
+	if (sscanf(lbl, "%8[^(](t - %d)", tmp, &lag) == 2) {
+	    tex_escape(myvar, tmp);
+	    sprintf(s, "%s$_{t-%d}$", myvar, lag);
+	    gotit = 1;
+	}
+    }
+	
+    if (!gotit) {
+	tex_escape(s, pdinfo->varname[v]); 
+    }
+}
+
 int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
 		     int c, PRN *prn)
 {
@@ -247,6 +275,8 @@ int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	tex_arma_coeff_name(tmp, pmod->params[c-1], 0);
     } else if (pmod->ci == GARCH) {
 	tex_garch_coeff_name(tmp, pmod->params[c-1], 0);
+    } else if (pmod->ci == VAR) {
+	tex_lagname(tmp, pdinfo, pmod->list[c]);
     } else {
 	tex_escape(tmp, pdinfo->varname[pmod->list[c]]);
     }
@@ -286,14 +316,14 @@ int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 
 /* ......................................................... */
 
-static int make_texfile (const PATHS *ppaths, int model_count,
-			 int equation, char *texfile, PRN *prn)
+static int make_texfile (int ID, int equation, 
+			 char *texfile, PRN *prn)
 {
     FILE *fp;
 
     if (*texfile == 0) {
-	sprintf(texfile, "%s%s_%d.tex", ppaths->userdir,
-		(equation)? "equation" : "model", model_count);
+	sprintf(texfile, "%s%s_%d.tex", gretl_user_dir(),
+		(equation)? "equation" : "model", ID);
     }
 
     fp = fopen(texfile, "w");
@@ -303,6 +333,92 @@ static int make_texfile (const PATHS *ppaths, int model_count,
     return 0;
 }
 
+/* mechanism for customizing gretl's tex preamble */
+
+static char tex_preamble_file[MAXLEN];
+
+#ifdef ENABLE_NLS
+static const char *get_gretltex_local (void)
+{
+    static char localtex[16] = {0};
+    char *lang = getenv("LANG");
+
+    if (lang != NULL) {
+	char lstr[3] = {0};
+
+	strncat(lstr, lang, 2);
+	sprintf(localtex, "gretlpre_%s.tex", lstr);
+    }
+
+    return localtex;
+}
+#endif
+
+void set_gretl_tex_preamble (void)
+{
+    FILE *fp;
+    const char *gretltex = "gretlpre.tex";
+#ifdef ENABLE_NLS
+    const char *localtex = get_gretltex_local();
+
+    /* first choice: localized preamble file */
+    sprintf(tex_preamble_file, "%s%s", gretl_user_dir(), localtex);
+    fp = fopen(tex_preamble_file, "r");
+    if (fp == NULL) {
+	tex_preamble_file[0] = '\0';
+    } else {
+	fclose(fp);
+	return;
+    }    
+#endif
+
+    /* preamble file on disk */
+    sprintf(tex_preamble_file, "%s%s", gretl_user_dir(), gretltex);
+    fp = fopen(tex_preamble_file, "r");
+    if (fp == NULL) {
+	tex_preamble_file[0] = '\0';
+    } else {
+	fclose(fp);
+    }
+}
+
+void gretl_tex_preamble (PRN *prn, int ams)
+{
+    FILE *fp = NULL;
+    int userfile = 0;
+
+    if (tex_preamble_file[0] != '\0') {
+	fp = fopen(tex_preamble_file, "r");
+	if (fp != NULL) {
+	    char line[128];
+
+	    while (fgets(line, sizeof line, fp)) {
+		pputs(prn, line);
+	    }
+	    userfile = 1;
+	    fclose(fp);
+	}
+    }
+
+    if (!userfile) {
+	pputs(prn, "\\documentclass[11pt]{article}\n");
+
+#ifdef ENABLE_NLS
+	pputs(prn, "\\usepackage[latin1]{inputenc}\n\n");
+#endif
+	if (ams) {
+	    pputs(prn, "\\usepackage{amsmath}\n\n");
+	} else {
+	    pputs(prn, "\\usepackage{dcolumn}\n\n");
+	}
+
+	pputs(prn, "\\begin{document}\n\n"
+	      "\\thispagestyle{empty}\n\n");
+    }
+}
+
+#define MAXCOEFF 4
+
 /**
  * tex_print_equation:
  * @pmod:  pointer to gretl MODEL struct.
@@ -323,23 +439,20 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     double tstat;
     char tmp[48];
     int i, nc = pmod->ncoeff;
+    int split = (nc > MAXCOEFF);
+    int cchars = 0, ccount = 0;
 
     if (standalone) {
-	pputs(prn, "\\documentclass[11pt]{article}\n");
-
-#ifdef ENABLE_NLS
-	pputs(prn, "\\usepackage[latin1]{inputenc}\n\n");
-#endif
-	pputs(prn, "\\usepackage{amsmath}\n\n");
-
-	pputs(prn, "\\begin{document}\n\n"
-		"\\thispagestyle{empty}\n\n");
+	gretl_tex_preamble(prn, 1);
     } else{
 	pputs(prn, "%%% the following needs the amsmath LaTeX package\n\n");
     }
 
     /* initial setup */
     pputs(prn, "\\begin{gather}\n");
+    if (split) {
+	pputs(prn, "\\begin{split}\n");
+    }
 
     /* dependent variable */
     *tmp = '\0';
@@ -348,7 +461,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     } else {
 	tex_escape(tmp, pdinfo->varname[pmod->list[1]]);
     }
-    pprintf(prn, "\\widehat{\\rm %s} = \n", tmp);
+    pprintf(prn, "\\widehat{\\rm %s} %s= \n", tmp, (split? "&" : ""));
 
     if (pmod->ci == GARCH) 
 	nc -= (1 + pmod->list[1] + pmod->list[2]);
@@ -364,17 +477,30 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	if (i > 0 || pmod->ifc == 0) {
 	    pputs(prn, "\\,");
 	    if (pmod->ci == ARMA) {
+		cchars += strlen(pmod->params[i+1]);
 		tex_arma_coeff_name(tmp, pmod->params[i+1], 1);
-		pprintf(prn, "%s", tmp);
+		pputs(prn, tmp);
 	    } else if (pmod->ci == GARCH) {
-		tex_garch_coeff_name(tmp, pmod->params[i+2], 1);
-		pprintf(prn, "%s", tmp);
+		cchars += strlen(pmod->params[i+1]);
+		tex_garch_coeff_name(tmp, pmod->params[i+1], 1);
+		pputs(prn, tmp);
 	    } else {
+		cchars += strlen(pdinfo->varname[pmod->list[i+2]]);
 		tex_escape(tmp, pdinfo->varname[pmod->list[i+2]]);
 		pprintf(prn, "\\mbox{%s}", tmp);
 	    }
 	}
-	pputc(prn, '\n');
+	ccount++;
+	if (split && (cchars > 30 || ccount > 3)) {
+	    pputs(prn, "\\\\\n& ");
+	    cchars = ccount = 0;
+	} else {
+	    pputc(prn, '\n');
+	}
+    }
+
+    if (split) {
+	pputs(prn, "\\end{split}\n");
     }
     pputs(prn, " \\notag \\\\\n");
 
@@ -466,22 +592,23 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion.
  */
 
-int tex_print_model (const MODEL *pmod, const DATAINFO *pdinfo, 
+int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo, 
 		     int standalone, PRN *prn)
 {
-    if (standalone) prn->format = GRETL_PRINT_FORMAT_TEX_DOC;
-    else prn->format = GRETL_PRINT_FORMAT_TEX;
+    if (standalone) {
+	prn->format = GRETL_PRINT_FORMAT_TEX_DOC;
+    } else {
+	prn->format = GRETL_PRINT_FORMAT_TEX;
+    }
     
-    return printmodel (pmod, pdinfo, prn);
+    return printmodel (pmod, pdinfo, OPT_NONE, prn);
 }
 
 /**
  * tabprint:
  * @pmod: pointer to gretl MODEL struct.
  * @pdinfo: information regarding the data set.
- * @ppaths: struct containing information on paths.
  * @texfile: name of file to save.
- * @model_count: count of models estimated so far.
  * @oflag: option: complete doc or fragment
  *
  * Prints to file a gretl model in the form of a LaTeX table, either as
@@ -491,17 +618,21 @@ int tex_print_model (const MODEL *pmod, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int tabprint (const MODEL *pmod, const DATAINFO *pdinfo,
-	      const PATHS *ppaths, char *texfile,
-	      int model_count, unsigned long oflag)
+int tabprint (MODEL *pmod, const DATAINFO *pdinfo,
+	      char *texfile, gretlopt oflag)
 {
     PRN prn;
 
-    if (make_texfile(ppaths, model_count, 0, texfile, &prn))
+    if (make_texfile(pmod->ID, 0, texfile, &prn)) {
 	return 1;
+    }
 
     tex_print_model(pmod, pdinfo, (oflag & OPT_O), &prn);
-    if (prn.fp != NULL) fclose(prn.fp);
+
+    if (prn.fp != NULL) {
+	fclose(prn.fp);
+    }
+
     return 0;
 }
 
@@ -509,9 +640,7 @@ int tabprint (const MODEL *pmod, const DATAINFO *pdinfo,
  * eqnprint:
  * @pmod: pointer to gretl MODEL struct.
  * @pdinfo: information regarding the data set.
- * @ppaths: struct containing information on paths.
  * @texfile: name of file to save.
- * @model_count: count of models estimated so far.
  * @oflag: if oflag & OPT_O, complete doc, else fragment
  *
  * Prints to file a gretl model in the form of a LaTeX equation, either as
@@ -521,16 +650,20 @@ int tabprint (const MODEL *pmod, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int eqnprint (const MODEL *pmod, const DATAINFO *pdinfo,
-	      const PATHS *ppaths, char *texfile,
-	      int model_count, unsigned long oflag)
+int eqnprint (MODEL *pmod, const DATAINFO *pdinfo,
+	      char *texfile, gretlopt oflag)
 {
     PRN prn;
 
-    if (make_texfile(ppaths, model_count, 1, texfile, &prn))
+    if (make_texfile(pmod->ID, 1, texfile, &prn)) {
 	return 1;
+    }
 
     tex_print_equation(pmod, pdinfo, (oflag & OPT_O), &prn);
-    if (prn.fp != NULL) fclose(prn.fp);
+
+    if (prn.fp != NULL) {
+	fclose(prn.fp);
+    }
+
     return 0;
 }
