@@ -24,23 +24,25 @@
 int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo, 
 			const int standalone, PRN *prn);
 
-int tex_print_model (const MODEL *pmod, const DATAINFO *pdinfo, 
+int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo, 
 		     const int standalone, PRN *prn);
 
 int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
 		      int c, PRN *prn);
 
-int tabprint (const MODEL *pmod, const DATAINFO *pdinfo,
-	      const PATHS *ppaths, char *texfile,
-	      const int model_count, unsigned long oflag);
+int tabprint (MODEL *pmod, const DATAINFO *pdinfo,
+	      char *texfile, gretlopt oflag);
 
-int eqnprint (const MODEL *pmod, const DATAINFO *pdinfo,
-	      const PATHS *ppaths, char *texfile,
-	      const int model_count, unsigned long oflag);
+int eqnprint (MODEL *pmod, const DATAINFO *pdinfo,
+	      char *texfile, gretlopt oflag);
 
 char *tex_escape (char *targ, const char *src);
 
 void tex_dcolumn_double (double xx, char *numstr);
 
+void set_gretl_tex_preamble (void);
+
+void gretl_tex_preamble (PRN *prn, int ams);
+
 
 
