@@ -22,38 +22,39 @@
 
 #include "gretl_matrix.h"
 
-int list_diffgenr (const LIST list, 
-		   double ***pZ, DATAINFO *pdinfo);
+enum ur_codes {
+    UR_NO_CONST = 1,
+    UR_CONST,
+    UR_TREND,
+    UR_TREND_SQUARED
+};
 
-int list_ldiffgenr (const LIST list, 
-		    double ***pZ, DATAINFO *pdinfo);
-
-int simple_var (int order, const LIST list, 
+int simple_var (int order, const int *list, 
 		double ***pZ, DATAINFO *pdinfo,
-		int pause, PRN *prn);
+		int pause, gretlopt opts, PRN *prn);
 
-GRETL_VAR *full_var (int order, const LIST list, 
+GRETL_VAR *full_var (int order, const int *list, 
 		     double ***pZ, DATAINFO *pdinfo,
-		     PRN *prn);
+		     gretlopt opts, PRN *prn);
 
 void gretl_var_free (GRETL_VAR *var);
 
 void gretl_var_free_unnamed (GRETL_VAR *var);
 
-int coint (int order, const LIST list, 
+int coint (int order, const int *list, 
 	   double ***pZ, DATAINFO *pdinfo, 
 	   PRN *prn);
 
-int adf_test (int order, const int varno, 
+int adf_test (int order, int varno, 
 	      double ***pZ, DATAINFO *pdinfo, 
-	      PRN *prn);
+	      gretlopt opt, PRN *prn);
 
-int ma_model (LIST list, double ***pZ, DATAINFO *pdinfo, 
-	      PRN *prn);
+int kpss_test (int order, int varno, double ***pZ,
+	       DATAINFO *pdinfo, gretlopt opt, PRN *prn);
 
-int johansen_test (int order, const LIST list, 
+int johansen_test (int order, const int *list, 
 		   double ***pZ, DATAINFO *pdinfo,
-		   unsigned long opt, PRN *prn);
+		   gretlopt opt, PRN *prn);
 
 double *
 gretl_var_get_impulse_responses (GRETL_VAR *var, int targ, int shock,
@@ -79,5 +80,8 @@ int gretl_var_get_n_equations (const GRETL_VAR *var);
 int gretl_var_add_resids_to_dataset (GRETL_VAR *var, int eqnum,
 				     double ***pZ, DATAINFO *pdinfo);
 
+int gretl_var_get_highest_variable (const GRETL_VAR *var,
+				    const DATAINFO *pdinfo);
+
 #endif /* VAR_H_ */
 
