@@ -189,7 +189,7 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
     pmod->dfd = pmod->nobs - pmod->dfn;
     pmod->ncoeff = p + q + 1;
 
-    copylist(&pmod->list, list);
+    pmod->list = copylist(list);
 
     pmod->ybar = gretl_mean(pmod->t1, pmod->t2, y);
     pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, y);
@@ -419,7 +419,7 @@ static int get_x12a_vcv (const char *fname, MODEL *pmod, int nc)
 		p += 22;
 		if (i >= j) {
 		    x = atof(valstr);
-		    k = ijton(i + 1, j + 1, nc);
+		    k = ijton(i, j, nc);
 		    pmod->vcv[k] = x;
 		}
 	    }
@@ -756,7 +756,8 @@ MODEL arma_x12_model (int *list, const double **Z,
     int t1, t2;
     MODEL armod;
 
-    gretl_model_init(&armod, pdinfo);  
+    gretl_model_init(&armod);  
+    gretl_model_smpl_init(&armod, pdinfo);
 
     if (check_arma_list(list)) {
 	armod.errcode = E_UNSPEC;
