@@ -35,18 +35,24 @@
 
 static void free_sheet (void);
 static int allocate_row_col (int row, int col, wbook *book);
-static char *copy_unicode_string (unsigned char **src);
+static char *copy_unicode_string (unsigned char *src, int remlen, 
+				  int *skip, int *slop);
 static char *convert8to7 (const unsigned char *s, int count);
 static char *convert16to7 (const unsigned char *s, int count);
 static char *mark_string (char *str);
 
+#undef EDEBUG
 #undef MEMDEBUG
 
 #define EXCEL_IMPORTER
 #include "import_common.c"
 
-#define cell_record(r) (r == BIFF_LABEL || r == BIFF_STRING || r == BIFF_NUMBER || \
-                        r == BIFF_RK || r == BIFF_MULRK || r == BIFF_FORMULA || \
+#define cell_record(r) (r == BIFF_LABEL || \
+                        r == BIFF_STRING || \
+                        r == BIFF_NUMBER || \
+                        r == BIFF_RK || \
+                        r == BIFF_MULRK || \
+                        r == BIFF_FORMULA || \
                         r == BIFF_LABELSST)
 
 struct rowdescr {
@@ -72,6 +78,7 @@ static double get_le_double (const unsigned char *rec)
         unsigned char cc[8];
         double d;
     } dconv;
+
     unsigned char *d;
     const unsigned char *s;
     int i;
@@ -212,20 +219,33 @@ static int check_copy_string (struct rowdescr *prow, int row, int col,
 static int process_item (BiffQuery *q, wbook *book, PRN *prn) 
 {
     struct rowdescr *prow = NULL;
-    static char **saved_ref;
-    int row = 0, col = 0;
+    static char **string_targ;
+    static int slop; /* SST overslop */
+    unsigned char *ptr = NULL;
+    int row = 0, col = 0, xfref = 0;
+    double val;
 
     if (cell_record(q->ls_op)) {
-	row = MS_OLE_GET_GUINT16(q->data + 0);
-	col = MS_OLE_GET_GUINT16(q->data + 2);
-	if (row_col_err(row, col, prn)) return 1;
+	row = EX_GETROW(q);
+	col = EX_GETCOL(q);
+	if (row_col_err(row, col, prn)) {
+	    return 1;
+	}
+	xfref = EX_GETXF(q);
     }
 
+#ifdef FORMAT_INFO
+    if (q->ls_op == BIFF_NUMBER || q->ls_op == BIFF_RK) {
+	fprintf(stderr, "Numeric cell (%d, %d), format ref = %d\n", 
+		row, col, xfref);
+    }
+#endif
+
     switch (q->ls_op) {
 
     case BIFF_SST: {
-	unsigned char *ptr = q->data + 8;
-	int i, sz, oldsz = sstsize;
+	int i, skip, remlen, oldsz = sstsize;
+	guint16 sz;
 
 	if (sst != NULL) {
 	    fprintf(stderr, "Got a second string table: this is nonsense\n");
@@ -235,17 +255,29 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	sz = MS_OLE_GET_GUINT16(q->data + 4);
 	sstsize += sz;
 	sst = realloc(sst, sstsize * sizeof *sst);
-	if (sst == NULL) return 1;
+	if (sst == NULL) {
+	    return 1;
+	}
 #ifdef EDEBUG
-	fprintf(stderr, "Got SST: allocated sst at size %d (%d bytes), %p\n", 
+	fprintf(stderr, "Got SST: allocated for %d strings (%d bytes), %p\n", 
 		sstsize, sstsize * sizeof *sst, (void *) sst);
 #endif
 	for (i=oldsz; i<sstsize; i++) {
 	    /* careful: initialize all pointers to NULL */
 	    sst[i] = NULL;
 	}
-	for (i=oldsz; i<sstsize && (ptr - q->data) < q->length; i++) {
-	    sst[i] = copy_unicode_string(&ptr);
+	ptr = q->data + 8;
+	for (i=oldsz; i<sstsize; i++) {
+	    remlen = q->length - (ptr - q->data);
+#ifdef EDEBUG
+	    fprintf(stderr, "Working on sst[%d], data offset=%d, remlen=%d\n", 
+		    i, (int) (ptr - q->data), remlen);
+#endif
+	    if (remlen <= 0) {
+		break;
+	    }
+	    sst[i] = copy_unicode_string(ptr, remlen, &skip, &slop);
+	    ptr += skip;
 	}
 	if (i < sstsize) {
 	    sstnext = i;
@@ -253,186 +285,185 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	break;
     }	
 
-    case BIFF_CONTINUE: {
-	int i;
-	unsigned char *ptr = q->data;
-
+    case BIFF_CONTINUE: 
 #ifdef EDEBUG
-	fprintf(stderr, "Got CONTINUE, with sstnext = %d\n", sstnext);
+	fprintf(stderr, "Got CONTINUE, sstnext = %d, len = %d\n", 
+		sstnext, (int) q->length);
 #endif
+	if (sstnext > 0) {
+	    int i, skip, remlen;
 
-	if (sstnext == 0) {
-	    break;
-	}
-	for (i=sstnext; i<sstsize && (ptr - q->data) < q->length; i++) {
-	    sst[i] = copy_unicode_string(&ptr);
-	}
-	if (i < sstsize) {
-	    sstnext = i;
+	    ptr = q->data;
+	    if (slop > 0) {
+		unsigned char flags = *ptr;
+		int csize = (flags & 0x01)? 2 : 1;
+
+		ptr += 1 + csize * slop;
+	    }
+	    for (i=sstnext; i<sstsize; i++) {
+		remlen = q->length - (ptr - q->data);
+		if (remlen <= 0) {
+		    break;
+		}
+#ifdef EDEBUG
+		fprintf(stderr, "Working on sst[%d]\n", i);
+#endif
+		sst[i] = copy_unicode_string(ptr, remlen, &skip, &slop);
+		ptr += skip;
+	    }
+	    if (i < sstsize) {
+		sstnext = i;
+	    }
 	}
 	break;
-    }
 			   
-    case BIFF_LABEL: {
-	unsigned int len = MS_OLE_GET_GUINT16(q->data + 6);
-
-	saved_ref = NULL;
-
+    case BIFF_LABEL: 
 #ifdef EDEBUG
 	fprintf(stderr, "Got LABEL, row=%d, col=%d\n", row, col);
 #endif
-	if (allocate_row_col(row, col, book)) return 1;
-	prow = rowptr + row;
-
-	prow->cells[col] = mark_string(convert8to7(q->data + 8, len));
+	if (allocate_row_col(row, col, book)) {
+	    return 1;
+	} else {
+	    unsigned int len = MS_OLE_GET_GUINT16(q->data + 6);
+	
+	    prow = rowptr + row;
+	    ptr = q->data + 8;
+	    prow->cells[col] = mark_string(convert8to7(ptr, len));
+	}
 	break;
-    }   
   
-    case BIFF_LABELSST: {
-	unsigned int sstidx = MS_OLE_GET_GUINT16(q->data + 6);
-
+    case BIFF_LABELSST:
 #ifdef EDEBUG
 	fprintf(stderr, "Got LABELSST, row=%d, col=%d\n", row, col);
 #endif
-	saved_ref = NULL;
-
-	if (allocate_row_col(row, col, book)) return 1;
-	prow = rowptr + row;
-
-	if (sstidx >= sstsize) {
-	    pprintf(prn, _("String index too large"));
-	    pputc(prn, '\n');
-	} else if (sst[sstidx] != NULL) {
-	    check_copy_string(prow, row, col, sstidx, sst[sstidx]);
+	if (allocate_row_col(row, col, book)) {
+	    return 1;
 	} else {
-	    prow->cells[col] = malloc(2);
-	    if (prow->cells[col] != NULL) {
-		*prow->cells[col] = '\0';
+	    unsigned int sstidx = MS_OLE_GET_GUINT16(q->data + 6);
+
+	    prow = rowptr + row;
+	    if (sstidx >= sstsize) {
+		pprintf(prn, _("String index too large"));
+		pputc(prn, '\n');
+	    } else if (sst[sstidx] != NULL) {
+		check_copy_string(prow, row, col, sstidx, sst[sstidx]);
+	    } else {
+		prow->cells[col] = malloc(2);
+		if (prow->cells[col] != NULL) {
+		    prow->cells[col][0] = '\0';
+		}
 	    }
 	}	
 	break;
-    }
 
-    case BIFF_NUMBER: {
-	double v;
-
-	saved_ref = NULL;
-
-	if (allocate_row_col(row, col, book)) return 1;
-	prow = rowptr + row;
-
-	v = get_le_double(q->data + 6);
-	prow->cells[col] = g_strdup_printf("%.10g", v);
+    case BIFF_NUMBER: 
+	if (allocate_row_col(row, col, book)) {
+	    return 1;
+	} else {
+	    val = get_le_double(q->data + 6);
+	    prow = rowptr + row;
+	    prow->cells[col] = g_strdup_printf("%.10g", val);
 #ifdef EDEBUG
-	fprintf(stderr, "Got NUMBER (%g), row=%d, col=%d\n", v, row, col);
+	    fprintf(stderr, "Got NUMBER (%g), row=%d, col=%d\n", val, 
+		    row, col);
 #endif
+	}
 	break;
-    }
 
-    case BIFF_RK: {
-	double v;
-
-	saved_ref = NULL;
-
-	if (allocate_row_col(row, col, book)) return 1;
-	prow = rowptr + row;
-
-	v = biff_get_rk(q->data + 6);
-	prow->cells[col] = g_strdup_printf("%.10g", v);
+    case BIFF_RK: 
+	if (allocate_row_col(row, col, book)) {
+	    return 1;
+	} else {
+	    val = biff_get_rk(q->data + 6);
+	    prow = rowptr + row;
+	    prow->cells[col] = g_strdup_printf("%.10g", val);
 #ifdef EDEBUG
-	fprintf(stderr, "Got RK (%g), row=%d, col=%d\n", v, row, col);
+	    fprintf(stderr, "Got RK (%g), row=%d, col=%d\n", val, 
+		    row, col);
 #endif
+	}
 	break;
-    }
 
     case BIFF_MULRK: {
 	int i, ncols = (q->length - 6) / 6;
-	double v;
-
-	saved_ref = NULL;
 
 #ifdef EDEBUG
 	fprintf(stderr, "Got MULRK, row=%d, first_col=%d, ncols=%d\n", 
 		row, col, ncols);
 #endif
 	for (i=0; i<ncols; i++) {
-	    if (allocate_row_col(row, col, book)) return 1;
+	    if (allocate_row_col(row, col, book)) {
+		return 1;
+	    }
+	    val = biff_get_rk(q->data + 6 + 6 * i);
 	    prow = rowptr + row;
-	    v = biff_get_rk(q->data + 6 + 6 * i);
-	    prow->cells[col] = g_strdup_printf("%.10g", v);
+	    prow->cells[col] = g_strdup_printf("%.10g", val);
 #ifdef EDEBUG
-	    fprintf(stderr, " MULRK[col=%d] = %g\n", col, v);
+	    fprintf(stderr, " MULRK[col=%d] = %g\n", col, val);
 #endif
 	    col++;
 	}
 	break;
     }
 
-    case BIFF_FORMULA: { 
-	const guint8 *result = q->data + 6;
-
-	saved_ref = NULL;
-
+    case BIFF_FORMULA:  
 #ifdef EDEBUG
 	fprintf(stderr, "Got FORMULA, row=%d, col=%d\n", row, col);
 #endif
-	if (allocate_row_col(row, col, book)) return 1;
-	prow = rowptr + row;
+	if (allocate_row_col(row, col, book)) {
+	    return 1;
+	} else {
+	    ptr = q->data + 6;
+	    prow = rowptr + row;
+	    if (ptr[6] == 0xff && ptr[7] == 0xff) {
+		unsigned char fcode = ptr[0];
 
-	if (result[6] == 0xff && result[7] == 0xff) {
-	    /* not a floating point value */
-
-	    switch (*result) {
-	    case 0x0: /* string formula */
-		saved_ref = prow->cells + col;
-		break;
-	    case 0x1: /* boolean value */
-		prow->cells[col] = g_strdup((result[2])? "1" : "0");
-		break;
-	    case 0x2: /* error code */
-		prow->cells[col] = g_strdup("-999.0");
-		break;
-	    default:
-		fprintf(stderr, "Bad formula code 0x%u\n", 
-			(unsigned) *result);
-		break;
-	    }
+		if (fcode == 0x0) {
+		    /* string formula: record target for following 
+		       STRING record */
+		    string_targ = prow->cells + col;
+		} else if (fcode == 0x1) {
+		    /* boolean value */
+		    prow->cells[col] = g_strdup((ptr[2])? "1" : "0");
+		} else if (fcode == 0x2 || fcode == 0x3) {
+		    /* error code or empty */
+		    prow->cells[col] = g_strdup("-999.0");
+		} else {
+		    fprintf(stderr, "Bad formula code 0x%u\n", 
+			    (unsigned) fcode);
+		    prow->cells[col] = g_strdup("-999.0");
+		}
+	    } else {
+		/* floating-point */
+		val = get_le_double(ptr);
 #ifdef EDEBUG
-	    fprintf(stderr, " non floating-point value, code = 0x%u\n", 
-		    (unsigned) *result);
+		fprintf(stderr, " floating-point value = %g\n", val);
+#endif
+		if (isnan(val)) {
+		    fprintf(stderr, "Got a NaN\n");
+		    prow->cells[col] = g_strdup("-999.0");
+		} else {
+		    prow->cells[col] = g_strdup_printf("%.10g", val);
+		}
+	    }
+	}
+	break;
+
+    case BIFF_STRING: 
+	if (string_targ == NULL) {
+#ifdef EDEBUG
+	    fprintf(stderr, "String record without preceding string formula\n");
 #endif
 	} else {
-	    /* floating-point */
-	    double v = get_le_double(result);
+	    char *tmp = copy_unicode_string(q->data, 0, NULL, NULL);
 
+	    *string_targ = mark_string(tmp);
 #ifdef EDEBUG
-	    fprintf(stderr, " floating-point value = %g\n", v);
+	    fprintf(stderr, "Filled out string formula with '%s'\n", *string_targ);	
 #endif
-	    if (isnan(v)) {
-		fprintf(stderr, "Got a NaN\n");
-		prow->cells[col] = g_strdup("-999.0");
-	    } else {
-		prow->cells[col] = g_strdup_printf("%.10g", v);
-	    }
+	    string_targ = NULL;
 	}
 	break;
-    }
-
-    case BIFF_STRING: {
-	unsigned int len;
-
-	if (saved_ref == NULL) {
-	    pprintf(prn, _("String record without preceding string formula"));
-	    pputc(prn, '\n');
-	    break;
-	}
-	len = MS_OLE_GET_GUINT16(q->data + 0);
-	*saved_ref = mark_string(convert8to7(q->data + 2, len + 1));
-#ifdef EDEBUG
-	fprintf(stderr, "Got STRING, reference = '%s'\n", *saved_ref);	
-#endif
-	break;
-    }
 
     case BIFF_BOF: 
 	if (rowptr) {
@@ -451,6 +482,37 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 #endif
 	break;
 
+#ifdef FORMAT_INFO
+    case BIFF_COLINFO:
+	fprintf(stderr, "Got BIFF_COLINFO: col range (%d, %d), XF index %d\n",
+		(int) MS_OLE_GET_GUINT16(q->data + 0),
+		(int) MS_OLE_GET_GUINT16(q->data + 2),
+		(int) MS_OLE_GET_GUINT16(q->data + 6));
+	break;
+
+    case BIFF_XF: {
+	unsigned short tp = MS_OLE_GET_GUINT16(q->data + 4);
+
+	fprintf(stderr, "Got BIFF_XF: format record index %d ",
+		(int) MS_OLE_GET_GUINT16(q->data + 2));
+	if (tp & 0x04) {
+	    fprintf(stderr, "(style XF)\n");
+	} else {
+	    fprintf(stderr, "(cell XF)\n");
+	}
+	break;
+    }
+
+    case BIFF_FORMAT: {
+	int idx = MS_OLE_GET_GUINT16(q->data + 0);
+
+	if ((idx >= 14 && idx <= 17) || idx >= 164) {
+	    fprintf(stderr, "Got BIFF_FORMAT: index %d\n", idx);
+	}
+	break;
+    }
+#endif
+
     default: 
 	break;
     }
@@ -477,6 +539,18 @@ static int handled_record (BiffQuery *q)
 	}
     }
 
+#ifdef FORMAT_INFO
+    if (q->opcode == BIFF_COLINFO ||
+	q->opcode == BIFF_XF) {
+	return 1;
+    }
+    if (q->ms_op == 0x04) {
+	if (q->ls_op == BIFF_FORMAT) {
+	    return 1;
+	}
+    } 
+#endif   
+
     if (q->ms_op == 0x08) {
 	if (q->ls_op == BIFF_BOF) return 1;
     }
@@ -494,9 +568,12 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
     BiffQuery *q;
     MsOle *file;
 
-    if (ms_ole_open(&file, filename)) return 1;
+    if (ms_ole_open(&file, filename)) {
+	return 1;
+    }
 
     result = ms_ole_stream_open(&stream, file, "/", "workbook", 'r');
+
     if (result != MS_OLE_ERR_OK) {
 	ms_ole_stream_close (&stream);
 
@@ -525,8 +602,8 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
 
     while (!err && ms_biff_query_next(q)) {
 #ifdef EDEBUG
-	    fprintf(stderr, "At %lu: q->opcode=0x%02x\n", 
-		    (unsigned long) q->streamPos, q->opcode);
+	fprintf(stderr, "At %lu: q->opcode=0x%02x\n", 
+		(unsigned long) q->streamPos, q->opcode);
 #endif
 	if (q->opcode == BIFF_EOF) {
 #ifdef EDEBUG
@@ -544,8 +621,11 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
 			    (unsigned long) stream->position);
 		}
 	    }
-	    if (eofcount == 2) break;
-	    else continue;
+	    if (eofcount == 2) {
+		break;
+	    } else {
+		continue;
+	    }
 	} 
 
 	if (handled_record(q)) {
@@ -579,51 +659,66 @@ static char *mark_string (char *str)
     return ret;
 }    
 
-static char *copy_unicode_string (unsigned char **src) 
+static char *
+copy_unicode_string (unsigned char *src, int remlen, 
+		     int *skip, int *slop) 
 {
-    int count = MS_OLE_GET_GUINT16(*src);
-    int flags = *(*src + 2);
-    int this_skip = 3;
-    int skip_to_next = 3;
+    int count = MS_OLE_GET_GUINT16(src);
+    unsigned char flags = *(src + 2);
+    int this_skip = 3, skip_to_next = 3;
     int csize = (flags & 0x01)? 2 : 1;
-    char *realstart;
+
+#ifdef EDEBUG
+    fprintf(stderr, "copy_unicode_string: count = %d, csize = %d\n",
+	    count, csize);
+    if (flags & 0x08) {
+	fprintf(stderr, " contains Rich-Text info\n");
+    }
+    if (flags & 0x04) {
+	fprintf(stderr, " contains Far-East info\n");
+    }    
+#endif
 
     skip_to_next += count * csize;
 
     if (flags & 0x08) {
-	unsigned short rich_text_info_len = 0;
+	guint16 rich_text_info_len = 0;
 
-#ifdef EDEBUG
-	fprintf(stderr, "copy_unicode_string: contains Rich-Text info\n");
-#endif
-	rich_text_info_len = 4 * MS_OLE_GET_GUINT16(*src + 3);
+	rich_text_info_len = 4 * MS_OLE_GET_GUINT16(src + 3);
 	this_skip += 2;
 	skip_to_next += 2 + rich_text_info_len;
     } 
 
     if (flags & 0x04) {
-	unsigned far_east_info_len = 0;
+	guint32 far_east_info_len = 0;
 	int far_east_offset = 3;
 
-#ifdef EDEBUG
-	fprintf(stderr, "copy_unicode_string: contains Far-East info\n");
-#endif
-	if (flags & 0x08) far_east_offset = 5;
-	far_east_info_len = MS_OLE_GET_GUINT32 (*src + far_east_offset);
+	if (flags & 0x08) {
+	    far_east_offset = 5;
+	}
+	far_east_info_len = MS_OLE_GET_GUINT32(src + far_east_offset);
 	this_skip += 4;
 	skip_to_next += 4 + far_east_info_len;
     }
 
-    /* for this read */
-    realstart = *src + this_skip;
+    /* skip for the next read */
+    if (skip != NULL) {
+	*skip = skip_to_next;
+    }
 
-    /* set up for the next read */
-    *src += skip_to_next;
+    /* size check */
+    if (slop != NULL) {
+	if (remlen > 0 && this_skip + count > remlen) {
+	    *slop = this_skip + count - remlen;
+	} else {
+	    *slop = 0;
+	}
+    }
 
     if (csize == 1) {
-	return convert8to7(realstart, count);
+	return convert8to7(src + this_skip, count);
     } else { 
-	return convert16to7(realstart, count);
+	return convert16to7(src + this_skip, count);
     }
 }
 
@@ -631,9 +726,13 @@ static char *convert8to7 (const unsigned char *s, int count)
 {
     char *dest;
 
-    dest = malloc(VNAMELEN);
+    if (count > USER_VLEN - 1) {
+	count = USER_VLEN - 1;
+    }
+
+    dest = malloc(USER_VLEN);
     *dest = '\0';
-    strncat(dest, s, VNAMELEN - 1);
+    strncat(dest, s, count);
     iso_to_ascii(dest);
 
     if (*dest == '\0') {
@@ -653,13 +752,16 @@ static char *convert16to7 (const unsigned char *s, int count)
     int i, j;
     guint16 u;
 
-    dest = malloc(VNAMELEN);
-    if (dest == NULL) return NULL;
-    memset(dest, 0, VNAMELEN);
+    dest = malloc(USER_VLEN);
+    if (dest == NULL) {
+	return NULL;
+    }
+
+    memset(dest, 0, USER_VLEN);
 
     p = dest;
     j = 0;
-    for (i=0; i<count && j<VNAMELEN-1; i++) {
+    for (i=0; i<count && j<USER_VLEN-1; i++) {
 	u = MS_OLE_GET_GUINT16(s);
 	s += 2;
 	if ((isalnum(u) || ispunct(u)) && u < 128) {
@@ -672,6 +774,10 @@ static char *convert16to7 (const unsigned char *s, int count)
 	strcpy(dest, "varname");
     }
 
+#ifdef EDEBUG
+    fprintf(stderr, "convert16to7: returning '%s'\n", dest);
+#endif
+
     return dest;    
 }
 
@@ -691,6 +797,7 @@ static int allocate_row_col (int row, int col, wbook *book)
 	book->row_offset = row;
 	fprintf(stderr, "Missing rows: trying an offset of %d\n", row);
     }
+
     started = 1;
 
 #ifdef MEMDEBUG
@@ -700,7 +807,9 @@ static int allocate_row_col (int row, int col, wbook *book)
     if (row >= lastrow) {
 	newlastrow = (row/16 + 1) * 16;
 	rowptr = realloc(rowptr, newlastrow * sizeof *rowptr);
-	if (rowptr == NULL) return 1;
+	if (rowptr == NULL) {
+	    return 1;
+	}
 	for (i=lastrow; i<newlastrow; i++) {
 #ifdef MEMDEBUG
 	    fprintf(stderr, "allocate: initing rowptr[%d]\n", i);
@@ -725,14 +834,18 @@ static int allocate_row_col (int row, int col, wbook *book)
 		newcol-rowptr[row].end, row);
 #endif
 	rowptr[row].cells = realloc(rowptr[row].cells, newcol * sizeof(char *));
-	if (rowptr[row].cells == NULL) return 1;
+	if (rowptr[row].cells == NULL) {
+	    return 1;
+	}
 	for (i=rowptr[row].end; i<newcol; i++) {
 	    rowptr[row].cells[i] = NULL;
 	}
 	rowptr[row].end = newcol;
     } 
  
-    if (col > rowptr[row].last) rowptr[row].last = col;
+    if (col > rowptr[row].last) {
+	rowptr[row].last = col;
+    }
 
     return 0;
 }
@@ -836,12 +949,33 @@ static int first_col_strings (wbook *book)
 		rowptr[t].cells[i]);
 #endif
 	if (rowptr == NULL || rowptr[t].cells[i] == NULL ||
-	    !IS_STRING(rowptr[t].cells[i]))
+	    !IS_STRING(rowptr[t].cells[i])) {
 	    return 0;
+	}
     }
+
     return 1;
 }
 
+#define obs_string(s) (!strcmp(s, "obs") || !strcmp(s, "id"))
+
+static int fix_varname (char *vname)
+{
+    int i, n = strlen(vname);
+    int bad = 0;
+
+    for (i=1; i<n; i++) {
+        if (!(isalpha((unsigned char) vname[i]))  
+            && !(isdigit((unsigned char) vname[i]))
+            && vname[i] != '_') {
+	    vname[i] = '_';
+	    bad++;
+	}
+    }
+
+    return (bad == n);
+}   
+
 static int check_all_varnames (wbook *book, int ncols, int skip)
 {
     int i, t = book->row_offset;
@@ -863,32 +997,44 @@ static int check_all_varnames (wbook *book, int ncols, int skip)
 	    return VARNAMES_NOTSTR;
 	}
 	test = rowptr[t].cells[i] + 1;
-	/* "obs" in first col is OK, though not thereafter */
-	if (i == skip+book->col_offset && !strcmp(test, "obs")) {
-	    ; /* pass along */
-	} else if (check_varname(test)) {
-	    return VARNAMES_INVALID;
+	/* "obs" or "id" in first col is OK, though not thereafter */
+	if (i == skip + book->col_offset && obs_string(test)) {
+	    /* pass along */
+	    ;
+	} else {
+	    int verr = check_varname(test);
+
+	    if (verr == VARNAME_BADCHAR) {
+		verr = fix_varname(test);
+	    }
+	    
+	    if (verr) {
+		return VARNAMES_INVALID;
+	    }
 	}
     }
+
     return VARNAMES_OK;
 }
 
 static int missval_string (const char *s)
 {
-    if (s + 1 == 0) return 1;
-    else {
-	char *p, test[6];
+    if (s + 1 == 0) {
+	return 1;
+    } else {
+	char test[6] = {0};
 
-	*test = 0;
 	strncat(test, s + 1, 4);
-	p = test;
-	while (*p) {
-	    *p = tolower(*p);
-	    p++;
-	}
-	if (!strcmp(test, "na") || !strcmp(test, "n.a."))
+	tailstrip(test);
+	lower(test);
+	if (!strcmp(test, "na") || 
+	    !strcmp(test, "n.a.") ||
+	    !strcmp(test, "..") ||
+	    !strcmp(test, "?")) {
 	    return 1;
+	}
     }
+
     return 0;
 }
 
@@ -910,16 +1056,14 @@ static int check_data_block (wbook *book, int ncols, int skip,
 		fprintf(stderr, "data_block: rowptr[%d].cells is NULL\n", t);
 #endif
 		ret = -1;
-	    } 
-	    else if (rowptr[t].cells[i] == NULL) {
+	    } else if (rowptr[t].cells[i] == NULL) {
 #ifdef EDEBUG
 		fprintf(stderr, "data_block: rowptr[%d].cells[%d] is NULL\n",
 			t, i);
 #endif
 		rowptr[t].cells[i] = g_strdup("-999.0");
 		ret = -1;
-	    }
-	    else if (IS_STRING(rowptr[t].cells[i])) {
+	    } else if (IS_STRING(rowptr[t].cells[i])) {
 		if (missval_string(rowptr[t].cells[i])) {
 		    free(rowptr[t].cells[i]);
 		    rowptr[t].cells[i] = g_strdup("-999.0");
@@ -970,8 +1114,11 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	if (book.nsheets > 1) wsheet_menu(&book, 1);
-	else wsheet_menu(&book, 0);
+	if (book.nsheets > 1) {
+	    wsheet_menu(&book, 1);
+	} else {
+	    wsheet_menu(&book, 0);
+	}
     }
 
 #ifdef EDEBUG
@@ -1002,16 +1149,24 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	strerr.str = NULL;
 
 	lastrow--;
-	while (lastrow > 0 && !rowptr[lastrow].cells) lastrow--;
+	while (lastrow > 0 && !rowptr[lastrow].cells) {
+	    lastrow--;
+	}
 
 	for (i=0; i<=lastrow; i++) {
 	    if (rowptr[i].cells != NULL) {
 		ncols = 0;
-		for (j=0; j<=rowptr[i].last; j++) 
-		    if (rowptr[i].cells[j] != NULL) ncols++;
-		if (ncols > maxcols) maxcols = ncols;
+		for (j=0; j<=rowptr[i].last; j++) {
+		    if (rowptr[i].cells[j] != NULL) {
+			ncols++;
+		    }
+		}
+		if (ncols > maxcols) {
+		    maxcols = ncols;
+		}
 	    }
 	}
+
 	ncols = maxcols;
 	printf("nrows=%d, ncols=%d\n", lastrow + 1, ncols);
 
@@ -1023,14 +1178,17 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	label_strings = first_col_strings(&book);
-	puts("found label strings in first column"); 
+	if (label_strings) {
+	    puts("found label strings in first column");
+	} else {
+	    puts("check for label strings in first column: not found");
+	}
 
 	err = check_all_varnames(&book, ncols, label_strings);
 	if (err == VARNAMES_NULL || err == VARNAMES_NOTSTR) {
 	    pputs(prn, _("One or more variable names are missing.\n"));
 	    pputs(prn, _(adjust_rc));
-	}
-	else if (err == VARNAMES_INVALID) {
+	} else if (err == VARNAMES_INVALID) {
 	    invalid_varname(prn);
 	}
 	if (err) goto getout; 
@@ -1091,7 +1249,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    }
 	    newinfo->varname[i][0] = 0;
 	    strncat(newinfo->varname[i], 
-		    rowptr[book.row_offset].cells[i_sheet] + 1, VNAMELEN - 1);
+		    rowptr[book.row_offset].cells[i_sheet] + 1, USER_VLEN - 1);
 	    for (t=0; t<newinfo->n; t++) {
 		t_sheet = t + 1 + book.row_offset;
 		if (rowptr[t_sheet].cells == NULL ||
@@ -1105,24 +1263,34 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 			rowptr[t_sheet].cells[i_sheet]);
 #endif
 		newZ[i][t] = atof(rowptr[t_sheet].cells[i_sheet]);
+		if (newZ[i][t] == -999.0) {
+		    newZ[i][t] = NADBL;
+		}
+	    }
+	}
+	
+	if (!err) {
+	    if (fix_varname_duplicates(newinfo)) {
+		pputs(prn, _("warning: some variable names were duplicated\n"));
 	    }
 	}
 
 	if (!err && label_strings) {
-	    char **S = NULL;
+	    char **S;
 
-	    newinfo->markers = 1;
-	    if (allocate_case_markers(&S, newinfo->n) == 0) {
+	    S = allocate_case_markers(newinfo->n);
+	    
+	    if (S != NULL) {
 		newinfo->markers = 1;
 		i = book.col_offset;
 		for (t=0; t<newinfo->n; t++) {
 		    t_sheet = t + 1 + book.row_offset;
-		    S[t][0] = 0;
 		    strncat(S[t], rowptr[t_sheet].cells[i] + 1, OBSLEN - 1);
 		}
 		newinfo->S = S;
 	    }
 	}
+
 	if (*pZ == NULL) {
 	    *pZ = newZ;
 	    *pdinfo = *newinfo;
