@@ -39,6 +39,8 @@
 #define SMALL_HT      1.0e-7 
 
 int global_np;
+double gscale;
+int gncoeff;
 
 #define vix(i,j) ((i) + global_np * (j))
 
@@ -211,12 +213,21 @@ static void print_iter_info (int iter, double *theta, int m, double ll,
 			     int hess, PRN *prn)
 {
     int i;
+    double x;
 
     pprintf(prn, "\n*** %s %d%s: theta, ll ***\n", ("iteration"), iter,
 	    (hess)? _(" (using Hessian)") : "");
     for (i=0; i<m; i++) {
-	if (i && i % 5 == 0) pputc(prn, '\n');
-	pprintf(prn, "%#12.5g ", theta[i]);
+	if (i && i % 5 == 0) {
+	    pputc(prn, '\n');
+	}
+	x = theta[i];
+	if (i < gncoeff - 1) {
+	    x *= gscale;
+	} else if (i == gncoeff - 1) {
+	    x *= gscale * gscale;
+	}
+	pprintf(prn, "%#12.5g ", x);
     }
     pprintf(prn, "\n    ll = %f\n", ll);
 }
@@ -477,7 +488,7 @@ int garch_estimate (int t1, int t2, int nobs,
 		    double *coeff, int nc, double *vcv, 
 		    double *res2, double *res, double *h,
 		    const double *y, double *amax, double *b, 
-		    int *iters, PRN *prn, int vopt)
+		    double scale, int *iters, PRN *prn, int vopt)
 {
     int i, j;
 
@@ -503,9 +514,13 @@ int garch_estimate (int t1, int t2, int nobs,
     q = (int) amax[1];
     p = (int) amax[2];
 
+    /* "export" scale as file-scope global */
+    gscale = scale;
+
     /* number of parameters of unconcentrated likelihood */
     nparam = nc + 1 + q + p;
     global_np = nparam;
+    gncoeff = nc + 1;
 
     if (vs_allocate(&dhdp, &g, &param, &aux3, &svc5, 
 		    &c, &aux, &vc5, &parpre, &partrc,
@@ -576,12 +591,16 @@ int garch_estimate (int t1, int t2, int nobs,
 	fprintf(stderr, "*** Calling garch_info_matrix, count=%d\n", count);	    
 #endif
 
-	garch_info_matrix(t1, t2, X, nx, yhat, c, 
-			  nc, res2, res, y,
-			  toler1, &count, vc5, (const double **) g, 
-			  aux3, param, nparam, b, 
-			  &a0, alfa, beta, q,
-			  p, h, dhdp, zt);
+	err = garch_info_matrix(t1, t2, X, nx, yhat, c, 
+				nc, res2, res, y,
+				toler1, &count, vc5, (const double **) g, 
+				aux3, param, nparam, b, 
+				&a0, alfa, beta, q,
+				p, h, dhdp, zt);
+
+	if (err) {
+	    return E_NOCONV;
+	}
 
 	/* if relative euclidean distance is used as converg. */
 	s_1 = s_2 = 0.0;
@@ -824,7 +843,9 @@ garch_ll (double *c, int nc, double *res2,
 
     ll = 0.0;
     for (t = t1; t <= t2; ++t) {
-	ll -= 0.5 * log(h[t]) + 0.5 * res2[t] / h[t] + LN_SQRT_2_PI;
+	double hts = h[t] * gscale * gscale;
+
+	ll -= 0.5 * log(hts) + 0.5 * res2[t] / h[t] + LN_SQRT_2_PI;
     }
 
     return ll;
@@ -1321,7 +1342,8 @@ garch_info_matrix (int t1, int t2,
     /* invert the information matrix */
     err = invert(vcv, nparam);
     if (err) {
-	fprintf(stderr, "matrix inversion failed\n");
+	fprintf(stderr, "garch_info_matrix: matrix inversion failed\n");
+	return 1;
     }
 
     if (count == NULL) {
@@ -1753,7 +1775,7 @@ garch_full_hessian (int t1, int t2,
     /* invert the Hessian */
     err = invert(vcv, nparam);
     if (err) {
-	fprintf(stderr, "matrix inversion failed\n");
+	fprintf(stderr, "garch_full_hessian: matrix inversion failed\n");
     }
 
     /* Start iteration here */
