@@ -280,10 +280,12 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
     }
 
     if (!err) {
-	for (i=0; i<cols; i++)
+	for (i=0; i<cols; i++) {
 	    if (leftcols[i]) sheet->text_cols += 1;
-	for (t=0; t<rows; t++)
+	}
+	for (t=0; t<rows; t++) {
 	    if (toprows[t]) sheet->text_rows += 1;
+	}
 
 	if (sheet->text_rows > 1) {
 	    pputs(prn, _("Found an extraneous row of text"));
@@ -661,6 +663,10 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    }
 	}
 
+	if (fix_varname_duplicates(newinfo)) {
+	    pputs(prn, _("warning: some variable names were duplicated\n"));
+	}
+
 	if (blank_cols > 0) {
 	    fprintf(stderr, "Dropping %d apparently blank column(s)\n", 
 		    blank_cols);
@@ -668,16 +674,18 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	if (label_strings && wsheet_labels_complete(&sheet)) {
-	    char **S = NULL;
+	    char **S;
 
-	    newinfo->markers = 1;
-	    if (allocate_case_markers(&S, newinfo->n) == 0) {
+	    S = allocate_case_markers(newinfo->n);
+	    if (S != NULL) {
 		newinfo->markers = 1;
-		for (t=0; t<newinfo->n; t++)
+		for (t=0; t<newinfo->n; t++) {
 		    strcpy(S[t], sheet.label[t+1]);
+		}
 		newinfo->S = S;
 	    }
 	}
+
 	if (*pZ == NULL) {
 	    *pZ = newZ;
 	    *pdinfo = *newinfo;
