@@ -137,11 +137,13 @@ static int tramo_x12a_spawn (const char *workdir, const char *fmt, ...)
     if (sout != NULL) g_free(sout);
 
     if (ret != 0) fputc(' ', stderr);
+
     for (i=0; i<nargs; i++) {
 	if (ret != 0) fprintf(stderr, "%s ", argv[i]);
 	free(argv[i]);
     }
     free(argv);
+
     if (ret != 0) fputc('\n', stderr);
     
     return ret;
@@ -168,7 +170,11 @@ static int tx_dialog (tx_request *request)
 {
     GtkWidget *hbox, *vbox, *tmp;
 #if GTK_MAJOR_VERSION >= 2
-    gint ret = 0;
+    gint i, ret = 0;
+
+    for (i=0; i<N_COMMON_OPTS; i++) {
+	request->opt[i].check = NULL;
+    }
 
     request->dialog = 
 	gtk_dialog_new_with_buttons (
@@ -285,14 +291,13 @@ static void truncate (char *str, int n)
     if (len > n) str[n] = 0;
 }
 
-static int graph_series (double **Z, DATAINFO *pdinfo, 
-			 PATHS *paths, int opt)
+static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
 {
     FILE *fp = NULL;
     char title[32];
     int t;
 
-    if (gnuplot_init(paths, PLOT_TRI_GRAPH, &fp)) return E_FOPEN;
+    if (gnuplot_init(PLOT_TRI_GRAPH, &fp)) return E_FOPEN;
 
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "C");
@@ -523,8 +528,9 @@ static void set_opts (tx_request *request)
 
     request->savevars = 0;
 
-    for (i=0; i<4; i++) {
-	if (GTK_TOGGLE_BUTTON(request->opt[i].check)->active) {
+    for (i=0; i<N_COMMON_OPTS; i++) {
+	if (request->opt[i].check != NULL && 
+	    GTK_TOGGLE_BUTTON(request->opt[i].check)->active) {
 	    request->opt[i].save = 1;
 	    if (i != TRIGRAPH) request->savevars++;
 	} else {
@@ -539,7 +545,7 @@ static void cancel_savevars (tx_request *request)
 
     request->savevars = 0;
 
-    for (i=0; i<4; i++) {
+    for (i=0; i<N_COMMON_OPTS; i++) {
 	request->opt[i].save = 0;
     } 
 }
@@ -733,8 +739,7 @@ static int make_x_axis_var (double ***pZ, DATAINFO *pdinfo)
 }
 
 int write_tx_data (char *fname, int varnum, 
-		   double ***pZ, DATAINFO *pdinfo, 
-		   PATHS *paths, int *graph,
+		   double ***pZ, DATAINFO *pdinfo, int *graph, 
 		   const char *prog, const char *workdir,
 		   char *errmsg)
 {
@@ -791,7 +796,10 @@ int write_tx_data (char *fname, int varnum,
 
     /* create little temporary dataset */
     tmpinfo = create_new_dataset(&tmpZ, 4, pdinfo->n, 0);
-    if (tmpinfo == NULL) return E_ALLOC;
+    if (tmpinfo == NULL) {
+	return E_ALLOC;
+    }
+
     copy_basic_data_info(tmpinfo, pdinfo);
 
     if (request.code == X12A) { 
@@ -834,13 +842,13 @@ int write_tx_data (char *fname, int varnum,
 	err = winfork(cmd, workdir, SW_SHOWMINIMIZED, 
 		      CREATE_NEW_CONSOLE | HIGH_PRIORITY_CLASS);
 #elif defined(GLIB2)
-       err = tramo_x12a_spawn(workdir, prog, varname, "-r", "-p", "-q", NULL);
+	err = tramo_x12a_spawn(workdir, prog, varname, "-r", "-p", "-q", NULL);
 #else
-       sprintf(cmd, "cd \"%s\" && \"%s\" %s -r -p -q >/dev/null", 
-	       workdir, prog, varname);
-       err = gretl_spawn(cmd);
+	sprintf(cmd, "cd \"%s\" && \"%s\" %s -r -p -q >/dev/null", 
+		workdir, prog, varname);
+	err = gretl_spawn(cmd);
 #endif
-       } else { /* TRAMO_SEATS */
+    } else { /* TRAMO_SEATS */
 	char seats[MAXLEN];
 
 	/* ensure any stale files get deleted first, just in case */
@@ -897,7 +905,7 @@ int write_tx_data (char *fname, int varnum,
 
 		if (pv < 0) err = 1;
 		if (!err) {
-		    err = graph_series(tmpZ, tmpinfo, paths, request.code);
+		    err = graph_series(tmpZ, tmpinfo, request.code);
 		    if (err) {
 			fprintf(stderr, "graph_series() failed\n");
 		    }
