@@ -774,7 +774,7 @@ int run_gretl_comparison (const char *datname,
 	}
     }
 
-    model = gretl_model_new(dinfo);
+    model = gretl_model_new();
     *model = lsq(list, pZ, dinfo, OLS, OPT_D, 0.0);
 
     if (model->errcode) {
@@ -794,7 +794,7 @@ int run_gretl_comparison (const char *datname,
 	int i;
 
 	model->ID = ++modelnum;
-	printmodel(model, dinfo, prn);
+	printmodel(model, dinfo, OPT_NONE, prn);
 
 	for (i=0; i<model->ncoeff; i++) {
 	    pprintf(prn, " gretl coefficient[%d] = %#.10g\n", i, 
